/**
 * nmr-parser - Read and convert any NMR file
 * @version v1.1.1
 * @link https://github.com/cheminfo/nmr-parser#readme
 * @license MIT
 */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.NMRparser = {}));
}(this, (function (exports) { 'use strict';

  const toString = Object.prototype.toString;
  function isAnyArray(object) {
    return toString.call(object).endsWith('Array]');
  }

  /**
   * a number that correspond to a type of numeric
   * @typedef {number} numericType
   * @const
   */
  const numericTypeTable = {
    0: 'uint8',
    1: 'uint16',
    2: 'uint32',
    3: 'uint64',
    4: 'int8',
    5: 'int16',
    6: 'int32',
    7: 'int64',
    8: 'float32',
    9: 'float64',
    10: 'complex64',
    11: 'complex128'
  };
  /**
   * a number that corresponds to a type of quantity
   * @typedef {number} quantityType
   * @const
   */

  const quantityTypeTable = {
    0: 'scalar',
    1: 'vector',
    2: 'matrix',
    3: 'symetricMatrix',
    4: 'pixel'
  };

  /**
   * a class for dependent variable
   * @param {object || array} data - the dependent variable
   * @param {numericType} numericType - a number that correspond to a type of numeric used to store the components
   * @param {object} [options] - an object with options (name, unit, quantityName, componentLabels, sparseSampling, application, description)
   * @param {string} [options.name] - a name of the dependent variable
   * @param {string} [options.unit] - the unit of the dependent variable
   * @param {string} [options.quantityName] - a name of the quantity
   * @param {array} [options.componentLabels] - an array of labels for each component of the dependent variable
   * @return {object} - an dependent variable
   */

  function formatDependentVariable(data, numericType, options = {}) {
    let {
      quantityType = 0,
      encoding = 'none',
      name = '',
      unit = '',
      quantityName = '',
      componentLabels = [],
      sparseSampling = {},
      from = 0,
      to = -1
    } = options;
    let components;

    if (isAnyArray(data)) {
      throw new Error('not yet implemented');
    } else if (Object.keys(data).length === 2) {
      components = fromReIm(data, from, to);
    }

    if (componentLabels.length === 0) {
      componentLabels = components.componentLabels;
    }

    return {
      type: 'internal',
      quantityType: quantityTypeTable[quantityType],
      numericType: numericTypeTable[numericType],
      encoding,
      name,
      unit,
      quantityName,
      componentLabels,
      sparseSampling,
      description: options.description || '',
      application: options.application || '',
      components: components.components,
      dataLength: components.dataLength
    };
  }
  /**
   * import object {re:[], im:[]} to component
   * @param {object} reIm - a reIm object to import
   * @param {number} from - lower limit
   * @param {number} to - upper limit
   * @return {array} - components
   */

  function fromReIm(reIm, from, to) {
    let dataLength = [];
    let componentLabels = [];
    let components = [];

    if (isAnyArray(reIm.re) & isAnyArray(reIm.im)) {
      if (typeof reIm.re[0] === 'number') {
        // if 1D
        dataLength[0] = setLengthComplex(from[0], to[0], reIm.re.length);
        let component = new Float64Array(dataLength[0]);

        for (let i = 0; i < dataLength[0]; i += 2) {
          let idx = i + from[0] * 2;
          component[i] = reIm.re[idx / 2];
          component[i + 1] = reIm.im[idx / 2];
        }

        components.push(component);
        componentLabels.push('complex');
      } else if (isAnyArray(reIm.re[0])) {
        // if 2D
        dataLength[0] = setLength(from[1], to[1], reIm.re.length);
        dataLength[1] = setLengthComplex(from[0], to[0], reIm.re[0].length);

        for (let j = 0; j < dataLength[0]; j++) {
          let component = new Float64Array(dataLength[1]);

          for (let i = 0; i < dataLength[1]; i += 2) {
            let idx = i + from[0] * 2;
            component[i] = reIm.re[j][idx / 2];
            component[i + 1] = reIm.im[j][idx / 2];
          }

          components.push(component);
        }
      } else {
        throw new Error('check your object');
      }
    } else if (isAnyArray(reIm.re.re)) {
      dataLength[0] = reIm.re.re.length * 2;
      let re = fromReIm(reIm.re, from, to).components;
      let im = fromReIm(reIm.im, from, to).components;

      for (let j = 0; j < dataLength[0] / 2; j++) {
        components.push(re[j]);
        components.push(im[j]);
      }
    } else {
      throw new Error('check the dimension or the type of data in your array');
    }

    return {
      dataLength,
      componentLabels,
      components
    };
  }

  function setLength(from, to, length) {
    if (to - from + 1 < length) {
      return to - from + 1;
    } else {
      return length;
    }
  }

  function setLengthComplex(from, to, length) {
    if (to - from + 1 < length) {
      return (to - from + 1) * 2;
    } else {
      return length * 2;
    }
  } // /**
  //  * add component to components from 1D array.
  //  * @param {array} array - a 1D or 2D array to import
  //  * @return {Float64Array} - component
  //  */
  // function add1DArray(array) {
  //   let component;
  //   component = new Float64Array(array.length);
  //   for (let i = 0; i < array.length; i++) {
  //     component[i] = array[i];
  //   }
  //   return component;
  // }
  // /**
  //  * import component to InternalDEPENDENTVAR class object from 1D or 2D array.
  //  * @param {array} array - a 1D or 2D array to import
  //  */
  // function fromArray(array) {
  //   this.dataLength[0] = array.length;
  //   if (typeof array[0] === 'number') {
  //     this.components = [this.add1DArray(array)];
  //   } else if (Array.isArray(array[0])) {
  //     this.dataLength[1] = array[0].length;
  //     for (let j = 0; j < this.dataLength[1]; j++) {
  //       this.components.push(this.add1DArray(array[j]));
  //     }
  //   } else {
  //     throw new Error('check the dimension or the type of data in your array');
  //   }
  //   return this;
  // }

  /**
   *
   * @param {*} label
   * @param {*} count
   * @param {*} increment
   * @param {*} options
   */
  function formatLinearDimension(label, count, increment, options = {}) {
    return {
      label: String(label),
      count: Number(count),
      increment: increment,
      type: 'linear',
      description: String(options.description) || '',
      application: options.application || {},
      coordinatesOffset: options.coordinatesOffset || 0,
      originOffset: options.originOffset || 0,
      quantityName: String(options.quantityName) || '',
      reciprocal: options.reciprocal || {},
      period: options.period || 0,
      complexFFT: options.complexFFT || false
    };
  }

  // eslint-disable-next-line import/no-unassigned-import
  const decoder = new TextDecoder('utf-8');
  function decode(bytes) {
    return decoder.decode(bytes);
  }
  const encoder = new TextEncoder();
  function encode(str) {
    return encoder.encode(str);
  }

  const defaultByteLength = 1024 * 8;
  class IOBuffer {
    /**
     * @param data - The data to construct the IOBuffer with.
     * If data is a number, it will be the new buffer's length<br>
     * If data is `undefined`, the buffer will be initialized with a default length of 8Kb<br>
     * If data is an ArrayBuffer, SharedArrayBuffer, an ArrayBufferView (Typed Array), an IOBuffer instance,
     * or a Node.js Buffer, a view will be created over the underlying ArrayBuffer.
     * @param options
     */
    constructor(data = defaultByteLength, options = {}) {
      let dataIsGiven = false;

      if (typeof data === 'number') {
        data = new ArrayBuffer(data);
      } else {
        dataIsGiven = true;
        this.lastWrittenByte = data.byteLength;
      }

      const offset = options.offset ? options.offset >>> 0 : 0;
      const byteLength = data.byteLength - offset;
      let dvOffset = offset;

      if (ArrayBuffer.isView(data) || data instanceof IOBuffer) {
        if (data.byteLength !== data.buffer.byteLength) {
          dvOffset = data.byteOffset + offset;
        }

        data = data.buffer;
      }

      if (dataIsGiven) {
        this.lastWrittenByte = byteLength;
      } else {
        this.lastWrittenByte = 0;
      }

      this.buffer = data;
      this.length = byteLength;
      this.byteLength = byteLength;
      this.byteOffset = dvOffset;
      this.offset = 0;
      this.littleEndian = true;
      this._data = new DataView(this.buffer, dvOffset, byteLength);
      this._mark = 0;
      this._marks = [];
    }
    /**
     * Checks if the memory allocated to the buffer is sufficient to store more
     * bytes after the offset.
     * @param byteLength - The needed memory in bytes.
     * @returns `true` if there is sufficient space and `false` otherwise.
     */


    available(byteLength = 1) {
      return this.offset + byteLength <= this.length;
    }
    /**
     * Check if little-endian mode is used for reading and writing multi-byte
     * values.
     * @returns `true` if little-endian mode is used, `false` otherwise.
     */


    isLittleEndian() {
      return this.littleEndian;
    }
    /**
     * Set little-endian mode for reading and writing multi-byte values.
     */


    setLittleEndian() {
      this.littleEndian = true;
      return this;
    }
    /**
     * Check if big-endian mode is used for reading and writing multi-byte values.
     * @returns `true` if big-endian mode is used, `false` otherwise.
     */


    isBigEndian() {
      return !this.littleEndian;
    }
    /**
     * Switches to big-endian mode for reading and writing multi-byte values.
     */


    setBigEndian() {
      this.littleEndian = false;
      return this;
    }
    /**
     * Move the pointer n bytes forward.
     * @param n - Number of bytes to skip.
     */


    skip(n = 1) {
      this.offset += n;
      return this;
    }
    /**
     * Move the pointer to the given offset.
     * @param offset
     */


    seek(offset) {
      this.offset = offset;
      return this;
    }
    /**
     * Store the current pointer offset.
     * @see {@link IOBuffer#reset}
     */


    mark() {
      this._mark = this.offset;
      return this;
    }
    /**
     * Move the pointer back to the last pointer offset set by mark.
     * @see {@link IOBuffer#mark}
     */


    reset() {
      this.offset = this._mark;
      return this;
    }
    /**
     * Push the current pointer offset to the mark stack.
     * @see {@link IOBuffer#popMark}
     */


    pushMark() {
      this._marks.push(this.offset);

      return this;
    }
    /**
     * Pop the last pointer offset from the mark stack, and set the current
     * pointer offset to the popped value.
     * @see {@link IOBuffer#pushMark}
     */


    popMark() {
      const offset = this._marks.pop();

      if (offset === undefined) {
        throw new Error('Mark stack empty');
      }

      this.seek(offset);
      return this;
    }
    /**
     * Move the pointer offset back to 0.
     */


    rewind() {
      this.offset = 0;
      return this;
    }
    /**
     * Make sure the buffer has sufficient memory to write a given byteLength at
     * the current pointer offset.
     * If the buffer's memory is insufficient, this method will create a new
     * buffer (a copy) with a length that is twice (byteLength + current offset).
     * @param byteLength
     */


    ensureAvailable(byteLength = 1) {
      if (!this.available(byteLength)) {
        const lengthNeeded = this.offset + byteLength;
        const newLength = lengthNeeded * 2;
        const newArray = new Uint8Array(newLength);
        newArray.set(new Uint8Array(this.buffer));
        this.buffer = newArray.buffer;
        this.length = this.byteLength = newLength;
        this._data = new DataView(this.buffer);
      }

      return this;
    }
    /**
     * Read a byte and return false if the byte's value is 0, or true otherwise.
     * Moves pointer forward by one byte.
     */


    readBoolean() {
      return this.readUint8() !== 0;
    }
    /**
     * Read a signed 8-bit integer and move pointer forward by 1 byte.
     */


    readInt8() {
      return this._data.getInt8(this.offset++);
    }
    /**
     * Read an unsigned 8-bit integer and move pointer forward by 1 byte.
     */


    readUint8() {
      return this._data.getUint8(this.offset++);
    }
    /**
     * Alias for {@link IOBuffer#readUint8}.
     */


    readByte() {
      return this.readUint8();
    }
    /**
     * Read `n` bytes and move pointer forward by `n` bytes.
     */


    readBytes(n = 1) {
      const bytes = new Uint8Array(n);

      for (let i = 0; i < n; i++) {
        bytes[i] = this.readByte();
      }

      return bytes;
    }
    /**
     * Read a 16-bit signed integer and move pointer forward by 2 bytes.
     */


    readInt16() {
      const value = this._data.getInt16(this.offset, this.littleEndian);

      this.offset += 2;
      return value;
    }
    /**
     * Read a 16-bit unsigned integer and move pointer forward by 2 bytes.
     */


    readUint16() {
      const value = this._data.getUint16(this.offset, this.littleEndian);

      this.offset += 2;
      return value;
    }
    /**
     * Read a 32-bit signed integer and move pointer forward by 4 bytes.
     */


    readInt32() {
      const value = this._data.getInt32(this.offset, this.littleEndian);

      this.offset += 4;
      return value;
    }
    /**
     * Read a 32-bit unsigned integer and move pointer forward by 4 bytes.
     */


    readUint32() {
      const value = this._data.getUint32(this.offset, this.littleEndian);

      this.offset += 4;
      return value;
    }
    /**
     * Read a 32-bit floating number and move pointer forward by 4 bytes.
     */


    readFloat32() {
      const value = this._data.getFloat32(this.offset, this.littleEndian);

      this.offset += 4;
      return value;
    }
    /**
     * Read a 64-bit floating number and move pointer forward by 8 bytes.
     */


    readFloat64() {
      const value = this._data.getFloat64(this.offset, this.littleEndian);

      this.offset += 8;
      return value;
    }
    /**
     * Read a 1-byte ASCII character and move pointer forward by 1 byte.
     */


    readChar() {
      return String.fromCharCode(this.readInt8());
    }
    /**
     * Read `n` 1-byte ASCII characters and move pointer forward by `n` bytes.
     */


    readChars(n = 1) {
      let result = '';

      for (let i = 0; i < n; i++) {
        result += this.readChar();
      }

      return result;
    }
    /**
     * Read the next `n` bytes, return a UTF-8 decoded string and move pointer
     * forward by `n` bytes.
     */


    readUtf8(n = 1) {
      return decode(this.readBytes(n));
    }
    /**
     * Write 0xff if the passed value is truthy, 0x00 otherwise and move pointer
     * forward by 1 byte.
     */


    writeBoolean(value) {
      this.writeUint8(value ? 0xff : 0x00);
      return this;
    }
    /**
     * Write `value` as an 8-bit signed integer and move pointer forward by 1 byte.
     */


    writeInt8(value) {
      this.ensureAvailable(1);

      this._data.setInt8(this.offset++, value);

      this._updateLastWrittenByte();

      return this;
    }
    /**
     * Write `value` as an 8-bit unsigned integer and move pointer forward by 1
     * byte.
     */


    writeUint8(value) {
      this.ensureAvailable(1);

      this._data.setUint8(this.offset++, value);

      this._updateLastWrittenByte();

      return this;
    }
    /**
     * An alias for {@link IOBuffer#writeUint8}.
     */


    writeByte(value) {
      return this.writeUint8(value);
    }
    /**
     * Write all elements of `bytes` as uint8 values and move pointer forward by
     * `bytes.length` bytes.
     */


    writeBytes(bytes) {
      this.ensureAvailable(bytes.length);

      for (let i = 0; i < bytes.length; i++) {
        this._data.setUint8(this.offset++, bytes[i]);
      }

      this._updateLastWrittenByte();

      return this;
    }
    /**
     * Write `value` as a 16-bit signed integer and move pointer forward by 2
     * bytes.
     */


    writeInt16(value) {
      this.ensureAvailable(2);

      this._data.setInt16(this.offset, value, this.littleEndian);

      this.offset += 2;

      this._updateLastWrittenByte();

      return this;
    }
    /**
     * Write `value` as a 16-bit unsigned integer and move pointer forward by 2
     * bytes.
     */


    writeUint16(value) {
      this.ensureAvailable(2);

      this._data.setUint16(this.offset, value, this.littleEndian);

      this.offset += 2;

      this._updateLastWrittenByte();

      return this;
    }
    /**
     * Write `value` as a 32-bit signed integer and move pointer forward by 4
     * bytes.
     */


    writeInt32(value) {
      this.ensureAvailable(4);

      this._data.setInt32(this.offset, value, this.littleEndian);

      this.offset += 4;

      this._updateLastWrittenByte();

      return this;
    }
    /**
     * Write `value` as a 32-bit unsigned integer and move pointer forward by 4
     * bytes.
     */


    writeUint32(value) {
      this.ensureAvailable(4);

      this._data.setUint32(this.offset, value, this.littleEndian);

      this.offset += 4;

      this._updateLastWrittenByte();

      return this;
    }
    /**
     * Write `value` as a 32-bit floating number and move pointer forward by 4
     * bytes.
     */


    writeFloat32(value) {
      this.ensureAvailable(4);

      this._data.setFloat32(this.offset, value, this.littleEndian);

      this.offset += 4;

      this._updateLastWrittenByte();

      return this;
    }
    /**
     * Write `value` as a 64-bit floating number and move pointer forward by 8
     * bytes.
     */


    writeFloat64(value) {
      this.ensureAvailable(8);

      this._data.setFloat64(this.offset, value, this.littleEndian);

      this.offset += 8;

      this._updateLastWrittenByte();

      return this;
    }
    /**
     * Write the charCode of `str`'s first character as an 8-bit unsigned integer
     * and move pointer forward by 1 byte.
     */


    writeChar(str) {
      return this.writeUint8(str.charCodeAt(0));
    }
    /**
     * Write the charCodes of all `str`'s characters as 8-bit unsigned integers
     * and move pointer forward by `str.length` bytes.
     */


    writeChars(str) {
      for (let i = 0; i < str.length; i++) {
        this.writeUint8(str.charCodeAt(i));
      }

      return this;
    }
    /**
     * UTF-8 encode and write `str` to the current pointer offset and move pointer
     * forward according to the encoded length.
     */


    writeUtf8(str) {
      return this.writeBytes(encode(str));
    }
    /**
     * Export a Uint8Array view of the internal buffer.
     * The view starts at the byte offset and its length
     * is calculated to stop at the last written byte or the original length.
     */


    toArray() {
      return new Uint8Array(this.buffer, this.byteOffset, this.lastWrittenByte);
    }
    /**
     * Update the last written byte offset
     * @private
     */


    _updateLastWrittenByte() {
      if (this.offset > this.lastWrittenByte) {
        this.lastWrittenByte = this.offset;
      }
    }

  }

  const endianness = {
    0: 'bigEndian',
    1: 'littleEndian'
  };
  const instrumentTable = {
    0: 'NONE',
    1: 'GSX',
    2: 'ALPHA',
    3: 'ECLIPSE',
    4: 'MASS_SPEC',
    5: 'COMPILER',
    6: 'OTHER_NMR',
    7: 'UNKNOWN',
    8: 'GEMINI',
    9: 'UNITY',
    10: 'ASPECT',
    11: 'UX',
    12: 'FELIX',
    13: 'LAMBDA',
    14: 'GE_1280',
    15: 'GE_OMEGA',
    16: 'CHEMAGNETICS',
    17: 'CDFF',
    18: 'GALACTIC',
    19: 'TRIAD',
    20: 'GENERIC_NMR',
    21: 'GAMMA',
    22: 'JCAMP_DX',
    23: 'AMX',
    24: 'DMX',
    25: 'ECA',
    26: 'ALICE',
    27: 'NMR_PIPE',
    28: 'SIMPSON'
  };
  const dataTypeTable = {
    0: '64Bit Float',
    1: '32Bit Float',
    2: 'Reserved',
    3: 'Reserved'
  };
  const dataFormatTable = {
    1: 'One_D',
    2: 'Two_D',
    3: 'Three_D',
    4: 'Four_D',
    5: 'Five_D',
    6: 'Six_D',
    7: 'Seven_D',
    8: 'Eight_D',
    9: 'not for NMR data formats',
    10: 'not for NMR data formats',
    11: 'not for NMR data formats',
    12: 'Small_Two_D',
    13: 'Small_Three_D',
    14: 'Small_Four_D'
  };
  const dataAxisTypeTable = {
    0: 'None',
    //Axis is not used.
    1: 'Real',
    //Axis has real data only, no imaginary.
    2: 'TPPI',
    3: 'Complex',
    4: 'Real_Complex',

    /* Axis should be accessed as complex when it is the major axis,
              accessed as real otherwise.  This is only valid when all axes in
              use have this setting.*/
    5: 'Envelope'
    /* Behaves the same way as a Real_Complex dimension but the data
          has different meaning.  Instead of being treated as real and
          imaginary parts of a complex number, the data should be treated as minimum and maximum parts of a projection.  This is used
          for the data that results from an envelope projection.*/

  };
  const prefixTable = {
    '-8': 'Yotta',
    '-6': 'Exa',
    '-7': 'Zetta',
    '-5': 'Pecta',
    '-4': 'Tera',
    '-3': 'Giga',
    '-2': 'Mega',
    '-1': 'Kilo',
    0: 'None',
    1: 'Milli',
    2: 'Micro',
    3: 'Nano',
    4: 'Pico',
    5: 'Femto',
    6: 'Atto',
    7: 'Zepto'
  };
  const unitPrefixTable = {
    Yotta: 24,
    Exa: 21,
    Zetta: 18,
    Pecta: 15,
    Tera: 12,
    Giga: 9,
    Mega: 6,
    Kilo: 3,
    None: 0,
    Milli: -3,
    Micro: -6,
    Nano: -9,
    Pico: -12,
    Femto: -15,
    Atto: -18,
    Zepto: -21
  };
  const baseTable = {
    0: 'None',
    1: 'Abundance',
    2: 'Ampere',
    3: 'Candela',
    4: 'Celsius',
    5: 'Coulomb',
    6: 'Degree',
    7: 'Electronvolt',
    8: 'Farad',
    9: 'Sievert',
    10: 'Gram',
    11: 'Gray',
    12: 'Henry',
    13: 'Hertz',
    14: 'Kelvin',
    15: 'Joule',
    16: 'Liter',
    17: 'Lumen',
    18: 'Lux',
    19: 'Meter',
    20: 'Mole',
    21: 'Newton',
    22: 'Ohm',
    23: 'Pascal',
    24: 'Percent',
    25: 'Point',
    26: 'Ppm',
    27: 'Radian',
    28: 'Second',
    29: 'Siemens',
    30: 'Steradian',
    31: 'Tesla',
    32: 'Volt',
    33: 'Watt',
    34: 'Weber',
    35: 'Decibel',
    36: 'Dalton',
    37: 'Thompson',
    38: 'Ugeneric',
    // Treated as None, but never displayed',
    39: 'LPercent ',
    // Treated as percent for display, but different for comparison',
    40: 'PPT',
    // Parts per trillion (Private, do not use)',
    41: 'PPB ',
    // Parts per billion (Private, do not use)',
    42: 'Index'
  };
  const dataAxisRangedTable = {
    0: 'Ranged',

    /* The ruler for the axis ranges from Data_Axis_Start[n] to
          Data_Axis_Stop[n] with a step function of
              (Data_Axis_Stop[n] - Data_Axis_Start[n]) /
              (Data_Offset_Stop[n] - Data_Offset_Start[n]) */
    1: 'Listed',
    // (deprecated)

    /* The ruler for the axis is a list of doubles stored in the
          List Section.  Values in the ruler may be anything.*/
    2: 'Sparse',

    /*The ruler for the axis is a list of doubles stored in the
          List Section.  Values in the rulers must be strictly monotonically
          increasing or decreasing.*/
    3: 'Listed'
    /* The ruler for the axis is a list of doubles stored in the
          List Section.  Values in the rulers do not fit definition of Sparse.*/

  };
  const valueTypeTable = {
    0: 'String',
    1: 'Integer',
    2: 'Float',
    3: 'Complex',
    4: 'Infinity'
  };

  function getPar(param, searchStr) {
    return param.paramArray.find(o => o.name === searchStr) || '';
  }
  function getDigitalFilter(param) {
    const orders = param.paramArray.find(e => e.name === 'orders');
    const factors = param.paramArray.find(e => e.name === 'factors');
    const sweep = param.paramArray.find(e => e.name === 'X_SWEEP');
    const acqTime = param.paramArray.find(e => e.name === 'x_acq_time');
    const nbPoints = param.paramArray.find(e => e.name === 'X_POINTS');
    const s = parseInt(orders.value.slice(0, 1), 10);
    const jump = orders.value.slice(1).length / s;
    let arg = 0;
    let factorNumber = new Int8Array(s);
    let offsetO = 1;
    let offsetF = 0;

    for (let i = 0; i < s; i++) {
      factorNumber[i] = parseInt(factors.value.slice(offsetF, offsetF + 1), 10);
      offsetF += 1;
    }

    for (let i = 0; i < s; i++) {
      let productorial = 1;

      for (let j = i; j < s; j++) {
        productorial *= factorNumber[j];
      }

      arg += (parseInt(orders.value.slice(offsetO, offsetO + jump), 10) - 1) / productorial;
      offsetO += jump;
    }

    arg /= 2;
    const delaySec = arg / sweep.value;
    return delaySec / acqTime.value * (nbPoints.value - 1);
  }
  function getMagnitude(param, searchStr) {
    let par = getPar(param, searchStr) || 'NA';

    if (par === 'NA') {
      return {
        magnitude: 'NA',
        unit: 'NA'
      };
    }

    let unit = par.unit[0].base;
    let unitMult = unitPrefixTable[par.unit[0].prefix];
    let magnitude = par.value * 10 ** unitMult;
    return {
      magnitude,
      unit
    };
  }
  function getUnit(buffer, size) {
    let unit = [];

    for (let i = 0; i < size; i++) {
      let byte = buffer.readByte();
      let prefix = prefixTable[byte >> 4];
      let power = byte & 0b00001111;
      let base = baseTable[buffer.readInt8()];
      unit.push({
        prefix,
        power,
        base
      });
    }

    return unit;
  }
  function getString(buffer, size) {
    let string = [];

    for (let i = 0; i < size; i++) {
      let char = buffer.readChar();

      if (char !== '\u0000') {
        string.push(char);
      }
    }

    return string.join('');
  }
  function getParamName(buffer, size) {
    let string = [];

    for (let i = 0; i < size; i++) {
      let char = buffer.readChar();

      if (char !== ' ') {
        string.push(char);
      }
    }

    return string.join('');
  }
  function getArray(buffer, size, format) {
    let double = [];

    for (let i = 0; i < size; i++) {
      switch (format) {
        case 'readUint32':
          double.push(buffer.readUint32());
          break;

        case 'readFloat64':
          double.push(buffer.readFloat64());
          break;

        case 'readFloat32':
          double.push(buffer.readFloat32());
          break;

        case 'readUint8':
          double.push(buffer.readUint8());
          break;

        case 'readBoolean':
          double.push(buffer.readBoolean());
          break;
      }
    }

    return double;
  }

  /**
   * A parser for 1D and 2D JDL NMR Files
   * @param {ArrayBuffer} buffer - a buffer object containing the JDL file
   * @return {Object} - an Object with converted data
   */

  function parseJEOL(buffer) {
    let ioBuffer = new IOBuffer(buffer);
    ioBuffer.setBigEndian(); // read header section

    let byte;
    let header = {};
    let byteArray = [];
    header.fileIdentifier = ioBuffer.readChars(8);
    header.endian = endianness[ioBuffer.readInt8()];
    header.majorVersion = ioBuffer.readUint8();
    header.minorVersion = ioBuffer.readUint16();
    header.dataDimensionNumber = ioBuffer.readUint8();
    header.dataDimensionExist = ioBuffer.readByte().toString(2).split('').map(x => Boolean(Number(x)));
    byte = ioBuffer.readByte();
    header.dataType = dataTypeTable[byte >> 6];
    header.dataFormat = dataFormatTable[byte & 0b00111111];
    header.dataInstrument = instrumentTable[ioBuffer.readInt8()];
    header.translate = getArray(ioBuffer, 8, 'readUint8');
    header.dataAxisType = getArray(ioBuffer, 8, 'readUint8').map(x => dataAxisTypeTable[x]);
    header.dataUnits = getUnit(ioBuffer, 8);
    header.title = getString(ioBuffer, 124);

    for (byte in getArray(ioBuffer, 4, 'readUint8')) {
      byteArray.push(dataAxisRangedTable[byte >> 4]);
      byteArray.push(dataAxisRangedTable[byte & 0b00001111]);
    }

    header.dataAxisRanged = byteArray;
    header.dataPoints = getArray(ioBuffer, 8, 'readUint32');
    header.dataOffsetStart = getArray(ioBuffer, 8, 'readUint32');
    header.dataOffsetStop = getArray(ioBuffer, 8, 'readUint32');
    header.dataAxisStart = getArray(ioBuffer, 8, 'readFloat64');
    header.dataAxisStop = getArray(ioBuffer, 8, 'readFloat64');
    byteArray = new Uint8Array(4);

    for (let i = 0; i < 4; i++) {
      byteArray[i] = ioBuffer.readByte();
    }

    let year = 1990 + (byteArray[0] >> 1);
    let month = (byteArray[0] << 3 & 0b00001000) + (byteArray[1] >> 5);
    let day = byteArray[2] & 0b00011111;
    header.creationTime = {
      year,
      month,
      day
    };

    for (let i = 0; i < 4; i++) {
      byteArray[i] = ioBuffer.readByte();
    }

    year = 1990 + (byteArray[0] >> 1);
    month = (byteArray[0] << 3 & 0b00001000) + (byteArray[1] >> 5);
    day = byteArray[2] & 0b00011111;
    header.revisionTime = {
      year,
      month,
      day
    };
    header.nodeName = getString(ioBuffer, 16);
    header.site = getString(ioBuffer, 128);
    header.author = getString(ioBuffer, 128);
    header.comment = getString(ioBuffer, 128);
    let dataAxisTitles = [];

    for (let i = 0; i < 8; i++) {
      dataAxisTitles.push(getString(ioBuffer, 32));
    }

    header.dataAxisTitles = dataAxisTitles;
    header.baseFreq = getArray(ioBuffer, 8, 'readFloat64');
    header.zeroPoint = getArray(ioBuffer, 8, 'readFloat64');
    header.reversed = getArray(ioBuffer, 8, 'readBoolean');
    ioBuffer.skip(3);
    header.annotationOK = Boolean(ioBuffer.readByte() >> 7);
    header.historyUsed = ioBuffer.readUint32();
    header.historyLength = ioBuffer.readUint32();
    header.paramStart = ioBuffer.readUint32();
    header.paramLength = ioBuffer.readUint32();
    header.ListStart = getArray(ioBuffer, 8, 'readUint32');
    header.ListLength = getArray(ioBuffer, 8, 'readUint32');
    header.dataStart = ioBuffer.readUint32();
    header.dataLength = ioBuffer.readUint32() << 32 | ioBuffer.readUint32();
    header.contextStart = ioBuffer.readUint32() << 32 | ioBuffer.readUint32();
    header.contextLength = ioBuffer.readUint32();
    header.annoteStart = ioBuffer.readUint32() << 32 | ioBuffer.readUint32();
    header.annoteLength = ioBuffer.readUint32();
    header.totalSize = ioBuffer.readUint32() << 32 | ioBuffer.readUint32();
    header.unitLocation = getArray(ioBuffer, 8, 'readUint8');
    let compoundUnit = [];

    for (let i = 0; i < 2; i++) {
      let unit = [];
      let scaler = ioBuffer.readInt16();

      for (let j = 0; j < 5; j++) {
        byte = ioBuffer.readInt16();
        unit.push(byte);
      }

      compoundUnit.push({
        scaler,
        unit
      });
    }

    header.compoundUnit = compoundUnit; // section parameters (param header and array)

    if (header.endian === 'littleEndian') {
      ioBuffer.setLittleEndian();
    }

    ioBuffer.seek(header.paramStart);
    let parameters = {
      parameterSize: ioBuffer.readUint32(),
      lowIndex: ioBuffer.readUint32(),
      highIndex: ioBuffer.readUint32(),
      totalSize: ioBuffer.readUint32()
    };
    let paramArray = [];

    for (let p = 0; p < parameters.highIndex + 1; p++) {
      ioBuffer.skip(4);
      let scaler = ioBuffer.readInt16();
      let unit = getUnit(ioBuffer, 5);
      ioBuffer.skip(16);
      let valueType = valueTypeTable[ioBuffer.readInt32()];
      ioBuffer.seek(ioBuffer.offset - 20);
      let value;

      switch (valueType) {
        case 'String':
          value = getParamName(ioBuffer, 16);
          break;

        case 'Integer':
          value = ioBuffer.readInt32();
          ioBuffer.skip(12);
          break;

        case 'Float':
          value = ioBuffer.readFloat64();
          ioBuffer.skip(8);
          break;

        case 'Complex':
          value.Real = ioBuffer.readFloat64();
          value.Imag = ioBuffer.readFloat64();
          break;

        case 'Infinity':
          value = ioBuffer.readInt32();
          ioBuffer.skip(12);
          break;

        default:
          ioBuffer.skip(16);
          break;
      }

      ioBuffer.skip(4);
      let name = getParamName(ioBuffer, 28);
      paramArray.push({
        name,
        scaler,
        unit,
        value,
        valueType
      });
    }

    parameters.paramArray = paramArray; // data section

    ioBuffer.seek(header.dataStart);

    if (header.endian === 'littleEndian') {
      ioBuffer.setLittleEndian();
    }

    let data = {};
    let dataSectionCount = 1;
    let realComplex = 0;

    for (let type of header.dataAxisType) {
      if (type === 'Real_Complex' && realComplex === 0) {
        dataSectionCount += 1;
        realComplex += 1;
      }

      if (type === 'Complex') {
        dataSectionCount *= 2;
      }
    }

    if (header.dataFormat !== 'One_D' && header.dataFormat !== 'Two_D') {
      throw new Error('Only One_D and two_D data formats are implemented yet');
    }

    if (header.dataFormat === 'One_D') {
      for (let s = 0; s < dataSectionCount; s++) {
        let section;

        if (header.dataType === '32Bit Float') {
          section = getArray(ioBuffer, header.dataPoints[0], 'readFloat32');
        } else if (header.dataType === '64Bit Float') {
          section = getArray(ioBuffer, header.dataPoints[0], 'readFloat64');
        }

        if (s === 0) data.re = section;
        if (s === 1) data.im = section;
      }
    }

    if (header.dataFormat === 'Two_D') {
      let me = 32;
      let dim1 = header.dataPoints[0];
      let dim2 = header.dataPoints[1]; // console.log(
      // `dim1: ${dim1},
      // dim2: ${dim2},
      // total: ${dim1 * dim2},
      // total(byte): ${dim1 * dim2 * 8},
      // total(length): ${dim1 * dim2 * 8 * dataSectionCount}
      // m size: ${dim1 / me} / ${dim2 / me}`,
      // );

      let I = dim2 / me;
      let J = dim1 / me;

      for (let s = 0; s < dataSectionCount; s++) {
        let section;

        for (let i = 0; i < I; i++) {
          let row = [];

          for (let j = 0; j < J; j++) {
            for (let k = 0; k < me; k++) {
              if (j === 0) {
                if (header.dataType === '32Bit Float') {
                  row[k] = getArray(ioBuffer, me, 'readFloat32');
                } else if (header.dataType === '64Bit Float') {
                  row[k] = getArray(ioBuffer, me, 'readFloat64');
                }
              } else {
                if (header.dataType === '32Bit Float') {
                  row[k] = row[k].concat(getArray(ioBuffer, me, 'readFloat32'));
                } else if (header.dataType === '64Bit Float') {
                  row[k] = row[k].concat(getArray(ioBuffer, me, 'readFloat64'));
                }
              }
            }
          }

          if (i === 0) {
            section = row;
          } else {
            section = section.concat(row);
          }
        }

        if (dataSectionCount === 2) {
          if (s === 0) data.re = section;
          if (s === 1) data.im = section;
        }

        if (dataSectionCount === 4) {
          if (s === 0) {
            data.re = {};
            data.re.re = section;
          }

          if (s === 1) data.re.im = section;

          if (s === 2) {
            data.im = {};
            data.im.re = section;
          }

          if (s === 3) data.im.im = section;
        }
      }
    } // format output


    let nucleus = [];
    let acquisitionTime = [];
    let spectralWidth = [];
    let spectralWidthClipped = [];
    let resolution = [];
    let originFrequency = [];
    let frequencyOffset = [];
    let dataUnits = [];

    if (header.dataFormat === 'One_D' || header.dataFormat === 'Two_D') {
      nucleus.push(getPar(parameters, 'X_DOMAIN').value);
      acquisitionTime.push(getMagnitude(parameters, 'x_acq_time'));
      spectralWidth.push(getMagnitude(parameters, 'X_SWEEP'));
      spectralWidthClipped.push(getMagnitude(parameters, 'X_SWEEP_CLIPPED'));
      resolution.push(getMagnitude(parameters, 'X_RESOLUTION'));
      originFrequency.push(getMagnitude(parameters, 'X_FREQ'));
      frequencyOffset.push(getMagnitude(parameters, 'X_OFFSET'));
      dataUnits.push(header.dataUnits[0].base);
    }

    if (header.dataFormat === 'Two_D') {
      nucleus.push(getPar(parameters, 'Y_DOMAIN').value);
      acquisitionTime.push(getMagnitude(parameters, 'y_acq_time'));
      spectralWidth.push(getMagnitude(parameters, 'Y_SWEEP'));
      resolution.push(getMagnitude(parameters, 'Y_RESOLUTION'));
      originFrequency.push(getMagnitude(parameters, 'Y_FREQ'));
      frequencyOffset.push(getMagnitude(parameters, 'Y_OFFSET'));
      dataUnits.push(header.dataUnits[1].base);
    }

    let digest = {
      info: {
        sampleName: getPar(parameters, 'sample_id').value,
        creationTime: header.creationTime,
        revisionTime: header.revisionTime,
        author: header.author,
        comment: header.comment,
        solvent: getPar(parameters, 'solvent').value,
        temperature: getMagnitude(parameters, 'temp_get'),
        probeName: getPar(parameters, 'probe_id').value,
        fieldStrength: getMagnitude(parameters, 'field_strength'),
        experiment: getPar(parameters, 'experiment').value,
        dimension: header.dataDimensionNumber,
        nucleus: nucleus,
        pulseStrength90: getMagnitude(parameters, 'X90'),
        numberOfScans: getPar(parameters, 'SCANS').value,
        relaxationTime: getMagnitude(parameters, 'relaxation_delay'),
        dataPoints: header.dataPoints.slice(0, header.dataDimensionNumber),
        dataOffsetStart: header.dataOffsetStart,
        dataOffsetStop: header.dataOffsetStop,
        dataUnits: dataUnits,
        dataSections: Object.keys(data),
        originFrequency,
        frequencyOffset,
        acquisitionTime,
        spectralWidth,
        spectralWidthClipped,
        dataAxisStart: header.dataAxisStart,
        dataAxisStop: header.dataAxisStop,
        resolution: resolution,
        digitalFilter: getDigitalFilter(parameters),
        //getPar(parameters, 'FILTER_FACTOR').value,
        decimationRate: getPar(parameters, 'decimation_rate').value,
        paramList: JSON.stringify(parameters.paramArray.map(par => par.name))
      },
      headers: header,
      parameters: parameters,
      data: data
    };
    return digest;
  }

  var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

  function createCommonjsModule(fn) {
    var module = { exports: {} };
  	return fn(module, module.exports), module.exports;
  }

  function commonjsRequire (target) {
  	throw new Error('Could not dynamically require "' + target + '". Please configure the dynamicRequireTargets option of @rollup/plugin-commonjs appropriately for this require call to behave properly.');
  }

  var medianQuickselect_min = createCommonjsModule(function (module) {
    (function () {
      function a(d) {
        for (var e = 0, f = d.length - 1, g = void 0, h = void 0, i = void 0, j = c(e, f); !0;) {
          if (f <= e) return d[j];
          if (f == e + 1) return d[e] > d[f] && b(d, e, f), d[j];

          for (g = c(e, f), d[g] > d[f] && b(d, g, f), d[e] > d[f] && b(d, e, f), d[g] > d[e] && b(d, g, e), b(d, g, e + 1), h = e + 1, i = f; !0;) {
            do h++; while (d[e] > d[h]);

            do i--; while (d[i] > d[e]);

            if (i < h) break;
            b(d, h, i);
          }

          b(d, e, i), i <= j && (e = h), i >= j && (f = i - 1);
        }
      }

      var b = function b(d, e, f) {
        var _ref;

        return _ref = [d[f], d[e]], d[e] = _ref[0], d[f] = _ref[1], _ref;
      },
          c = function c(d, e) {
        return ~~((d + e) / 2);
      };

      module.exports ? module.exports = a : window.median = a;
    })();
  });

  function median(input) {
    if (!isAnyArray(input)) {
      throw new TypeError('input must be an array');
    }

    if (input.length === 0) {
      throw new TypeError('input must not be empty');
    }

    return medianQuickselect_min(input.slice());
  }

  // sources:
  // https://en.wikipedia.org/wiki/Gyromagnetic_ratio
  // TODO: #13 can we have a better source and more digits ? @jwist
  const gyromagneticRatio = {
    '1H': 267.52218744e6,
    '2H': 41.065e6,
    '3H': 285.3508e6,
    '3He': -203.789e6,
    '7Li': 103.962e6,
    '13C': 67.28284e6,
    '14N': 19.331e6,
    '15N': -27.116e6,
    '17O': -36.264e6,
    '19F': 251.662e6,
    '23Na': 70.761e6,
    '27Al': 69.763e6,
    '29Si': -53.19e6,
    '31P': 108.291e6,
    '57Fe': 8.681e6,
    '63Cu': 71.118e6,
    '67Zn': 16.767e6,
    '129Xe': -73.997e6
  };

  var name = "nmr-parser";
  var version = "1.1.1";
  var description = "Read and convert any NMR file";
  var main = "lib/index.js";
  var module = "src/index.js";
  var files = [
  	"lib",
  	"src"
  ];
  var scripts = {
  	eslint: "eslint src",
  	"eslint-fix": "npm run eslint -- --fix",
  	compile: "rollup -c",
  	prepublishOnly: "npm run compile",
  	test: "npm run compile && npm run test-coverage && npm run eslint",
  	"test-coverage": "jest --coverage",
  	"test-only": "jest",
  	build: "cheminfo-build --entry src/index.js --root NMRparser"
  };
  var repository = {
  	type: "git",
  	url: "git+https://github.com/cheminfo/nmr-parser.git"
  };
  var keywords = [
  	"nmr",
  	"magnetic resonance",
  	"parser",
  	"bruker",
  	"JEOL",
  	"CSDM",
  	"data analysis"
  ];
  var author = "Julien Wist";
  var license = "MIT";
  var bugs = {
  	url: "https://github.com/cheminfo/nmr-parser/issues"
  };
  var homepage = "https://github.com/cheminfo/nmr-parser#readme";
  var jest = {
  	testEnvironment: "node"
  };
  var prettier = {
  	arrowParens: "always",
  	semi: true,
  	singleQuote: true,
  	tabWidth: 2,
  	trailingComma: "all"
  };
  var devDependencies = {
  	"@babel/plugin-transform-modules-commonjs": "^7.14.0",
  	"@rollup/plugin-json": "^4.1.0",
  	"@types/jest": "^26.0.23",
  	"bruker-data-test": "^0.1.0",
  	"cheminfo-build": "^1.1.10",
  	eslint: "^7.25.0",
  	"eslint-config-cheminfo": "^5.2.3",
  	"jcamp-data-test": "^0.0.5",
  	"jeol-data-test": "^0.2.3",
  	jest: "^26.6.3",
  	prettier: "^2.2.1",
  	rollup: "^2.47.0"
  };
  var dependencies = {
  	brukerconverter: "^3.3.0",
  	"is-any-array": "^1.0.0",
  	jcampconverter: "^8.1.1",
  	jeolconverter: "^1.0.0",
  	"nmr-processing": "^1.2.0"
  };
  var packageJson = {
  	name: name,
  	version: version,
  	description: description,
  	main: main,
  	module: module,
  	files: files,
  	scripts: scripts,
  	repository: repository,
  	keywords: keywords,
  	author: author,
  	license: license,
  	bugs: bugs,
  	homepage: homepage,
  	jest: jest,
  	prettier: prettier,
  	devDependencies: devDependencies,
  	dependencies: dependencies
  };

  function toKeyValue(object) {
    let newObject = {};

    for (let key in object) {
      if (typeof object[key] !== 'string') {
        newObject[key] = JSON.stringify(object[key]);
      } else {
        newObject[key] = object[key];
      }
    }

    return newObject;
  }

  function fromJEOL(buffer) {
    let parsedData = parseJEOL(buffer);
    let info = parsedData.info;
    let headers = parsedData.headers;
    let parameters = parsedData.parameters;
    let paramArray = Object.assign({}, parameters.paramArray);
    delete parameters.paramArray;
    let data = parsedData.data; // curation of parameters

    let newInfo = {};
    newInfo.title = `title: ${headers.title} / comment: ${headers.comment} / author:${headers.author} / site: ${headers.site}`;
    newInfo.nucleus = info.nucleus.map(x => {
      if (x === 'Proton') {
        x = '1H';
      }

      if (x === 'Carbon13') {
        x = '13C';
      }

      if (x === 'Nitrogen15') {
        x = '15N';
      }

      return x;
    });
    newInfo.sampleName = info.sampleName;
    newInfo.date = JSON.stringify(info.creationTime);
    newInfo.author = info.author; //newInfo.comment = info.comment;

    newInfo.solvent = info.solvent;
    newInfo.temperature = info.temperature.magnitude;
    newInfo.probeName = info.probeName || '';
    newInfo.fieldStrength = info.fieldStrength.magnitude;
    let gyromagneticRatioConstants = newInfo.nucleus.map(nucleus => gyromagneticRatio[nucleus]);
    newInfo.baseFrequency = gyromagneticRatioConstants.map(gmr => info.fieldStrength.magnitude * gmr / (2 * Math.PI * 1e6));
    newInfo.pulseSequence = info.experiment;
    newInfo.temperature = info.temperature.unit.toLowerCase() === 'celsius' ? 273.15 + info.temperature.magnitude : info.temperature.magnitude;
    newInfo.digitalFilter = info.digitalFilter;
    newInfo.pulseStrength90 = 1 / (4 * info.pulseStrength90.magnitude);
    newInfo.numberOfScans = info.numberOfScans;
    newInfo.relaxationTime = info.relaxationTime.magnitude;
    newInfo.isComplex = info.dataSections.includes('im');
    newInfo.isFid = info.dataUnits[0] === 'Second';
    newInfo.isFt = info.dataUnits[0] === 'Ppm';
    newInfo.dimension = info.dimension;
    const dimension = newInfo.dimension;
    newInfo.originFrequency = info.originFrequency.map(d => d.magnitude / 1e6).slice(0, dimension);
    newInfo.numberOfPoints = info.dataPoints.slice(0, 1);
    newInfo.frequencyOffset = info.frequencyOffset.map((f, i) => f.magnitude * newInfo.baseFrequency[i]).slice(0, dimension);
    newInfo.acquisitionTime = info.acquisitionTime.map(a => a.magnitude).slice(0, dimension);
    newInfo.spectralWidth = info.spectralWidth.map((sw, i) => sw.magnitude / info.originFrequency[i].magnitude * 1e6).slice(0, dimension); // set options for dimensions

    let dimensions = [];
    let options = {};
    let increment;

    for (let d = 0; d < info.dimension; d++) {
      increment = {
        magnitude: info.acquisitionTime[d].magnitude / (info.dataPoints[d] - 1),
        unit: 's'
      };

      if (info.dataUnits[d] === 'Second') {
        options.quantityName = 'time';
        options.originOffset = {
          magnitude: 0,
          unit: 's'
        };

        if (d === 0) {
          options.coordinatesOffset = {
            magnitude: info.digitalFilter * increment.magnitude,
            unit: 's'
          };
        } else {
          options.coordinatesOffset = {
            magnitude: 0,
            unit: 's'
          };
        }

        options.reciprocal = {
          originOffset: {
            magnitude: info.originFrequency[d].magnitude,
            unit: 'Hz'
          },
          quantityName: 'frequency',
          coordinatesOffset: {
            magnitude: info.frequencyOffset[d].magnitude * info.originFrequency[d].magnitude / 1000000,
            unit: 'Hz'
          }
        };
      } else if (info.dataUnits[d] === 'Ppm') {
        options.quantityName = 'frequency';
        let origin = info.originFrequency[d].magnitude;
        options.originOffset = {
          magnitude: origin,
          unit: 'Hz'
        };
        let firstPoint = info.dataOffsetStart[0];
        let lastPoint = info.dataOffsetStop[0];
        let dataLength = lastPoint - firstPoint + 1;
        let spectralWidth = info.spectralWidth[d].magnitude;
        let incr = spectralWidth / info.dataPoints[d];
        increment = {
          magnitude: incr,
          unit: 'Hz'
        };
        let offset = info.dataAxisStop[0] * origin / 1000000;
        options.coordinatesOffset = {
          magnitude: offset,
          unit: 'Hz'
        }; // after increment is computed with whole frequency
        // and original number of points, we recast the
        // number of point for export

        if (dataLength < info.dataPoints[d]) {
          info.dataPoints[d] = dataLength;
        }
      }

      if (d === 0) {
        options.description = 'direct dimension';
      } else {
        options.description = 'indirect dimension';
      }

      dimensions.push(formatLinearDimension(headers.dataAxisTitles[d], info.dataPoints[d], increment, options));
    } // set options for dependentVariable


    options = {
      unit: 'none',
      quantityName: 'relative intensity',
      from: info.dataOffsetStart,
      to: info.dataOffsetStop
    };
    let dependentVariables = [];
    dependentVariables.push(formatDependentVariable(data, 11, options));
    let description = Object.assign({}, newInfo);
    delete description.paramList;
    description.metadata = Object.assign({}, toKeyValue(headers), toKeyValue(parameters), toKeyValue(paramArray));
    let dataStructure = {
      timeStamp: Date.now(),
      version: packageJson.version,
      description,
      tags: ['magnetic resonance'].concat(newInfo.nucleus),
      application: {
        spectralWidthClipped: info.spectralWidthClipped[0].magnitude / newInfo.baseFrequency[0]
      },
      dimensions: dimensions,
      dependentVariables: dependentVariables
    };
    return dataStructure;
  }

  /*!

  JSZip v3.6.0 - A JavaScript class for generating and reading zip files
  <http://stuartk.com/jszip>

  (c) 2009-2016 Stuart Knightley <stuart [at] stuartk.com>
  Dual licenced under the MIT license or GPLv3. See https://raw.github.com/Stuk/jszip/master/LICENSE.markdown.

  JSZip uses the library pako released under the MIT license :
  https://github.com/nodeca/pako/blob/master/LICENSE
  */
  var jszip = createCommonjsModule(function (module, exports) {
    (function (f) {
      {
        module.exports = f();
      }
    })(function () {
      return function e(t, n, r) {
        function s(o, u) {
          if (!n[o]) {
            if (!t[o]) {
              var a = typeof commonjsRequire == "function" && commonjsRequire;
              if (!u && a) return a(o, !0);
              if (i) return i(o, !0);
              var f = new Error("Cannot find module '" + o + "'");
              throw f.code = "MODULE_NOT_FOUND", f;
            }

            var l = n[o] = {
              exports: {}
            };
            t[o][0].call(l.exports, function (e) {
              var n = t[o][1][e];
              return s(n ? n : e);
            }, l, l.exports, e, t, n, r);
          }

          return n[o].exports;
        }

        var i = typeof commonjsRequire == "function" && commonjsRequire;

        for (var o = 0; o < r.length; o++) s(r[o]);

        return s;
      }({
        1: [function (require, module, exports) {
          (function (global) {
            /*!
            
            JSZip v3.5.0 - A JavaScript class for generating and reading zip files
            <http://stuartk.com/jszip>
            
            (c) 2009-2016 Stuart Knightley <stuart [at] stuartk.com>
            Dual licenced under the MIT license or GPLv3. See https://raw.github.com/Stuk/jszip/master/LICENSE.markdown.
            
            JSZip uses the library pako released under the MIT license :
            https://github.com/nodeca/pako/blob/master/LICENSE
            */
            !function (e) {
              if ("object" == typeof exports && "undefined" != typeof module) module.exports = e();else {
                ("undefined" != typeof window ? window : "undefined" != typeof global ? global : "undefined" != typeof self ? self : this).JSZip = e();
              }
            }(function () {
              return function s(a, o, u) {
                function h(r, e) {
                  if (!o[r]) {
                    if (!a[r]) {
                      var t = "function" == typeof require && require;
                      if (!e && t) return t(r, !0);
                      if (f) return f(r, !0);
                      var n = new Error("Cannot find module '" + r + "'");
                      throw n.code = "MODULE_NOT_FOUND", n;
                    }

                    var i = o[r] = {
                      exports: {}
                    };
                    a[r][0].call(i.exports, function (e) {
                      var t = a[r][1][e];
                      return h(t || e);
                    }, i, i.exports, s, a, o, u);
                  }

                  return o[r].exports;
                }

                for (var f = "function" == typeof require && require, e = 0; e < u.length; e++) h(u[e]);

                return h;
              }({
                1: [function (l, t, n) {
                  (function (r) {
                    !function (e) {
                      "object" == typeof n && void 0 !== t ? t.exports = e() : ("undefined" != typeof window ? window : void 0 !== r ? r : "undefined" != typeof self ? self : this).JSZip = e();
                    }(function () {
                      return function s(a, o, u) {
                        function h(t, e) {
                          if (!o[t]) {
                            if (!a[t]) {
                              var r = "function" == typeof l && l;
                              if (!e && r) return r(t, !0);
                              if (f) return f(t, !0);
                              var n = new Error("Cannot find module '" + t + "'");
                              throw n.code = "MODULE_NOT_FOUND", n;
                            }

                            var i = o[t] = {
                              exports: {}
                            };
                            a[t][0].call(i.exports, function (e) {
                              return h(a[t][1][e] || e);
                            }, i, i.exports, s, a, o, u);
                          }

                          return o[t].exports;
                        }

                        for (var f = "function" == typeof l && l, e = 0; e < u.length; e++) h(u[e]);

                        return h;
                      }({
                        1: [function (l, t, n) {
                          (function (r) {
                            !function (e) {
                              "object" == typeof n && void 0 !== t ? t.exports = e() : ("undefined" != typeof window ? window : void 0 !== r ? r : "undefined" != typeof self ? self : this).JSZip = e();
                            }(function () {
                              return function s(a, o, u) {
                                function h(t, e) {
                                  if (!o[t]) {
                                    if (!a[t]) {
                                      var r = "function" == typeof l && l;
                                      if (!e && r) return r(t, !0);
                                      if (f) return f(t, !0);
                                      var n = new Error("Cannot find module '" + t + "'");
                                      throw n.code = "MODULE_NOT_FOUND", n;
                                    }

                                    var i = o[t] = {
                                      exports: {}
                                    };
                                    a[t][0].call(i.exports, function (e) {
                                      return h(a[t][1][e] || e);
                                    }, i, i.exports, s, a, o, u);
                                  }

                                  return o[t].exports;
                                }

                                for (var f = "function" == typeof l && l, e = 0; e < u.length; e++) h(u[e]);

                                return h;
                              }({
                                1: [function (l, t, n) {
                                  (function (r) {
                                    !function (e) {
                                      "object" == typeof n && void 0 !== t ? t.exports = e() : ("undefined" != typeof window ? window : void 0 !== r ? r : "undefined" != typeof self ? self : this).JSZip = e();
                                    }(function () {
                                      return function s(a, o, u) {
                                        function h(t, e) {
                                          if (!o[t]) {
                                            if (!a[t]) {
                                              var r = "function" == typeof l && l;
                                              if (!e && r) return r(t, !0);
                                              if (f) return f(t, !0);
                                              var n = new Error("Cannot find module '" + t + "'");
                                              throw n.code = "MODULE_NOT_FOUND", n;
                                            }

                                            var i = o[t] = {
                                              exports: {}
                                            };
                                            a[t][0].call(i.exports, function (e) {
                                              return h(a[t][1][e] || e);
                                            }, i, i.exports, s, a, o, u);
                                          }

                                          return o[t].exports;
                                        }

                                        for (var f = "function" == typeof l && l, e = 0; e < u.length; e++) h(u[e]);

                                        return h;
                                      }({
                                        1: [function (l, t, n) {
                                          (function (r) {
                                            !function (e) {
                                              "object" == typeof n && void 0 !== t ? t.exports = e() : ("undefined" != typeof window ? window : void 0 !== r ? r : "undefined" != typeof self ? self : this).JSZip = e();
                                            }(function () {
                                              return function s(a, o, u) {
                                                function h(t, e) {
                                                  if (!o[t]) {
                                                    if (!a[t]) {
                                                      var r = "function" == typeof l && l;
                                                      if (!e && r) return r(t, !0);
                                                      if (f) return f(t, !0);
                                                      var n = new Error("Cannot find module '" + t + "'");
                                                      throw n.code = "MODULE_NOT_FOUND", n;
                                                    }

                                                    var i = o[t] = {
                                                      exports: {}
                                                    };
                                                    a[t][0].call(i.exports, function (e) {
                                                      return h(a[t][1][e] || e);
                                                    }, i, i.exports, s, a, o, u);
                                                  }

                                                  return o[t].exports;
                                                }

                                                for (var f = "function" == typeof l && l, e = 0; e < u.length; e++) h(u[e]);

                                                return h;
                                              }({
                                                1: [function (e, t, r) {

                                                  var c = e("./utils"),
                                                      l = e("./support"),
                                                      p = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
                                                  r.encode = function (e) {
                                                    for (var t, r, n, i, s, a, o, u = [], h = 0, f = e.length, l = f, d = "string" !== c.getTypeOf(e); h < e.length;) l = f - h, n = d ? (t = e[h++], r = h < f ? e[h++] : 0, h < f ? e[h++] : 0) : (t = e.charCodeAt(h++), r = h < f ? e.charCodeAt(h++) : 0, h < f ? e.charCodeAt(h++) : 0), i = t >> 2, s = (3 & t) << 4 | r >> 4, a = 1 < l ? (15 & r) << 2 | n >> 6 : 64, o = 2 < l ? 63 & n : 64, u.push(p.charAt(i) + p.charAt(s) + p.charAt(a) + p.charAt(o));

                                                    return u.join("");
                                                  }, r.decode = function (e) {
                                                    var t,
                                                        r,
                                                        n,
                                                        i,
                                                        s,
                                                        a,
                                                        o = 0,
                                                        u = 0;
                                                    if ("data:" === e.substr(0, "data:".length)) throw new Error("Invalid base64 input, it looks like a data url.");
                                                    var h,
                                                        f = 3 * (e = e.replace(/[^A-Za-z0-9\+\/\=]/g, "")).length / 4;
                                                    if (e.charAt(e.length - 1) === p.charAt(64) && f--, e.charAt(e.length - 2) === p.charAt(64) && f--, f % 1 != 0) throw new Error("Invalid base64 input, bad content length.");

                                                    for (h = l.uint8array ? new Uint8Array(0 | f) : new Array(0 | f); o < e.length;) t = p.indexOf(e.charAt(o++)) << 2 | (i = p.indexOf(e.charAt(o++))) >> 4, r = (15 & i) << 4 | (s = p.indexOf(e.charAt(o++))) >> 2, n = (3 & s) << 6 | (a = p.indexOf(e.charAt(o++))), h[u++] = t, 64 !== s && (h[u++] = r), 64 !== a && (h[u++] = n);

                                                    return h;
                                                  };
                                                }, {
                                                  "./support": 30,
                                                  "./utils": 32
                                                }],
                                                2: [function (e, t, r) {

                                                  var n = e("./external"),
                                                      i = e("./stream/DataWorker"),
                                                      s = e("./stream/Crc32Probe"),
                                                      a = e("./stream/DataLengthProbe");

                                                  function o(e, t, r, n, i) {
                                                    this.compressedSize = e, this.uncompressedSize = t, this.crc32 = r, this.compression = n, this.compressedContent = i;
                                                  }

                                                  o.prototype = {
                                                    getContentWorker: function () {
                                                      var e = new i(n.Promise.resolve(this.compressedContent)).pipe(this.compression.uncompressWorker()).pipe(new a("data_length")),
                                                          t = this;
                                                      return e.on("end", function () {
                                                        if (this.streamInfo.data_length !== t.uncompressedSize) throw new Error("Bug : uncompressed data size mismatch");
                                                      }), e;
                                                    },
                                                    getCompressedWorker: function () {
                                                      return new i(n.Promise.resolve(this.compressedContent)).withStreamInfo("compressedSize", this.compressedSize).withStreamInfo("uncompressedSize", this.uncompressedSize).withStreamInfo("crc32", this.crc32).withStreamInfo("compression", this.compression);
                                                    }
                                                  }, o.createWorkerFrom = function (e, t, r) {
                                                    return e.pipe(new s()).pipe(new a("uncompressedSize")).pipe(t.compressWorker(r)).pipe(new a("compressedSize")).withStreamInfo("compression", t);
                                                  }, t.exports = o;
                                                }, {
                                                  "./external": 6,
                                                  "./stream/Crc32Probe": 25,
                                                  "./stream/DataLengthProbe": 26,
                                                  "./stream/DataWorker": 27
                                                }],
                                                3: [function (e, t, r) {

                                                  var n = e("./stream/GenericWorker");
                                                  r.STORE = {
                                                    magic: "\0\0",
                                                    compressWorker: function (e) {
                                                      return new n("STORE compression");
                                                    },
                                                    uncompressWorker: function () {
                                                      return new n("STORE decompression");
                                                    }
                                                  }, r.DEFLATE = e("./flate");
                                                }, {
                                                  "./flate": 7,
                                                  "./stream/GenericWorker": 28
                                                }],
                                                4: [function (e, t, r) {

                                                  var n = e("./utils"),
                                                      a = function () {
                                                    for (var e, t = [], r = 0; r < 256; r++) {
                                                      e = r;

                                                      for (var n = 0; n < 8; n++) e = 1 & e ? 3988292384 ^ e >>> 1 : e >>> 1;

                                                      t[r] = e;
                                                    }

                                                    return t;
                                                  }();

                                                  t.exports = function (e, t) {
                                                    return void 0 !== e && e.length ? "string" !== n.getTypeOf(e) ? function (e, t, r) {
                                                      var n = a,
                                                          i = 0 + r;
                                                      e ^= -1;

                                                      for (var s = 0; s < i; s++) e = e >>> 8 ^ n[255 & (e ^ t[s])];

                                                      return -1 ^ e;
                                                    }(0 | t, e, e.length) : function (e, t, r) {
                                                      var n = a,
                                                          i = 0 + r;
                                                      e ^= -1;

                                                      for (var s = 0; s < i; s++) e = e >>> 8 ^ n[255 & (e ^ t.charCodeAt(s))];

                                                      return -1 ^ e;
                                                    }(0 | t, e, e.length) : 0;
                                                  };
                                                }, {
                                                  "./utils": 32
                                                }],
                                                5: [function (e, t, r) {

                                                  r.base64 = !1, r.binary = !1, r.dir = !1, r.createFolders = !0, r.date = null, r.compression = null, r.compressionOptions = null, r.comment = null, r.unixPermissions = null, r.dosPermissions = null;
                                                }, {}],
                                                6: [function (e, t, r) {

                                                  var n;
                                                  n = "undefined" != typeof Promise ? Promise : e("lie"), t.exports = {
                                                    Promise: n
                                                  };
                                                }, {
                                                  lie: 37
                                                }],
                                                7: [function (e, t, r) {

                                                  var n = "undefined" != typeof Uint8Array && "undefined" != typeof Uint16Array && "undefined" != typeof Uint32Array,
                                                      i = e("pako"),
                                                      s = e("./utils"),
                                                      a = e("./stream/GenericWorker"),
                                                      o = n ? "uint8array" : "array";

                                                  function u(e, t) {
                                                    a.call(this, "FlateWorker/" + e), this._pako = null, this._pakoAction = e, this._pakoOptions = t, this.meta = {};
                                                  }

                                                  r.magic = "\b\0", s.inherits(u, a), u.prototype.processChunk = function (e) {
                                                    this.meta = e.meta, null === this._pako && this._createPako(), this._pako.push(s.transformTo(o, e.data), !1);
                                                  }, u.prototype.flush = function () {
                                                    a.prototype.flush.call(this), null === this._pako && this._createPako(), this._pako.push([], !0);
                                                  }, u.prototype.cleanUp = function () {
                                                    a.prototype.cleanUp.call(this), this._pako = null;
                                                  }, u.prototype._createPako = function () {
                                                    this._pako = new i[this._pakoAction]({
                                                      raw: !0,
                                                      level: this._pakoOptions.level || -1
                                                    });
                                                    var t = this;

                                                    this._pako.onData = function (e) {
                                                      t.push({
                                                        data: e,
                                                        meta: t.meta
                                                      });
                                                    };
                                                  }, r.compressWorker = function (e) {
                                                    return new u("Deflate", e);
                                                  }, r.uncompressWorker = function () {
                                                    return new u("Inflate", {});
                                                  };
                                                }, {
                                                  "./stream/GenericWorker": 28,
                                                  "./utils": 32,
                                                  pako: 38
                                                }],
                                                8: [function (e, t, r) {

                                                  function I(e, t) {
                                                    var r,
                                                        n = "";

                                                    for (r = 0; r < t; r++) n += String.fromCharCode(255 & e), e >>>= 8;

                                                    return n;
                                                  }

                                                  function i(e, t, r, n, i, s) {
                                                    var a,
                                                        o,
                                                        u = e.file,
                                                        h = e.compression,
                                                        f = s !== B.utf8encode,
                                                        l = O.transformTo("string", s(u.name)),
                                                        d = O.transformTo("string", B.utf8encode(u.name)),
                                                        c = u.comment,
                                                        p = O.transformTo("string", s(c)),
                                                        m = O.transformTo("string", B.utf8encode(c)),
                                                        _ = d.length !== u.name.length,
                                                        g = m.length !== c.length,
                                                        v = "",
                                                        b = "",
                                                        w = "",
                                                        y = u.dir,
                                                        k = u.date,
                                                        x = {
                                                      crc32: 0,
                                                      compressedSize: 0,
                                                      uncompressedSize: 0
                                                    };

                                                    t && !r || (x.crc32 = e.crc32, x.compressedSize = e.compressedSize, x.uncompressedSize = e.uncompressedSize);
                                                    var S = 0;
                                                    t && (S |= 8), f || !_ && !g || (S |= 2048);
                                                    var z,
                                                        C = 0,
                                                        E = 0;
                                                    y && (C |= 16), "UNIX" === i ? (E = 798, C |= ((z = u.unixPermissions) || (z = y ? 16893 : 33204), (65535 & z) << 16)) : (E = 20, C |= 63 & (u.dosPermissions || 0)), a = k.getUTCHours(), a <<= 6, a |= k.getUTCMinutes(), a <<= 5, a |= k.getUTCSeconds() / 2, o = k.getUTCFullYear() - 1980, o <<= 4, o |= k.getUTCMonth() + 1, o <<= 5, o |= k.getUTCDate(), _ && (v += "up" + I((b = I(1, 1) + I(R(l), 4) + d).length, 2) + b), g && (v += "uc" + I((w = I(1, 1) + I(R(p), 4) + m).length, 2) + w);
                                                    var A = "";
                                                    return A += "\n\0", A += I(S, 2), A += h.magic, A += I(a, 2), A += I(o, 2), A += I(x.crc32, 4), A += I(x.compressedSize, 4), A += I(x.uncompressedSize, 4), A += I(l.length, 2), A += I(v.length, 2), {
                                                      fileRecord: T.LOCAL_FILE_HEADER + A + l + v,
                                                      dirRecord: T.CENTRAL_FILE_HEADER + I(E, 2) + A + I(p.length, 2) + "\0\0\0\0" + I(C, 4) + I(n, 4) + l + v + p
                                                    };
                                                  }

                                                  var O = e("../utils"),
                                                      s = e("../stream/GenericWorker"),
                                                      B = e("../utf8"),
                                                      R = e("../crc32"),
                                                      T = e("../signature");

                                                  function n(e, t, r, n) {
                                                    s.call(this, "ZipFileWorker"), this.bytesWritten = 0, this.zipComment = t, this.zipPlatform = r, this.encodeFileName = n, this.streamFiles = e, this.accumulate = !1, this.contentBuffer = [], this.dirRecords = [], this.currentSourceOffset = 0, this.entriesCount = 0, this.currentFile = null, this._sources = [];
                                                  }

                                                  O.inherits(n, s), n.prototype.push = function (e) {
                                                    var t = e.meta.percent || 0,
                                                        r = this.entriesCount,
                                                        n = this._sources.length;
                                                    this.accumulate ? this.contentBuffer.push(e) : (this.bytesWritten += e.data.length, s.prototype.push.call(this, {
                                                      data: e.data,
                                                      meta: {
                                                        currentFile: this.currentFile,
                                                        percent: r ? (t + 100 * (r - n - 1)) / r : 100
                                                      }
                                                    }));
                                                  }, n.prototype.openedSource = function (e) {
                                                    this.currentSourceOffset = this.bytesWritten, this.currentFile = e.file.name;
                                                    var t = this.streamFiles && !e.file.dir;

                                                    if (t) {
                                                      var r = i(e, t, !1, this.currentSourceOffset, this.zipPlatform, this.encodeFileName);
                                                      this.push({
                                                        data: r.fileRecord,
                                                        meta: {
                                                          percent: 0
                                                        }
                                                      });
                                                    } else this.accumulate = !0;
                                                  }, n.prototype.closedSource = function (e) {
                                                    this.accumulate = !1;
                                                    var t,
                                                        r = this.streamFiles && !e.file.dir,
                                                        n = i(e, r, !0, this.currentSourceOffset, this.zipPlatform, this.encodeFileName);
                                                    if (this.dirRecords.push(n.dirRecord), r) this.push({
                                                      data: (t = e, T.DATA_DESCRIPTOR + I(t.crc32, 4) + I(t.compressedSize, 4) + I(t.uncompressedSize, 4)),
                                                      meta: {
                                                        percent: 100
                                                      }
                                                    });else for (this.push({
                                                      data: n.fileRecord,
                                                      meta: {
                                                        percent: 0
                                                      }
                                                    }); this.contentBuffer.length;) this.push(this.contentBuffer.shift());
                                                    this.currentFile = null;
                                                  }, n.prototype.flush = function () {
                                                    for (var e = this.bytesWritten, t = 0; t < this.dirRecords.length; t++) this.push({
                                                      data: this.dirRecords[t],
                                                      meta: {
                                                        percent: 100
                                                      }
                                                    });

                                                    var r,
                                                        n,
                                                        i,
                                                        s,
                                                        a,
                                                        o,
                                                        u = this.bytesWritten - e,
                                                        h = (r = this.dirRecords.length, n = u, i = e, s = this.zipComment, a = this.encodeFileName, o = O.transformTo("string", a(s)), T.CENTRAL_DIRECTORY_END + "\0\0\0\0" + I(r, 2) + I(r, 2) + I(n, 4) + I(i, 4) + I(o.length, 2) + o);
                                                    this.push({
                                                      data: h,
                                                      meta: {
                                                        percent: 100
                                                      }
                                                    });
                                                  }, n.prototype.prepareNextSource = function () {
                                                    this.previous = this._sources.shift(), this.openedSource(this.previous.streamInfo), this.isPaused ? this.previous.pause() : this.previous.resume();
                                                  }, n.prototype.registerPrevious = function (e) {
                                                    this._sources.push(e);

                                                    var t = this;
                                                    return e.on("data", function (e) {
                                                      t.processChunk(e);
                                                    }), e.on("end", function () {
                                                      t.closedSource(t.previous.streamInfo), t._sources.length ? t.prepareNextSource() : t.end();
                                                    }), e.on("error", function (e) {
                                                      t.error(e);
                                                    }), this;
                                                  }, n.prototype.resume = function () {
                                                    return !!s.prototype.resume.call(this) && (!this.previous && this._sources.length ? (this.prepareNextSource(), !0) : this.previous || this._sources.length || this.generatedError ? void 0 : (this.end(), !0));
                                                  }, n.prototype.error = function (e) {
                                                    var t = this._sources;
                                                    if (!s.prototype.error.call(this, e)) return !1;

                                                    for (var r = 0; r < t.length; r++) try {
                                                      t[r].error(e);
                                                    } catch (e) {}

                                                    return !0;
                                                  }, n.prototype.lock = function () {
                                                    s.prototype.lock.call(this);

                                                    for (var e = this._sources, t = 0; t < e.length; t++) e[t].lock();
                                                  }, t.exports = n;
                                                }, {
                                                  "../crc32": 4,
                                                  "../signature": 23,
                                                  "../stream/GenericWorker": 28,
                                                  "../utf8": 31,
                                                  "../utils": 32
                                                }],
                                                9: [function (e, t, r) {

                                                  var h = e("../compressions"),
                                                      n = e("./ZipFileWorker");

                                                  r.generateWorker = function (e, a, t) {
                                                    var o = new n(a.streamFiles, t, a.platform, a.encodeFileName),
                                                        u = 0;

                                                    try {
                                                      e.forEach(function (e, t) {
                                                        u++;

                                                        var r = function (e, t) {
                                                          var r = e || t,
                                                              n = h[r];
                                                          if (!n) throw new Error(r + " is not a valid compression method !");
                                                          return n;
                                                        }(t.options.compression, a.compression),
                                                            n = t.options.compressionOptions || a.compressionOptions || {},
                                                            i = t.dir,
                                                            s = t.date;

                                                        t._compressWorker(r, n).withStreamInfo("file", {
                                                          name: e,
                                                          dir: i,
                                                          date: s,
                                                          comment: t.comment || "",
                                                          unixPermissions: t.unixPermissions,
                                                          dosPermissions: t.dosPermissions
                                                        }).pipe(o);
                                                      }), o.entriesCount = u;
                                                    } catch (e) {
                                                      o.error(e);
                                                    }

                                                    return o;
                                                  };
                                                }, {
                                                  "../compressions": 3,
                                                  "./ZipFileWorker": 8
                                                }],
                                                10: [function (e, t, r) {

                                                  function n() {
                                                    if (!(this instanceof n)) return new n();
                                                    if (arguments.length) throw new Error("The constructor with parameters has been removed in JSZip 3.0, please check the upgrade guide.");
                                                    this.files = {}, this.comment = null, this.root = "", this.clone = function () {
                                                      var e = new n();

                                                      for (var t in this) "function" != typeof this[t] && (e[t] = this[t]);

                                                      return e;
                                                    };
                                                  }

                                                  (n.prototype = e("./object")).loadAsync = e("./load"), n.support = e("./support"), n.defaults = e("./defaults"), n.version = "3.5.0", n.loadAsync = function (e, t) {
                                                    return new n().loadAsync(e, t);
                                                  }, n.external = e("./external"), t.exports = n;
                                                }, {
                                                  "./defaults": 5,
                                                  "./external": 6,
                                                  "./load": 11,
                                                  "./object": 15,
                                                  "./support": 30
                                                }],
                                                11: [function (e, t, r) {

                                                  var n = e("./utils"),
                                                      i = e("./external"),
                                                      o = e("./utf8"),
                                                      u = e("./zipEntries"),
                                                      s = e("./stream/Crc32Probe"),
                                                      h = e("./nodejsUtils");

                                                  function f(n) {
                                                    return new i.Promise(function (e, t) {
                                                      var r = n.decompressed.getContentWorker().pipe(new s());
                                                      r.on("error", function (e) {
                                                        t(e);
                                                      }).on("end", function () {
                                                        r.streamInfo.crc32 !== n.decompressed.crc32 ? t(new Error("Corrupted zip : CRC32 mismatch")) : e();
                                                      }).resume();
                                                    });
                                                  }

                                                  t.exports = function (e, s) {
                                                    var a = this;
                                                    return s = n.extend(s || {}, {
                                                      base64: !1,
                                                      checkCRC32: !1,
                                                      optimizedBinaryString: !1,
                                                      createFolders: !1,
                                                      decodeFileName: o.utf8decode
                                                    }), h.isNode && h.isStream(e) ? i.Promise.reject(new Error("JSZip can't accept a stream when loading a zip file.")) : n.prepareContent("the loaded zip file", e, !0, s.optimizedBinaryString, s.base64).then(function (e) {
                                                      var t = new u(s);
                                                      return t.load(e), t;
                                                    }).then(function (e) {
                                                      var t = [i.Promise.resolve(e)],
                                                          r = e.files;
                                                      if (s.checkCRC32) for (var n = 0; n < r.length; n++) t.push(f(r[n]));
                                                      return i.Promise.all(t);
                                                    }).then(function (e) {
                                                      for (var t = e.shift(), r = t.files, n = 0; n < r.length; n++) {
                                                        var i = r[n];
                                                        a.file(i.fileNameStr, i.decompressed, {
                                                          binary: !0,
                                                          optimizedBinaryString: !0,
                                                          date: i.date,
                                                          dir: i.dir,
                                                          comment: i.fileCommentStr.length ? i.fileCommentStr : null,
                                                          unixPermissions: i.unixPermissions,
                                                          dosPermissions: i.dosPermissions,
                                                          createFolders: s.createFolders
                                                        });
                                                      }

                                                      return t.zipComment.length && (a.comment = t.zipComment), a;
                                                    });
                                                  };
                                                }, {
                                                  "./external": 6,
                                                  "./nodejsUtils": 14,
                                                  "./stream/Crc32Probe": 25,
                                                  "./utf8": 31,
                                                  "./utils": 32,
                                                  "./zipEntries": 33
                                                }],
                                                12: [function (e, t, r) {

                                                  var n = e("../utils"),
                                                      i = e("../stream/GenericWorker");

                                                  function s(e, t) {
                                                    i.call(this, "Nodejs stream input adapter for " + e), this._upstreamEnded = !1, this._bindStream(t);
                                                  }

                                                  n.inherits(s, i), s.prototype._bindStream = function (e) {
                                                    var t = this;
                                                    (this._stream = e).pause(), e.on("data", function (e) {
                                                      t.push({
                                                        data: e,
                                                        meta: {
                                                          percent: 0
                                                        }
                                                      });
                                                    }).on("error", function (e) {
                                                      t.isPaused ? this.generatedError = e : t.error(e);
                                                    }).on("end", function () {
                                                      t.isPaused ? t._upstreamEnded = !0 : t.end();
                                                    });
                                                  }, s.prototype.pause = function () {
                                                    return !!i.prototype.pause.call(this) && (this._stream.pause(), !0);
                                                  }, s.prototype.resume = function () {
                                                    return !!i.prototype.resume.call(this) && (this._upstreamEnded ? this.end() : this._stream.resume(), !0);
                                                  }, t.exports = s;
                                                }, {
                                                  "../stream/GenericWorker": 28,
                                                  "../utils": 32
                                                }],
                                                13: [function (e, t, r) {

                                                  var i = e("readable-stream").Readable;

                                                  function n(e, t, r) {
                                                    i.call(this, t), this._helper = e;
                                                    var n = this;
                                                    e.on("data", function (e, t) {
                                                      n.push(e) || n._helper.pause(), r && r(t);
                                                    }).on("error", function (e) {
                                                      n.emit("error", e);
                                                    }).on("end", function () {
                                                      n.push(null);
                                                    });
                                                  }

                                                  e("../utils").inherits(n, i), n.prototype._read = function () {
                                                    this._helper.resume();
                                                  }, t.exports = n;
                                                }, {
                                                  "../utils": 32,
                                                  "readable-stream": 16
                                                }],
                                                14: [function (e, t, r) {

                                                  t.exports = {
                                                    isNode: "undefined" != typeof Buffer,
                                                    newBufferFrom: function (e, t) {
                                                      if (Buffer.from && Buffer.from !== Uint8Array.from) return Buffer.from(e, t);
                                                      if ("number" == typeof e) throw new Error('The "data" argument must not be a number');
                                                      return new Buffer(e, t);
                                                    },
                                                    allocBuffer: function (e) {
                                                      if (Buffer.alloc) return Buffer.alloc(e);
                                                      var t = new Buffer(e);
                                                      return t.fill(0), t;
                                                    },
                                                    isBuffer: function (e) {
                                                      return Buffer.isBuffer(e);
                                                    },
                                                    isStream: function (e) {
                                                      return e && "function" == typeof e.on && "function" == typeof e.pause && "function" == typeof e.resume;
                                                    }
                                                  };
                                                }, {}],
                                                15: [function (e, t, r) {

                                                  function s(e, t, r) {
                                                    var n,
                                                        i = f.getTypeOf(t),
                                                        s = f.extend(r || {}, d);
                                                    s.date = s.date || new Date(), null !== s.compression && (s.compression = s.compression.toUpperCase()), "string" == typeof s.unixPermissions && (s.unixPermissions = parseInt(s.unixPermissions, 8)), s.unixPermissions && 16384 & s.unixPermissions && (s.dir = !0), s.dosPermissions && 16 & s.dosPermissions && (s.dir = !0), s.dir && (e = h(e)), s.createFolders && (n = function (e) {
                                                      "/" === e.slice(-1) && (e = e.substring(0, e.length - 1));
                                                      var t = e.lastIndexOf("/");
                                                      return 0 < t ? e.substring(0, t) : "";
                                                    }(e)) && g.call(this, n, !0);
                                                    var a,
                                                        o = "string" === i && !1 === s.binary && !1 === s.base64;
                                                    r && void 0 !== r.binary || (s.binary = !o), (t instanceof c && 0 === t.uncompressedSize || s.dir || !t || 0 === t.length) && (s.base64 = !1, s.binary = !0, t = "", s.compression = "STORE", i = "string"), a = t instanceof c || t instanceof l ? t : m.isNode && m.isStream(t) ? new _(e, t) : f.prepareContent(e, t, s.binary, s.optimizedBinaryString, s.base64);
                                                    var u = new p(e, a, s);
                                                    this.files[e] = u;
                                                  }

                                                  function h(e) {
                                                    return "/" !== e.slice(-1) && (e += "/"), e;
                                                  }

                                                  var i = e("./utf8"),
                                                      f = e("./utils"),
                                                      l = e("./stream/GenericWorker"),
                                                      a = e("./stream/StreamHelper"),
                                                      d = e("./defaults"),
                                                      c = e("./compressedObject"),
                                                      p = e("./zipObject"),
                                                      o = e("./generate"),
                                                      m = e("./nodejsUtils"),
                                                      _ = e("./nodejs/NodejsStreamInputAdapter"),
                                                      g = function (e, t) {
                                                    return t = void 0 !== t ? t : d.createFolders, e = h(e), this.files[e] || s.call(this, e, null, {
                                                      dir: !0,
                                                      createFolders: t
                                                    }), this.files[e];
                                                  };

                                                  function u(e) {
                                                    return "[object RegExp]" === Object.prototype.toString.call(e);
                                                  }

                                                  var n = {
                                                    load: function () {
                                                      throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.");
                                                    },
                                                    forEach: function (e) {
                                                      var t, r, n;

                                                      for (t in this.files) this.files.hasOwnProperty(t) && (n = this.files[t], (r = t.slice(this.root.length, t.length)) && t.slice(0, this.root.length) === this.root && e(r, n));
                                                    },
                                                    filter: function (r) {
                                                      var n = [];
                                                      return this.forEach(function (e, t) {
                                                        r(e, t) && n.push(t);
                                                      }), n;
                                                    },
                                                    file: function (e, t, r) {
                                                      if (1 !== arguments.length) return e = this.root + e, s.call(this, e, t, r), this;

                                                      if (u(e)) {
                                                        var n = e;
                                                        return this.filter(function (e, t) {
                                                          return !t.dir && n.test(e);
                                                        });
                                                      }

                                                      var i = this.files[this.root + e];
                                                      return i && !i.dir ? i : null;
                                                    },
                                                    folder: function (r) {
                                                      if (!r) return this;
                                                      if (u(r)) return this.filter(function (e, t) {
                                                        return t.dir && r.test(e);
                                                      });
                                                      var e = this.root + r,
                                                          t = g.call(this, e),
                                                          n = this.clone();
                                                      return n.root = t.name, n;
                                                    },
                                                    remove: function (r) {
                                                      r = this.root + r;
                                                      var e = this.files[r];
                                                      if (e || ("/" !== r.slice(-1) && (r += "/"), e = this.files[r]), e && !e.dir) delete this.files[r];else for (var t = this.filter(function (e, t) {
                                                        return t.name.slice(0, r.length) === r;
                                                      }), n = 0; n < t.length; n++) delete this.files[t[n].name];
                                                      return this;
                                                    },
                                                    generate: function (e) {
                                                      throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.");
                                                    },
                                                    generateInternalStream: function (e) {
                                                      var t,
                                                          r = {};

                                                      try {
                                                        if ((r = f.extend(e || {}, {
                                                          streamFiles: !1,
                                                          compression: "STORE",
                                                          compressionOptions: null,
                                                          type: "",
                                                          platform: "DOS",
                                                          comment: null,
                                                          mimeType: "application/zip",
                                                          encodeFileName: i.utf8encode
                                                        })).type = r.type.toLowerCase(), r.compression = r.compression.toUpperCase(), "binarystring" === r.type && (r.type = "string"), !r.type) throw new Error("No output type specified.");
                                                        f.checkSupport(r.type), "darwin" !== r.platform && "freebsd" !== r.platform && "linux" !== r.platform && "sunos" !== r.platform || (r.platform = "UNIX"), "win32" === r.platform && (r.platform = "DOS");
                                                        var n = r.comment || this.comment || "";
                                                        t = o.generateWorker(this, r, n);
                                                      } catch (e) {
                                                        (t = new l("error")).error(e);
                                                      }

                                                      return new a(t, r.type || "string", r.mimeType);
                                                    },
                                                    generateAsync: function (e, t) {
                                                      return this.generateInternalStream(e).accumulate(t);
                                                    },
                                                    generateNodeStream: function (e, t) {
                                                      return (e = e || {}).type || (e.type = "nodebuffer"), this.generateInternalStream(e).toNodejsStream(t);
                                                    }
                                                  };
                                                  t.exports = n;
                                                }, {
                                                  "./compressedObject": 2,
                                                  "./defaults": 5,
                                                  "./generate": 9,
                                                  "./nodejs/NodejsStreamInputAdapter": 12,
                                                  "./nodejsUtils": 14,
                                                  "./stream/GenericWorker": 28,
                                                  "./stream/StreamHelper": 29,
                                                  "./utf8": 31,
                                                  "./utils": 32,
                                                  "./zipObject": 35
                                                }],
                                                16: [function (e, t, r) {
                                                  t.exports = e("stream");
                                                }, {
                                                  stream: void 0
                                                }],
                                                17: [function (e, t, r) {

                                                  var n = e("./DataReader");

                                                  function i(e) {
                                                    n.call(this, e);

                                                    for (var t = 0; t < this.data.length; t++) e[t] = 255 & e[t];
                                                  }

                                                  e("../utils").inherits(i, n), i.prototype.byteAt = function (e) {
                                                    return this.data[this.zero + e];
                                                  }, i.prototype.lastIndexOfSignature = function (e) {
                                                    for (var t = e.charCodeAt(0), r = e.charCodeAt(1), n = e.charCodeAt(2), i = e.charCodeAt(3), s = this.length - 4; 0 <= s; --s) if (this.data[s] === t && this.data[s + 1] === r && this.data[s + 2] === n && this.data[s + 3] === i) return s - this.zero;

                                                    return -1;
                                                  }, i.prototype.readAndCheckSignature = function (e) {
                                                    var t = e.charCodeAt(0),
                                                        r = e.charCodeAt(1),
                                                        n = e.charCodeAt(2),
                                                        i = e.charCodeAt(3),
                                                        s = this.readData(4);
                                                    return t === s[0] && r === s[1] && n === s[2] && i === s[3];
                                                  }, i.prototype.readData = function (e) {
                                                    if (this.checkOffset(e), 0 === e) return [];
                                                    var t = this.data.slice(this.zero + this.index, this.zero + this.index + e);
                                                    return this.index += e, t;
                                                  }, t.exports = i;
                                                }, {
                                                  "../utils": 32,
                                                  "./DataReader": 18
                                                }],
                                                18: [function (e, t, r) {

                                                  var n = e("../utils");

                                                  function i(e) {
                                                    this.data = e, this.length = e.length, this.index = 0, this.zero = 0;
                                                  }

                                                  i.prototype = {
                                                    checkOffset: function (e) {
                                                      this.checkIndex(this.index + e);
                                                    },
                                                    checkIndex: function (e) {
                                                      if (this.length < this.zero + e || e < 0) throw new Error("End of data reached (data length = " + this.length + ", asked index = " + e + "). Corrupted zip ?");
                                                    },
                                                    setIndex: function (e) {
                                                      this.checkIndex(e), this.index = e;
                                                    },
                                                    skip: function (e) {
                                                      this.setIndex(this.index + e);
                                                    },
                                                    byteAt: function (e) {},
                                                    readInt: function (e) {
                                                      var t,
                                                          r = 0;

                                                      for (this.checkOffset(e), t = this.index + e - 1; t >= this.index; t--) r = (r << 8) + this.byteAt(t);

                                                      return this.index += e, r;
                                                    },
                                                    readString: function (e) {
                                                      return n.transformTo("string", this.readData(e));
                                                    },
                                                    readData: function (e) {},
                                                    lastIndexOfSignature: function (e) {},
                                                    readAndCheckSignature: function (e) {},
                                                    readDate: function () {
                                                      var e = this.readInt(4);
                                                      return new Date(Date.UTC(1980 + (e >> 25 & 127), (e >> 21 & 15) - 1, e >> 16 & 31, e >> 11 & 31, e >> 5 & 63, (31 & e) << 1));
                                                    }
                                                  }, t.exports = i;
                                                }, {
                                                  "../utils": 32
                                                }],
                                                19: [function (e, t, r) {

                                                  var n = e("./Uint8ArrayReader");

                                                  function i(e) {
                                                    n.call(this, e);
                                                  }

                                                  e("../utils").inherits(i, n), i.prototype.readData = function (e) {
                                                    this.checkOffset(e);
                                                    var t = this.data.slice(this.zero + this.index, this.zero + this.index + e);
                                                    return this.index += e, t;
                                                  }, t.exports = i;
                                                }, {
                                                  "../utils": 32,
                                                  "./Uint8ArrayReader": 21
                                                }],
                                                20: [function (e, t, r) {

                                                  var n = e("./DataReader");

                                                  function i(e) {
                                                    n.call(this, e);
                                                  }

                                                  e("../utils").inherits(i, n), i.prototype.byteAt = function (e) {
                                                    return this.data.charCodeAt(this.zero + e);
                                                  }, i.prototype.lastIndexOfSignature = function (e) {
                                                    return this.data.lastIndexOf(e) - this.zero;
                                                  }, i.prototype.readAndCheckSignature = function (e) {
                                                    return e === this.readData(4);
                                                  }, i.prototype.readData = function (e) {
                                                    this.checkOffset(e);
                                                    var t = this.data.slice(this.zero + this.index, this.zero + this.index + e);
                                                    return this.index += e, t;
                                                  }, t.exports = i;
                                                }, {
                                                  "../utils": 32,
                                                  "./DataReader": 18
                                                }],
                                                21: [function (e, t, r) {

                                                  var n = e("./ArrayReader");

                                                  function i(e) {
                                                    n.call(this, e);
                                                  }

                                                  e("../utils").inherits(i, n), i.prototype.readData = function (e) {
                                                    if (this.checkOffset(e), 0 === e) return new Uint8Array(0);
                                                    var t = this.data.subarray(this.zero + this.index, this.zero + this.index + e);
                                                    return this.index += e, t;
                                                  }, t.exports = i;
                                                }, {
                                                  "../utils": 32,
                                                  "./ArrayReader": 17
                                                }],
                                                22: [function (e, t, r) {

                                                  var n = e("../utils"),
                                                      i = e("../support"),
                                                      s = e("./ArrayReader"),
                                                      a = e("./StringReader"),
                                                      o = e("./NodeBufferReader"),
                                                      u = e("./Uint8ArrayReader");

                                                  t.exports = function (e) {
                                                    var t = n.getTypeOf(e);
                                                    return n.checkSupport(t), "string" !== t || i.uint8array ? "nodebuffer" === t ? new o(e) : i.uint8array ? new u(n.transformTo("uint8array", e)) : new s(n.transformTo("array", e)) : new a(e);
                                                  };
                                                }, {
                                                  "../support": 30,
                                                  "../utils": 32,
                                                  "./ArrayReader": 17,
                                                  "./NodeBufferReader": 19,
                                                  "./StringReader": 20,
                                                  "./Uint8ArrayReader": 21
                                                }],
                                                23: [function (e, t, r) {

                                                  r.LOCAL_FILE_HEADER = "PK", r.CENTRAL_FILE_HEADER = "PK", r.CENTRAL_DIRECTORY_END = "PK", r.ZIP64_CENTRAL_DIRECTORY_LOCATOR = "PK", r.ZIP64_CENTRAL_DIRECTORY_END = "PK", r.DATA_DESCRIPTOR = "PK\b";
                                                }, {}],
                                                24: [function (e, t, r) {

                                                  var n = e("./GenericWorker"),
                                                      i = e("../utils");

                                                  function s(e) {
                                                    n.call(this, "ConvertWorker to " + e), this.destType = e;
                                                  }

                                                  i.inherits(s, n), s.prototype.processChunk = function (e) {
                                                    this.push({
                                                      data: i.transformTo(this.destType, e.data),
                                                      meta: e.meta
                                                    });
                                                  }, t.exports = s;
                                                }, {
                                                  "../utils": 32,
                                                  "./GenericWorker": 28
                                                }],
                                                25: [function (e, t, r) {

                                                  var n = e("./GenericWorker"),
                                                      i = e("../crc32");

                                                  function s() {
                                                    n.call(this, "Crc32Probe"), this.withStreamInfo("crc32", 0);
                                                  }

                                                  e("../utils").inherits(s, n), s.prototype.processChunk = function (e) {
                                                    this.streamInfo.crc32 = i(e.data, this.streamInfo.crc32 || 0), this.push(e);
                                                  }, t.exports = s;
                                                }, {
                                                  "../crc32": 4,
                                                  "../utils": 32,
                                                  "./GenericWorker": 28
                                                }],
                                                26: [function (e, t, r) {

                                                  var n = e("../utils"),
                                                      i = e("./GenericWorker");

                                                  function s(e) {
                                                    i.call(this, "DataLengthProbe for " + e), this.propName = e, this.withStreamInfo(e, 0);
                                                  }

                                                  n.inherits(s, i), s.prototype.processChunk = function (e) {
                                                    if (e) {
                                                      var t = this.streamInfo[this.propName] || 0;
                                                      this.streamInfo[this.propName] = t + e.data.length;
                                                    }

                                                    i.prototype.processChunk.call(this, e);
                                                  }, t.exports = s;
                                                }, {
                                                  "../utils": 32,
                                                  "./GenericWorker": 28
                                                }],
                                                27: [function (e, t, r) {

                                                  var n = e("../utils"),
                                                      i = e("./GenericWorker");

                                                  function s(e) {
                                                    i.call(this, "DataWorker");
                                                    var t = this;
                                                    this.dataIsReady = !1, this.index = 0, this.max = 0, this.data = null, this.type = "", this._tickScheduled = !1, e.then(function (e) {
                                                      t.dataIsReady = !0, t.data = e, t.max = e && e.length || 0, t.type = n.getTypeOf(e), t.isPaused || t._tickAndRepeat();
                                                    }, function (e) {
                                                      t.error(e);
                                                    });
                                                  }

                                                  n.inherits(s, i), s.prototype.cleanUp = function () {
                                                    i.prototype.cleanUp.call(this), this.data = null;
                                                  }, s.prototype.resume = function () {
                                                    return !!i.prototype.resume.call(this) && (!this._tickScheduled && this.dataIsReady && (this._tickScheduled = !0, n.delay(this._tickAndRepeat, [], this)), !0);
                                                  }, s.prototype._tickAndRepeat = function () {
                                                    this._tickScheduled = !1, this.isPaused || this.isFinished || (this._tick(), this.isFinished || (n.delay(this._tickAndRepeat, [], this), this._tickScheduled = !0));
                                                  }, s.prototype._tick = function () {
                                                    if (this.isPaused || this.isFinished) return !1;
                                                    var e = null,
                                                        t = Math.min(this.max, this.index + 16384);
                                                    if (this.index >= this.max) return this.end();

                                                    switch (this.type) {
                                                      case "string":
                                                        e = this.data.substring(this.index, t);
                                                        break;

                                                      case "uint8array":
                                                        e = this.data.subarray(this.index, t);
                                                        break;

                                                      case "array":
                                                      case "nodebuffer":
                                                        e = this.data.slice(this.index, t);
                                                    }

                                                    return this.index = t, this.push({
                                                      data: e,
                                                      meta: {
                                                        percent: this.max ? this.index / this.max * 100 : 0
                                                      }
                                                    });
                                                  }, t.exports = s;
                                                }, {
                                                  "../utils": 32,
                                                  "./GenericWorker": 28
                                                }],
                                                28: [function (e, t, r) {

                                                  function n(e) {
                                                    this.name = e || "default", this.streamInfo = {}, this.generatedError = null, this.extraStreamInfo = {}, this.isPaused = !0, this.isFinished = !1, this.isLocked = !1, this._listeners = {
                                                      data: [],
                                                      end: [],
                                                      error: []
                                                    }, this.previous = null;
                                                  }

                                                  n.prototype = {
                                                    push: function (e) {
                                                      this.emit("data", e);
                                                    },
                                                    end: function () {
                                                      if (this.isFinished) return !1;
                                                      this.flush();

                                                      try {
                                                        this.emit("end"), this.cleanUp(), this.isFinished = !0;
                                                      } catch (e) {
                                                        this.emit("error", e);
                                                      }

                                                      return !0;
                                                    },
                                                    error: function (e) {
                                                      return !this.isFinished && (this.isPaused ? this.generatedError = e : (this.isFinished = !0, this.emit("error", e), this.previous && this.previous.error(e), this.cleanUp()), !0);
                                                    },
                                                    on: function (e, t) {
                                                      return this._listeners[e].push(t), this;
                                                    },
                                                    cleanUp: function () {
                                                      this.streamInfo = this.generatedError = this.extraStreamInfo = null, this._listeners = [];
                                                    },
                                                    emit: function (e, t) {
                                                      if (this._listeners[e]) for (var r = 0; r < this._listeners[e].length; r++) this._listeners[e][r].call(this, t);
                                                    },
                                                    pipe: function (e) {
                                                      return e.registerPrevious(this);
                                                    },
                                                    registerPrevious: function (e) {
                                                      if (this.isLocked) throw new Error("The stream '" + this + "' has already been used.");
                                                      this.streamInfo = e.streamInfo, this.mergeStreamInfo(), this.previous = e;
                                                      var t = this;
                                                      return e.on("data", function (e) {
                                                        t.processChunk(e);
                                                      }), e.on("end", function () {
                                                        t.end();
                                                      }), e.on("error", function (e) {
                                                        t.error(e);
                                                      }), this;
                                                    },
                                                    pause: function () {
                                                      return !this.isPaused && !this.isFinished && (this.isPaused = !0, this.previous && this.previous.pause(), !0);
                                                    },
                                                    resume: function () {
                                                      if (!this.isPaused || this.isFinished) return !1;
                                                      var e = this.isPaused = !1;
                                                      return this.generatedError && (this.error(this.generatedError), e = !0), this.previous && this.previous.resume(), !e;
                                                    },
                                                    flush: function () {},
                                                    processChunk: function (e) {
                                                      this.push(e);
                                                    },
                                                    withStreamInfo: function (e, t) {
                                                      return this.extraStreamInfo[e] = t, this.mergeStreamInfo(), this;
                                                    },
                                                    mergeStreamInfo: function () {
                                                      for (var e in this.extraStreamInfo) this.extraStreamInfo.hasOwnProperty(e) && (this.streamInfo[e] = this.extraStreamInfo[e]);
                                                    },
                                                    lock: function () {
                                                      if (this.isLocked) throw new Error("The stream '" + this + "' has already been used.");
                                                      this.isLocked = !0, this.previous && this.previous.lock();
                                                    },
                                                    toString: function () {
                                                      var e = "Worker " + this.name;
                                                      return this.previous ? this.previous + " -> " + e : e;
                                                    }
                                                  }, t.exports = n;
                                                }, {}],
                                                29: [function (e, t, r) {

                                                  var h = e("../utils"),
                                                      i = e("./ConvertWorker"),
                                                      s = e("./GenericWorker"),
                                                      f = e("../base64"),
                                                      n = e("../support"),
                                                      a = e("../external"),
                                                      o = null;
                                                  if (n.nodestream) try {
                                                    o = e("../nodejs/NodejsStreamOutputAdapter");
                                                  } catch (e) {}

                                                  function u(e, t, r) {
                                                    var n = t;

                                                    switch (t) {
                                                      case "blob":
                                                      case "arraybuffer":
                                                        n = "uint8array";
                                                        break;

                                                      case "base64":
                                                        n = "string";
                                                    }

                                                    try {
                                                      this._internalType = n, this._outputType = t, this._mimeType = r, h.checkSupport(n), this._worker = e.pipe(new i(n)), e.lock();
                                                    } catch (e) {
                                                      this._worker = new s("error"), this._worker.error(e);
                                                    }
                                                  }

                                                  u.prototype = {
                                                    accumulate: function (e) {
                                                      return o = this, u = e, new a.Promise(function (t, r) {
                                                        var n = [],
                                                            i = o._internalType,
                                                            s = o._outputType,
                                                            a = o._mimeType;
                                                        o.on("data", function (e, t) {
                                                          n.push(e), u && u(t);
                                                        }).on("error", function (e) {
                                                          n = [], r(e);
                                                        }).on("end", function () {
                                                          try {
                                                            var e = function (e, t, r) {
                                                              switch (e) {
                                                                case "blob":
                                                                  return h.newBlob(h.transformTo("arraybuffer", t), r);

                                                                case "base64":
                                                                  return f.encode(t);

                                                                default:
                                                                  return h.transformTo(e, t);
                                                              }
                                                            }(s, function (e, t) {
                                                              var r,
                                                                  n = 0,
                                                                  i = null,
                                                                  s = 0;

                                                              for (r = 0; r < t.length; r++) s += t[r].length;

                                                              switch (e) {
                                                                case "string":
                                                                  return t.join("");

                                                                case "array":
                                                                  return Array.prototype.concat.apply([], t);

                                                                case "uint8array":
                                                                  for (i = new Uint8Array(s), r = 0; r < t.length; r++) i.set(t[r], n), n += t[r].length;

                                                                  return i;

                                                                case "nodebuffer":
                                                                  return Buffer.concat(t);

                                                                default:
                                                                  throw new Error("concat : unsupported type '" + e + "'");
                                                              }
                                                            }(i, n), a);

                                                            t(e);
                                                          } catch (e) {
                                                            r(e);
                                                          }

                                                          n = [];
                                                        }).resume();
                                                      });
                                                      var o, u;
                                                    },
                                                    on: function (e, t) {
                                                      var r = this;
                                                      return "data" === e ? this._worker.on(e, function (e) {
                                                        t.call(r, e.data, e.meta);
                                                      }) : this._worker.on(e, function () {
                                                        h.delay(t, arguments, r);
                                                      }), this;
                                                    },
                                                    resume: function () {
                                                      return h.delay(this._worker.resume, [], this._worker), this;
                                                    },
                                                    pause: function () {
                                                      return this._worker.pause(), this;
                                                    },
                                                    toNodejsStream: function (e) {
                                                      if (h.checkSupport("nodestream"), "nodebuffer" !== this._outputType) throw new Error(this._outputType + " is not supported by this method");
                                                      return new o(this, {
                                                        objectMode: "nodebuffer" !== this._outputType
                                                      }, e);
                                                    }
                                                  }, t.exports = u;
                                                }, {
                                                  "../base64": 1,
                                                  "../external": 6,
                                                  "../nodejs/NodejsStreamOutputAdapter": 13,
                                                  "../support": 30,
                                                  "../utils": 32,
                                                  "./ConvertWorker": 24,
                                                  "./GenericWorker": 28
                                                }],
                                                30: [function (e, t, r) {

                                                  if (r.base64 = !0, r.array = !0, r.string = !0, r.arraybuffer = "undefined" != typeof ArrayBuffer && "undefined" != typeof Uint8Array, r.nodebuffer = "undefined" != typeof Buffer, r.uint8array = "undefined" != typeof Uint8Array, "undefined" == typeof ArrayBuffer) r.blob = !1;else {
                                                    var n = new ArrayBuffer(0);

                                                    try {
                                                      r.blob = 0 === new Blob([n], {
                                                        type: "application/zip"
                                                      }).size;
                                                    } catch (e) {
                                                      try {
                                                        var i = new (self.BlobBuilder || self.WebKitBlobBuilder || self.MozBlobBuilder || self.MSBlobBuilder)();
                                                        i.append(n), r.blob = 0 === i.getBlob("application/zip").size;
                                                      } catch (e) {
                                                        r.blob = !1;
                                                      }
                                                    }
                                                  }

                                                  try {
                                                    r.nodestream = !!e("readable-stream").Readable;
                                                  } catch (e) {
                                                    r.nodestream = !1;
                                                  }
                                                }, {
                                                  "readable-stream": 16
                                                }],
                                                31: [function (e, t, s) {

                                                  for (var o = e("./utils"), u = e("./support"), r = e("./nodejsUtils"), n = e("./stream/GenericWorker"), h = new Array(256), i = 0; i < 256; i++) h[i] = 252 <= i ? 6 : 248 <= i ? 5 : 240 <= i ? 4 : 224 <= i ? 3 : 192 <= i ? 2 : 1;

                                                  function a() {
                                                    n.call(this, "utf-8 decode"), this.leftOver = null;
                                                  }

                                                  function f() {
                                                    n.call(this, "utf-8 encode");
                                                  }

                                                  h[254] = h[254] = 1, s.utf8encode = function (e) {
                                                    return u.nodebuffer ? r.newBufferFrom(e, "utf-8") : function (e) {
                                                      var t,
                                                          r,
                                                          n,
                                                          i,
                                                          s,
                                                          a = e.length,
                                                          o = 0;

                                                      for (i = 0; i < a; i++) 55296 == (64512 & (r = e.charCodeAt(i))) && i + 1 < a && 56320 == (64512 & (n = e.charCodeAt(i + 1))) && (r = 65536 + (r - 55296 << 10) + (n - 56320), i++), o += r < 128 ? 1 : r < 2048 ? 2 : r < 65536 ? 3 : 4;

                                                      for (t = u.uint8array ? new Uint8Array(o) : new Array(o), i = s = 0; s < o; i++) 55296 == (64512 & (r = e.charCodeAt(i))) && i + 1 < a && 56320 == (64512 & (n = e.charCodeAt(i + 1))) && (r = 65536 + (r - 55296 << 10) + (n - 56320), i++), r < 128 ? t[s++] = r : (r < 2048 ? t[s++] = 192 | r >>> 6 : (r < 65536 ? t[s++] = 224 | r >>> 12 : (t[s++] = 240 | r >>> 18, t[s++] = 128 | r >>> 12 & 63), t[s++] = 128 | r >>> 6 & 63), t[s++] = 128 | 63 & r);

                                                      return t;
                                                    }(e);
                                                  }, s.utf8decode = function (e) {
                                                    return u.nodebuffer ? o.transformTo("nodebuffer", e).toString("utf-8") : function (e) {
                                                      var t,
                                                          r,
                                                          n,
                                                          i,
                                                          s = e.length,
                                                          a = new Array(2 * s);

                                                      for (t = r = 0; t < s;) if ((n = e[t++]) < 128) a[r++] = n;else if (4 < (i = h[n])) a[r++] = 65533, t += i - 1;else {
                                                        for (n &= 2 === i ? 31 : 3 === i ? 15 : 7; 1 < i && t < s;) n = n << 6 | 63 & e[t++], i--;

                                                        1 < i ? a[r++] = 65533 : n < 65536 ? a[r++] = n : (n -= 65536, a[r++] = 55296 | n >> 10 & 1023, a[r++] = 56320 | 1023 & n);
                                                      }

                                                      return a.length !== r && (a.subarray ? a = a.subarray(0, r) : a.length = r), o.applyFromCharCode(a);
                                                    }(e = o.transformTo(u.uint8array ? "uint8array" : "array", e));
                                                  }, o.inherits(a, n), a.prototype.processChunk = function (e) {
                                                    var t = o.transformTo(u.uint8array ? "uint8array" : "array", e.data);

                                                    if (this.leftOver && this.leftOver.length) {
                                                      if (u.uint8array) {
                                                        var r = t;
                                                        (t = new Uint8Array(r.length + this.leftOver.length)).set(this.leftOver, 0), t.set(r, this.leftOver.length);
                                                      } else t = this.leftOver.concat(t);

                                                      this.leftOver = null;
                                                    }

                                                    var n = function (e, t) {
                                                      var r;

                                                      for ((t = t || e.length) > e.length && (t = e.length), r = t - 1; 0 <= r && 128 == (192 & e[r]);) r--;

                                                      return r < 0 ? t : 0 === r ? t : r + h[e[r]] > t ? r : t;
                                                    }(t),
                                                        i = t;

                                                    n !== t.length && (u.uint8array ? (i = t.subarray(0, n), this.leftOver = t.subarray(n, t.length)) : (i = t.slice(0, n), this.leftOver = t.slice(n, t.length))), this.push({
                                                      data: s.utf8decode(i),
                                                      meta: e.meta
                                                    });
                                                  }, a.prototype.flush = function () {
                                                    this.leftOver && this.leftOver.length && (this.push({
                                                      data: s.utf8decode(this.leftOver),
                                                      meta: {}
                                                    }), this.leftOver = null);
                                                  }, s.Utf8DecodeWorker = a, o.inherits(f, n), f.prototype.processChunk = function (e) {
                                                    this.push({
                                                      data: s.utf8encode(e.data),
                                                      meta: e.meta
                                                    });
                                                  }, s.Utf8EncodeWorker = f;
                                                }, {
                                                  "./nodejsUtils": 14,
                                                  "./stream/GenericWorker": 28,
                                                  "./support": 30,
                                                  "./utils": 32
                                                }],
                                                32: [function (e, t, o) {

                                                  var u = e("./support"),
                                                      h = e("./base64"),
                                                      r = e("./nodejsUtils"),
                                                      n = e("set-immediate-shim"),
                                                      f = e("./external");

                                                  function i(e) {
                                                    return e;
                                                  }

                                                  function l(e, t) {
                                                    for (var r = 0; r < e.length; ++r) t[r] = 255 & e.charCodeAt(r);

                                                    return t;
                                                  }

                                                  o.newBlob = function (t, r) {
                                                    o.checkSupport("blob");

                                                    try {
                                                      return new Blob([t], {
                                                        type: r
                                                      });
                                                    } catch (e) {
                                                      try {
                                                        var n = new (self.BlobBuilder || self.WebKitBlobBuilder || self.MozBlobBuilder || self.MSBlobBuilder)();
                                                        return n.append(t), n.getBlob(r);
                                                      } catch (e) {
                                                        throw new Error("Bug : can't construct the Blob.");
                                                      }
                                                    }
                                                  };

                                                  var s = {
                                                    stringifyByChunk: function (e, t, r) {
                                                      var n = [],
                                                          i = 0,
                                                          s = e.length;
                                                      if (s <= r) return String.fromCharCode.apply(null, e);

                                                      for (; i < s;) "array" === t || "nodebuffer" === t ? n.push(String.fromCharCode.apply(null, e.slice(i, Math.min(i + r, s)))) : n.push(String.fromCharCode.apply(null, e.subarray(i, Math.min(i + r, s)))), i += r;

                                                      return n.join("");
                                                    },
                                                    stringifyByChar: function (e) {
                                                      for (var t = "", r = 0; r < e.length; r++) t += String.fromCharCode(e[r]);

                                                      return t;
                                                    },
                                                    applyCanBeUsed: {
                                                      uint8array: function () {
                                                        try {
                                                          return u.uint8array && 1 === String.fromCharCode.apply(null, new Uint8Array(1)).length;
                                                        } catch (e) {
                                                          return !1;
                                                        }
                                                      }(),
                                                      nodebuffer: function () {
                                                        try {
                                                          return u.nodebuffer && 1 === String.fromCharCode.apply(null, r.allocBuffer(1)).length;
                                                        } catch (e) {
                                                          return !1;
                                                        }
                                                      }()
                                                    }
                                                  };

                                                  function a(e) {
                                                    var t = 65536,
                                                        r = o.getTypeOf(e),
                                                        n = !0;
                                                    if ("uint8array" === r ? n = s.applyCanBeUsed.uint8array : "nodebuffer" === r && (n = s.applyCanBeUsed.nodebuffer), n) for (; 1 < t;) try {
                                                      return s.stringifyByChunk(e, r, t);
                                                    } catch (e) {
                                                      t = Math.floor(t / 2);
                                                    }
                                                    return s.stringifyByChar(e);
                                                  }

                                                  function d(e, t) {
                                                    for (var r = 0; r < e.length; r++) t[r] = e[r];

                                                    return t;
                                                  }

                                                  o.applyFromCharCode = a;
                                                  var c = {};
                                                  c.string = {
                                                    string: i,
                                                    array: function (e) {
                                                      return l(e, new Array(e.length));
                                                    },
                                                    arraybuffer: function (e) {
                                                      return c.string.uint8array(e).buffer;
                                                    },
                                                    uint8array: function (e) {
                                                      return l(e, new Uint8Array(e.length));
                                                    },
                                                    nodebuffer: function (e) {
                                                      return l(e, r.allocBuffer(e.length));
                                                    }
                                                  }, c.array = {
                                                    string: a,
                                                    array: i,
                                                    arraybuffer: function (e) {
                                                      return new Uint8Array(e).buffer;
                                                    },
                                                    uint8array: function (e) {
                                                      return new Uint8Array(e);
                                                    },
                                                    nodebuffer: function (e) {
                                                      return r.newBufferFrom(e);
                                                    }
                                                  }, c.arraybuffer = {
                                                    string: function (e) {
                                                      return a(new Uint8Array(e));
                                                    },
                                                    array: function (e) {
                                                      return d(new Uint8Array(e), new Array(e.byteLength));
                                                    },
                                                    arraybuffer: i,
                                                    uint8array: function (e) {
                                                      return new Uint8Array(e);
                                                    },
                                                    nodebuffer: function (e) {
                                                      return r.newBufferFrom(new Uint8Array(e));
                                                    }
                                                  }, c.uint8array = {
                                                    string: a,
                                                    array: function (e) {
                                                      return d(e, new Array(e.length));
                                                    },
                                                    arraybuffer: function (e) {
                                                      return e.buffer;
                                                    },
                                                    uint8array: i,
                                                    nodebuffer: function (e) {
                                                      return r.newBufferFrom(e);
                                                    }
                                                  }, c.nodebuffer = {
                                                    string: a,
                                                    array: function (e) {
                                                      return d(e, new Array(e.length));
                                                    },
                                                    arraybuffer: function (e) {
                                                      return c.nodebuffer.uint8array(e).buffer;
                                                    },
                                                    uint8array: function (e) {
                                                      return d(e, new Uint8Array(e.length));
                                                    },
                                                    nodebuffer: i
                                                  }, o.transformTo = function (e, t) {
                                                    if (t = t || "", !e) return t;
                                                    o.checkSupport(e);
                                                    var r = o.getTypeOf(t);
                                                    return c[r][e](t);
                                                  }, o.getTypeOf = function (e) {
                                                    return "string" == typeof e ? "string" : "[object Array]" === Object.prototype.toString.call(e) ? "array" : u.nodebuffer && r.isBuffer(e) ? "nodebuffer" : u.uint8array && e instanceof Uint8Array ? "uint8array" : u.arraybuffer && e instanceof ArrayBuffer ? "arraybuffer" : void 0;
                                                  }, o.checkSupport = function (e) {
                                                    if (!u[e.toLowerCase()]) throw new Error(e + " is not supported by this platform");
                                                  }, o.MAX_VALUE_16BITS = 65535, o.MAX_VALUE_32BITS = -1, o.pretty = function (e) {
                                                    var t,
                                                        r,
                                                        n = "";

                                                    for (r = 0; r < (e || "").length; r++) n += "\\x" + ((t = e.charCodeAt(r)) < 16 ? "0" : "") + t.toString(16).toUpperCase();

                                                    return n;
                                                  }, o.delay = function (e, t, r) {
                                                    n(function () {
                                                      e.apply(r || null, t || []);
                                                    });
                                                  }, o.inherits = function (e, t) {
                                                    function r() {}

                                                    r.prototype = t.prototype, e.prototype = new r();
                                                  }, o.extend = function () {
                                                    var e,
                                                        t,
                                                        r = {};

                                                    for (e = 0; e < arguments.length; e++) for (t in arguments[e]) arguments[e].hasOwnProperty(t) && void 0 === r[t] && (r[t] = arguments[e][t]);

                                                    return r;
                                                  }, o.prepareContent = function (n, e, i, s, a) {
                                                    return f.Promise.resolve(e).then(function (n) {
                                                      return u.blob && (n instanceof Blob || -1 !== ["[object File]", "[object Blob]"].indexOf(Object.prototype.toString.call(n))) && "undefined" != typeof FileReader ? new f.Promise(function (t, r) {
                                                        var e = new FileReader();
                                                        e.onload = function (e) {
                                                          t(e.target.result);
                                                        }, e.onerror = function (e) {
                                                          r(e.target.error);
                                                        }, e.readAsArrayBuffer(n);
                                                      }) : n;
                                                    }).then(function (e) {
                                                      var t,
                                                          r = o.getTypeOf(e);
                                                      return r ? ("arraybuffer" === r ? e = o.transformTo("uint8array", e) : "string" === r && (a ? e = h.decode(e) : i && !0 !== s && (e = l(t = e, u.uint8array ? new Uint8Array(t.length) : new Array(t.length)))), e) : f.Promise.reject(new Error("Can't read the data of '" + n + "'. Is it in a supported JavaScript type (String, Blob, ArrayBuffer, etc) ?"));
                                                    });
                                                  };
                                                }, {
                                                  "./base64": 1,
                                                  "./external": 6,
                                                  "./nodejsUtils": 14,
                                                  "./support": 30,
                                                  "set-immediate-shim": 54
                                                }],
                                                33: [function (e, t, r) {

                                                  var n = e("./reader/readerFor"),
                                                      i = e("./utils"),
                                                      s = e("./signature"),
                                                      a = e("./zipEntry"),
                                                      o = (e("./utf8"), e("./support"));

                                                  function u(e) {
                                                    this.files = [], this.loadOptions = e;
                                                  }

                                                  u.prototype = {
                                                    checkSignature: function (e) {
                                                      if (!this.reader.readAndCheckSignature(e)) {
                                                        this.reader.index -= 4;
                                                        var t = this.reader.readString(4);
                                                        throw new Error("Corrupted zip or bug: unexpected signature (" + i.pretty(t) + ", expected " + i.pretty(e) + ")");
                                                      }
                                                    },
                                                    isSignature: function (e, t) {
                                                      var r = this.reader.index;
                                                      this.reader.setIndex(e);
                                                      var n = this.reader.readString(4) === t;
                                                      return this.reader.setIndex(r), n;
                                                    },
                                                    readBlockEndOfCentral: function () {
                                                      this.diskNumber = this.reader.readInt(2), this.diskWithCentralDirStart = this.reader.readInt(2), this.centralDirRecordsOnThisDisk = this.reader.readInt(2), this.centralDirRecords = this.reader.readInt(2), this.centralDirSize = this.reader.readInt(4), this.centralDirOffset = this.reader.readInt(4), this.zipCommentLength = this.reader.readInt(2);
                                                      var e = this.reader.readData(this.zipCommentLength),
                                                          t = o.uint8array ? "uint8array" : "array",
                                                          r = i.transformTo(t, e);
                                                      this.zipComment = this.loadOptions.decodeFileName(r);
                                                    },
                                                    readBlockZip64EndOfCentral: function () {
                                                      this.zip64EndOfCentralSize = this.reader.readInt(8), this.reader.skip(4), this.diskNumber = this.reader.readInt(4), this.diskWithCentralDirStart = this.reader.readInt(4), this.centralDirRecordsOnThisDisk = this.reader.readInt(8), this.centralDirRecords = this.reader.readInt(8), this.centralDirSize = this.reader.readInt(8), this.centralDirOffset = this.reader.readInt(8), this.zip64ExtensibleData = {};

                                                      for (var e, t, r, n = this.zip64EndOfCentralSize - 44; 0 < n;) e = this.reader.readInt(2), t = this.reader.readInt(4), r = this.reader.readData(t), this.zip64ExtensibleData[e] = {
                                                        id: e,
                                                        length: t,
                                                        value: r
                                                      };
                                                    },
                                                    readBlockZip64EndOfCentralLocator: function () {
                                                      if (this.diskWithZip64CentralDirStart = this.reader.readInt(4), this.relativeOffsetEndOfZip64CentralDir = this.reader.readInt(8), this.disksCount = this.reader.readInt(4), 1 < this.disksCount) throw new Error("Multi-volumes zip are not supported");
                                                    },
                                                    readLocalFiles: function () {
                                                      var e, t;

                                                      for (e = 0; e < this.files.length; e++) t = this.files[e], this.reader.setIndex(t.localHeaderOffset), this.checkSignature(s.LOCAL_FILE_HEADER), t.readLocalPart(this.reader), t.handleUTF8(), t.processAttributes();
                                                    },
                                                    readCentralDir: function () {
                                                      var e;

                                                      for (this.reader.setIndex(this.centralDirOffset); this.reader.readAndCheckSignature(s.CENTRAL_FILE_HEADER);) (e = new a({
                                                        zip64: this.zip64
                                                      }, this.loadOptions)).readCentralPart(this.reader), this.files.push(e);

                                                      if (this.centralDirRecords !== this.files.length && 0 !== this.centralDirRecords && 0 === this.files.length) throw new Error("Corrupted zip or bug: expected " + this.centralDirRecords + " records in central dir, got " + this.files.length);
                                                    },
                                                    readEndOfCentral: function () {
                                                      var e = this.reader.lastIndexOfSignature(s.CENTRAL_DIRECTORY_END);
                                                      if (e < 0) throw this.isSignature(0, s.LOCAL_FILE_HEADER) ? new Error("Corrupted zip: can't find end of central directory") : new Error("Can't find end of central directory : is this a zip file ? If it is, see https://stuk.github.io/jszip/documentation/howto/read_zip.html");
                                                      this.reader.setIndex(e);
                                                      var t = e;

                                                      if (this.checkSignature(s.CENTRAL_DIRECTORY_END), this.readBlockEndOfCentral(), this.diskNumber === i.MAX_VALUE_16BITS || this.diskWithCentralDirStart === i.MAX_VALUE_16BITS || this.centralDirRecordsOnThisDisk === i.MAX_VALUE_16BITS || this.centralDirRecords === i.MAX_VALUE_16BITS || this.centralDirSize === i.MAX_VALUE_32BITS || this.centralDirOffset === i.MAX_VALUE_32BITS) {
                                                        if (this.zip64 = !0, (e = this.reader.lastIndexOfSignature(s.ZIP64_CENTRAL_DIRECTORY_LOCATOR)) < 0) throw new Error("Corrupted zip: can't find the ZIP64 end of central directory locator");
                                                        if (this.reader.setIndex(e), this.checkSignature(s.ZIP64_CENTRAL_DIRECTORY_LOCATOR), this.readBlockZip64EndOfCentralLocator(), !this.isSignature(this.relativeOffsetEndOfZip64CentralDir, s.ZIP64_CENTRAL_DIRECTORY_END) && (this.relativeOffsetEndOfZip64CentralDir = this.reader.lastIndexOfSignature(s.ZIP64_CENTRAL_DIRECTORY_END), this.relativeOffsetEndOfZip64CentralDir < 0)) throw new Error("Corrupted zip: can't find the ZIP64 end of central directory");
                                                        this.reader.setIndex(this.relativeOffsetEndOfZip64CentralDir), this.checkSignature(s.ZIP64_CENTRAL_DIRECTORY_END), this.readBlockZip64EndOfCentral();
                                                      }

                                                      var r = this.centralDirOffset + this.centralDirSize;
                                                      this.zip64 && (r += 20, r += 12 + this.zip64EndOfCentralSize);
                                                      var n = t - r;
                                                      if (0 < n) this.isSignature(t, s.CENTRAL_FILE_HEADER) || (this.reader.zero = n);else if (n < 0) throw new Error("Corrupted zip: missing " + Math.abs(n) + " bytes.");
                                                    },
                                                    prepareReader: function (e) {
                                                      this.reader = n(e);
                                                    },
                                                    load: function (e) {
                                                      this.prepareReader(e), this.readEndOfCentral(), this.readCentralDir(), this.readLocalFiles();
                                                    }
                                                  }, t.exports = u;
                                                }, {
                                                  "./reader/readerFor": 22,
                                                  "./signature": 23,
                                                  "./support": 30,
                                                  "./utf8": 31,
                                                  "./utils": 32,
                                                  "./zipEntry": 34
                                                }],
                                                34: [function (e, t, r) {

                                                  var n = e("./reader/readerFor"),
                                                      s = e("./utils"),
                                                      i = e("./compressedObject"),
                                                      a = e("./crc32"),
                                                      o = e("./utf8"),
                                                      u = e("./compressions"),
                                                      h = e("./support");

                                                  function f(e, t) {
                                                    this.options = e, this.loadOptions = t;
                                                  }

                                                  f.prototype = {
                                                    isEncrypted: function () {
                                                      return 1 == (1 & this.bitFlag);
                                                    },
                                                    useUTF8: function () {
                                                      return 2048 == (2048 & this.bitFlag);
                                                    },
                                                    readLocalPart: function (e) {
                                                      var t, r;
                                                      if (e.skip(22), this.fileNameLength = e.readInt(2), r = e.readInt(2), this.fileName = e.readData(this.fileNameLength), e.skip(r), -1 === this.compressedSize || -1 === this.uncompressedSize) throw new Error("Bug or corrupted zip : didn't get enough information from the central directory (compressedSize === -1 || uncompressedSize === -1)");
                                                      if (null === (t = function (e) {
                                                        for (var t in u) if (u.hasOwnProperty(t) && u[t].magic === e) return u[t];

                                                        return null;
                                                      }(this.compressionMethod))) throw new Error("Corrupted zip : compression " + s.pretty(this.compressionMethod) + " unknown (inner file : " + s.transformTo("string", this.fileName) + ")");
                                                      this.decompressed = new i(this.compressedSize, this.uncompressedSize, this.crc32, t, e.readData(this.compressedSize));
                                                    },
                                                    readCentralPart: function (e) {
                                                      this.versionMadeBy = e.readInt(2), e.skip(2), this.bitFlag = e.readInt(2), this.compressionMethod = e.readString(2), this.date = e.readDate(), this.crc32 = e.readInt(4), this.compressedSize = e.readInt(4), this.uncompressedSize = e.readInt(4);
                                                      var t = e.readInt(2);
                                                      if (this.extraFieldsLength = e.readInt(2), this.fileCommentLength = e.readInt(2), this.diskNumberStart = e.readInt(2), this.internalFileAttributes = e.readInt(2), this.externalFileAttributes = e.readInt(4), this.localHeaderOffset = e.readInt(4), this.isEncrypted()) throw new Error("Encrypted zip are not supported");
                                                      e.skip(t), this.readExtraFields(e), this.parseZIP64ExtraField(e), this.fileComment = e.readData(this.fileCommentLength);
                                                    },
                                                    processAttributes: function () {
                                                      this.unixPermissions = null, this.dosPermissions = null;
                                                      var e = this.versionMadeBy >> 8;
                                                      this.dir = !!(16 & this.externalFileAttributes), 0 == e && (this.dosPermissions = 63 & this.externalFileAttributes), 3 == e && (this.unixPermissions = this.externalFileAttributes >> 16 & 65535), this.dir || "/" !== this.fileNameStr.slice(-1) || (this.dir = !0);
                                                    },
                                                    parseZIP64ExtraField: function (e) {
                                                      if (this.extraFields[1]) {
                                                        var t = n(this.extraFields[1].value);
                                                        this.uncompressedSize === s.MAX_VALUE_32BITS && (this.uncompressedSize = t.readInt(8)), this.compressedSize === s.MAX_VALUE_32BITS && (this.compressedSize = t.readInt(8)), this.localHeaderOffset === s.MAX_VALUE_32BITS && (this.localHeaderOffset = t.readInt(8)), this.diskNumberStart === s.MAX_VALUE_32BITS && (this.diskNumberStart = t.readInt(4));
                                                      }
                                                    },
                                                    readExtraFields: function (e) {
                                                      var t,
                                                          r,
                                                          n,
                                                          i = e.index + this.extraFieldsLength;

                                                      for (this.extraFields || (this.extraFields = {}); e.index + 4 < i;) t = e.readInt(2), r = e.readInt(2), n = e.readData(r), this.extraFields[t] = {
                                                        id: t,
                                                        length: r,
                                                        value: n
                                                      };

                                                      e.setIndex(i);
                                                    },
                                                    handleUTF8: function () {
                                                      var e = h.uint8array ? "uint8array" : "array";
                                                      if (this.useUTF8()) this.fileNameStr = o.utf8decode(this.fileName), this.fileCommentStr = o.utf8decode(this.fileComment);else {
                                                        var t = this.findExtraFieldUnicodePath();
                                                        if (null !== t) this.fileNameStr = t;else {
                                                          var r = s.transformTo(e, this.fileName);
                                                          this.fileNameStr = this.loadOptions.decodeFileName(r);
                                                        }
                                                        var n = this.findExtraFieldUnicodeComment();
                                                        if (null !== n) this.fileCommentStr = n;else {
                                                          var i = s.transformTo(e, this.fileComment);
                                                          this.fileCommentStr = this.loadOptions.decodeFileName(i);
                                                        }
                                                      }
                                                    },
                                                    findExtraFieldUnicodePath: function () {
                                                      var e = this.extraFields[28789];

                                                      if (e) {
                                                        var t = n(e.value);
                                                        return 1 !== t.readInt(1) ? null : a(this.fileName) !== t.readInt(4) ? null : o.utf8decode(t.readData(e.length - 5));
                                                      }

                                                      return null;
                                                    },
                                                    findExtraFieldUnicodeComment: function () {
                                                      var e = this.extraFields[25461];

                                                      if (e) {
                                                        var t = n(e.value);
                                                        return 1 !== t.readInt(1) ? null : a(this.fileComment) !== t.readInt(4) ? null : o.utf8decode(t.readData(e.length - 5));
                                                      }

                                                      return null;
                                                    }
                                                  }, t.exports = f;
                                                }, {
                                                  "./compressedObject": 2,
                                                  "./compressions": 3,
                                                  "./crc32": 4,
                                                  "./reader/readerFor": 22,
                                                  "./support": 30,
                                                  "./utf8": 31,
                                                  "./utils": 32
                                                }],
                                                35: [function (e, t, r) {

                                                  function n(e, t, r) {
                                                    this.name = e, this.dir = r.dir, this.date = r.date, this.comment = r.comment, this.unixPermissions = r.unixPermissions, this.dosPermissions = r.dosPermissions, this._data = t, this._dataBinary = r.binary, this.options = {
                                                      compression: r.compression,
                                                      compressionOptions: r.compressionOptions
                                                    };
                                                  }

                                                  var s = e("./stream/StreamHelper"),
                                                      i = e("./stream/DataWorker"),
                                                      a = e("./utf8"),
                                                      o = e("./compressedObject"),
                                                      u = e("./stream/GenericWorker");
                                                  n.prototype = {
                                                    internalStream: function (e) {
                                                      var t = null,
                                                          r = "string";

                                                      try {
                                                        if (!e) throw new Error("No output type specified.");
                                                        var n = "string" === (r = e.toLowerCase()) || "text" === r;
                                                        "binarystring" !== r && "text" !== r || (r = "string"), t = this._decompressWorker();
                                                        var i = !this._dataBinary;
                                                        i && !n && (t = t.pipe(new a.Utf8EncodeWorker())), !i && n && (t = t.pipe(new a.Utf8DecodeWorker()));
                                                      } catch (e) {
                                                        (t = new u("error")).error(e);
                                                      }

                                                      return new s(t, r, "");
                                                    },
                                                    async: function (e, t) {
                                                      return this.internalStream(e).accumulate(t);
                                                    },
                                                    nodeStream: function (e, t) {
                                                      return this.internalStream(e || "nodebuffer").toNodejsStream(t);
                                                    },
                                                    _compressWorker: function (e, t) {
                                                      if (this._data instanceof o && this._data.compression.magic === e.magic) return this._data.getCompressedWorker();

                                                      var r = this._decompressWorker();

                                                      return this._dataBinary || (r = r.pipe(new a.Utf8EncodeWorker())), o.createWorkerFrom(r, e, t);
                                                    },
                                                    _decompressWorker: function () {
                                                      return this._data instanceof o ? this._data.getContentWorker() : this._data instanceof u ? this._data : new i(this._data);
                                                    }
                                                  };

                                                  for (var h = ["asText", "asBinary", "asNodeBuffer", "asUint8Array", "asArrayBuffer"], f = function () {
                                                    throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.");
                                                  }, l = 0; l < h.length; l++) n.prototype[h[l]] = f;

                                                  t.exports = n;
                                                }, {
                                                  "./compressedObject": 2,
                                                  "./stream/DataWorker": 27,
                                                  "./stream/GenericWorker": 28,
                                                  "./stream/StreamHelper": 29,
                                                  "./utf8": 31
                                                }],
                                                36: [function (e, f, t) {
                                                  (function (t) {

                                                    var r,
                                                        n,
                                                        e = t.MutationObserver || t.WebKitMutationObserver;

                                                    if (e) {
                                                      var i = 0,
                                                          s = new e(h),
                                                          a = t.document.createTextNode("");
                                                      s.observe(a, {
                                                        characterData: !0
                                                      }), r = function () {
                                                        a.data = i = ++i % 2;
                                                      };
                                                    } else if (t.setImmediate || void 0 === t.MessageChannel) r = "document" in t && "onreadystatechange" in t.document.createElement("script") ? function () {
                                                      var e = t.document.createElement("script");
                                                      e.onreadystatechange = function () {
                                                        h(), e.onreadystatechange = null, e.parentNode.removeChild(e), e = null;
                                                      }, t.document.documentElement.appendChild(e);
                                                    } : function () {
                                                      setTimeout(h, 0);
                                                    };else {
                                                      var o = new t.MessageChannel();
                                                      o.port1.onmessage = h, r = function () {
                                                        o.port2.postMessage(0);
                                                      };
                                                    }

                                                    var u = [];

                                                    function h() {
                                                      var e, t;
                                                      n = !0;

                                                      for (var r = u.length; r;) {
                                                        for (t = u, u = [], e = -1; ++e < r;) t[e]();

                                                        r = u.length;
                                                      }

                                                      n = !1;
                                                    }

                                                    f.exports = function (e) {
                                                      1 !== u.push(e) || n || r();
                                                    };
                                                  }).call(this, void 0 !== r ? r : "undefined" != typeof self ? self : "undefined" != typeof window ? window : {});
                                                }, {}],
                                                37: [function (e, t, r) {

                                                  var i = e("immediate");

                                                  function h() {}

                                                  var f = {},
                                                      s = ["REJECTED"],
                                                      a = ["FULFILLED"],
                                                      n = ["PENDING"];

                                                  function o(e) {
                                                    if ("function" != typeof e) throw new TypeError("resolver must be a function");
                                                    this.state = n, this.queue = [], this.outcome = void 0, e !== h && c(this, e);
                                                  }

                                                  function u(e, t, r) {
                                                    this.promise = e, "function" == typeof t && (this.onFulfilled = t, this.callFulfilled = this.otherCallFulfilled), "function" == typeof r && (this.onRejected = r, this.callRejected = this.otherCallRejected);
                                                  }

                                                  function l(t, r, n) {
                                                    i(function () {
                                                      var e;

                                                      try {
                                                        e = r(n);
                                                      } catch (e) {
                                                        return f.reject(t, e);
                                                      }

                                                      e === t ? f.reject(t, new TypeError("Cannot resolve promise with itself")) : f.resolve(t, e);
                                                    });
                                                  }

                                                  function d(e) {
                                                    var t = e && e.then;
                                                    if (e && ("object" == typeof e || "function" == typeof e) && "function" == typeof t) return function () {
                                                      t.apply(e, arguments);
                                                    };
                                                  }

                                                  function c(t, e) {
                                                    var r = !1;

                                                    function n(e) {
                                                      r || (r = !0, f.reject(t, e));
                                                    }

                                                    function i(e) {
                                                      r || (r = !0, f.resolve(t, e));
                                                    }

                                                    var s = p(function () {
                                                      e(i, n);
                                                    });
                                                    "error" === s.status && n(s.value);
                                                  }

                                                  function p(e, t) {
                                                    var r = {};

                                                    try {
                                                      r.value = e(t), r.status = "success";
                                                    } catch (e) {
                                                      r.status = "error", r.value = e;
                                                    }

                                                    return r;
                                                  }

                                                  (t.exports = o).prototype.finally = function (t) {
                                                    if ("function" != typeof t) return this;
                                                    var r = this.constructor;
                                                    return this.then(function (e) {
                                                      return r.resolve(t()).then(function () {
                                                        return e;
                                                      });
                                                    }, function (e) {
                                                      return r.resolve(t()).then(function () {
                                                        throw e;
                                                      });
                                                    });
                                                  }, o.prototype.catch = function (e) {
                                                    return this.then(null, e);
                                                  }, o.prototype.then = function (e, t) {
                                                    if ("function" != typeof e && this.state === a || "function" != typeof t && this.state === s) return this;
                                                    var r = new this.constructor(h);
                                                    return this.state !== n ? l(r, this.state === a ? e : t, this.outcome) : this.queue.push(new u(r, e, t)), r;
                                                  }, u.prototype.callFulfilled = function (e) {
                                                    f.resolve(this.promise, e);
                                                  }, u.prototype.otherCallFulfilled = function (e) {
                                                    l(this.promise, this.onFulfilled, e);
                                                  }, u.prototype.callRejected = function (e) {
                                                    f.reject(this.promise, e);
                                                  }, u.prototype.otherCallRejected = function (e) {
                                                    l(this.promise, this.onRejected, e);
                                                  }, f.resolve = function (e, t) {
                                                    var r = p(d, t);
                                                    if ("error" === r.status) return f.reject(e, r.value);
                                                    var n = r.value;
                                                    if (n) c(e, n);else {
                                                      e.state = a, e.outcome = t;

                                                      for (var i = -1, s = e.queue.length; ++i < s;) e.queue[i].callFulfilled(t);
                                                    }
                                                    return e;
                                                  }, f.reject = function (e, t) {
                                                    e.state = s, e.outcome = t;

                                                    for (var r = -1, n = e.queue.length; ++r < n;) e.queue[r].callRejected(t);

                                                    return e;
                                                  }, o.resolve = function (e) {
                                                    return e instanceof this ? e : f.resolve(new this(h), e);
                                                  }, o.reject = function (e) {
                                                    var t = new this(h);
                                                    return f.reject(t, e);
                                                  }, o.all = function (e) {
                                                    var r = this;
                                                    if ("[object Array]" !== Object.prototype.toString.call(e)) return this.reject(new TypeError("must be an array"));
                                                    var n = e.length,
                                                        i = !1;
                                                    if (!n) return this.resolve([]);

                                                    for (var s = new Array(n), a = 0, t = -1, o = new this(h); ++t < n;) u(e[t], t);

                                                    return o;

                                                    function u(e, t) {
                                                      r.resolve(e).then(function (e) {
                                                        s[t] = e, ++a !== n || i || (i = !0, f.resolve(o, s));
                                                      }, function (e) {
                                                        i || (i = !0, f.reject(o, e));
                                                      });
                                                    }
                                                  }, o.race = function (e) {
                                                    if ("[object Array]" !== Object.prototype.toString.call(e)) return this.reject(new TypeError("must be an array"));
                                                    var t = e.length,
                                                        r = !1;
                                                    if (!t) return this.resolve([]);

                                                    for (var n, i = -1, s = new this(h); ++i < t;) n = e[i], this.resolve(n).then(function (e) {
                                                      r || (r = !0, f.resolve(s, e));
                                                    }, function (e) {
                                                      r || (r = !0, f.reject(s, e));
                                                    });

                                                    return s;
                                                  };
                                                }, {
                                                  immediate: 36
                                                }],
                                                38: [function (e, t, r) {

                                                  var n = {};
                                                  (0, e("./lib/utils/common").assign)(n, e("./lib/deflate"), e("./lib/inflate"), e("./lib/zlib/constants")), t.exports = n;
                                                }, {
                                                  "./lib/deflate": 39,
                                                  "./lib/inflate": 40,
                                                  "./lib/utils/common": 41,
                                                  "./lib/zlib/constants": 44
                                                }],
                                                39: [function (e, t, r) {

                                                  var a = e("./zlib/deflate"),
                                                      o = e("./utils/common"),
                                                      u = e("./utils/strings"),
                                                      i = e("./zlib/messages"),
                                                      s = e("./zlib/zstream"),
                                                      h = Object.prototype.toString,
                                                      f = 0,
                                                      l = -1,
                                                      d = 0,
                                                      c = 8;

                                                  function p(e) {
                                                    if (!(this instanceof p)) return new p(e);
                                                    this.options = o.assign({
                                                      level: l,
                                                      method: c,
                                                      chunkSize: 16384,
                                                      windowBits: 15,
                                                      memLevel: 8,
                                                      strategy: d,
                                                      to: ""
                                                    }, e || {});
                                                    var t = this.options;
                                                    t.raw && 0 < t.windowBits ? t.windowBits = -t.windowBits : t.gzip && 0 < t.windowBits && t.windowBits < 16 && (t.windowBits += 16), this.err = 0, this.msg = "", this.ended = !1, this.chunks = [], this.strm = new s(), this.strm.avail_out = 0;
                                                    var r = a.deflateInit2(this.strm, t.level, t.method, t.windowBits, t.memLevel, t.strategy);
                                                    if (r !== f) throw new Error(i[r]);

                                                    if (t.header && a.deflateSetHeader(this.strm, t.header), t.dictionary) {
                                                      var n;
                                                      if (n = "string" == typeof t.dictionary ? u.string2buf(t.dictionary) : "[object ArrayBuffer]" === h.call(t.dictionary) ? new Uint8Array(t.dictionary) : t.dictionary, (r = a.deflateSetDictionary(this.strm, n)) !== f) throw new Error(i[r]);
                                                      this._dict_set = !0;
                                                    }
                                                  }

                                                  function n(e, t) {
                                                    var r = new p(t);
                                                    if (r.push(e, !0), r.err) throw r.msg || i[r.err];
                                                    return r.result;
                                                  }

                                                  p.prototype.push = function (e, t) {
                                                    var r,
                                                        n,
                                                        i = this.strm,
                                                        s = this.options.chunkSize;
                                                    if (this.ended) return !1;
                                                    n = t === ~~t ? t : !0 === t ? 4 : 0, "string" == typeof e ? i.input = u.string2buf(e) : "[object ArrayBuffer]" === h.call(e) ? i.input = new Uint8Array(e) : i.input = e, i.next_in = 0, i.avail_in = i.input.length;

                                                    do {
                                                      if (0 === i.avail_out && (i.output = new o.Buf8(s), i.next_out = 0, i.avail_out = s), 1 !== (r = a.deflate(i, n)) && r !== f) return this.onEnd(r), !(this.ended = !0);
                                                      0 !== i.avail_out && (0 !== i.avail_in || 4 !== n && 2 !== n) || ("string" === this.options.to ? this.onData(u.buf2binstring(o.shrinkBuf(i.output, i.next_out))) : this.onData(o.shrinkBuf(i.output, i.next_out)));
                                                    } while ((0 < i.avail_in || 0 === i.avail_out) && 1 !== r);

                                                    return 4 === n ? (r = a.deflateEnd(this.strm), this.onEnd(r), this.ended = !0, r === f) : 2 !== n || (this.onEnd(f), !(i.avail_out = 0));
                                                  }, p.prototype.onData = function (e) {
                                                    this.chunks.push(e);
                                                  }, p.prototype.onEnd = function (e) {
                                                    e === f && ("string" === this.options.to ? this.result = this.chunks.join("") : this.result = o.flattenChunks(this.chunks)), this.chunks = [], this.err = e, this.msg = this.strm.msg;
                                                  }, r.Deflate = p, r.deflate = n, r.deflateRaw = function (e, t) {
                                                    return (t = t || {}).raw = !0, n(e, t);
                                                  }, r.gzip = function (e, t) {
                                                    return (t = t || {}).gzip = !0, n(e, t);
                                                  };
                                                }, {
                                                  "./utils/common": 41,
                                                  "./utils/strings": 42,
                                                  "./zlib/deflate": 46,
                                                  "./zlib/messages": 51,
                                                  "./zlib/zstream": 53
                                                }],
                                                40: [function (e, t, r) {

                                                  var d = e("./zlib/inflate"),
                                                      c = e("./utils/common"),
                                                      p = e("./utils/strings"),
                                                      m = e("./zlib/constants"),
                                                      n = e("./zlib/messages"),
                                                      i = e("./zlib/zstream"),
                                                      s = e("./zlib/gzheader"),
                                                      _ = Object.prototype.toString;

                                                  function a(e) {
                                                    if (!(this instanceof a)) return new a(e);
                                                    this.options = c.assign({
                                                      chunkSize: 16384,
                                                      windowBits: 0,
                                                      to: ""
                                                    }, e || {});
                                                    var t = this.options;
                                                    t.raw && 0 <= t.windowBits && t.windowBits < 16 && (t.windowBits = -t.windowBits, 0 === t.windowBits && (t.windowBits = -15)), !(0 <= t.windowBits && t.windowBits < 16) || e && e.windowBits || (t.windowBits += 32), 15 < t.windowBits && t.windowBits < 48 && 0 == (15 & t.windowBits) && (t.windowBits |= 15), this.err = 0, this.msg = "", this.ended = !1, this.chunks = [], this.strm = new i(), this.strm.avail_out = 0;
                                                    var r = d.inflateInit2(this.strm, t.windowBits);
                                                    if (r !== m.Z_OK) throw new Error(n[r]);
                                                    this.header = new s(), d.inflateGetHeader(this.strm, this.header);
                                                  }

                                                  function o(e, t) {
                                                    var r = new a(t);
                                                    if (r.push(e, !0), r.err) throw r.msg || n[r.err];
                                                    return r.result;
                                                  }

                                                  a.prototype.push = function (e, t) {
                                                    var r,
                                                        n,
                                                        i,
                                                        s,
                                                        a,
                                                        o,
                                                        u = this.strm,
                                                        h = this.options.chunkSize,
                                                        f = this.options.dictionary,
                                                        l = !1;
                                                    if (this.ended) return !1;
                                                    n = t === ~~t ? t : !0 === t ? m.Z_FINISH : m.Z_NO_FLUSH, "string" == typeof e ? u.input = p.binstring2buf(e) : "[object ArrayBuffer]" === _.call(e) ? u.input = new Uint8Array(e) : u.input = e, u.next_in = 0, u.avail_in = u.input.length;

                                                    do {
                                                      if (0 === u.avail_out && (u.output = new c.Buf8(h), u.next_out = 0, u.avail_out = h), (r = d.inflate(u, m.Z_NO_FLUSH)) === m.Z_NEED_DICT && f && (o = "string" == typeof f ? p.string2buf(f) : "[object ArrayBuffer]" === _.call(f) ? new Uint8Array(f) : f, r = d.inflateSetDictionary(this.strm, o)), r === m.Z_BUF_ERROR && !0 === l && (r = m.Z_OK, l = !1), r !== m.Z_STREAM_END && r !== m.Z_OK) return this.onEnd(r), !(this.ended = !0);
                                                      u.next_out && (0 !== u.avail_out && r !== m.Z_STREAM_END && (0 !== u.avail_in || n !== m.Z_FINISH && n !== m.Z_SYNC_FLUSH) || ("string" === this.options.to ? (i = p.utf8border(u.output, u.next_out), s = u.next_out - i, a = p.buf2string(u.output, i), u.next_out = s, u.avail_out = h - s, s && c.arraySet(u.output, u.output, i, s, 0), this.onData(a)) : this.onData(c.shrinkBuf(u.output, u.next_out)))), 0 === u.avail_in && 0 === u.avail_out && (l = !0);
                                                    } while ((0 < u.avail_in || 0 === u.avail_out) && r !== m.Z_STREAM_END);

                                                    return r === m.Z_STREAM_END && (n = m.Z_FINISH), n === m.Z_FINISH ? (r = d.inflateEnd(this.strm), this.onEnd(r), this.ended = !0, r === m.Z_OK) : n !== m.Z_SYNC_FLUSH || (this.onEnd(m.Z_OK), !(u.avail_out = 0));
                                                  }, a.prototype.onData = function (e) {
                                                    this.chunks.push(e);
                                                  }, a.prototype.onEnd = function (e) {
                                                    e === m.Z_OK && ("string" === this.options.to ? this.result = this.chunks.join("") : this.result = c.flattenChunks(this.chunks)), this.chunks = [], this.err = e, this.msg = this.strm.msg;
                                                  }, r.Inflate = a, r.inflate = o, r.inflateRaw = function (e, t) {
                                                    return (t = t || {}).raw = !0, o(e, t);
                                                  }, r.ungzip = o;
                                                }, {
                                                  "./utils/common": 41,
                                                  "./utils/strings": 42,
                                                  "./zlib/constants": 44,
                                                  "./zlib/gzheader": 47,
                                                  "./zlib/inflate": 49,
                                                  "./zlib/messages": 51,
                                                  "./zlib/zstream": 53
                                                }],
                                                41: [function (e, t, r) {

                                                  var n = "undefined" != typeof Uint8Array && "undefined" != typeof Uint16Array && "undefined" != typeof Int32Array;
                                                  r.assign = function (e) {
                                                    for (var t = Array.prototype.slice.call(arguments, 1); t.length;) {
                                                      var r = t.shift();

                                                      if (r) {
                                                        if ("object" != typeof r) throw new TypeError(r + "must be non-object");

                                                        for (var n in r) r.hasOwnProperty(n) && (e[n] = r[n]);
                                                      }
                                                    }

                                                    return e;
                                                  }, r.shrinkBuf = function (e, t) {
                                                    return e.length === t ? e : e.subarray ? e.subarray(0, t) : (e.length = t, e);
                                                  };
                                                  var i = {
                                                    arraySet: function (e, t, r, n, i) {
                                                      if (t.subarray && e.subarray) e.set(t.subarray(r, r + n), i);else for (var s = 0; s < n; s++) e[i + s] = t[r + s];
                                                    },
                                                    flattenChunks: function (e) {
                                                      var t, r, n, i, s, a;

                                                      for (t = n = 0, r = e.length; t < r; t++) n += e[t].length;

                                                      for (a = new Uint8Array(n), t = i = 0, r = e.length; t < r; t++) s = e[t], a.set(s, i), i += s.length;

                                                      return a;
                                                    }
                                                  },
                                                      s = {
                                                    arraySet: function (e, t, r, n, i) {
                                                      for (var s = 0; s < n; s++) e[i + s] = t[r + s];
                                                    },
                                                    flattenChunks: function (e) {
                                                      return [].concat.apply([], e);
                                                    }
                                                  };
                                                  r.setTyped = function (e) {
                                                    e ? (r.Buf8 = Uint8Array, r.Buf16 = Uint16Array, r.Buf32 = Int32Array, r.assign(r, i)) : (r.Buf8 = Array, r.Buf16 = Array, r.Buf32 = Array, r.assign(r, s));
                                                  }, r.setTyped(n);
                                                }, {}],
                                                42: [function (e, t, r) {

                                                  var u = e("./common"),
                                                      i = !0,
                                                      s = !0;

                                                  try {
                                                    String.fromCharCode.apply(null, [0]);
                                                  } catch (e) {
                                                    i = !1;
                                                  }

                                                  try {
                                                    String.fromCharCode.apply(null, new Uint8Array(1));
                                                  } catch (e) {
                                                    s = !1;
                                                  }

                                                  for (var h = new u.Buf8(256), n = 0; n < 256; n++) h[n] = 252 <= n ? 6 : 248 <= n ? 5 : 240 <= n ? 4 : 224 <= n ? 3 : 192 <= n ? 2 : 1;

                                                  function f(e, t) {
                                                    if (t < 65537 && (e.subarray && s || !e.subarray && i)) return String.fromCharCode.apply(null, u.shrinkBuf(e, t));

                                                    for (var r = "", n = 0; n < t; n++) r += String.fromCharCode(e[n]);

                                                    return r;
                                                  }

                                                  h[254] = h[254] = 1, r.string2buf = function (e) {
                                                    var t,
                                                        r,
                                                        n,
                                                        i,
                                                        s,
                                                        a = e.length,
                                                        o = 0;

                                                    for (i = 0; i < a; i++) 55296 == (64512 & (r = e.charCodeAt(i))) && i + 1 < a && 56320 == (64512 & (n = e.charCodeAt(i + 1))) && (r = 65536 + (r - 55296 << 10) + (n - 56320), i++), o += r < 128 ? 1 : r < 2048 ? 2 : r < 65536 ? 3 : 4;

                                                    for (t = new u.Buf8(o), i = s = 0; s < o; i++) 55296 == (64512 & (r = e.charCodeAt(i))) && i + 1 < a && 56320 == (64512 & (n = e.charCodeAt(i + 1))) && (r = 65536 + (r - 55296 << 10) + (n - 56320), i++), r < 128 ? t[s++] = r : (r < 2048 ? t[s++] = 192 | r >>> 6 : (r < 65536 ? t[s++] = 224 | r >>> 12 : (t[s++] = 240 | r >>> 18, t[s++] = 128 | r >>> 12 & 63), t[s++] = 128 | r >>> 6 & 63), t[s++] = 128 | 63 & r);

                                                    return t;
                                                  }, r.buf2binstring = function (e) {
                                                    return f(e, e.length);
                                                  }, r.binstring2buf = function (e) {
                                                    for (var t = new u.Buf8(e.length), r = 0, n = t.length; r < n; r++) t[r] = e.charCodeAt(r);

                                                    return t;
                                                  }, r.buf2string = function (e, t) {
                                                    var r,
                                                        n,
                                                        i,
                                                        s,
                                                        a = t || e.length,
                                                        o = new Array(2 * a);

                                                    for (r = n = 0; r < a;) if ((i = e[r++]) < 128) o[n++] = i;else if (4 < (s = h[i])) o[n++] = 65533, r += s - 1;else {
                                                      for (i &= 2 === s ? 31 : 3 === s ? 15 : 7; 1 < s && r < a;) i = i << 6 | 63 & e[r++], s--;

                                                      1 < s ? o[n++] = 65533 : i < 65536 ? o[n++] = i : (i -= 65536, o[n++] = 55296 | i >> 10 & 1023, o[n++] = 56320 | 1023 & i);
                                                    }

                                                    return f(o, n);
                                                  }, r.utf8border = function (e, t) {
                                                    var r;

                                                    for ((t = t || e.length) > e.length && (t = e.length), r = t - 1; 0 <= r && 128 == (192 & e[r]);) r--;

                                                    return r < 0 ? t : 0 === r ? t : r + h[e[r]] > t ? r : t;
                                                  };
                                                }, {
                                                  "./common": 41
                                                }],
                                                43: [function (e, t, r) {

                                                  t.exports = function (e, t, r, n) {
                                                    for (var i = 65535 & e | 0, s = e >>> 16 & 65535 | 0, a = 0; 0 !== r;) {
                                                      for (r -= a = 2e3 < r ? 2e3 : r; s = s + (i = i + t[n++] | 0) | 0, --a;);

                                                      i %= 65521, s %= 65521;
                                                    }

                                                    return i | s << 16 | 0;
                                                  };
                                                }, {}],
                                                44: [function (e, t, r) {

                                                  t.exports = {
                                                    Z_NO_FLUSH: 0,
                                                    Z_PARTIAL_FLUSH: 1,
                                                    Z_SYNC_FLUSH: 2,
                                                    Z_FULL_FLUSH: 3,
                                                    Z_FINISH: 4,
                                                    Z_BLOCK: 5,
                                                    Z_TREES: 6,
                                                    Z_OK: 0,
                                                    Z_STREAM_END: 1,
                                                    Z_NEED_DICT: 2,
                                                    Z_ERRNO: -1,
                                                    Z_STREAM_ERROR: -2,
                                                    Z_DATA_ERROR: -3,
                                                    Z_BUF_ERROR: -5,
                                                    Z_NO_COMPRESSION: 0,
                                                    Z_BEST_SPEED: 1,
                                                    Z_BEST_COMPRESSION: 9,
                                                    Z_DEFAULT_COMPRESSION: -1,
                                                    Z_FILTERED: 1,
                                                    Z_HUFFMAN_ONLY: 2,
                                                    Z_RLE: 3,
                                                    Z_FIXED: 4,
                                                    Z_DEFAULT_STRATEGY: 0,
                                                    Z_BINARY: 0,
                                                    Z_TEXT: 1,
                                                    Z_UNKNOWN: 2,
                                                    Z_DEFLATED: 8
                                                  };
                                                }, {}],
                                                45: [function (e, t, r) {

                                                  var o = function () {
                                                    for (var e, t = [], r = 0; r < 256; r++) {
                                                      e = r;

                                                      for (var n = 0; n < 8; n++) e = 1 & e ? 3988292384 ^ e >>> 1 : e >>> 1;

                                                      t[r] = e;
                                                    }

                                                    return t;
                                                  }();

                                                  t.exports = function (e, t, r, n) {
                                                    var i = o,
                                                        s = n + r;
                                                    e ^= -1;

                                                    for (var a = n; a < s; a++) e = e >>> 8 ^ i[255 & (e ^ t[a])];

                                                    return -1 ^ e;
                                                  };
                                                }, {}],
                                                46: [function (e, t, r) {

                                                  var u,
                                                      d = e("../utils/common"),
                                                      h = e("./trees"),
                                                      c = e("./adler32"),
                                                      p = e("./crc32"),
                                                      n = e("./messages"),
                                                      f = 0,
                                                      l = 0,
                                                      m = -2,
                                                      i = 2,
                                                      _ = 8,
                                                      s = 286,
                                                      a = 30,
                                                      o = 19,
                                                      g = 2 * s + 1,
                                                      v = 15,
                                                      b = 3,
                                                      w = 258,
                                                      y = w + b + 1,
                                                      k = 42,
                                                      x = 113;

                                                  function S(e, t) {
                                                    return e.msg = n[t], t;
                                                  }

                                                  function z(e) {
                                                    return (e << 1) - (4 < e ? 9 : 0);
                                                  }

                                                  function C(e) {
                                                    for (var t = e.length; 0 <= --t;) e[t] = 0;
                                                  }

                                                  function E(e) {
                                                    var t = e.state,
                                                        r = t.pending;
                                                    r > e.avail_out && (r = e.avail_out), 0 !== r && (d.arraySet(e.output, t.pending_buf, t.pending_out, r, e.next_out), e.next_out += r, t.pending_out += r, e.total_out += r, e.avail_out -= r, t.pending -= r, 0 === t.pending && (t.pending_out = 0));
                                                  }

                                                  function A(e, t) {
                                                    h._tr_flush_block(e, 0 <= e.block_start ? e.block_start : -1, e.strstart - e.block_start, t), e.block_start = e.strstart, E(e.strm);
                                                  }

                                                  function I(e, t) {
                                                    e.pending_buf[e.pending++] = t;
                                                  }

                                                  function O(e, t) {
                                                    e.pending_buf[e.pending++] = t >>> 8 & 255, e.pending_buf[e.pending++] = 255 & t;
                                                  }

                                                  function B(e, t) {
                                                    var r,
                                                        n,
                                                        i = e.max_chain_length,
                                                        s = e.strstart,
                                                        a = e.prev_length,
                                                        o = e.nice_match,
                                                        u = e.strstart > e.w_size - y ? e.strstart - (e.w_size - y) : 0,
                                                        h = e.window,
                                                        f = e.w_mask,
                                                        l = e.prev,
                                                        d = e.strstart + w,
                                                        c = h[s + a - 1],
                                                        p = h[s + a];
                                                    e.prev_length >= e.good_match && (i >>= 2), o > e.lookahead && (o = e.lookahead);

                                                    do {
                                                      if (h[(r = t) + a] === p && h[r + a - 1] === c && h[r] === h[s] && h[++r] === h[s + 1]) {
                                                        s += 2, r++;

                                                        do {} while (h[++s] === h[++r] && h[++s] === h[++r] && h[++s] === h[++r] && h[++s] === h[++r] && h[++s] === h[++r] && h[++s] === h[++r] && h[++s] === h[++r] && h[++s] === h[++r] && s < d);

                                                        if (n = w - (d - s), s = d - w, a < n) {
                                                          if (e.match_start = t, o <= (a = n)) break;
                                                          c = h[s + a - 1], p = h[s + a];
                                                        }
                                                      }
                                                    } while ((t = l[t & f]) > u && 0 != --i);

                                                    return a <= e.lookahead ? a : e.lookahead;
                                                  }

                                                  function R(e) {
                                                    var t,
                                                        r,
                                                        n,
                                                        i,
                                                        s,
                                                        a,
                                                        o,
                                                        u,
                                                        h,
                                                        f,
                                                        l = e.w_size;

                                                    do {
                                                      if (i = e.window_size - e.lookahead - e.strstart, e.strstart >= l + (l - y)) {
                                                        for (d.arraySet(e.window, e.window, l, l, 0), e.match_start -= l, e.strstart -= l, e.block_start -= l, t = r = e.hash_size; n = e.head[--t], e.head[t] = l <= n ? n - l : 0, --r;);

                                                        for (t = r = l; n = e.prev[--t], e.prev[t] = l <= n ? n - l : 0, --r;);

                                                        i += l;
                                                      }

                                                      if (0 === e.strm.avail_in) break;
                                                      if (a = e.strm, o = e.window, u = e.strstart + e.lookahead, f = void 0, (h = i) < (f = a.avail_in) && (f = h), r = 0 === f ? 0 : (a.avail_in -= f, d.arraySet(o, a.input, a.next_in, f, u), 1 === a.state.wrap ? a.adler = c(a.adler, o, f, u) : 2 === a.state.wrap && (a.adler = p(a.adler, o, f, u)), a.next_in += f, a.total_in += f, f), e.lookahead += r, e.lookahead + e.insert >= b) for (s = e.strstart - e.insert, e.ins_h = e.window[s], e.ins_h = (e.ins_h << e.hash_shift ^ e.window[s + 1]) & e.hash_mask; e.insert && (e.ins_h = (e.ins_h << e.hash_shift ^ e.window[s + b - 1]) & e.hash_mask, e.prev[s & e.w_mask] = e.head[e.ins_h], e.head[e.ins_h] = s, s++, e.insert--, !(e.lookahead + e.insert < b)););
                                                    } while (e.lookahead < y && 0 !== e.strm.avail_in);
                                                  }

                                                  function T(e, t) {
                                                    for (var r, n;;) {
                                                      if (e.lookahead < y) {
                                                        if (R(e), e.lookahead < y && t === f) return 1;
                                                        if (0 === e.lookahead) break;
                                                      }

                                                      if (r = 0, e.lookahead >= b && (e.ins_h = (e.ins_h << e.hash_shift ^ e.window[e.strstart + b - 1]) & e.hash_mask, r = e.prev[e.strstart & e.w_mask] = e.head[e.ins_h], e.head[e.ins_h] = e.strstart), 0 !== r && e.strstart - r <= e.w_size - y && (e.match_length = B(e, r)), e.match_length >= b) {
                                                        if (n = h._tr_tally(e, e.strstart - e.match_start, e.match_length - b), e.lookahead -= e.match_length, e.match_length <= e.max_lazy_match && e.lookahead >= b) {
                                                          for (e.match_length--; e.strstart++, e.ins_h = (e.ins_h << e.hash_shift ^ e.window[e.strstart + b - 1]) & e.hash_mask, r = e.prev[e.strstart & e.w_mask] = e.head[e.ins_h], e.head[e.ins_h] = e.strstart, 0 != --e.match_length;);

                                                          e.strstart++;
                                                        } else e.strstart += e.match_length, e.match_length = 0, e.ins_h = e.window[e.strstart], e.ins_h = (e.ins_h << e.hash_shift ^ e.window[e.strstart + 1]) & e.hash_mask;
                                                      } else n = h._tr_tally(e, 0, e.window[e.strstart]), e.lookahead--, e.strstart++;
                                                      if (n && (A(e, !1), 0 === e.strm.avail_out)) return 1;
                                                    }

                                                    return e.insert = e.strstart < b - 1 ? e.strstart : b - 1, 4 === t ? (A(e, !0), 0 === e.strm.avail_out ? 3 : 4) : e.last_lit && (A(e, !1), 0 === e.strm.avail_out) ? 1 : 2;
                                                  }

                                                  function D(e, t) {
                                                    for (var r, n, i;;) {
                                                      if (e.lookahead < y) {
                                                        if (R(e), e.lookahead < y && t === f) return 1;
                                                        if (0 === e.lookahead) break;
                                                      }

                                                      if (r = 0, e.lookahead >= b && (e.ins_h = (e.ins_h << e.hash_shift ^ e.window[e.strstart + b - 1]) & e.hash_mask, r = e.prev[e.strstart & e.w_mask] = e.head[e.ins_h], e.head[e.ins_h] = e.strstart), e.prev_length = e.match_length, e.prev_match = e.match_start, e.match_length = b - 1, 0 !== r && e.prev_length < e.max_lazy_match && e.strstart - r <= e.w_size - y && (e.match_length = B(e, r), e.match_length <= 5 && (1 === e.strategy || e.match_length === b && 4096 < e.strstart - e.match_start) && (e.match_length = b - 1)), e.prev_length >= b && e.match_length <= e.prev_length) {
                                                        for (i = e.strstart + e.lookahead - b, n = h._tr_tally(e, e.strstart - 1 - e.prev_match, e.prev_length - b), e.lookahead -= e.prev_length - 1, e.prev_length -= 2; ++e.strstart <= i && (e.ins_h = (e.ins_h << e.hash_shift ^ e.window[e.strstart + b - 1]) & e.hash_mask, r = e.prev[e.strstart & e.w_mask] = e.head[e.ins_h], e.head[e.ins_h] = e.strstart), 0 != --e.prev_length;);

                                                        if (e.match_available = 0, e.match_length = b - 1, e.strstart++, n && (A(e, !1), 0 === e.strm.avail_out)) return 1;
                                                      } else if (e.match_available) {
                                                        if ((n = h._tr_tally(e, 0, e.window[e.strstart - 1])) && A(e, !1), e.strstart++, e.lookahead--, 0 === e.strm.avail_out) return 1;
                                                      } else e.match_available = 1, e.strstart++, e.lookahead--;
                                                    }

                                                    return e.match_available && (n = h._tr_tally(e, 0, e.window[e.strstart - 1]), e.match_available = 0), e.insert = e.strstart < b - 1 ? e.strstart : b - 1, 4 === t ? (A(e, !0), 0 === e.strm.avail_out ? 3 : 4) : e.last_lit && (A(e, !1), 0 === e.strm.avail_out) ? 1 : 2;
                                                  }

                                                  function F(e, t, r, n, i) {
                                                    this.good_length = e, this.max_lazy = t, this.nice_length = r, this.max_chain = n, this.func = i;
                                                  }

                                                  function N() {
                                                    this.strm = null, this.status = 0, this.pending_buf = null, this.pending_buf_size = 0, this.pending_out = 0, this.pending = 0, this.wrap = 0, this.gzhead = null, this.gzindex = 0, this.method = _, this.last_flush = -1, this.w_size = 0, this.w_bits = 0, this.w_mask = 0, this.window = null, this.window_size = 0, this.prev = null, this.head = null, this.ins_h = 0, this.hash_size = 0, this.hash_bits = 0, this.hash_mask = 0, this.hash_shift = 0, this.block_start = 0, this.match_length = 0, this.prev_match = 0, this.match_available = 0, this.strstart = 0, this.match_start = 0, this.lookahead = 0, this.prev_length = 0, this.max_chain_length = 0, this.max_lazy_match = 0, this.level = 0, this.strategy = 0, this.good_match = 0, this.nice_match = 0, this.dyn_ltree = new d.Buf16(2 * g), this.dyn_dtree = new d.Buf16(2 * (2 * a + 1)), this.bl_tree = new d.Buf16(2 * (2 * o + 1)), C(this.dyn_ltree), C(this.dyn_dtree), C(this.bl_tree), this.l_desc = null, this.d_desc = null, this.bl_desc = null, this.bl_count = new d.Buf16(v + 1), this.heap = new d.Buf16(2 * s + 1), C(this.heap), this.heap_len = 0, this.heap_max = 0, this.depth = new d.Buf16(2 * s + 1), C(this.depth), this.l_buf = 0, this.lit_bufsize = 0, this.last_lit = 0, this.d_buf = 0, this.opt_len = 0, this.static_len = 0, this.matches = 0, this.insert = 0, this.bi_buf = 0, this.bi_valid = 0;
                                                  }

                                                  function U(e) {
                                                    var t;
                                                    return e && e.state ? (e.total_in = e.total_out = 0, e.data_type = i, (t = e.state).pending = 0, t.pending_out = 0, t.wrap < 0 && (t.wrap = -t.wrap), t.status = t.wrap ? k : x, e.adler = 2 === t.wrap ? 0 : 1, t.last_flush = f, h._tr_init(t), l) : S(e, m);
                                                  }

                                                  function P(e) {
                                                    var t,
                                                        r = U(e);
                                                    return r === l && ((t = e.state).window_size = 2 * t.w_size, C(t.head), t.max_lazy_match = u[t.level].max_lazy, t.good_match = u[t.level].good_length, t.nice_match = u[t.level].nice_length, t.max_chain_length = u[t.level].max_chain, t.strstart = 0, t.block_start = 0, t.lookahead = 0, t.insert = 0, t.match_length = t.prev_length = b - 1, t.match_available = 0, t.ins_h = 0), r;
                                                  }

                                                  function L(e, t, r, n, i, s) {
                                                    if (!e) return m;
                                                    var a = 1;
                                                    if (-1 === t && (t = 6), n < 0 ? (a = 0, n = -n) : 15 < n && (a = 2, n -= 16), i < 1 || 9 < i || r !== _ || n < 8 || 15 < n || t < 0 || 9 < t || s < 0 || 4 < s) return S(e, m);
                                                    8 === n && (n = 9);
                                                    var o = new N();
                                                    return (e.state = o).strm = e, o.wrap = a, o.gzhead = null, o.w_bits = n, o.w_size = 1 << o.w_bits, o.w_mask = o.w_size - 1, o.hash_bits = i + 7, o.hash_size = 1 << o.hash_bits, o.hash_mask = o.hash_size - 1, o.hash_shift = ~~((o.hash_bits + b - 1) / b), o.window = new d.Buf8(2 * o.w_size), o.head = new d.Buf16(o.hash_size), o.prev = new d.Buf16(o.w_size), o.lit_bufsize = 1 << i + 6, o.pending_buf_size = 4 * o.lit_bufsize, o.pending_buf = new d.Buf8(o.pending_buf_size), o.d_buf = 1 * o.lit_bufsize, o.l_buf = 3 * o.lit_bufsize, o.level = t, o.strategy = s, o.method = r, P(e);
                                                  }

                                                  u = [new F(0, 0, 0, 0, function (e, t) {
                                                    var r = 65535;

                                                    for (r > e.pending_buf_size - 5 && (r = e.pending_buf_size - 5);;) {
                                                      if (e.lookahead <= 1) {
                                                        if (R(e), 0 === e.lookahead && t === f) return 1;
                                                        if (0 === e.lookahead) break;
                                                      }

                                                      e.strstart += e.lookahead, e.lookahead = 0;
                                                      var n = e.block_start + r;
                                                      if ((0 === e.strstart || e.strstart >= n) && (e.lookahead = e.strstart - n, e.strstart = n, A(e, !1), 0 === e.strm.avail_out)) return 1;
                                                      if (e.strstart - e.block_start >= e.w_size - y && (A(e, !1), 0 === e.strm.avail_out)) return 1;
                                                    }

                                                    return e.insert = 0, 4 === t ? (A(e, !0), 0 === e.strm.avail_out ? 3 : 4) : (e.strstart > e.block_start && (A(e, !1), e.strm.avail_out), 1);
                                                  }), new F(4, 4, 8, 4, T), new F(4, 5, 16, 8, T), new F(4, 6, 32, 32, T), new F(4, 4, 16, 16, D), new F(8, 16, 32, 32, D), new F(8, 16, 128, 128, D), new F(8, 32, 128, 256, D), new F(32, 128, 258, 1024, D), new F(32, 258, 258, 4096, D)], r.deflateInit = function (e, t) {
                                                    return L(e, t, _, 15, 8, 0);
                                                  }, r.deflateInit2 = L, r.deflateReset = P, r.deflateResetKeep = U, r.deflateSetHeader = function (e, t) {
                                                    return e && e.state ? 2 !== e.state.wrap ? m : (e.state.gzhead = t, l) : m;
                                                  }, r.deflate = function (e, t) {
                                                    var r, n, i, s;
                                                    if (!e || !e.state || 5 < t || t < 0) return e ? S(e, m) : m;
                                                    if (n = e.state, !e.output || !e.input && 0 !== e.avail_in || 666 === n.status && 4 !== t) return S(e, 0 === e.avail_out ? -5 : m);
                                                    if (n.strm = e, r = n.last_flush, n.last_flush = t, n.status === k) if (2 === n.wrap) e.adler = 0, I(n, 31), I(n, 139), I(n, 8), n.gzhead ? (I(n, (n.gzhead.text ? 1 : 0) + (n.gzhead.hcrc ? 2 : 0) + (n.gzhead.extra ? 4 : 0) + (n.gzhead.name ? 8 : 0) + (n.gzhead.comment ? 16 : 0)), I(n, 255 & n.gzhead.time), I(n, n.gzhead.time >> 8 & 255), I(n, n.gzhead.time >> 16 & 255), I(n, n.gzhead.time >> 24 & 255), I(n, 9 === n.level ? 2 : 2 <= n.strategy || n.level < 2 ? 4 : 0), I(n, 255 & n.gzhead.os), n.gzhead.extra && n.gzhead.extra.length && (I(n, 255 & n.gzhead.extra.length), I(n, n.gzhead.extra.length >> 8 & 255)), n.gzhead.hcrc && (e.adler = p(e.adler, n.pending_buf, n.pending, 0)), n.gzindex = 0, n.status = 69) : (I(n, 0), I(n, 0), I(n, 0), I(n, 0), I(n, 0), I(n, 9 === n.level ? 2 : 2 <= n.strategy || n.level < 2 ? 4 : 0), I(n, 3), n.status = x);else {
                                                      var a = _ + (n.w_bits - 8 << 4) << 8;
                                                      a |= (2 <= n.strategy || n.level < 2 ? 0 : n.level < 6 ? 1 : 6 === n.level ? 2 : 3) << 6, 0 !== n.strstart && (a |= 32), a += 31 - a % 31, n.status = x, O(n, a), 0 !== n.strstart && (O(n, e.adler >>> 16), O(n, 65535 & e.adler)), e.adler = 1;
                                                    }
                                                    if (69 === n.status) if (n.gzhead.extra) {
                                                      for (i = n.pending; n.gzindex < (65535 & n.gzhead.extra.length) && (n.pending !== n.pending_buf_size || (n.gzhead.hcrc && n.pending > i && (e.adler = p(e.adler, n.pending_buf, n.pending - i, i)), E(e), i = n.pending, n.pending !== n.pending_buf_size));) I(n, 255 & n.gzhead.extra[n.gzindex]), n.gzindex++;

                                                      n.gzhead.hcrc && n.pending > i && (e.adler = p(e.adler, n.pending_buf, n.pending - i, i)), n.gzindex === n.gzhead.extra.length && (n.gzindex = 0, n.status = 73);
                                                    } else n.status = 73;
                                                    if (73 === n.status) if (n.gzhead.name) {
                                                      i = n.pending;

                                                      do {
                                                        if (n.pending === n.pending_buf_size && (n.gzhead.hcrc && n.pending > i && (e.adler = p(e.adler, n.pending_buf, n.pending - i, i)), E(e), i = n.pending, n.pending === n.pending_buf_size)) {
                                                          s = 1;
                                                          break;
                                                        }

                                                        s = n.gzindex < n.gzhead.name.length ? 255 & n.gzhead.name.charCodeAt(n.gzindex++) : 0, I(n, s);
                                                      } while (0 !== s);

                                                      n.gzhead.hcrc && n.pending > i && (e.adler = p(e.adler, n.pending_buf, n.pending - i, i)), 0 === s && (n.gzindex = 0, n.status = 91);
                                                    } else n.status = 91;
                                                    if (91 === n.status) if (n.gzhead.comment) {
                                                      i = n.pending;

                                                      do {
                                                        if (n.pending === n.pending_buf_size && (n.gzhead.hcrc && n.pending > i && (e.adler = p(e.adler, n.pending_buf, n.pending - i, i)), E(e), i = n.pending, n.pending === n.pending_buf_size)) {
                                                          s = 1;
                                                          break;
                                                        }

                                                        s = n.gzindex < n.gzhead.comment.length ? 255 & n.gzhead.comment.charCodeAt(n.gzindex++) : 0, I(n, s);
                                                      } while (0 !== s);

                                                      n.gzhead.hcrc && n.pending > i && (e.adler = p(e.adler, n.pending_buf, n.pending - i, i)), 0 === s && (n.status = 103);
                                                    } else n.status = 103;

                                                    if (103 === n.status && (n.gzhead.hcrc ? (n.pending + 2 > n.pending_buf_size && E(e), n.pending + 2 <= n.pending_buf_size && (I(n, 255 & e.adler), I(n, e.adler >> 8 & 255), e.adler = 0, n.status = x)) : n.status = x), 0 !== n.pending) {
                                                      if (E(e), 0 === e.avail_out) return n.last_flush = -1, l;
                                                    } else if (0 === e.avail_in && z(t) <= z(r) && 4 !== t) return S(e, -5);

                                                    if (666 === n.status && 0 !== e.avail_in) return S(e, -5);

                                                    if (0 !== e.avail_in || 0 !== n.lookahead || t !== f && 666 !== n.status) {
                                                      var o = 2 === n.strategy ? function (e, t) {
                                                        for (var r;;) {
                                                          if (0 === e.lookahead && (R(e), 0 === e.lookahead)) {
                                                            if (t === f) return 1;
                                                            break;
                                                          }

                                                          if (e.match_length = 0, r = h._tr_tally(e, 0, e.window[e.strstart]), e.lookahead--, e.strstart++, r && (A(e, !1), 0 === e.strm.avail_out)) return 1;
                                                        }

                                                        return e.insert = 0, 4 === t ? (A(e, !0), 0 === e.strm.avail_out ? 3 : 4) : e.last_lit && (A(e, !1), 0 === e.strm.avail_out) ? 1 : 2;
                                                      }(n, t) : 3 === n.strategy ? function (e, t) {
                                                        for (var r, n, i, s, a = e.window;;) {
                                                          if (e.lookahead <= w) {
                                                            if (R(e), e.lookahead <= w && t === f) return 1;
                                                            if (0 === e.lookahead) break;
                                                          }

                                                          if (e.match_length = 0, e.lookahead >= b && 0 < e.strstart && (n = a[i = e.strstart - 1]) === a[++i] && n === a[++i] && n === a[++i]) {
                                                            s = e.strstart + w;

                                                            do {} while (n === a[++i] && n === a[++i] && n === a[++i] && n === a[++i] && n === a[++i] && n === a[++i] && n === a[++i] && n === a[++i] && i < s);

                                                            e.match_length = w - (s - i), e.match_length > e.lookahead && (e.match_length = e.lookahead);
                                                          }

                                                          if (e.match_length >= b ? (r = h._tr_tally(e, 1, e.match_length - b), e.lookahead -= e.match_length, e.strstart += e.match_length, e.match_length = 0) : (r = h._tr_tally(e, 0, e.window[e.strstart]), e.lookahead--, e.strstart++), r && (A(e, !1), 0 === e.strm.avail_out)) return 1;
                                                        }

                                                        return e.insert = 0, 4 === t ? (A(e, !0), 0 === e.strm.avail_out ? 3 : 4) : e.last_lit && (A(e, !1), 0 === e.strm.avail_out) ? 1 : 2;
                                                      }(n, t) : u[n.level].func(n, t);
                                                      if (3 !== o && 4 !== o || (n.status = 666), 1 === o || 3 === o) return 0 === e.avail_out && (n.last_flush = -1), l;
                                                      if (2 === o && (1 === t ? h._tr_align(n) : 5 !== t && (h._tr_stored_block(n, 0, 0, !1), 3 === t && (C(n.head), 0 === n.lookahead && (n.strstart = 0, n.block_start = 0, n.insert = 0))), E(e), 0 === e.avail_out)) return n.last_flush = -1, l;
                                                    }

                                                    return 4 !== t ? l : n.wrap <= 0 ? 1 : (2 === n.wrap ? (I(n, 255 & e.adler), I(n, e.adler >> 8 & 255), I(n, e.adler >> 16 & 255), I(n, e.adler >> 24 & 255), I(n, 255 & e.total_in), I(n, e.total_in >> 8 & 255), I(n, e.total_in >> 16 & 255), I(n, e.total_in >> 24 & 255)) : (O(n, e.adler >>> 16), O(n, 65535 & e.adler)), E(e), 0 < n.wrap && (n.wrap = -n.wrap), 0 !== n.pending ? l : 1);
                                                  }, r.deflateEnd = function (e) {
                                                    var t;
                                                    return e && e.state ? (t = e.state.status) !== k && 69 !== t && 73 !== t && 91 !== t && 103 !== t && t !== x && 666 !== t ? S(e, m) : (e.state = null, t === x ? S(e, -3) : l) : m;
                                                  }, r.deflateSetDictionary = function (e, t) {
                                                    var r,
                                                        n,
                                                        i,
                                                        s,
                                                        a,
                                                        o,
                                                        u,
                                                        h,
                                                        f = t.length;
                                                    if (!e || !e.state) return m;
                                                    if (2 === (s = (r = e.state).wrap) || 1 === s && r.status !== k || r.lookahead) return m;

                                                    for (1 === s && (e.adler = c(e.adler, t, f, 0)), r.wrap = 0, f >= r.w_size && (0 === s && (C(r.head), r.strstart = 0, r.block_start = 0, r.insert = 0), h = new d.Buf8(r.w_size), d.arraySet(h, t, f - r.w_size, r.w_size, 0), t = h, f = r.w_size), a = e.avail_in, o = e.next_in, u = e.input, e.avail_in = f, e.next_in = 0, e.input = t, R(r); r.lookahead >= b;) {
                                                      for (n = r.strstart, i = r.lookahead - (b - 1); r.ins_h = (r.ins_h << r.hash_shift ^ r.window[n + b - 1]) & r.hash_mask, r.prev[n & r.w_mask] = r.head[r.ins_h], r.head[r.ins_h] = n, n++, --i;);

                                                      r.strstart = n, r.lookahead = b - 1, R(r);
                                                    }

                                                    return r.strstart += r.lookahead, r.block_start = r.strstart, r.insert = r.lookahead, r.lookahead = 0, r.match_length = r.prev_length = b - 1, r.match_available = 0, e.next_in = o, e.input = u, e.avail_in = a, r.wrap = s, l;
                                                  }, r.deflateInfo = "pako deflate (from Nodeca project)";
                                                }, {
                                                  "../utils/common": 41,
                                                  "./adler32": 43,
                                                  "./crc32": 45,
                                                  "./messages": 51,
                                                  "./trees": 52
                                                }],
                                                47: [function (e, t, r) {

                                                  t.exports = function () {
                                                    this.text = 0, this.time = 0, this.xflags = 0, this.os = 0, this.extra = null, this.extra_len = 0, this.name = "", this.comment = "", this.hcrc = 0, this.done = !1;
                                                  };
                                                }, {}],
                                                48: [function (e, t, r) {

                                                  t.exports = function (e, t) {
                                                    var r, n, i, s, a, o, u, h, f, l, d, c, p, m, _, g, v, b, w, y, k, x, S, z, C;

                                                    r = e.state, n = e.next_in, z = e.input, i = n + (e.avail_in - 5), s = e.next_out, C = e.output, a = s - (t - e.avail_out), o = s + (e.avail_out - 257), u = r.dmax, h = r.wsize, f = r.whave, l = r.wnext, d = r.window, c = r.hold, p = r.bits, m = r.lencode, _ = r.distcode, g = (1 << r.lenbits) - 1, v = (1 << r.distbits) - 1;

                                                    e: do {
                                                      p < 15 && (c += z[n++] << p, p += 8, c += z[n++] << p, p += 8), b = m[c & g];

                                                      t: for (;;) {
                                                        if (c >>>= w = b >>> 24, p -= w, 0 == (w = b >>> 16 & 255)) C[s++] = 65535 & b;else {
                                                          if (!(16 & w)) {
                                                            if (0 == (64 & w)) {
                                                              b = m[(65535 & b) + (c & (1 << w) - 1)];
                                                              continue t;
                                                            }

                                                            if (32 & w) {
                                                              r.mode = 12;
                                                              break e;
                                                            }

                                                            e.msg = "invalid literal/length code", r.mode = 30;
                                                            break e;
                                                          }

                                                          y = 65535 & b, (w &= 15) && (p < w && (c += z[n++] << p, p += 8), y += c & (1 << w) - 1, c >>>= w, p -= w), p < 15 && (c += z[n++] << p, p += 8, c += z[n++] << p, p += 8), b = _[c & v];

                                                          r: for (;;) {
                                                            if (c >>>= w = b >>> 24, p -= w, !(16 & (w = b >>> 16 & 255))) {
                                                              if (0 == (64 & w)) {
                                                                b = _[(65535 & b) + (c & (1 << w) - 1)];
                                                                continue r;
                                                              }

                                                              e.msg = "invalid distance code", r.mode = 30;
                                                              break e;
                                                            }

                                                            if (k = 65535 & b, p < (w &= 15) && (c += z[n++] << p, (p += 8) < w && (c += z[n++] << p, p += 8)), u < (k += c & (1 << w) - 1)) {
                                                              e.msg = "invalid distance too far back", r.mode = 30;
                                                              break e;
                                                            }

                                                            if (c >>>= w, p -= w, (w = s - a) < k) {
                                                              if (f < (w = k - w) && r.sane) {
                                                                e.msg = "invalid distance too far back", r.mode = 30;
                                                                break e;
                                                              }

                                                              if (S = d, (x = 0) === l) {
                                                                if (x += h - w, w < y) {
                                                                  for (y -= w; C[s++] = d[x++], --w;);

                                                                  x = s - k, S = C;
                                                                }
                                                              } else if (l < w) {
                                                                if (x += h + l - w, (w -= l) < y) {
                                                                  for (y -= w; C[s++] = d[x++], --w;);

                                                                  if (x = 0, l < y) {
                                                                    for (y -= w = l; C[s++] = d[x++], --w;);

                                                                    x = s - k, S = C;
                                                                  }
                                                                }
                                                              } else if (x += l - w, w < y) {
                                                                for (y -= w; C[s++] = d[x++], --w;);

                                                                x = s - k, S = C;
                                                              }

                                                              for (; 2 < y;) C[s++] = S[x++], C[s++] = S[x++], C[s++] = S[x++], y -= 3;

                                                              y && (C[s++] = S[x++], 1 < y && (C[s++] = S[x++]));
                                                            } else {
                                                              for (x = s - k; C[s++] = C[x++], C[s++] = C[x++], C[s++] = C[x++], 2 < (y -= 3););

                                                              y && (C[s++] = C[x++], 1 < y && (C[s++] = C[x++]));
                                                            }

                                                            break;
                                                          }
                                                        }
                                                        break;
                                                      }
                                                    } while (n < i && s < o);

                                                    n -= y = p >> 3, c &= (1 << (p -= y << 3)) - 1, e.next_in = n, e.next_out = s, e.avail_in = n < i ? i - n + 5 : 5 - (n - i), e.avail_out = s < o ? o - s + 257 : 257 - (s - o), r.hold = c, r.bits = p;
                                                  };
                                                }, {}],
                                                49: [function (e, t, r) {

                                                  var I = e("../utils/common"),
                                                      O = e("./adler32"),
                                                      B = e("./crc32"),
                                                      R = e("./inffast"),
                                                      T = e("./inftrees"),
                                                      D = 1,
                                                      F = 2,
                                                      N = 0,
                                                      U = -2,
                                                      P = 1,
                                                      n = 852,
                                                      i = 592;

                                                  function L(e) {
                                                    return (e >>> 24 & 255) + (e >>> 8 & 65280) + ((65280 & e) << 8) + ((255 & e) << 24);
                                                  }

                                                  function s() {
                                                    this.mode = 0, this.last = !1, this.wrap = 0, this.havedict = !1, this.flags = 0, this.dmax = 0, this.check = 0, this.total = 0, this.head = null, this.wbits = 0, this.wsize = 0, this.whave = 0, this.wnext = 0, this.window = null, this.hold = 0, this.bits = 0, this.length = 0, this.offset = 0, this.extra = 0, this.lencode = null, this.distcode = null, this.lenbits = 0, this.distbits = 0, this.ncode = 0, this.nlen = 0, this.ndist = 0, this.have = 0, this.next = null, this.lens = new I.Buf16(320), this.work = new I.Buf16(288), this.lendyn = null, this.distdyn = null, this.sane = 0, this.back = 0, this.was = 0;
                                                  }

                                                  function a(e) {
                                                    var t;
                                                    return e && e.state ? (t = e.state, e.total_in = e.total_out = t.total = 0, e.msg = "", t.wrap && (e.adler = 1 & t.wrap), t.mode = P, t.last = 0, t.havedict = 0, t.dmax = 32768, t.head = null, t.hold = 0, t.bits = 0, t.lencode = t.lendyn = new I.Buf32(n), t.distcode = t.distdyn = new I.Buf32(i), t.sane = 1, t.back = -1, N) : U;
                                                  }

                                                  function o(e) {
                                                    var t;
                                                    return e && e.state ? ((t = e.state).wsize = 0, t.whave = 0, t.wnext = 0, a(e)) : U;
                                                  }

                                                  function u(e, t) {
                                                    var r, n;
                                                    return e && e.state ? (n = e.state, t < 0 ? (r = 0, t = -t) : (r = 1 + (t >> 4), t < 48 && (t &= 15)), t && (t < 8 || 15 < t) ? U : (null !== n.window && n.wbits !== t && (n.window = null), n.wrap = r, n.wbits = t, o(e))) : U;
                                                  }

                                                  function h(e, t) {
                                                    var r, n;
                                                    return e ? (n = new s(), (e.state = n).window = null, (r = u(e, t)) !== N && (e.state = null), r) : U;
                                                  }

                                                  var f,
                                                      l,
                                                      d = !0;

                                                  function j(e) {
                                                    if (d) {
                                                      var t;

                                                      for (f = new I.Buf32(512), l = new I.Buf32(32), t = 0; t < 144;) e.lens[t++] = 8;

                                                      for (; t < 256;) e.lens[t++] = 9;

                                                      for (; t < 280;) e.lens[t++] = 7;

                                                      for (; t < 288;) e.lens[t++] = 8;

                                                      for (T(D, e.lens, 0, 288, f, 0, e.work, {
                                                        bits: 9
                                                      }), t = 0; t < 32;) e.lens[t++] = 5;

                                                      T(F, e.lens, 0, 32, l, 0, e.work, {
                                                        bits: 5
                                                      }), d = !1;
                                                    }

                                                    e.lencode = f, e.lenbits = 9, e.distcode = l, e.distbits = 5;
                                                  }

                                                  function Z(e, t, r, n) {
                                                    var i,
                                                        s = e.state;
                                                    return null === s.window && (s.wsize = 1 << s.wbits, s.wnext = 0, s.whave = 0, s.window = new I.Buf8(s.wsize)), n >= s.wsize ? (I.arraySet(s.window, t, r - s.wsize, s.wsize, 0), s.wnext = 0, s.whave = s.wsize) : (n < (i = s.wsize - s.wnext) && (i = n), I.arraySet(s.window, t, r - n, i, s.wnext), (n -= i) ? (I.arraySet(s.window, t, r - n, n, 0), s.wnext = n, s.whave = s.wsize) : (s.wnext += i, s.wnext === s.wsize && (s.wnext = 0), s.whave < s.wsize && (s.whave += i))), 0;
                                                  }

                                                  r.inflateReset = o, r.inflateReset2 = u, r.inflateResetKeep = a, r.inflateInit = function (e) {
                                                    return h(e, 15);
                                                  }, r.inflateInit2 = h, r.inflate = function (e, t) {
                                                    var r,
                                                        n,
                                                        i,
                                                        s,
                                                        a,
                                                        o,
                                                        u,
                                                        h,
                                                        f,
                                                        l,
                                                        d,
                                                        c,
                                                        p,
                                                        m,
                                                        _,
                                                        g,
                                                        v,
                                                        b,
                                                        w,
                                                        y,
                                                        k,
                                                        x,
                                                        S,
                                                        z,
                                                        C = 0,
                                                        E = new I.Buf8(4),
                                                        A = [16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15];

                                                    if (!e || !e.state || !e.output || !e.input && 0 !== e.avail_in) return U;
                                                    12 === (r = e.state).mode && (r.mode = 13), a = e.next_out, i = e.output, u = e.avail_out, s = e.next_in, n = e.input, o = e.avail_in, h = r.hold, f = r.bits, l = o, d = u, x = N;

                                                    e: for (;;) switch (r.mode) {
                                                      case P:
                                                        if (0 === r.wrap) {
                                                          r.mode = 13;
                                                          break;
                                                        }

                                                        for (; f < 16;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        if (2 & r.wrap && 35615 === h) {
                                                          E[r.check = 0] = 255 & h, E[1] = h >>> 8 & 255, r.check = B(r.check, E, 2, 0), f = h = 0, r.mode = 2;
                                                          break;
                                                        }

                                                        if (r.flags = 0, r.head && (r.head.done = !1), !(1 & r.wrap) || (((255 & h) << 8) + (h >> 8)) % 31) {
                                                          e.msg = "incorrect header check", r.mode = 30;
                                                          break;
                                                        }

                                                        if (8 != (15 & h)) {
                                                          e.msg = "unknown compression method", r.mode = 30;
                                                          break;
                                                        }

                                                        if (f -= 4, k = 8 + (15 & (h >>>= 4)), 0 === r.wbits) r.wbits = k;else if (k > r.wbits) {
                                                          e.msg = "invalid window size", r.mode = 30;
                                                          break;
                                                        }
                                                        r.dmax = 1 << k, e.adler = r.check = 1, r.mode = 512 & h ? 10 : 12, f = h = 0;
                                                        break;

                                                      case 2:
                                                        for (; f < 16;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        if (r.flags = h, 8 != (255 & r.flags)) {
                                                          e.msg = "unknown compression method", r.mode = 30;
                                                          break;
                                                        }

                                                        if (57344 & r.flags) {
                                                          e.msg = "unknown header flags set", r.mode = 30;
                                                          break;
                                                        }

                                                        r.head && (r.head.text = h >> 8 & 1), 512 & r.flags && (E[0] = 255 & h, E[1] = h >>> 8 & 255, r.check = B(r.check, E, 2, 0)), f = h = 0, r.mode = 3;

                                                      case 3:
                                                        for (; f < 32;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        r.head && (r.head.time = h), 512 & r.flags && (E[0] = 255 & h, E[1] = h >>> 8 & 255, E[2] = h >>> 16 & 255, E[3] = h >>> 24 & 255, r.check = B(r.check, E, 4, 0)), f = h = 0, r.mode = 4;

                                                      case 4:
                                                        for (; f < 16;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        r.head && (r.head.xflags = 255 & h, r.head.os = h >> 8), 512 & r.flags && (E[0] = 255 & h, E[1] = h >>> 8 & 255, r.check = B(r.check, E, 2, 0)), f = h = 0, r.mode = 5;

                                                      case 5:
                                                        if (1024 & r.flags) {
                                                          for (; f < 16;) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          r.length = h, r.head && (r.head.extra_len = h), 512 & r.flags && (E[0] = 255 & h, E[1] = h >>> 8 & 255, r.check = B(r.check, E, 2, 0)), f = h = 0;
                                                        } else r.head && (r.head.extra = null);

                                                        r.mode = 6;

                                                      case 6:
                                                        if (1024 & r.flags && (o < (c = r.length) && (c = o), c && (r.head && (k = r.head.extra_len - r.length, r.head.extra || (r.head.extra = new Array(r.head.extra_len)), I.arraySet(r.head.extra, n, s, c, k)), 512 & r.flags && (r.check = B(r.check, n, c, s)), o -= c, s += c, r.length -= c), r.length)) break e;
                                                        r.length = 0, r.mode = 7;

                                                      case 7:
                                                        if (2048 & r.flags) {
                                                          if (0 === o) break e;

                                                          for (c = 0; k = n[s + c++], r.head && k && r.length < 65536 && (r.head.name += String.fromCharCode(k)), k && c < o;);

                                                          if (512 & r.flags && (r.check = B(r.check, n, c, s)), o -= c, s += c, k) break e;
                                                        } else r.head && (r.head.name = null);

                                                        r.length = 0, r.mode = 8;

                                                      case 8:
                                                        if (4096 & r.flags) {
                                                          if (0 === o) break e;

                                                          for (c = 0; k = n[s + c++], r.head && k && r.length < 65536 && (r.head.comment += String.fromCharCode(k)), k && c < o;);

                                                          if (512 & r.flags && (r.check = B(r.check, n, c, s)), o -= c, s += c, k) break e;
                                                        } else r.head && (r.head.comment = null);

                                                        r.mode = 9;

                                                      case 9:
                                                        if (512 & r.flags) {
                                                          for (; f < 16;) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          if (h !== (65535 & r.check)) {
                                                            e.msg = "header crc mismatch", r.mode = 30;
                                                            break;
                                                          }

                                                          f = h = 0;
                                                        }

                                                        r.head && (r.head.hcrc = r.flags >> 9 & 1, r.head.done = !0), e.adler = r.check = 0, r.mode = 12;
                                                        break;

                                                      case 10:
                                                        for (; f < 32;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        e.adler = r.check = L(h), f = h = 0, r.mode = 11;

                                                      case 11:
                                                        if (0 === r.havedict) return e.next_out = a, e.avail_out = u, e.next_in = s, e.avail_in = o, r.hold = h, r.bits = f, 2;
                                                        e.adler = r.check = 1, r.mode = 12;

                                                      case 12:
                                                        if (5 === t || 6 === t) break e;

                                                      case 13:
                                                        if (r.last) {
                                                          h >>>= 7 & f, f -= 7 & f, r.mode = 27;
                                                          break;
                                                        }

                                                        for (; f < 3;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        switch (r.last = 1 & h, f -= 1, 3 & (h >>>= 1)) {
                                                          case 0:
                                                            r.mode = 14;
                                                            break;

                                                          case 1:
                                                            if (j(r), r.mode = 20, 6 !== t) break;
                                                            h >>>= 2, f -= 2;
                                                            break e;

                                                          case 2:
                                                            r.mode = 17;
                                                            break;

                                                          case 3:
                                                            e.msg = "invalid block type", r.mode = 30;
                                                        }

                                                        h >>>= 2, f -= 2;
                                                        break;

                                                      case 14:
                                                        for (h >>>= 7 & f, f -= 7 & f; f < 32;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        if ((65535 & h) != (h >>> 16 ^ 65535)) {
                                                          e.msg = "invalid stored block lengths", r.mode = 30;
                                                          break;
                                                        }

                                                        if (r.length = 65535 & h, f = h = 0, r.mode = 15, 6 === t) break e;

                                                      case 15:
                                                        r.mode = 16;

                                                      case 16:
                                                        if (c = r.length) {
                                                          if (o < c && (c = o), u < c && (c = u), 0 === c) break e;
                                                          I.arraySet(i, n, s, c, a), o -= c, s += c, u -= c, a += c, r.length -= c;
                                                          break;
                                                        }

                                                        r.mode = 12;
                                                        break;

                                                      case 17:
                                                        for (; f < 14;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        if (r.nlen = 257 + (31 & h), h >>>= 5, f -= 5, r.ndist = 1 + (31 & h), h >>>= 5, f -= 5, r.ncode = 4 + (15 & h), h >>>= 4, f -= 4, 286 < r.nlen || 30 < r.ndist) {
                                                          e.msg = "too many length or distance symbols", r.mode = 30;
                                                          break;
                                                        }

                                                        r.have = 0, r.mode = 18;

                                                      case 18:
                                                        for (; r.have < r.ncode;) {
                                                          for (; f < 3;) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          r.lens[A[r.have++]] = 7 & h, h >>>= 3, f -= 3;
                                                        }

                                                        for (; r.have < 19;) r.lens[A[r.have++]] = 0;

                                                        if (r.lencode = r.lendyn, r.lenbits = 7, S = {
                                                          bits: r.lenbits
                                                        }, x = T(0, r.lens, 0, 19, r.lencode, 0, r.work, S), r.lenbits = S.bits, x) {
                                                          e.msg = "invalid code lengths set", r.mode = 30;
                                                          break;
                                                        }

                                                        r.have = 0, r.mode = 19;

                                                      case 19:
                                                        for (; r.have < r.nlen + r.ndist;) {
                                                          for (; g = (C = r.lencode[h & (1 << r.lenbits) - 1]) >>> 16 & 255, v = 65535 & C, !((_ = C >>> 24) <= f);) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          if (v < 16) h >>>= _, f -= _, r.lens[r.have++] = v;else {
                                                            if (16 === v) {
                                                              for (z = _ + 2; f < z;) {
                                                                if (0 === o) break e;
                                                                o--, h += n[s++] << f, f += 8;
                                                              }

                                                              if (h >>>= _, f -= _, 0 === r.have) {
                                                                e.msg = "invalid bit length repeat", r.mode = 30;
                                                                break;
                                                              }

                                                              k = r.lens[r.have - 1], c = 3 + (3 & h), h >>>= 2, f -= 2;
                                                            } else if (17 === v) {
                                                              for (z = _ + 3; f < z;) {
                                                                if (0 === o) break e;
                                                                o--, h += n[s++] << f, f += 8;
                                                              }

                                                              f -= _, k = 0, c = 3 + (7 & (h >>>= _)), h >>>= 3, f -= 3;
                                                            } else {
                                                              for (z = _ + 7; f < z;) {
                                                                if (0 === o) break e;
                                                                o--, h += n[s++] << f, f += 8;
                                                              }

                                                              f -= _, k = 0, c = 11 + (127 & (h >>>= _)), h >>>= 7, f -= 7;
                                                            }

                                                            if (r.have + c > r.nlen + r.ndist) {
                                                              e.msg = "invalid bit length repeat", r.mode = 30;
                                                              break;
                                                            }

                                                            for (; c--;) r.lens[r.have++] = k;
                                                          }
                                                        }

                                                        if (30 === r.mode) break;

                                                        if (0 === r.lens[256]) {
                                                          e.msg = "invalid code -- missing end-of-block", r.mode = 30;
                                                          break;
                                                        }

                                                        if (r.lenbits = 9, S = {
                                                          bits: r.lenbits
                                                        }, x = T(D, r.lens, 0, r.nlen, r.lencode, 0, r.work, S), r.lenbits = S.bits, x) {
                                                          e.msg = "invalid literal/lengths set", r.mode = 30;
                                                          break;
                                                        }

                                                        if (r.distbits = 6, r.distcode = r.distdyn, S = {
                                                          bits: r.distbits
                                                        }, x = T(F, r.lens, r.nlen, r.ndist, r.distcode, 0, r.work, S), r.distbits = S.bits, x) {
                                                          e.msg = "invalid distances set", r.mode = 30;
                                                          break;
                                                        }

                                                        if (r.mode = 20, 6 === t) break e;

                                                      case 20:
                                                        r.mode = 21;

                                                      case 21:
                                                        if (6 <= o && 258 <= u) {
                                                          e.next_out = a, e.avail_out = u, e.next_in = s, e.avail_in = o, r.hold = h, r.bits = f, R(e, d), a = e.next_out, i = e.output, u = e.avail_out, s = e.next_in, n = e.input, o = e.avail_in, h = r.hold, f = r.bits, 12 === r.mode && (r.back = -1);
                                                          break;
                                                        }

                                                        for (r.back = 0; g = (C = r.lencode[h & (1 << r.lenbits) - 1]) >>> 16 & 255, v = 65535 & C, !((_ = C >>> 24) <= f);) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        if (g && 0 == (240 & g)) {
                                                          for (b = _, w = g, y = v; g = (C = r.lencode[y + ((h & (1 << b + w) - 1) >> b)]) >>> 16 & 255, v = 65535 & C, !(b + (_ = C >>> 24) <= f);) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          h >>>= b, f -= b, r.back += b;
                                                        }

                                                        if (h >>>= _, f -= _, r.back += _, r.length = v, 0 === g) {
                                                          r.mode = 26;
                                                          break;
                                                        }

                                                        if (32 & g) {
                                                          r.back = -1, r.mode = 12;
                                                          break;
                                                        }

                                                        if (64 & g) {
                                                          e.msg = "invalid literal/length code", r.mode = 30;
                                                          break;
                                                        }

                                                        r.extra = 15 & g, r.mode = 22;

                                                      case 22:
                                                        if (r.extra) {
                                                          for (z = r.extra; f < z;) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          r.length += h & (1 << r.extra) - 1, h >>>= r.extra, f -= r.extra, r.back += r.extra;
                                                        }

                                                        r.was = r.length, r.mode = 23;

                                                      case 23:
                                                        for (; g = (C = r.distcode[h & (1 << r.distbits) - 1]) >>> 16 & 255, v = 65535 & C, !((_ = C >>> 24) <= f);) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        if (0 == (240 & g)) {
                                                          for (b = _, w = g, y = v; g = (C = r.distcode[y + ((h & (1 << b + w) - 1) >> b)]) >>> 16 & 255, v = 65535 & C, !(b + (_ = C >>> 24) <= f);) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          h >>>= b, f -= b, r.back += b;
                                                        }

                                                        if (h >>>= _, f -= _, r.back += _, 64 & g) {
                                                          e.msg = "invalid distance code", r.mode = 30;
                                                          break;
                                                        }

                                                        r.offset = v, r.extra = 15 & g, r.mode = 24;

                                                      case 24:
                                                        if (r.extra) {
                                                          for (z = r.extra; f < z;) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          r.offset += h & (1 << r.extra) - 1, h >>>= r.extra, f -= r.extra, r.back += r.extra;
                                                        }

                                                        if (r.offset > r.dmax) {
                                                          e.msg = "invalid distance too far back", r.mode = 30;
                                                          break;
                                                        }

                                                        r.mode = 25;

                                                      case 25:
                                                        if (0 === u) break e;

                                                        if (c = d - u, r.offset > c) {
                                                          if ((c = r.offset - c) > r.whave && r.sane) {
                                                            e.msg = "invalid distance too far back", r.mode = 30;
                                                            break;
                                                          }

                                                          p = c > r.wnext ? (c -= r.wnext, r.wsize - c) : r.wnext - c, c > r.length && (c = r.length), m = r.window;
                                                        } else m = i, p = a - r.offset, c = r.length;

                                                        for (u < c && (c = u), u -= c, r.length -= c; i[a++] = m[p++], --c;);

                                                        0 === r.length && (r.mode = 21);
                                                        break;

                                                      case 26:
                                                        if (0 === u) break e;
                                                        i[a++] = r.length, u--, r.mode = 21;
                                                        break;

                                                      case 27:
                                                        if (r.wrap) {
                                                          for (; f < 32;) {
                                                            if (0 === o) break e;
                                                            o--, h |= n[s++] << f, f += 8;
                                                          }

                                                          if (d -= u, e.total_out += d, r.total += d, d && (e.adler = r.check = r.flags ? B(r.check, i, d, a - d) : O(r.check, i, d, a - d)), d = u, (r.flags ? h : L(h)) !== r.check) {
                                                            e.msg = "incorrect data check", r.mode = 30;
                                                            break;
                                                          }

                                                          f = h = 0;
                                                        }

                                                        r.mode = 28;

                                                      case 28:
                                                        if (r.wrap && r.flags) {
                                                          for (; f < 32;) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          if (h !== (4294967295 & r.total)) {
                                                            e.msg = "incorrect length check", r.mode = 30;
                                                            break;
                                                          }

                                                          f = h = 0;
                                                        }

                                                        r.mode = 29;

                                                      case 29:
                                                        x = 1;
                                                        break e;

                                                      case 30:
                                                        x = -3;
                                                        break e;

                                                      case 31:
                                                        return -4;

                                                      case 32:
                                                      default:
                                                        return U;
                                                    }

                                                    return e.next_out = a, e.avail_out = u, e.next_in = s, e.avail_in = o, r.hold = h, r.bits = f, (r.wsize || d !== e.avail_out && r.mode < 30 && (r.mode < 27 || 4 !== t)) && Z(e, e.output, e.next_out, d - e.avail_out) ? (r.mode = 31, -4) : (l -= e.avail_in, d -= e.avail_out, e.total_in += l, e.total_out += d, r.total += d, r.wrap && d && (e.adler = r.check = r.flags ? B(r.check, i, d, e.next_out - d) : O(r.check, i, d, e.next_out - d)), e.data_type = r.bits + (r.last ? 64 : 0) + (12 === r.mode ? 128 : 0) + (20 === r.mode || 15 === r.mode ? 256 : 0), (0 == l && 0 === d || 4 === t) && x === N && (x = -5), x);
                                                  }, r.inflateEnd = function (e) {
                                                    if (!e || !e.state) return U;
                                                    var t = e.state;
                                                    return t.window && (t.window = null), e.state = null, N;
                                                  }, r.inflateGetHeader = function (e, t) {
                                                    var r;
                                                    return e && e.state ? 0 == (2 & (r = e.state).wrap) ? U : ((r.head = t).done = !1, N) : U;
                                                  }, r.inflateSetDictionary = function (e, t) {
                                                    var r,
                                                        n = t.length;
                                                    return e && e.state ? 0 !== (r = e.state).wrap && 11 !== r.mode ? U : 11 === r.mode && O(1, t, n, 0) !== r.check ? -3 : Z(e, t, n, n) ? (r.mode = 31, -4) : (r.havedict = 1, N) : U;
                                                  }, r.inflateInfo = "pako inflate (from Nodeca project)";
                                                }, {
                                                  "../utils/common": 41,
                                                  "./adler32": 43,
                                                  "./crc32": 45,
                                                  "./inffast": 48,
                                                  "./inftrees": 50
                                                }],
                                                50: [function (e, t, r) {

                                                  var D = e("../utils/common"),
                                                      F = [3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, 17, 19, 23, 27, 31, 35, 43, 51, 59, 67, 83, 99, 115, 131, 163, 195, 227, 258, 0, 0],
                                                      N = [16, 16, 16, 16, 16, 16, 16, 16, 17, 17, 17, 17, 18, 18, 18, 18, 19, 19, 19, 19, 20, 20, 20, 20, 21, 21, 21, 21, 16, 72, 78],
                                                      U = [1, 2, 3, 4, 5, 7, 9, 13, 17, 25, 33, 49, 65, 97, 129, 193, 257, 385, 513, 769, 1025, 1537, 2049, 3073, 4097, 6145, 8193, 12289, 16385, 24577, 0, 0],
                                                      P = [16, 16, 16, 16, 17, 17, 18, 18, 19, 19, 20, 20, 21, 21, 22, 22, 23, 23, 24, 24, 25, 25, 26, 26, 27, 27, 28, 28, 29, 29, 64, 64];

                                                  t.exports = function (e, t, r, n, i, s, a, o) {
                                                    var u,
                                                        h,
                                                        f,
                                                        l,
                                                        d,
                                                        c,
                                                        p,
                                                        m,
                                                        _,
                                                        g = o.bits,
                                                        v = 0,
                                                        b = 0,
                                                        w = 0,
                                                        y = 0,
                                                        k = 0,
                                                        x = 0,
                                                        S = 0,
                                                        z = 0,
                                                        C = 0,
                                                        E = 0,
                                                        A = null,
                                                        I = 0,
                                                        O = new D.Buf16(16),
                                                        B = new D.Buf16(16),
                                                        R = null,
                                                        T = 0;

                                                    for (v = 0; v <= 15; v++) O[v] = 0;

                                                    for (b = 0; b < n; b++) O[t[r + b]]++;

                                                    for (k = g, y = 15; 1 <= y && 0 === O[y]; y--);

                                                    if (y < k && (k = y), 0 === y) return i[s++] = 20971520, i[s++] = 20971520, o.bits = 1, 0;

                                                    for (w = 1; w < y && 0 === O[w]; w++);

                                                    for (k < w && (k = w), v = z = 1; v <= 15; v++) if (z <<= 1, (z -= O[v]) < 0) return -1;

                                                    if (0 < z && (0 === e || 1 !== y)) return -1;

                                                    for (B[1] = 0, v = 1; v < 15; v++) B[v + 1] = B[v] + O[v];

                                                    for (b = 0; b < n; b++) 0 !== t[r + b] && (a[B[t[r + b]]++] = b);

                                                    if (c = 0 === e ? (A = R = a, 19) : 1 === e ? (A = F, I -= 257, R = N, T -= 257, 256) : (A = U, R = P, -1), v = w, d = s, S = b = E = 0, f = -1, l = (C = 1 << (x = k)) - 1, 1 === e && 852 < C || 2 === e && 592 < C) return 1;

                                                    for (;;) {
                                                      for (p = v - S, _ = a[b] < c ? (m = 0, a[b]) : a[b] > c ? (m = R[T + a[b]], A[I + a[b]]) : (m = 96, 0), u = 1 << v - S, w = h = 1 << x; i[d + (E >> S) + (h -= u)] = p << 24 | m << 16 | _ | 0, 0 !== h;);

                                                      for (u = 1 << v - 1; E & u;) u >>= 1;

                                                      if (0 !== u ? (E &= u - 1, E += u) : E = 0, b++, 0 == --O[v]) {
                                                        if (v === y) break;
                                                        v = t[r + a[b]];
                                                      }

                                                      if (k < v && (E & l) !== f) {
                                                        for (0 === S && (S = k), d += w, z = 1 << (x = v - S); x + S < y && !((z -= O[x + S]) <= 0);) x++, z <<= 1;

                                                        if (C += 1 << x, 1 === e && 852 < C || 2 === e && 592 < C) return 1;
                                                        i[f = E & l] = k << 24 | x << 16 | d - s | 0;
                                                      }
                                                    }

                                                    return 0 !== E && (i[d + E] = v - S << 24 | 64 << 16 | 0), o.bits = k, 0;
                                                  };
                                                }, {
                                                  "../utils/common": 41
                                                }],
                                                51: [function (e, t, r) {

                                                  t.exports = {
                                                    2: "need dictionary",
                                                    1: "stream end",
                                                    0: "",
                                                    "-1": "file error",
                                                    "-2": "stream error",
                                                    "-3": "data error",
                                                    "-4": "insufficient memory",
                                                    "-5": "buffer error",
                                                    "-6": "incompatible version"
                                                  };
                                                }, {}],
                                                52: [function (e, t, r) {

                                                  var o = e("../utils/common");

                                                  function n(e) {
                                                    for (var t = e.length; 0 <= --t;) e[t] = 0;
                                                  }

                                                  var _ = 15,
                                                      i = 16,
                                                      u = [0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 0],
                                                      h = [0, 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 10, 11, 11, 12, 12, 13, 13],
                                                      a = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 3, 7],
                                                      f = [16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15],
                                                      l = new Array(576);
                                                  n(l);
                                                  var d = new Array(60);
                                                  n(d);
                                                  var c = new Array(512);
                                                  n(c);
                                                  var p = new Array(256);
                                                  n(p);
                                                  var m = new Array(29);
                                                  n(m);
                                                  var g,
                                                      v,
                                                      b,
                                                      w = new Array(30);

                                                  function y(e, t, r, n, i) {
                                                    this.static_tree = e, this.extra_bits = t, this.extra_base = r, this.elems = n, this.max_length = i, this.has_stree = e && e.length;
                                                  }

                                                  function s(e, t) {
                                                    this.dyn_tree = e, this.max_code = 0, this.stat_desc = t;
                                                  }

                                                  function k(e) {
                                                    return e < 256 ? c[e] : c[256 + (e >>> 7)];
                                                  }

                                                  function x(e, t) {
                                                    e.pending_buf[e.pending++] = 255 & t, e.pending_buf[e.pending++] = t >>> 8 & 255;
                                                  }

                                                  function S(e, t, r) {
                                                    e.bi_valid > i - r ? (e.bi_buf |= t << e.bi_valid & 65535, x(e, e.bi_buf), e.bi_buf = t >> i - e.bi_valid, e.bi_valid += r - i) : (e.bi_buf |= t << e.bi_valid & 65535, e.bi_valid += r);
                                                  }

                                                  function z(e, t, r) {
                                                    S(e, r[2 * t], r[2 * t + 1]);
                                                  }

                                                  function C(e, t) {
                                                    for (var r = 0; r |= 1 & e, e >>>= 1, r <<= 1, 0 < --t;);

                                                    return r >>> 1;
                                                  }

                                                  function E(e, t, r) {
                                                    var n,
                                                        i,
                                                        s = new Array(_ + 1),
                                                        a = 0;

                                                    for (n = 1; n <= _; n++) s[n] = a = a + r[n - 1] << 1;

                                                    for (i = 0; i <= t; i++) {
                                                      var o = e[2 * i + 1];
                                                      0 !== o && (e[2 * i] = C(s[o]++, o));
                                                    }
                                                  }

                                                  function A(e) {
                                                    var t;

                                                    for (t = 0; t < 286; t++) e.dyn_ltree[2 * t] = 0;

                                                    for (t = 0; t < 30; t++) e.dyn_dtree[2 * t] = 0;

                                                    for (t = 0; t < 19; t++) e.bl_tree[2 * t] = 0;

                                                    e.dyn_ltree[512] = 1, e.opt_len = e.static_len = 0, e.last_lit = e.matches = 0;
                                                  }

                                                  function I(e) {
                                                    8 < e.bi_valid ? x(e, e.bi_buf) : 0 < e.bi_valid && (e.pending_buf[e.pending++] = e.bi_buf), e.bi_buf = 0, e.bi_valid = 0;
                                                  }

                                                  function O(e, t, r, n) {
                                                    var i = 2 * t,
                                                        s = 2 * r;
                                                    return e[i] < e[s] || e[i] === e[s] && n[t] <= n[r];
                                                  }

                                                  function B(e, t, r) {
                                                    for (var n = e.heap[r], i = r << 1; i <= e.heap_len && (i < e.heap_len && O(t, e.heap[i + 1], e.heap[i], e.depth) && i++, !O(t, n, e.heap[i], e.depth));) e.heap[r] = e.heap[i], r = i, i <<= 1;

                                                    e.heap[r] = n;
                                                  }

                                                  function R(e, t, r) {
                                                    var n,
                                                        i,
                                                        s,
                                                        a,
                                                        o = 0;
                                                    if (0 !== e.last_lit) for (; n = e.pending_buf[e.d_buf + 2 * o] << 8 | e.pending_buf[e.d_buf + 2 * o + 1], i = e.pending_buf[e.l_buf + o], o++, 0 === n ? z(e, i, t) : (z(e, (s = p[i]) + 256 + 1, t), 0 !== (a = u[s]) && S(e, i -= m[s], a), z(e, s = k(--n), r), 0 !== (a = h[s]) && S(e, n -= w[s], a)), o < e.last_lit;);
                                                    z(e, 256, t);
                                                  }

                                                  function T(e, t) {
                                                    var r,
                                                        n,
                                                        i,
                                                        s = t.dyn_tree,
                                                        a = t.stat_desc.static_tree,
                                                        o = t.stat_desc.has_stree,
                                                        u = t.stat_desc.elems,
                                                        h = -1;

                                                    for (e.heap_len = 0, e.heap_max = 573, r = 0; r < u; r++) 0 !== s[2 * r] ? (e.heap[++e.heap_len] = h = r, e.depth[r] = 0) : s[2 * r + 1] = 0;

                                                    for (; e.heap_len < 2;) s[2 * (i = e.heap[++e.heap_len] = h < 2 ? ++h : 0)] = 1, e.depth[i] = 0, e.opt_len--, o && (e.static_len -= a[2 * i + 1]);

                                                    for (t.max_code = h, r = e.heap_len >> 1; 1 <= r; r--) B(e, s, r);

                                                    for (i = u; r = e.heap[1], e.heap[1] = e.heap[e.heap_len--], B(e, s, 1), n = e.heap[1], e.heap[--e.heap_max] = r, e.heap[--e.heap_max] = n, s[2 * i] = s[2 * r] + s[2 * n], e.depth[i] = (e.depth[r] >= e.depth[n] ? e.depth[r] : e.depth[n]) + 1, s[2 * r + 1] = s[2 * n + 1] = i, e.heap[1] = i++, B(e, s, 1), 2 <= e.heap_len;);

                                                    e.heap[--e.heap_max] = e.heap[1], function (e, t) {
                                                      var r,
                                                          n,
                                                          i,
                                                          s,
                                                          a,
                                                          o,
                                                          u = t.dyn_tree,
                                                          h = t.max_code,
                                                          f = t.stat_desc.static_tree,
                                                          l = t.stat_desc.has_stree,
                                                          d = t.stat_desc.extra_bits,
                                                          c = t.stat_desc.extra_base,
                                                          p = t.stat_desc.max_length,
                                                          m = 0;

                                                      for (s = 0; s <= _; s++) e.bl_count[s] = 0;

                                                      for (u[2 * e.heap[e.heap_max] + 1] = 0, r = e.heap_max + 1; r < 573; r++) p < (s = u[2 * u[2 * (n = e.heap[r]) + 1] + 1] + 1) && (s = p, m++), u[2 * n + 1] = s, h < n || (e.bl_count[s]++, a = 0, c <= n && (a = d[n - c]), o = u[2 * n], e.opt_len += o * (s + a), l && (e.static_len += o * (f[2 * n + 1] + a)));

                                                      if (0 !== m) {
                                                        do {
                                                          for (s = p - 1; 0 === e.bl_count[s];) s--;

                                                          e.bl_count[s]--, e.bl_count[s + 1] += 2, e.bl_count[p]--, m -= 2;
                                                        } while (0 < m);

                                                        for (s = p; 0 !== s; s--) for (n = e.bl_count[s]; 0 !== n;) h < (i = e.heap[--r]) || (u[2 * i + 1] !== s && (e.opt_len += (s - u[2 * i + 1]) * u[2 * i], u[2 * i + 1] = s), n--);
                                                      }
                                                    }(e, t), E(s, h, e.bl_count);
                                                  }

                                                  function D(e, t, r) {
                                                    var n,
                                                        i,
                                                        s = -1,
                                                        a = t[1],
                                                        o = 0,
                                                        u = 7,
                                                        h = 4;

                                                    for (0 === a && (u = 138, h = 3), t[2 * (r + 1) + 1] = 65535, n = 0; n <= r; n++) i = a, a = t[2 * (n + 1) + 1], ++o < u && i === a || (o < h ? e.bl_tree[2 * i] += o : 0 !== i ? (i !== s && e.bl_tree[2 * i]++, e.bl_tree[32]++) : o <= 10 ? e.bl_tree[34]++ : e.bl_tree[36]++, s = i, h = (o = 0) === a ? (u = 138, 3) : i === a ? (u = 6, 3) : (u = 7, 4));
                                                  }

                                                  function F(e, t, r) {
                                                    var n,
                                                        i,
                                                        s = -1,
                                                        a = t[1],
                                                        o = 0,
                                                        u = 7,
                                                        h = 4;

                                                    for (0 === a && (u = 138, h = 3), n = 0; n <= r; n++) if (i = a, a = t[2 * (n + 1) + 1], !(++o < u && i === a)) {
                                                      if (o < h) for (; z(e, i, e.bl_tree), 0 != --o;);else 0 !== i ? (i !== s && (z(e, i, e.bl_tree), o--), z(e, 16, e.bl_tree), S(e, o - 3, 2)) : o <= 10 ? (z(e, 17, e.bl_tree), S(e, o - 3, 3)) : (z(e, 18, e.bl_tree), S(e, o - 11, 7));
                                                      s = i, h = (o = 0) === a ? (u = 138, 3) : i === a ? (u = 6, 3) : (u = 7, 4);
                                                    }
                                                  }

                                                  n(w);
                                                  var N = !1;

                                                  function U(e, t, r, n) {
                                                    var i, s, a;
                                                    S(e, 0 + (n ? 1 : 0), 3), s = t, a = r, I(i = e), x(i, a), x(i, ~a), o.arraySet(i.pending_buf, i.window, s, a, i.pending), i.pending += a;
                                                  }

                                                  r._tr_init = function (e) {
                                                    N || (function () {
                                                      var e,
                                                          t,
                                                          r,
                                                          n,
                                                          i,
                                                          s = new Array(_ + 1);

                                                      for (n = r = 0; n < 28; n++) for (m[n] = r, e = 0; e < 1 << u[n]; e++) p[r++] = n;

                                                      for (p[r - 1] = n, n = i = 0; n < 16; n++) for (w[n] = i, e = 0; e < 1 << h[n]; e++) c[i++] = n;

                                                      for (i >>= 7; n < 30; n++) for (w[n] = i << 7, e = 0; e < 1 << h[n] - 7; e++) c[256 + i++] = n;

                                                      for (t = 0; t <= _; t++) s[t] = 0;

                                                      for (e = 0; e <= 143;) l[2 * e + 1] = 8, e++, s[8]++;

                                                      for (; e <= 255;) l[2 * e + 1] = 9, e++, s[9]++;

                                                      for (; e <= 279;) l[2 * e + 1] = 7, e++, s[7]++;

                                                      for (; e <= 287;) l[2 * e + 1] = 8, e++, s[8]++;

                                                      for (E(l, 287, s), e = 0; e < 30; e++) d[2 * e + 1] = 5, d[2 * e] = C(e, 5);

                                                      g = new y(l, u, 257, 286, _), v = new y(d, h, 0, 30, _), b = new y(new Array(0), a, 0, 19, 7);
                                                    }(), N = !0), e.l_desc = new s(e.dyn_ltree, g), e.d_desc = new s(e.dyn_dtree, v), e.bl_desc = new s(e.bl_tree, b), e.bi_buf = 0, e.bi_valid = 0, A(e);
                                                  }, r._tr_stored_block = U, r._tr_flush_block = function (e, t, r, n) {
                                                    var i,
                                                        s,
                                                        a = 0;
                                                    0 < e.level ? (2 === e.strm.data_type && (e.strm.data_type = function (e) {
                                                      var t,
                                                          r = 4093624447;

                                                      for (t = 0; t <= 31; t++, r >>>= 1) if (1 & r && 0 !== e.dyn_ltree[2 * t]) return 0;

                                                      if (0 !== e.dyn_ltree[18] || 0 !== e.dyn_ltree[20] || 0 !== e.dyn_ltree[26]) return 1;

                                                      for (t = 32; t < 256; t++) if (0 !== e.dyn_ltree[2 * t]) return 1;

                                                      return 0;
                                                    }(e)), T(e, e.l_desc), T(e, e.d_desc), a = function (e) {
                                                      var t;

                                                      for (D(e, e.dyn_ltree, e.l_desc.max_code), D(e, e.dyn_dtree, e.d_desc.max_code), T(e, e.bl_desc), t = 18; 3 <= t && 0 === e.bl_tree[2 * f[t] + 1]; t--);

                                                      return e.opt_len += 3 * (t + 1) + 5 + 5 + 4, t;
                                                    }(e), i = e.opt_len + 3 + 7 >>> 3, (s = e.static_len + 3 + 7 >>> 3) <= i && (i = s)) : i = s = r + 5, r + 4 <= i && -1 !== t ? U(e, t, r, n) : 4 === e.strategy || s === i ? (S(e, 2 + (n ? 1 : 0), 3), R(e, l, d)) : (S(e, 4 + (n ? 1 : 0), 3), function (e, t, r, n) {
                                                      var i;

                                                      for (S(e, t - 257, 5), S(e, r - 1, 5), S(e, n - 4, 4), i = 0; i < n; i++) S(e, e.bl_tree[2 * f[i] + 1], 3);

                                                      F(e, e.dyn_ltree, t - 1), F(e, e.dyn_dtree, r - 1);
                                                    }(e, e.l_desc.max_code + 1, e.d_desc.max_code + 1, a + 1), R(e, e.dyn_ltree, e.dyn_dtree)), A(e), n && I(e);
                                                  }, r._tr_tally = function (e, t, r) {
                                                    return e.pending_buf[e.d_buf + 2 * e.last_lit] = t >>> 8 & 255, e.pending_buf[e.d_buf + 2 * e.last_lit + 1] = 255 & t, e.pending_buf[e.l_buf + e.last_lit] = 255 & r, e.last_lit++, 0 === t ? e.dyn_ltree[2 * r]++ : (e.matches++, t--, e.dyn_ltree[2 * (p[r] + 256 + 1)]++, e.dyn_dtree[2 * k(t)]++), e.last_lit === e.lit_bufsize - 1;
                                                  }, r._tr_align = function (e) {
                                                    var t;
                                                    S(e, 2, 3), z(e, 256, l), 16 === (t = e).bi_valid ? (x(t, t.bi_buf), t.bi_buf = 0, t.bi_valid = 0) : 8 <= t.bi_valid && (t.pending_buf[t.pending++] = 255 & t.bi_buf, t.bi_buf >>= 8, t.bi_valid -= 8);
                                                  };
                                                }, {
                                                  "../utils/common": 41
                                                }],
                                                53: [function (e, t, r) {

                                                  t.exports = function () {
                                                    this.input = null, this.next_in = 0, this.avail_in = 0, this.total_in = 0, this.output = null, this.next_out = 0, this.avail_out = 0, this.total_out = 0, this.msg = "", this.state = null, this.data_type = 2, this.adler = 0;
                                                  };
                                                }, {}],
                                                54: [function (e, t, r) {

                                                  t.exports = "function" == typeof setImmediate ? setImmediate : function () {
                                                    var e = [].slice.apply(arguments);
                                                    e.splice(1, 0, 0), setTimeout.apply(null, e);
                                                  };
                                                }, {}]
                                              }, {}, [10])(10);
                                            });
                                          }).call(this, void 0 !== r ? r : "undefined" != typeof self ? self : "undefined" != typeof window ? window : {});
                                        }, {}]
                                      }, {}, [1])(1);
                                    });
                                  }).call(this, void 0 !== r ? r : "undefined" != typeof self ? self : "undefined" != typeof window ? window : {});
                                }, {}]
                              }, {}, [1])(1);
                            });
                          }).call(this, void 0 !== r ? r : "undefined" != typeof self ? self : "undefined" != typeof window ? window : {});
                        }, {}]
                      }, {}, [1])(1);
                    });
                  }).call(this, "undefined" != typeof global ? global : "undefined" != typeof self ? self : "undefined" != typeof window ? window : {});
                }, {}]
              }, {}, [1])(1);
            });
          }).call(this, typeof commonjsGlobal !== "undefined" ? commonjsGlobal : typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {});
        }, {}]
      }, {}, [1])(1);
    });
  });

  /**
   * Those functions should disappear if add2D becomes accessible in jcampconvert
   * @param spectra
   * @returns {{z: Array, minX: *, maxX: *, minY: *, maxY: *, minZ: *, maxZ: *, noise: number}}
   */
  function convertTo3DZ$1(spectra) {
    let noise = 0;
    let minZ = spectra[0].data[0];
    let maxZ = minZ;
    let ySize = spectra.length;
    let xSize = spectra[0].data.length / 2;
    let z = new Array(ySize);

    for (let i = 0; i < ySize; i++) {
      z[i] = new Float64Array(xSize);

      for (let j = 0; j < xSize; j++) {
        z[i][j] = spectra[i].data[j * 2 + 1];
        if (z[i][j] < minZ) minZ = spectra[i].data[j * 2 + 1];
        if (z[i][j] > maxZ) maxZ = spectra[i].data[j * 2 + 1];

        if (i !== 0 && j !== 0) {
          noise += Math.abs(z[i][j] - z[i][j - 1]) + Math.abs(z[i][j] - z[i - 1][j]);
        }
      }
    }

    const firstX = spectra[0].data[0];
    const lastX = spectra[0].data[spectra[0].data.length - 2]; // has to be -2 because it is a 1D array [x,y,x,y,...]

    const firstY = spectra[0].pageValue;
    const lastY = spectra[ySize - 1].pageValue; // Because the min / max value are the only information about the matrix if we invert
    // min and max we need to invert the array

    if (firstX > lastX) {
      for (let spectrum of z) {
        spectrum.reverse();
      }
    }

    if (firstY > lastY) {
      z.reverse();
    }

    let minMaxX = firstX < lastX ? {
      minX: firstX,
      maxX: lastX
    } : {
      minX: lastX,
      maxX: firstX
    };
    let minMaxY = firstY < lastY ? {
      minY: firstY,
      maxY: lastY
    } : {
      minY: lastY,
      maxY: firstY
    };
    return {
      z,
      minZ: minZ,
      maxZ: maxZ,
      ...minMaxX,
      ...minMaxY,
      noise: noise / ((ySize - 1) * (xSize - 1) * 2)
    };
  }

  function generateContourLines$1(zData) {
    let noise = zData.noise;
    let z = zData.z;
    let contourLevels = [];
    let nbLevels = 7;
    let povarHeight = new Float32Array(4);
    let isOver = [];
    let nbSubSpectra = z.length;
    let nbPovars = z[0].length;
    let pAx, pAy, pBx, pBy;
    let x0 = zData.minX;
    let xN = zData.maxX;
    let dx = (xN - x0) / (nbPovars - 1);
    let y0 = zData.minY;
    let yN = zData.maxY;
    let dy = (yN - y0) / (nbSubSpectra - 1);
    let minZ = zData.minZ;
    let maxZ = zData.maxZ; // System.out.prvarln('y0 '+y0+' yN '+yN);
    // -------------------------
    // Povars attribution
    //
    // 0----1
    // |  / |
    // | /  |
    // 2----3
    //
    // ---------------------d------

    let lineZValue;

    for (let level = 0; level < nbLevels * 2; level++) {
      // multiply by 2 for positif and negatif
      let contourLevel = {};
      contourLevels.push(contourLevel);
      let side = level % 2;

      if (side === 0) {
        lineZValue = (maxZ - 5 * noise) * Math.exp(level / 2 - nbLevels) + 5 * noise;
      } else {
        lineZValue = -(maxZ - 5 * noise) * Math.exp(level / 2 - nbLevels) - 5 * noise;
      }

      let lines = [];
      contourLevel.zValue = lineZValue;
      contourLevel.lines = lines;
      if (lineZValue <= minZ || lineZValue >= maxZ) continue;

      for (let iSubSpectra = 0; iSubSpectra < nbSubSpectra - 1; iSubSpectra++) {
        for (let povar = 0; povar < nbPovars - 1; povar++) {
          povarHeight[0] = z[iSubSpectra][povar];
          povarHeight[1] = z[iSubSpectra][povar + 1];
          povarHeight[2] = z[iSubSpectra + 1][povar];
          povarHeight[3] = z[iSubSpectra + 1][povar + 1];

          for (let i = 0; i < 4; i++) {
            isOver[i] = povarHeight[i] > lineZValue;
          } // Example povar0 is over the plane and povar1 and
          // povar2 are below, we find the varersections and add
          // the segment


          if (isOver[0] !== isOver[1] && isOver[0] !== isOver[2]) {
            pAx = povar + (lineZValue - povarHeight[0]) / (povarHeight[1] - povarHeight[0]);
            pAy = iSubSpectra;
            pBx = povar;
            pBy = iSubSpectra + (lineZValue - povarHeight[0]) / (povarHeight[2] - povarHeight[0]);
            lines.push(pAx * dx + x0, pAy * dy + y0, pBx * dx + x0, pBy * dy + y0);
          }

          if (isOver[3] !== isOver[1] && isOver[3] !== isOver[2]) {
            pAx = povar + 1;
            pAy = iSubSpectra + 1 - (lineZValue - povarHeight[3]) / (povarHeight[1] - povarHeight[3]);
            pBx = povar + 1 - (lineZValue - povarHeight[3]) / (povarHeight[2] - povarHeight[3]);
            pBy = iSubSpectra + 1;
            lines.push(pAx * dx + x0, pAy * dy + y0, pBx * dx + x0, pBy * dy + y0);
          } // test around the diagonal


          if (isOver[1] !== isOver[2]) {
            pAx = povar + 1 - (lineZValue - povarHeight[1]) / (povarHeight[2] - povarHeight[1]);
            pAy = iSubSpectra + (lineZValue - povarHeight[1]) / (povarHeight[2] - povarHeight[1]);

            if (isOver[1] !== isOver[0]) {
              pBx = povar + 1 - (lineZValue - povarHeight[1]) / (povarHeight[0] - povarHeight[1]);
              pBy = iSubSpectra;
              lines.push(pAx * dx + x0, pAy * dy + y0, pBx * dx + x0, pBy * dy + y0);
            }

            if (isOver[2] !== isOver[0]) {
              pBx = povar;
              pBy = iSubSpectra + 1 - (lineZValue - povarHeight[2]) / (povarHeight[0] - povarHeight[2]);
              lines.push(pAx * dx + x0, pAy * dy + y0, pBx * dx + x0, pBy * dy + y0);
            }

            if (isOver[1] !== isOver[3]) {
              pBx = povar + 1;
              pBy = iSubSpectra + (lineZValue - povarHeight[1]) / (povarHeight[3] - povarHeight[1]);
              lines.push(pAx * dx + x0, pAy * dy + y0, pBx * dx + x0, pBy * dy + y0);
            }

            if (isOver[2] !== isOver[3]) {
              pBx = povar + (lineZValue - povarHeight[2]) / (povarHeight[3] - povarHeight[2]);
              pBy = iSubSpectra + 1;
              lines.push(pAx * dx + x0, pAy * dy + y0, pBx * dx + x0, pBy * dy + y0);
            }
          }
        }
      }
    }

    return {
      minX: zData.minX,
      maxX: zData.maxX,
      minY: zData.minY,
      maxY: zData.maxY,
      segments: contourLevels
    };
  }

  function add2D$1(result, options) {
    let zData = convertTo3DZ$1(result.spectra);

    if (!options.noContours) {
      result.contourLines = generateContourLines$1(zData);
    }

    result.minMax = zData;
  }

  /**
   * Ensure that the data is string. If it is an ArrayBuffer it will be converted to string using TextDecoder.
   * @param {string|ArrayBuffer} blob
   * @param {object} [options={}]
   * @param {string} [options.encoding='utf8']
   * @returns {string}
   */
  function ensureString(blob, options = {}) {
    const {
      encoding = 'utf8'
    } = options;

    if (ArrayBuffer.isView(blob) || blob instanceof ArrayBuffer) {
      const decoder = new TextDecoder(encoding);
      return decoder.decode(blob);
    }

    return blob;
  }

  const GC_MS_FIELDS = ['TIC', '.RIC', 'SCANNUMBER'];
  function complexChromatogram(result) {
    let spectra = result.spectra;
    let length = spectra.length;
    let chromatogram = {
      times: new Array(length),
      series: {
        ms: {
          dimension: 2,
          data: new Array(length)
        }
      }
    };
    let existingGCMSFields = [];

    for (let i = 0; i < GC_MS_FIELDS.length; i++) {
      let label = convertMSFieldToLabel(GC_MS_FIELDS[i]);

      if (spectra[0][label]) {
        existingGCMSFields.push(label);
        chromatogram.series[label] = {
          dimension: 1,
          data: new Array(length)
        };
      }
    }

    for (let i = 0; i < length; i++) {
      let spectrum = spectra[i];
      chromatogram.times[i] = spectrum.pageValue;

      for (let j = 0; j < existingGCMSFields.length; j++) {
        chromatogram.series[existingGCMSFields[j]].data[i] = parseFloat(spectrum[existingGCMSFields[j]]);
      }

      if (spectrum.data) {
        chromatogram.series.ms.data[i] = [spectrum.data.x, spectrum.data.y];
      }
    }

    result.chromatogram = chromatogram;
  }
  function isMSField(canonicDataLabel) {
    return GC_MS_FIELDS.indexOf(canonicDataLabel) !== -1;
  }
  function convertMSFieldToLabel(value) {
    return value.toLowerCase().replace(/[^a-z0-9]/g, '');
  }

  function convertToFloatArray$1(stringArray) {
    let floatArray = [];

    for (let i = 0; i < stringArray.length; i++) {
      floatArray.push(parseFloat(stringArray[i]));
    }

    return floatArray;
  }

  function fastParseXYData(spectrum, value) {
    // TODO need to deal with result
    //  console.log(value);
    // we check if deltaX is defined otherwise we calculate it
    let yFactor = spectrum.yFactor;
    let deltaX = spectrum.deltaX;
    spectrum.isXYdata = true;
    let currentData = {
      x: [],
      y: []
    };
    spectrum.data = currentData;
    let currentX = spectrum.firstX;
    let currentY = spectrum.firstY; // we skip the first line
    //

    let endLine = false;
    let ascii;
    let i = 0;

    for (; i < value.length; i++) {
      ascii = value.charCodeAt(i);

      if (ascii === 13 || ascii === 10) {
        endLine = true;
      } else {
        if (endLine) break;
      }
    } // we proceed taking the i after the first line


    let newLine = true;
    let isDifference = false;
    let isLastDifference = false;
    let lastDifference = 0;
    let isDuplicate = false;
    let inComment = false;
    let currentValue = 0; // can be a difference or a duplicate

    let lastValue = 0; // must be the real last value

    let isNegative = false;
    let inValue = false;
    let skipFirstValue = false;
    let decimalPosition = 0;

    for (; i <= value.length; i++) {
      if (i === value.length) ascii = 13;else ascii = value.charCodeAt(i);

      if (inComment) {
        // we should ignore the text if we are after $$
        if (ascii === 13 || ascii === 10) {
          newLine = true;
          inComment = false;
        }
      } else {
        // when is it a new value ?
        // when it is not a digit, . or comma
        // it is a number that is either new or we continue
        if (ascii <= 57 && ascii >= 48) {
          // a number
          inValue = true;

          if (decimalPosition > 0) {
            currentValue += (ascii - 48) / Math.pow(10, decimalPosition++);
          } else {
            currentValue *= 10;
            currentValue += ascii - 48;
          }
        } else if (ascii === 44 || ascii === 46) {
          // a "," or "."
          inValue = true;
          decimalPosition++;
        } else {
          if (inValue) {
            // need to process the previous value
            if (newLine) {
              newLine = false; // we don't check the X value
              // console.log("NEW LINE",isDifference, lastDifference);
              // if new line and lastDifference, the first value is just a check !
              // that we don't check ...

              if (isLastDifference) skipFirstValue = true;
            } else {
              // need to deal with duplicate and differences
              if (skipFirstValue) {
                skipFirstValue = false;
              } else {
                if (isDifference) {
                  lastDifference = isNegative ? 0 - currentValue : currentValue;
                  isLastDifference = true;
                  isDifference = false;
                } else if (!isDuplicate) {
                  lastValue = isNegative ? 0 - currentValue : currentValue;
                }

                let duplicate = isDuplicate ? currentValue - 1 : 1;

                for (let j = 0; j < duplicate; j++) {
                  if (isLastDifference) {
                    currentY += lastDifference;
                  } else {
                    currentY = lastValue;
                  }

                  currentData.x.push(currentX);
                  currentData.y.push(currentY * yFactor);
                  currentX += deltaX;
                }
              }
            }

            isNegative = false;
            currentValue = 0;
            decimalPosition = 0;
            inValue = false;
            isDuplicate = false;
          } // positive SQZ digits @ A B C D E F G H I (ascii 64-73)


          if (ascii < 74 && ascii > 63) {
            inValue = true;
            isLastDifference = false;
            currentValue = ascii - 64;
          } else if (ascii > 96 && ascii < 106) {
            // negative SQZ digits a b c d e f g h i (ascii 97-105)
            inValue = true;
            isLastDifference = false;
            currentValue = ascii - 96;
            isNegative = true;
          } else if (ascii === 115) {
            // DUP digits S T U V W X Y Z s (ascii 83-90, 115)
            inValue = true;
            isDuplicate = true;
            currentValue = 9;
          } else if (ascii > 82 && ascii < 91) {
            inValue = true;
            isDuplicate = true;
            currentValue = ascii - 82;
          } else if (ascii > 73 && ascii < 83) {
            // positive DIF digits % J K L M N O P Q R (ascii 37, 74-82)
            inValue = true;
            isDifference = true;
            currentValue = ascii - 73;
          } else if (ascii > 105 && ascii < 115) {
            // negative DIF digits j k l m n o p q r (ascii 106-114)
            inValue = true;
            isDifference = true;
            currentValue = ascii - 105;
            isNegative = true;
          } else if (ascii === 36 && value.charCodeAt(i + 1) === 36) {
            // $ sign, we need to check the next one
            inValue = true;
            inComment = true;
          } else if (ascii === 37) {
            // positive DIF digits % J K L M N O P Q R (ascii 37, 74-82)
            inValue = true;
            isDifference = true;
            currentValue = 0;
            isNegative = false;
          } else if (ascii === 45) {
            // a "-"
            // check if after there is a number, decimal or comma
            let ascii2 = value.charCodeAt(i + 1);

            if (ascii2 >= 48 && ascii2 <= 57 || ascii2 === 44 || ascii2 === 46) {
              inValue = true;
              if (!newLine) isLastDifference = false;
              isNegative = true;
            }
          } else if (ascii === 13 || ascii === 10) {
            newLine = true;
            inComment = false;
          } // and now analyse the details ... space or tabulation
          // if "+" we just don't care

        }
      }
    }
  }

  const removeCommentRegExp = /\$\$.*/;
  const peakTableSplitRegExp = /[,\t ]+/;
  function parsePeakTable(spectrum, value, result) {
    spectrum.isPeaktable = true;

    if (!spectrum.variables || Object.keys(spectrum.variables) === 2) {
      parseXY(spectrum, value, result);
    } else {
      parseXYZ(spectrum, value, result);
    } // we will add the data in the variables


    if (spectrum.variables) {
      for (let key in spectrum.variables) {
        spectrum.variables[key].data = spectrum.data[key];
      }
    }
  }

  function parseXY(spectrum, value, result) {
    let currentData = {
      x: [],
      y: []
    };
    spectrum.data = currentData; // counts for around 20% of the time

    let lines = value.split(/,? *,?[;\r\n]+ */);

    for (let i = 1; i < lines.length; i++) {
      let values = lines[i].trim().replace(removeCommentRegExp, '').split(peakTableSplitRegExp);

      if (values.length % 2 === 0) {
        for (let j = 0; j < values.length; j = j + 2) {
          // takes around 40% of the time to add and parse the 2 values nearly exclusively because of parseFloat
          currentData.x.push(parseFloat(values[j]) * spectrum.xFactor);
          currentData.y.push(parseFloat(values[j + 1]) * spectrum.yFactor);
        }
      } else {
        result.logs.push(`Format error: ${values}`);
      }
    }
  }

  function parseXYZ(spectrum, value, result) {
    let currentData = {};
    let variables = Object.keys(spectrum.variables);
    let numberOfVariables = variables.length;
    variables.forEach(variable => currentData[variable] = []);
    spectrum.data = currentData; // counts for around 20% of the time

    let lines = value.split(/,? *,?[;\r\n]+ */);

    for (let i = 1; i < lines.length; i++) {
      let values = lines[i].trim().replace(removeCommentRegExp, '').split(peakTableSplitRegExp);

      if (values.length % numberOfVariables === 0) {
        for (let j = 0; j < values.length; j++) {
          // todo should try to find a xFactor (y, ...)
          currentData[variables[j % numberOfVariables]].push(parseFloat(values[j]));
        }
      } else {
        result.logs.push(`Format error: ${values}`);
      }
    }
  }

  function parseXYA(spectrum, value) {
    let removeSymbolRegExp = /(\(+|\)+|<+|>+|\s+)/g;
    spectrum.isXYAdata = true;
    let values;
    let currentData = {
      x: [],
      y: []
    };
    spectrum.data = currentData;
    let lines = value.split(/,? *,?[;\r\n]+ */);

    for (let i = 1; i < lines.length; i++) {
      values = lines[i].trim().replace(removeSymbolRegExp, '').split(',');
      currentData.x.push(parseFloat(values[0]));
      currentData.y.push(parseFloat(values[1]));
    }
  }

  function convertTo3DZ(spectra) {
    let minZ = spectra[0].data.y[0];
    let maxZ = minZ;
    let ySize = spectra.length;
    let xSize = spectra[0].data.x.length;
    let z = new Array(ySize);

    for (let i = 0; i < ySize; i++) {
      z[i] = spectra[i].data.y;

      for (let j = 0; j < xSize; j++) {
        let value = z[i][j];
        if (value < minZ) minZ = value;
        if (value > maxZ) maxZ = value;
      }
    }

    const firstX = spectra[0].data.x[0];
    const lastX = spectra[0].data.x[spectra[0].data.x.length - 1]; // has to be -2 because it is a 1D array [x,y,x,y,...]

    const firstY = spectra[0].pageValue;
    const lastY = spectra[ySize - 1].pageValue; // Because the min / max value are the only information about the matrix if we invert
    // min and max we need to invert the array

    if (firstX > lastX) {
      for (let spectrum of z) {
        spectrum.reverse();
      }
    }

    if (firstY > lastY) {
      z.reverse();
    }

    const medians = [];

    for (let i = 0; i < z.length; i++) {
      const row = Float64Array.from(z[i]);

      for (let i = 0; i < row.length; i++) {
        if (row[i] < 0) row[i] = -row[i];
      }

      medians.push(median(row));
    }

    const median$1 = median(medians);
    return {
      z: z,
      minX: Math.min(firstX, lastX),
      maxX: Math.max(firstX, lastX),
      minY: Math.min(firstY, lastY),
      maxY: Math.max(firstY, lastY),
      minZ: minZ,
      maxZ: maxZ,
      noise: median$1
    };
  }

  function generateContourLines(zData, options) {
    let noise = zData.noise;
    let z = zData.z;
    let povarHeight0, povarHeight1, povarHeight2, povarHeight3;
    let isOver0, isOver1, isOver2, isOver3;
    let nbSubSpectra = z.length;
    let nbPovars = z[0].length;
    let pAx, pAy, pBx, pBy;
    let x0 = zData.minX;
    let xN = zData.maxX;
    let dx = (xN - x0) / (nbPovars - 1);
    let y0 = zData.minY;
    let yN = zData.maxY;
    let dy = (yN - y0) / (nbSubSpectra - 1);
    let minZ = zData.minZ;
    let maxZ = zData.maxZ; // System.out.prvarln('y0 '+y0+' yN '+yN);
    // -------------------------
    // Povars attribution
    //
    // 0----1
    // |  / |
    // | /  |
    // 2----3
    //
    // ---------------------d------

    let iter = options.nbContourLevels * 2;
    let contourLevels = new Array(iter);
    let lineZValue;

    for (let level = 0; level < iter; level++) {
      // multiply by 2 for positif and negatif
      let contourLevel = {};
      contourLevels[level] = contourLevel;
      let side = level % 2;
      let factor = (maxZ - options.noiseMultiplier * noise) * Math.exp((level >> 1) - options.nbContourLevels);

      if (side === 0) {
        lineZValue = factor + options.noiseMultiplier * noise;
      } else {
        lineZValue = 0 - factor - options.noiseMultiplier * noise;
      }

      let lines = [];
      contourLevel.zValue = lineZValue;
      contourLevel.lines = lines;
      if (lineZValue <= minZ || lineZValue >= maxZ) continue;

      for (let iSubSpectra = 0; iSubSpectra < nbSubSpectra - 1; iSubSpectra++) {
        let subSpectra = z[iSubSpectra];
        let subSpectraAfter = z[iSubSpectra + 1];

        for (let povar = 0; povar < nbPovars - 1; povar++) {
          povarHeight0 = subSpectra[povar];
          povarHeight1 = subSpectra[povar + 1];
          povarHeight2 = subSpectraAfter[povar];
          povarHeight3 = subSpectraAfter[povar + 1];
          isOver0 = povarHeight0 > lineZValue;
          isOver1 = povarHeight1 > lineZValue;
          isOver2 = povarHeight2 > lineZValue;
          isOver3 = povarHeight3 > lineZValue; // Example povar0 is over the plane and povar1 and
          // povar2 are below, we find the varersections and add
          // the segment

          if (isOver0 !== isOver1 && isOver0 !== isOver2) {
            pAx = povar + (lineZValue - povarHeight0) / (povarHeight1 - povarHeight0);
            pAy = iSubSpectra;
            pBx = povar;
            pBy = iSubSpectra + (lineZValue - povarHeight0) / (povarHeight2 - povarHeight0);
            lines.push(pAx * dx + x0);
            lines.push(pAy * dy + y0);
            lines.push(pBx * dx + x0);
            lines.push(pBy * dy + y0);
          } // remove push does not help !!!!


          if (isOver3 !== isOver1 && isOver3 !== isOver2) {
            pAx = povar + 1;
            pAy = iSubSpectra + 1 - (lineZValue - povarHeight3) / (povarHeight1 - povarHeight3);
            pBx = povar + 1 - (lineZValue - povarHeight3) / (povarHeight2 - povarHeight3);
            pBy = iSubSpectra + 1;
            lines.push(pAx * dx + x0);
            lines.push(pAy * dy + y0);
            lines.push(pBx * dx + x0);
            lines.push(pBy * dy + y0);
          } // test around the diagonal


          if (isOver1 !== isOver2) {
            pAx = (povar + 1 - (lineZValue - povarHeight1) / (povarHeight2 - povarHeight1)) * dx + x0;
            pAy = (iSubSpectra + (lineZValue - povarHeight1) / (povarHeight2 - povarHeight1)) * dy + y0;

            if (isOver1 !== isOver0) {
              pBx = povar + 1 - (lineZValue - povarHeight1) / (povarHeight0 - povarHeight1);
              pBy = iSubSpectra;
              lines.push(pAx);
              lines.push(pAy);
              lines.push(pBx * dx + x0);
              lines.push(pBy * dy + y0);
            }

            if (isOver2 !== isOver0) {
              pBx = povar;
              pBy = iSubSpectra + 1 - (lineZValue - povarHeight2) / (povarHeight0 - povarHeight2);
              lines.push(pAx);
              lines.push(pAy);
              lines.push(pBx * dx + x0);
              lines.push(pBy * dy + y0);
            }

            if (isOver1 !== isOver3) {
              pBx = povar + 1;
              pBy = iSubSpectra + (lineZValue - povarHeight1) / (povarHeight3 - povarHeight1);
              lines.push(pAx);
              lines.push(pAy);
              lines.push(pBx * dx + x0);
              lines.push(pBy * dy + y0);
            }

            if (isOver2 !== isOver3) {
              pBx = povar + (lineZValue - povarHeight2) / (povarHeight3 - povarHeight2);
              pBy = iSubSpectra + 1;
              lines.push(pAx);
              lines.push(pAy);
              lines.push(pBx * dx + x0);
              lines.push(pBy * dy + y0);
            }
          }
        }
      }
    }

    return {
      minX: zData.minX,
      maxX: zData.maxX,
      minY: zData.minY,
      maxY: zData.maxY,
      segments: contourLevels
    };
  }

  function add2D(result, options) {
    let zData = convertTo3DZ(result.spectra);

    if (!options.noContour) {
      result.contourLines = generateContourLines(zData, options);
      delete zData.z;
    }

    result.minMax = zData;
  }

  function postProcessingNMR(entriesFlat) {
    // specific NMR functions
    for (let entry of entriesFlat) {
      let observeFrequency = 0;
      let shiftOffsetVal = 0;

      for (let spectrum of entry.spectra) {
        if (entry.ntuples && entry.ntuples.symbol) {
          if (!observeFrequency && spectrum.observeFrequency) {
            observeFrequency = spectrum.observeFrequency;
          }

          if (!shiftOffsetVal && spectrum.shiftOffsetVal) {
            shiftOffsetVal = spectrum.shiftOffsetVal;
          }
        } else {
          observeFrequency = spectrum.observeFrequency;
          shiftOffsetVal = spectrum.shiftOffsetVal;
        }

        if (observeFrequency) {
          if (spectrum.xUnits && spectrum.xUnits.toUpperCase().includes('HZ')) {
            spectrum.xUnits = 'PPM';
            spectrum.xFactor = spectrum.xFactor / observeFrequency;
            spectrum.firstX = spectrum.firstX / observeFrequency;
            spectrum.lastX = spectrum.lastX / observeFrequency;
            spectrum.deltaX = spectrum.deltaX / observeFrequency;

            for (let i = 0; i < spectrum.data.x.length; i++) {
              spectrum.data.x[i] /= observeFrequency;
            }
          }
        }

        if (shiftOffsetVal) {
          let shift = spectrum.firstX - shiftOffsetVal;
          spectrum.firstX = spectrum.firstX - shift;
          spectrum.lastX = spectrum.lastX - shift;

          for (let i = 0; i < spectrum.data.x.length; i++) {
            spectrum.data.x[i] -= shift;
          }
        } // we will check if some nucleus are missing ...


        if (entry.ntuples && entry.ntuples.nucleus && entry.ntuples.symbol) {
          for (let i = 0; i < entry.ntuples.nucleus.length; i++) {
            let symbol = entry.ntuples.symbol[i];
            let nucleus = entry.ntuples.nucleus[i];

            if (symbol.startsWith('F') && !nucleus) {
              if (symbol === 'F1') {
                // if F1 is defined we will use F2
                if (entry.tmp.$NUC2) {
                  entry.ntuples.nucleus[i] = entry.tmp.$NUC2;
                } else {
                  let f2index = entry.ntuples.symbol.indexOf('F2');

                  if (f2index && entry.ntuples.nucleus[f2index]) {
                    entry.ntuples.nucleus[i] = entry.ntuples.nucleus[f2index];
                  }
                }
              }

              if (symbol === 'F2') entry.ntuples.nucleus[i] = entry.tmp.$NUC1;
            }

            if (symbol === 'F2') {
              entry.yType = entry.ntuples.nucleus[0];
            }
          }
        }

        if (observeFrequency && entry.ntuples && entry.ntuples.symbol && entry.ntuples.nucleus) {
          let unit = '';
          let pageSymbolIndex = entry.ntuples.symbol.indexOf(spectrum.pageSymbol);

          if (entry.ntuples.units && entry.ntuples.units[pageSymbolIndex]) {
            unit = entry.ntuples.units[pageSymbolIndex];
          }

          if (unit !== 'PPM') {
            if (pageSymbolIndex !== 0) {
              throw Error('Not sure about this ntuples format');
            }

            let ratio0 = gyromagneticRatio[entry.ntuples.nucleus[0]];
            let ratio1 = gyromagneticRatio[entry.ntuples.nucleus[1]];

            if (!ratio0 || !ratio1) {
              throw Error('Problem with determination of gyromagnetic ratio');
            }

            let ratio = ratio0 / ratio1 * observeFrequency;
            spectrum.pageValue /= ratio;
          }
        }
      }
    }
  }

  function profiling(result, action, options) {
    if (result.profiling) {
      result.profiling.push({
        action,
        time: Date.now() - options.start
      });
    }
  }

  function simpleChromatogram(result) {
    let data = result.spectra[0].data;
    result.chromatogram = {
      times: data.x.slice(),
      series: {
        intensity: {
          dimension: 1,
          data: data.y.slice()
        }
      }
    };
  }

  function postProcessing(entriesFlat, result, options) {
    // converting Hz to ppm
    postProcessingNMR(entriesFlat);

    for (let entry of entriesFlat) {
      if (Object.keys(entry.ntuples).length > 0) {
        let newNtuples = [];
        let keys = Object.keys(entry.ntuples);

        for (let i = 0; i < keys.length; i++) {
          let key = keys[i];
          let values = entry.ntuples[key];

          for (let j = 0; j < values.length; j++) {
            if (!newNtuples[j]) newNtuples[j] = {};
            newNtuples[j][key] = values[j];
          }
        }

        entry.ntuples = newNtuples;
      }

      if (entry.twoD && options.wantXY) {
        add2D(entry, options);
        profiling(result, 'Finished countour plot calculation', options);

        if (!options.keepSpectra) {
          delete entry.spectra;
        }
      } // maybe it is a GC (HPLC) / MS. In this case we add a new format


      if (options.chromatogram) {
        if (entry.spectra.length > 1) {
          complexChromatogram(entry);
        } else {
          simpleChromatogram(entry);
        }

        profiling(result, 'Finished chromatogram calculation', options);
      }

      delete entry.tmp;
    }
  }

  function prepareNtuplesDatatable(currentEntry, spectrum, kind) {
    let xIndex = -1;
    let yIndex = -1;
    let firstVariable = '';
    let secondVariable = '';

    if (kind.indexOf('++') > 0) {
      firstVariable = kind.replace(/.*\(([a-zA-Z0-9]+)\+\+.*/, '$1');
      secondVariable = kind.replace(/.*\.\.([a-zA-Z0-9]+).*/, '$1');
    } else {
      kind = kind.replace(/[^a-zA-Z]/g, '');
      firstVariable = kind.charAt(0);
      secondVariable = kind.charAt(1);
      spectrum.variables = {};

      for (let symbol of kind) {
        let lowerCaseSymbol = symbol.toLowerCase();
        let index = currentEntry.ntuples.symbol.indexOf(symbol);
        if (index === -1) throw Error(`Symbol undefined: ${symbol}`);
        spectrum.variables[lowerCaseSymbol] = {};

        for (let key in currentEntry.ntuples) {
          if (currentEntry.ntuples[key][index]) {
            spectrum.variables[lowerCaseSymbol][key.replace(/^var/, '')] = currentEntry.ntuples[key][index];
          }
        }
      }
    }

    xIndex = currentEntry.ntuples.symbol.indexOf(firstVariable);
    yIndex = currentEntry.ntuples.symbol.indexOf(secondVariable);
    if (xIndex === -1) xIndex = 0;
    if (yIndex === -1) yIndex = 0;

    if (currentEntry.ntuples.first) {
      if (currentEntry.ntuples.first.length > xIndex) {
        spectrum.firstX = currentEntry.ntuples.first[xIndex];
      }

      if (currentEntry.ntuples.first.length > yIndex) {
        spectrum.firstY = currentEntry.ntuples.first[yIndex];
      }
    }

    if (currentEntry.ntuples.last) {
      if (currentEntry.ntuples.last.length > xIndex) {
        spectrum.lastX = currentEntry.ntuples.last[xIndex];
      }

      if (currentEntry.ntuples.last.length > yIndex) {
        spectrum.lastY = currentEntry.ntuples.last[yIndex];
      }
    }

    if (currentEntry.ntuples.vardim && currentEntry.ntuples.vardim.length > xIndex) {
      spectrum.nbPoints = currentEntry.ntuples.vardim[xIndex];
    }

    if (currentEntry.ntuples.factor) {
      if (currentEntry.ntuples.factor.length > xIndex) {
        spectrum.xFactor = currentEntry.ntuples.factor[xIndex];
      }

      if (currentEntry.ntuples.factor.length > yIndex) {
        spectrum.yFactor = currentEntry.ntuples.factor[yIndex];
      }
    }

    if (currentEntry.ntuples.units) {
      if (currentEntry.ntuples.units.length > xIndex) {
        if (currentEntry.ntuples.varname && currentEntry.ntuples.varname[xIndex]) {
          spectrum.xUnits = `${currentEntry.ntuples.varname[xIndex]} [${currentEntry.ntuples.units[xIndex]}]`;
        } else {
          spectrum.xUnits = currentEntry.ntuples.units[xIndex];
        }
      }

      if (currentEntry.ntuples.units.length > yIndex) {
        if (currentEntry.ntuples.varname && currentEntry.ntuples.varname[yIndex]) {
          spectrum.yUnits = `${currentEntry.ntuples.varname[yIndex]} [${currentEntry.ntuples.units[yIndex]}]`;
        } else {
          spectrum.yUnits = currentEntry.ntuples.units[yIndex];
        }
      }
    }
  }

  function prepareSpectrum(spectrum) {
    if (!spectrum.xFactor) spectrum.xFactor = 1;
    if (!spectrum.yFactor) spectrum.yFactor = 1;
  }

  const ntuplesSeparatorRegExp = /[ \t]*,[ \t]*/;
  const numberRegExp = /^[-+]?[0-9]*\.?[0-9]+(e[-+]?[0-9]+)?$/;

  class Spectrum {}

  const defaultOptions$1 = {
    keepRecordsRegExp: /^$/,
    canonicDataLabels: true,
    canonicMetadataLabels: false,
    dynamicTyping: true,
    withoutXY: false,
    chromatogram: false,
    keepSpectra: false,
    noContour: false,
    nbContourLevels: 7,
    noiseMultiplier: 5,
    profiling: false
  };
  /**
   *
   * @param {string|ArrayBuffer} jcamp
   * @param {object} [options]
   * @param {number} [options.keepRecordsRegExp=/^$/] By default we don't keep meta information
   * @param {number} [options.canonicDataLabels=true] Canonize the Labels (uppercase without symbol)
   * @param {number} [options.canonicMetadataLabels=false] Canonize the metadata Labels (uppercase without symbol)
   * @param {number} [options.dynamicTyping=false] Convert numbers to Number
   * @param {number} [options.withoutXY=false] Remove the XY data
   * @param {number} [options.chromatogram=false] Special post-processing for GC / HPLC / MS
   * @param {number} [options.keepSpectra=false] Force to keep the spectra in case of 2D
   * @param {number} [options.noContour=false] Don't calculate countour in case of 2D
   * @param {number} [options.nbContourLevels=7] Number of positive / negative contour levels to calculate
   * @param {number} [options.noiseMultiplier=5] Define for 2D the level as 5 times the median as default
   * @param {number} [options.profiling=false] Add profiling information
   */

  function convert(jcamp, options = {}) {
    jcamp = ensureString(jcamp);
    options = { ...defaultOptions$1,
      ...options
    };
    options.wantXY = !options.withoutXY;
    options.start = Date.now();
    let entriesFlat = [];
    let result = {
      profiling: options.profiling ? [] : false,
      logs: [],
      entries: []
    };
    let tmpResult = {
      children: []
    };
    let currentEntry = tmpResult;
    let parentsStack = [];
    let spectrum = new Spectrum();

    if (typeof jcamp !== 'string') {
      throw new TypeError('the JCAMP should be a string');
    }

    profiling(result, 'Before split to LDRS', options);
    let ldrs = jcamp.replace(/[\r\n]+##/g, '\n##').split('\n##');
    profiling(result, 'Split to LDRS', options);
    if (ldrs[0]) ldrs[0] = ldrs[0].replace(/^[\r\n ]*##/, '');

    for (let ldr of ldrs) {
      // This is a new LDR
      let position = ldr.indexOf('=');
      let dataLabel = position > 0 ? ldr.substring(0, position) : ldr;
      let dataValue = position > 0 ? ldr.substring(position + 1).trim() : '';
      let canonicDataLabel = dataLabel.replace(/[_ -]/g, '').toUpperCase();

      if (canonicDataLabel === 'DATATABLE') {
        let endLine = dataValue.indexOf('\n');
        if (endLine === -1) endLine = dataValue.indexOf('\r');

        if (endLine > 0) {
          // ##DATA TABLE= (X++(I..I)), XYDATA
          // We need to find the variables
          let infos = dataValue.substring(0, endLine).split(/[ ,;\t]+/);
          prepareNtuplesDatatable(currentEntry, spectrum, infos[0]);
          spectrum.datatable = infos[0];

          if (infos[1] && infos[1].indexOf('PEAKS') > -1) {
            canonicDataLabel = 'PEAKTABLE';
          } else if (infos[1] && (infos[1].indexOf('XYDATA') || infos[0].indexOf('++') > 0)) {
            canonicDataLabel = 'XYDATA';
            spectrum.deltaX = (spectrum.lastX - spectrum.firstX) / (spectrum.nbPoints - 1);
          }
        }
      }

      if (canonicDataLabel === 'XYDATA') {
        if (options.wantXY) {
          prepareSpectrum(spectrum); // well apparently we should still consider it is a PEAK TABLE if there are no '++' after

          if (dataValue.match(/.*\+\+.*/)) {
            // ex: (X++(Y..Y))
            spectrum.deltaX = (spectrum.lastX - spectrum.firstX) / (spectrum.nbPoints - 1);
            fastParseXYData(spectrum, dataValue);
          } else {
            parsePeakTable(spectrum, dataValue, result);
          }

          currentEntry.spectra.push(spectrum);
          spectrum = new Spectrum();
        }

        continue;
      } else if (canonicDataLabel === 'PEAKTABLE') {
        if (options.wantXY) {
          prepareSpectrum(spectrum);
          parsePeakTable(spectrum, dataValue, result);
          currentEntry.spectra.push(spectrum);
          spectrum = new Spectrum();
        }

        continue;
      }

      if (canonicDataLabel === 'PEAKASSIGNMENTS') {
        if (options.wantXY) {
          if (dataValue.match(/.*(XYA).*/)) {
            // ex: (XYA)
            parseXYA(spectrum, dataValue);
          }

          currentEntry.spectra.push(spectrum);
          spectrum = new Spectrum();
        }

        continue;
      }

      if (canonicDataLabel === 'TITLE') {
        let parentEntry = currentEntry;

        if (!parentEntry.children) {
          parentEntry.children = [];
        }

        currentEntry = {
          spectra: [],
          ntuples: {},
          info: {},
          meta: {},
          tmp: {} // tmp information we need to keep for postprocessing

        };
        parentEntry.children.push(currentEntry);
        parentsStack.push(parentEntry);
        entriesFlat.push(currentEntry);
        currentEntry.title = dataValue;
      } else if (canonicDataLabel === 'DATATYPE') {
        currentEntry.dataType = dataValue;

        if (dataValue.toLowerCase().indexOf('nd') > -1) {
          currentEntry.twoD = true;
        }
      } else if (canonicDataLabel === 'NTUPLES') {
        if (dataValue.toLowerCase().indexOf('nd') > -1) {
          currentEntry.twoD = true;
        }
      } else if (canonicDataLabel === 'DATACLASS') {
        currentEntry.dataClass = dataValue;
      } else if (canonicDataLabel === 'XUNITS') {
        spectrum.xUnits = dataValue;
      } else if (canonicDataLabel === 'YUNITS') {
        spectrum.yUnits = dataValue;
      } else if (canonicDataLabel === 'FIRSTX') {
        spectrum.firstX = parseFloat(dataValue);
      } else if (canonicDataLabel === 'LASTX') {
        spectrum.lastX = parseFloat(dataValue);
      } else if (canonicDataLabel === 'FIRSTY') {
        spectrum.firstY = parseFloat(dataValue);
      } else if (canonicDataLabel === 'LASTY') {
        spectrum.lastY = parseFloat(dataValue);
      } else if (canonicDataLabel === 'NPOINTS') {
        spectrum.nbPoints = parseFloat(dataValue);
      } else if (canonicDataLabel === 'XFACTOR') {
        spectrum.xFactor = parseFloat(dataValue);
      } else if (canonicDataLabel === 'YFACTOR') {
        spectrum.yFactor = parseFloat(dataValue);
      } else if (canonicDataLabel === 'MAXX') {
        spectrum.maxX = parseFloat(dataValue);
      } else if (canonicDataLabel === 'MINX') {
        spectrum.minX = parseFloat(dataValue);
      } else if (canonicDataLabel === 'MAXY') {
        spectrum.maxY = parseFloat(dataValue);
      } else if (canonicDataLabel === 'MINY') {
        spectrum.minY = parseFloat(dataValue);
      } else if (canonicDataLabel === 'DELTAX') {
        spectrum.deltaX = parseFloat(dataValue);
      } else if (canonicDataLabel === '.OBSERVEFREQUENCY' || canonicDataLabel === '$SFO1') {
        if (!spectrum.observeFrequency) {
          spectrum.observeFrequency = parseFloat(dataValue);
        }
      } else if (canonicDataLabel === '.OBSERVENUCLEUS') {
        if (!spectrum.xType) {
          currentEntry.xType = dataValue.replace(/[^a-zA-Z0-9]/g, '');
        }
      } else if (canonicDataLabel === '$OFFSET') {
        // OFFSET for Bruker spectra
        currentEntry.shiftOffsetNum = 0;

        if (!spectrum.shiftOffsetVal) {
          spectrum.shiftOffsetVal = parseFloat(dataValue);
        }
      } else if (canonicDataLabel === '$REFERENCEPOINT') ; else if (canonicDataLabel === 'VARNAME') {
        currentEntry.ntuples.varname = dataValue.split(ntuplesSeparatorRegExp);
      } else if (canonicDataLabel === 'SYMBOL') {
        currentEntry.ntuples.symbol = dataValue.split(ntuplesSeparatorRegExp);
      } else if (canonicDataLabel === 'VARTYPE') {
        currentEntry.ntuples.vartype = dataValue.split(ntuplesSeparatorRegExp);
      } else if (canonicDataLabel === 'VARFORM') {
        currentEntry.ntuples.varform = dataValue.split(ntuplesSeparatorRegExp);
      } else if (canonicDataLabel === 'VARDIM') {
        currentEntry.ntuples.vardim = convertToFloatArray$1(dataValue.split(ntuplesSeparatorRegExp));
      } else if (canonicDataLabel === 'UNITS') {
        currentEntry.ntuples.units = dataValue.split(ntuplesSeparatorRegExp);
      } else if (canonicDataLabel === 'FACTOR') {
        currentEntry.ntuples.factor = convertToFloatArray$1(dataValue.split(ntuplesSeparatorRegExp));
      } else if (canonicDataLabel === 'FIRST') {
        currentEntry.ntuples.first = convertToFloatArray$1(dataValue.split(ntuplesSeparatorRegExp));
      } else if (canonicDataLabel === 'LAST') {
        currentEntry.ntuples.last = convertToFloatArray$1(dataValue.split(ntuplesSeparatorRegExp));
      } else if (canonicDataLabel === 'MIN') {
        currentEntry.ntuples.min = convertToFloatArray$1(dataValue.split(ntuplesSeparatorRegExp));
      } else if (canonicDataLabel === 'MAX') {
        currentEntry.ntuples.max = convertToFloatArray$1(dataValue.split(ntuplesSeparatorRegExp));
      } else if (canonicDataLabel === '.NUCLEUS') {
        if (currentEntry.ntuples) {
          currentEntry.ntuples.nucleus = dataValue.split(ntuplesSeparatorRegExp);
        }
      } else if (canonicDataLabel === 'PAGE') {
        spectrum.page = dataValue.trim();
        spectrum.pageValue = parseFloat(dataValue.replace(/^.*=/, ''));
        spectrum.pageSymbol = spectrum.page.replace(/[=].*/, '');
      } else if (canonicDataLabel === 'RETENTIONTIME') {
        spectrum.pageValue = parseFloat(dataValue);
      } else if (isMSField(canonicDataLabel)) {
        spectrum[convertMSFieldToLabel(canonicDataLabel)] = dataValue;
      } else if (canonicDataLabel === 'SAMPLEDESCRIPTION') {
        spectrum.sampleDescription = dataValue;
      } else if (canonicDataLabel.startsWith('$NUC')) {
        if (!currentEntry.tmp[canonicDataLabel] && !dataValue.includes('off')) {
          currentEntry.tmp[canonicDataLabel] = dataValue.replace(/[<>]/g, '');
        }
      } else if (canonicDataLabel === 'END') {
        currentEntry = parentsStack.pop();
      }

      if (currentEntry && currentEntry.info && currentEntry.meta && canonicDataLabel.match(options.keepRecordsRegExp)) {
        let value = dataValue.trim();
        let target, label;

        if (dataLabel.startsWith('$')) {
          label = options.canonicMetadataLabels ? canonicDataLabel.substring(1) : dataLabel.substring(1);
          target = currentEntry.meta;
        } else {
          label = options.canonicDataLabels ? canonicDataLabel : dataLabel;
          target = currentEntry.info;
        }

        if (options.dynamicTyping) {
          if (value.match(numberRegExp)) {
            value = Number.parseFloat(value);
          }
        }

        if (target[label]) {
          if (!Array.isArray(target[label])) {
            target[label] = [target[label]];
          }

          target[label].push(value);
        } else {
          target[label] = value;
        }
      }
    }

    profiling(result, 'Finished parsing', options);
    postProcessing(entriesFlat, result, options);
    profiling(result, 'Total time', options);
    /*
    if (result.children && result.children.length>0) {
      result = { ...result, ...result.children[0] };
    }
    */

    result.entries = tmpResult.children;
    result.flatten = entriesFlat;
    return result;
  }

  function parseData(file, options) {
    let {
      keepRecordsRegExp = /.*/
    } = options;
    let result = convert(file, {
      keepRecordsRegExp: keepRecordsRegExp
    });
    return result.flatten.length === 0 ? {} : result.flatten[0];
  }

  function ensureIOBuffer(data) {
    if (data instanceof Array || data instanceof Uint8Array) {
      data = new ArrayBuffer(data);
    }

    if (data instanceof ArrayBuffer) {
      return new IOBuffer(data);
    }

    return data;
  }

  function setFIDSpectrumData(file, spectra) {
    file = ensureIOBuffer(file);
    let td = parseInt(spectra.meta.TD[0], 10);
    let SW_H = parseFloat(spectra.meta.SW_h[0]);
    let SF = parseFloat(spectra.meta.SFO1[0]);
    spectra.meta.DATATYPE = 'NMR FID';
    let DW = 1 / (2 * SW_H);
    let AQ = td * DW;
    let endian = parseInt(spectra.meta.BYTORDA, 10);
    endian = endian ? 0 : 1;

    if (endian) {
      file.setLittleEndian();
    } else {
      file.setBigEndian();
    }

    let nbSubSpectra = spectra.meta.nbSubSpectra ? spectra.meta.nbSubSpectra : 1;
    spectra.spectra = new Array(nbSubSpectra);

    for (let j = 0; j < nbSubSpectra / 2; j++) {
      let toSave = {
        dataType: 'NMR FID',
        dataTable: '(X++(R..R))',
        nbPoints: td,
        firstX: 0,
        lastX: AQ,
        nucleus: spectra.meta.NUC1,
        xUnit: 'Sec',
        yUnit: 'Arbitrary',
        data: [new Float64Array(2 * td)],
        isXYdata: true,
        observeFrequency: SF,
        title: spectra.meta.TITLE,
        deltaX: DW
      };
      spectra.spectra[j * 2] = toSave;
      toSave = {
        dataType: 'NMR FID',
        dataTable: '(X++(I..I))',
        nbPoints: td,
        firstX: 0,
        lastX: AQ,
        nucleus: spectra.meta.NUC1,
        xUnit: 'Sec',
        yUnit: 'Arbitrary',
        data: new Float64Array(2 * td),
        isXYdata: true,
        directFrequency: SF,
        title: spectra.meta.TITLE,
        deltaX: DW
      };
      spectra.spectra[j * 2 + 1] = toSave;
      let i = 0;
      let x = 0;

      for (; file.available(8) && i < td; i++, x = i * DW) {
        let y = file.readInt32();

        if (y === null || isNaN(y)) {
          y = 0;
        }

        spectra.spectra[j * 2].data[2 * i + 1] = y;
        spectra.spectra[j * 2].data[2 * i] = x;
        y = file.readInt32();

        if (y === null || isNaN(y)) {
          y = 0;
        }

        spectra.spectra[j * 2 + 1].data[2 * i + 1] = y;
        spectra.spectra[j * 2 + 1].data[2 * i] = x;
      }

      for (; i < td; i++, x = i * DW) {
        spectra.spectra[j * 2].data[2 * i + 1] = 0;
        spectra.spectra[j * 2].data[2 * i] = x;
        spectra.spectra[j * 2 + 1].data[2 * i + 1] = 0;
        spectra.spectra[j * 2 + 1].data[2 * i] = x;
      }
    }
  }

  function setXYSpectrumData(file, spectra, real) {
    file = ensureIOBuffer(file);
    let td = parseInt(spectra.meta.SI, 10);
    let swP = parseFloat(spectra.meta.SW_p);
    let sf = parseFloat(spectra.meta.SF);
    let bf = sf;
    let offset = spectra.shiftOffsetVal || parseFloat(spectra.meta.OFFSET);
    spectra.meta.observeFrequency = sf;
    spectra.meta.brukerReference = bf;
    spectra.meta.DATATYPE = 'NMR Spectrum';
    let endian = parseInt(spectra.meta.BYTORDP, 10);
    endian = endian ? 0 : 1;
    let nbSubSpectra = spectra.meta.nbSubSpectra ? spectra.meta.nbSubSpectra : 1;

    if (endian) {
      file.setLittleEndian();
    } else {
      file.setBigEndian();
    }

    for (let i = 0; i < nbSubSpectra; i++) {
      let toSave = {
        dataType: 'NMR Spectrum',
        dataTable: '(X++(R..R))',
        nbPoints: td,
        firstX: offset,
        lastX: offset - swP / sf,
        xUnit: 'PPM',
        yUnit: 'Arbitrary',
        data: new Float64Array(td * 2),
        isXYdata: true,
        observeFrequency: sf,
        title: spectra.meta.TITLE,
        deltaX: -(swP / sf) / (td - 1)
      };
      let x = offset;
      let deltaX = toSave.deltaX;

      if (real) {
        for (let k = 0; k < td; ++k) {
          toSave.data[2 * k] = x;
          toSave.data[2 * k + 1] = file.readInt32();

          if (toSave.data[2 * k + 1] === null || isNaN(toSave.data[2 * k + 1])) {
            toSave.data[2 * k + 1] = 0;
          }

          x += deltaX;
        }
      } else {
        for (let k = 0; k < td; ++k) {
          toSave.data[2 * k] = x;
          toSave.data[2 * k + 1] = file.readInt32();

          if (toSave.data[2 * k + 1] === null || isNaN(toSave.data[2 * k + 1])) {
            toSave.data[2 * k + 1] = 0;
          }

          x += deltaX;
        }
      }

      spectra.spectra.push(toSave);
    }
  }

  function convert1D(files, options) {
    let result = parseData(files.procs || '', options);
    let temp = parseData(files.acqus || '', options);

    if (!Object.keys(result).length) {
      result = temp;
    }

    for (let key in result.meta) {
      result.meta[key] = [result.meta[key]];
    }

    for (let currKey in temp.meta) {
      if (result.meta[currKey] === undefined) {
        result.meta[currKey] = [temp.meta[currKey]];
      }
    }

    if (files['1r'] || files['1i']) {
      if (files['1r']) {
        setXYSpectrumData(files['1r'], result, true);
      }

      if (files['1i']) {
        setXYSpectrumData(files['1i'], result, false);
      }
    } else if (files.fid) {
      setFIDSpectrumData(files.fid, result);
    }

    return result;
  }

  function mergeMetadata(main, complement) {
    for (let key in complement.meta) {
      if (main.meta[key]) {
        if (!Array.isArray(main.meta[key])) {
          main.meta[key] = [main.meta[key]];
        }

        main.meta[key].push(complement.meta[key]);
      } else if (main.meta[key] === undefined) {
        main.meta[key] = [complement.meta[key]];
      }
    }

    return main;
  }

  function convert2D(files, options) {
    let temp, temp2, result;

    if (files.proc2s && files.procs) {
      result = parseData(files.procs, options);
      temp = parseData(files.proc2s, options);
      result = mergeMetadata(result, temp);
    }

    temp = parseData(files.acqus, options);
    temp2 = parseData(files.acqu2s, options);
    temp = mergeMetadata(temp, temp2);

    if (!result) {
      result = temp;
    }

    for (let key in temp.meta) {
      if (result.meta[key] === undefined) {
        result.meta[key] = temp.meta[key];
      }
    }

    for (let key in result.meta) {
      if (!Array.isArray(result.meta[key])) {
        result.meta[key] = [result.meta[key]];
      }
    }

    result.meta.nbSubSpectra = files['2rr'] ? parseInt(result.meta.SI[1], 10) : parseInt(result.meta.TD[1], 10);

    if (!result.meta.SW_p) {
      // eslint-disable-next-line camelcase
      result.meta.SW_p = result.meta.SW_h;
    }

    if (!result.meta.SF) {
      result.meta.SF = result.meta.SFO1;
    }

    let firstY, lastY, xOffset, yOffset;

    if (files['2rr']) {
      let sf = parseFloat(result.meta.SF[1]);
      let swP = parseFloat(result.meta.SW_p[1] || result.meta.SW[1]);
      yOffset = parseFloat(result.meta.OFFSET[1]);
      xOffset = parseFloat(result.meta.OFFSET[0]);
      firstY = yOffset;
      lastY = yOffset - swP / sf;
      result.meta.firstY = firstY;
      result.meta.lastY = lastY;
      setXYSpectrumData(files['2rr'], result, true);
    } else if (files.ser) {
      firstY = 0;
      lastY = result.meta.nbSubSpectra;
      let xWindowSize = parseFloat(result.meta.SW[0]);
      let yWindowSize = parseFloat(result.meta.SW[1]);
      let xTransmitterFrequency = parseFloat(result.meta.SFO1[0]);
      let yTransmitterFrequency = parseFloat(result.meta.SFO1[1]);
      let xTransmitterFrequencyOffset = parseFloat(result.meta.O1[0]);
      let yTransmitterFrequencyOffset = parseFloat(result.meta.O1[1]);
      xOffset = xTransmitterFrequencyOffset / xTransmitterFrequency + xWindowSize / 2;
      yOffset = yTransmitterFrequencyOffset / yTransmitterFrequency + yWindowSize / 2;
      setFIDSpectrumData(files.ser, result);
    }

    let pageValue = firstY;
    let nbSubSpectra = result.meta.nbSubSpectra;
    let deltaY = (lastY - firstY) / (nbSubSpectra - 1);

    for (let i = 0; i < nbSubSpectra; i++) {
      pageValue += deltaY;
      result.spectra[i].pageValue = pageValue;
    }

    let {
      NUC1: nuc1,
      AXNUC: axnuc,
      SF: sf
    } = result.meta;
    const nucleus = axnuc ? axnuc : nuc1 ? nuc1 : [];
    result.info['2D_Y_NUCLEUS'] = nucleus[1];
    result.info['2D_X_NUCLEUS'] = nucleus[0];
    result.info['2D_Y_FRECUENCY'] = sf[1];
    result.info['2D_X_FRECUENCY'] = sf[0];
    result.info['2D_Y_OFFSET'] = yOffset;
    result.info['2D_X_OFFSET'] = xOffset;
    result.info.twoD = result.twoD = true;
    return result;
  }

  /**
   * Extract information and data from bruker files.
   * @param {object} brukerFiles - Needed bruker files to parse raw data.
   * @param {object} [options = {}] - options.
   * @param {boolean} [options.xy] - if true, spectra data is a object with x and y
   * @param {RegExp} [options.keepRecordsRegExp='\/.*\/'] - regular expresion to parse the metadata of the spectrum.
   * @param {boolean} [options.noContours=false] - if true the contour data is not generated.
   * @returns
   */

  function convertFolder(brukerFiles, options = {}) {
    let start = new Date();
    let result;

    if (brukerFiles.ser || brukerFiles['2rr']) {
      result = convert2D(brukerFiles, options);
    } else if (brukerFiles['1r'] || brukerFiles['1i'] || brukerFiles.fid) {
      result = convert1D(brukerFiles, options);
    } else {
      throw new RangeError('The current files are invalid');
    } //normalizing info


    result.meta.DATE = parseFloat(result.meta.DATE);

    if (result.meta.GRPDLY) {
      result.meta.GRPDLY = parseFloat(result.meta.GRPDLY);
      result.meta.DSPFVS = parseFloat(result.meta.DSPFVS);
      result.meta.DECIM = parseFloat(result.meta.DECIM);
    }

    for (let key in result.meta) {
      if (!Array.isArray(result.meta[key])) {
        continue;
      }

      if (result.meta[key].length === 1) {
        result.meta[key] = result.meta[key][0];
      } else if (typeof result.meta[key][0] === 'string' && result.meta[key][0].indexOf('(0..') > -1) {
        result.meta[key] = result.meta[key][0];
      }
    }

    if (result.twoD) {
      add2D$1(result, options);

      if (result.profiling) {
        result.profiling.push({
          action: 'Finished countour plot calculation',
          time: new Date() - start
        });
      }

      if (!options.keepSpectra) {
        delete result.spectra;
      }
    }

    let spectra = result.spectra;

    if (options.xy && !!spectra) {
      //the spectraData should not be a oneD Array but an object with x and y
      if (spectra.length > 0) {
        for (let i = 0; i < spectra.length; i++) {
          let spectrum = spectra[i];

          if (spectrum.data.length) {
            let data = spectrum.data;
            let newData = {
              x: new Float64Array(data.length / 2),
              y: new Float64Array(data.length / 2)
            };

            for (let k = 0; k < data.length; k = k + 2) {
              newData.x[k / 2] = data[k];
              newData.y[k / 2] = data[k + 1];
            }

            spectrum.data = newData;
          }
        }
      }
    }

    const {
      file
    } = brukerFiles;

    if (file) {
      result.source = {
        file
      };
    }

    return result;
  }

  /*!

  JSZip v3.6.0 - A JavaScript class for generating and reading zip files
  <http://stuartk.com/jszip>

  (c) 2009-2016 Stuart Knightley <stuart [at] stuartk.com>
  Dual licenced under the MIT license or GPLv3. See https://raw.github.com/Stuk/jszip/master/LICENSE.markdown.

  JSZip uses the library pako released under the MIT license :
  https://github.com/nodeca/pako/blob/master/LICENSE
  */
  var jszip_min = createCommonjsModule(function (module, exports) {
    !function (e) {
      module.exports = e();
    }(function () {
      return function s(a, o, u) {
        function h(r, e) {
          if (!o[r]) {
            if (!a[r]) {
              var t = "function" == typeof commonjsRequire && commonjsRequire;
              if (!e && t) return t(r, !0);
              if (f) return f(r, !0);
              var n = new Error("Cannot find module '" + r + "'");
              throw n.code = "MODULE_NOT_FOUND", n;
            }

            var i = o[r] = {
              exports: {}
            };
            a[r][0].call(i.exports, function (e) {
              var t = a[r][1][e];
              return h(t || e);
            }, i, i.exports, s, a, o, u);
          }

          return o[r].exports;
        }

        for (var f = "function" == typeof commonjsRequire && commonjsRequire, e = 0; e < u.length; e++) h(u[e]);

        return h;
      }({
        1: [function (l, t, n) {
          (function (r) {
            !function (e) {
              "object" == typeof n && void 0 !== t ? t.exports = e() : ("undefined" != typeof window ? window : void 0 !== r ? r : "undefined" != typeof self ? self : this).JSZip = e();
            }(function () {
              return function s(a, o, u) {
                function h(t, e) {
                  if (!o[t]) {
                    if (!a[t]) {
                      var r = "function" == typeof l && l;
                      if (!e && r) return r(t, !0);
                      if (f) return f(t, !0);
                      var n = new Error("Cannot find module '" + t + "'");
                      throw n.code = "MODULE_NOT_FOUND", n;
                    }

                    var i = o[t] = {
                      exports: {}
                    };
                    a[t][0].call(i.exports, function (e) {
                      return h(a[t][1][e] || e);
                    }, i, i.exports, s, a, o, u);
                  }

                  return o[t].exports;
                }

                for (var f = "function" == typeof l && l, e = 0; e < u.length; e++) h(u[e]);

                return h;
              }({
                1: [function (l, t, n) {
                  (function (r) {
                    !function (e) {
                      "object" == typeof n && void 0 !== t ? t.exports = e() : ("undefined" != typeof window ? window : void 0 !== r ? r : "undefined" != typeof self ? self : this).JSZip = e();
                    }(function () {
                      return function s(a, o, u) {
                        function h(t, e) {
                          if (!o[t]) {
                            if (!a[t]) {
                              var r = "function" == typeof l && l;
                              if (!e && r) return r(t, !0);
                              if (f) return f(t, !0);
                              var n = new Error("Cannot find module '" + t + "'");
                              throw n.code = "MODULE_NOT_FOUND", n;
                            }

                            var i = o[t] = {
                              exports: {}
                            };
                            a[t][0].call(i.exports, function (e) {
                              return h(a[t][1][e] || e);
                            }, i, i.exports, s, a, o, u);
                          }

                          return o[t].exports;
                        }

                        for (var f = "function" == typeof l && l, e = 0; e < u.length; e++) h(u[e]);

                        return h;
                      }({
                        1: [function (l, t, n) {
                          (function (r) {
                            !function (e) {
                              "object" == typeof n && void 0 !== t ? t.exports = e() : ("undefined" != typeof window ? window : void 0 !== r ? r : "undefined" != typeof self ? self : this).JSZip = e();
                            }(function () {
                              return function s(a, o, u) {
                                function h(t, e) {
                                  if (!o[t]) {
                                    if (!a[t]) {
                                      var r = "function" == typeof l && l;
                                      if (!e && r) return r(t, !0);
                                      if (f) return f(t, !0);
                                      var n = new Error("Cannot find module '" + t + "'");
                                      throw n.code = "MODULE_NOT_FOUND", n;
                                    }

                                    var i = o[t] = {
                                      exports: {}
                                    };
                                    a[t][0].call(i.exports, function (e) {
                                      return h(a[t][1][e] || e);
                                    }, i, i.exports, s, a, o, u);
                                  }

                                  return o[t].exports;
                                }

                                for (var f = "function" == typeof l && l, e = 0; e < u.length; e++) h(u[e]);

                                return h;
                              }({
                                1: [function (l, t, n) {
                                  (function (r) {
                                    !function (e) {
                                      "object" == typeof n && void 0 !== t ? t.exports = e() : ("undefined" != typeof window ? window : void 0 !== r ? r : "undefined" != typeof self ? self : this).JSZip = e();
                                    }(function () {
                                      return function s(a, o, u) {
                                        function h(t, e) {
                                          if (!o[t]) {
                                            if (!a[t]) {
                                              var r = "function" == typeof l && l;
                                              if (!e && r) return r(t, !0);
                                              if (f) return f(t, !0);
                                              var n = new Error("Cannot find module '" + t + "'");
                                              throw n.code = "MODULE_NOT_FOUND", n;
                                            }

                                            var i = o[t] = {
                                              exports: {}
                                            };
                                            a[t][0].call(i.exports, function (e) {
                                              return h(a[t][1][e] || e);
                                            }, i, i.exports, s, a, o, u);
                                          }

                                          return o[t].exports;
                                        }

                                        for (var f = "function" == typeof l && l, e = 0; e < u.length; e++) h(u[e]);

                                        return h;
                                      }({
                                        1: [function (l, t, n) {
                                          (function (r) {
                                            !function (e) {
                                              "object" == typeof n && void 0 !== t ? t.exports = e() : ("undefined" != typeof window ? window : void 0 !== r ? r : "undefined" != typeof self ? self : this).JSZip = e();
                                            }(function () {
                                              return function s(a, o, u) {
                                                function h(t, e) {
                                                  if (!o[t]) {
                                                    if (!a[t]) {
                                                      var r = "function" == typeof l && l;
                                                      if (!e && r) return r(t, !0);
                                                      if (f) return f(t, !0);
                                                      var n = new Error("Cannot find module '" + t + "'");
                                                      throw n.code = "MODULE_NOT_FOUND", n;
                                                    }

                                                    var i = o[t] = {
                                                      exports: {}
                                                    };
                                                    a[t][0].call(i.exports, function (e) {
                                                      return h(a[t][1][e] || e);
                                                    }, i, i.exports, s, a, o, u);
                                                  }

                                                  return o[t].exports;
                                                }

                                                for (var f = "function" == typeof l && l, e = 0; e < u.length; e++) h(u[e]);

                                                return h;
                                              }({
                                                1: [function (e, t, r) {

                                                  var c = e("./utils"),
                                                      l = e("./support"),
                                                      p = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
                                                  r.encode = function (e) {
                                                    for (var t, r, n, i, s, a, o, u = [], h = 0, f = e.length, l = f, d = "string" !== c.getTypeOf(e); h < e.length;) l = f - h, n = d ? (t = e[h++], r = h < f ? e[h++] : 0, h < f ? e[h++] : 0) : (t = e.charCodeAt(h++), r = h < f ? e.charCodeAt(h++) : 0, h < f ? e.charCodeAt(h++) : 0), i = t >> 2, s = (3 & t) << 4 | r >> 4, a = 1 < l ? (15 & r) << 2 | n >> 6 : 64, o = 2 < l ? 63 & n : 64, u.push(p.charAt(i) + p.charAt(s) + p.charAt(a) + p.charAt(o));

                                                    return u.join("");
                                                  }, r.decode = function (e) {
                                                    var t,
                                                        r,
                                                        n,
                                                        i,
                                                        s,
                                                        a,
                                                        o = 0,
                                                        u = 0;
                                                    if ("data:" === e.substr(0, "data:".length)) throw new Error("Invalid base64 input, it looks like a data url.");
                                                    var h,
                                                        f = 3 * (e = e.replace(/[^A-Za-z0-9\+\/\=]/g, "")).length / 4;
                                                    if (e.charAt(e.length - 1) === p.charAt(64) && f--, e.charAt(e.length - 2) === p.charAt(64) && f--, f % 1 != 0) throw new Error("Invalid base64 input, bad content length.");

                                                    for (h = l.uint8array ? new Uint8Array(0 | f) : new Array(0 | f); o < e.length;) t = p.indexOf(e.charAt(o++)) << 2 | (i = p.indexOf(e.charAt(o++))) >> 4, r = (15 & i) << 4 | (s = p.indexOf(e.charAt(o++))) >> 2, n = (3 & s) << 6 | (a = p.indexOf(e.charAt(o++))), h[u++] = t, 64 !== s && (h[u++] = r), 64 !== a && (h[u++] = n);

                                                    return h;
                                                  };
                                                }, {
                                                  "./support": 30,
                                                  "./utils": 32
                                                }],
                                                2: [function (e, t, r) {

                                                  var n = e("./external"),
                                                      i = e("./stream/DataWorker"),
                                                      s = e("./stream/Crc32Probe"),
                                                      a = e("./stream/DataLengthProbe");

                                                  function o(e, t, r, n, i) {
                                                    this.compressedSize = e, this.uncompressedSize = t, this.crc32 = r, this.compression = n, this.compressedContent = i;
                                                  }

                                                  o.prototype = {
                                                    getContentWorker: function () {
                                                      var e = new i(n.Promise.resolve(this.compressedContent)).pipe(this.compression.uncompressWorker()).pipe(new a("data_length")),
                                                          t = this;
                                                      return e.on("end", function () {
                                                        if (this.streamInfo.data_length !== t.uncompressedSize) throw new Error("Bug : uncompressed data size mismatch");
                                                      }), e;
                                                    },
                                                    getCompressedWorker: function () {
                                                      return new i(n.Promise.resolve(this.compressedContent)).withStreamInfo("compressedSize", this.compressedSize).withStreamInfo("uncompressedSize", this.uncompressedSize).withStreamInfo("crc32", this.crc32).withStreamInfo("compression", this.compression);
                                                    }
                                                  }, o.createWorkerFrom = function (e, t, r) {
                                                    return e.pipe(new s()).pipe(new a("uncompressedSize")).pipe(t.compressWorker(r)).pipe(new a("compressedSize")).withStreamInfo("compression", t);
                                                  }, t.exports = o;
                                                }, {
                                                  "./external": 6,
                                                  "./stream/Crc32Probe": 25,
                                                  "./stream/DataLengthProbe": 26,
                                                  "./stream/DataWorker": 27
                                                }],
                                                3: [function (e, t, r) {

                                                  var n = e("./stream/GenericWorker");
                                                  r.STORE = {
                                                    magic: "\0\0",
                                                    compressWorker: function (e) {
                                                      return new n("STORE compression");
                                                    },
                                                    uncompressWorker: function () {
                                                      return new n("STORE decompression");
                                                    }
                                                  }, r.DEFLATE = e("./flate");
                                                }, {
                                                  "./flate": 7,
                                                  "./stream/GenericWorker": 28
                                                }],
                                                4: [function (e, t, r) {

                                                  var n = e("./utils"),
                                                      a = function () {
                                                    for (var e, t = [], r = 0; r < 256; r++) {
                                                      e = r;

                                                      for (var n = 0; n < 8; n++) e = 1 & e ? 3988292384 ^ e >>> 1 : e >>> 1;

                                                      t[r] = e;
                                                    }

                                                    return t;
                                                  }();

                                                  t.exports = function (e, t) {
                                                    return void 0 !== e && e.length ? "string" !== n.getTypeOf(e) ? function (e, t, r) {
                                                      var n = a,
                                                          i = 0 + r;
                                                      e ^= -1;

                                                      for (var s = 0; s < i; s++) e = e >>> 8 ^ n[255 & (e ^ t[s])];

                                                      return -1 ^ e;
                                                    }(0 | t, e, e.length) : function (e, t, r) {
                                                      var n = a,
                                                          i = 0 + r;
                                                      e ^= -1;

                                                      for (var s = 0; s < i; s++) e = e >>> 8 ^ n[255 & (e ^ t.charCodeAt(s))];

                                                      return -1 ^ e;
                                                    }(0 | t, e, e.length) : 0;
                                                  };
                                                }, {
                                                  "./utils": 32
                                                }],
                                                5: [function (e, t, r) {

                                                  r.base64 = !1, r.binary = !1, r.dir = !1, r.createFolders = !0, r.date = null, r.compression = null, r.compressionOptions = null, r.comment = null, r.unixPermissions = null, r.dosPermissions = null;
                                                }, {}],
                                                6: [function (e, t, r) {

                                                  var n;
                                                  n = "undefined" != typeof Promise ? Promise : e("lie"), t.exports = {
                                                    Promise: n
                                                  };
                                                }, {
                                                  lie: 37
                                                }],
                                                7: [function (e, t, r) {

                                                  var n = "undefined" != typeof Uint8Array && "undefined" != typeof Uint16Array && "undefined" != typeof Uint32Array,
                                                      i = e("pako"),
                                                      s = e("./utils"),
                                                      a = e("./stream/GenericWorker"),
                                                      o = n ? "uint8array" : "array";

                                                  function u(e, t) {
                                                    a.call(this, "FlateWorker/" + e), this._pako = null, this._pakoAction = e, this._pakoOptions = t, this.meta = {};
                                                  }

                                                  r.magic = "\b\0", s.inherits(u, a), u.prototype.processChunk = function (e) {
                                                    this.meta = e.meta, null === this._pako && this._createPako(), this._pako.push(s.transformTo(o, e.data), !1);
                                                  }, u.prototype.flush = function () {
                                                    a.prototype.flush.call(this), null === this._pako && this._createPako(), this._pako.push([], !0);
                                                  }, u.prototype.cleanUp = function () {
                                                    a.prototype.cleanUp.call(this), this._pako = null;
                                                  }, u.prototype._createPako = function () {
                                                    this._pako = new i[this._pakoAction]({
                                                      raw: !0,
                                                      level: this._pakoOptions.level || -1
                                                    });
                                                    var t = this;

                                                    this._pako.onData = function (e) {
                                                      t.push({
                                                        data: e,
                                                        meta: t.meta
                                                      });
                                                    };
                                                  }, r.compressWorker = function (e) {
                                                    return new u("Deflate", e);
                                                  }, r.uncompressWorker = function () {
                                                    return new u("Inflate", {});
                                                  };
                                                }, {
                                                  "./stream/GenericWorker": 28,
                                                  "./utils": 32,
                                                  pako: 38
                                                }],
                                                8: [function (e, t, r) {

                                                  function I(e, t) {
                                                    var r,
                                                        n = "";

                                                    for (r = 0; r < t; r++) n += String.fromCharCode(255 & e), e >>>= 8;

                                                    return n;
                                                  }

                                                  function i(e, t, r, n, i, s) {
                                                    var a,
                                                        o,
                                                        u = e.file,
                                                        h = e.compression,
                                                        f = s !== B.utf8encode,
                                                        l = O.transformTo("string", s(u.name)),
                                                        d = O.transformTo("string", B.utf8encode(u.name)),
                                                        c = u.comment,
                                                        p = O.transformTo("string", s(c)),
                                                        m = O.transformTo("string", B.utf8encode(c)),
                                                        _ = d.length !== u.name.length,
                                                        g = m.length !== c.length,
                                                        v = "",
                                                        b = "",
                                                        w = "",
                                                        y = u.dir,
                                                        k = u.date,
                                                        x = {
                                                      crc32: 0,
                                                      compressedSize: 0,
                                                      uncompressedSize: 0
                                                    };

                                                    t && !r || (x.crc32 = e.crc32, x.compressedSize = e.compressedSize, x.uncompressedSize = e.uncompressedSize);
                                                    var S = 0;
                                                    t && (S |= 8), f || !_ && !g || (S |= 2048);
                                                    var z,
                                                        E = 0,
                                                        C = 0;
                                                    y && (E |= 16), "UNIX" === i ? (C = 798, E |= ((z = u.unixPermissions) || (z = y ? 16893 : 33204), (65535 & z) << 16)) : (C = 20, E |= 63 & (u.dosPermissions || 0)), a = k.getUTCHours(), a <<= 6, a |= k.getUTCMinutes(), a <<= 5, a |= k.getUTCSeconds() / 2, o = k.getUTCFullYear() - 1980, o <<= 4, o |= k.getUTCMonth() + 1, o <<= 5, o |= k.getUTCDate(), _ && (v += "up" + I((b = I(1, 1) + I(T(l), 4) + d).length, 2) + b), g && (v += "uc" + I((w = I(1, 1) + I(T(p), 4) + m).length, 2) + w);
                                                    var A = "";
                                                    return A += "\n\0", A += I(S, 2), A += h.magic, A += I(a, 2), A += I(o, 2), A += I(x.crc32, 4), A += I(x.compressedSize, 4), A += I(x.uncompressedSize, 4), A += I(l.length, 2), A += I(v.length, 2), {
                                                      fileRecord: R.LOCAL_FILE_HEADER + A + l + v,
                                                      dirRecord: R.CENTRAL_FILE_HEADER + I(C, 2) + A + I(p.length, 2) + "\0\0\0\0" + I(E, 4) + I(n, 4) + l + v + p
                                                    };
                                                  }

                                                  var O = e("../utils"),
                                                      s = e("../stream/GenericWorker"),
                                                      B = e("../utf8"),
                                                      T = e("../crc32"),
                                                      R = e("../signature");

                                                  function n(e, t, r, n) {
                                                    s.call(this, "ZipFileWorker"), this.bytesWritten = 0, this.zipComment = t, this.zipPlatform = r, this.encodeFileName = n, this.streamFiles = e, this.accumulate = !1, this.contentBuffer = [], this.dirRecords = [], this.currentSourceOffset = 0, this.entriesCount = 0, this.currentFile = null, this._sources = [];
                                                  }

                                                  O.inherits(n, s), n.prototype.push = function (e) {
                                                    var t = e.meta.percent || 0,
                                                        r = this.entriesCount,
                                                        n = this._sources.length;
                                                    this.accumulate ? this.contentBuffer.push(e) : (this.bytesWritten += e.data.length, s.prototype.push.call(this, {
                                                      data: e.data,
                                                      meta: {
                                                        currentFile: this.currentFile,
                                                        percent: r ? (t + 100 * (r - n - 1)) / r : 100
                                                      }
                                                    }));
                                                  }, n.prototype.openedSource = function (e) {
                                                    this.currentSourceOffset = this.bytesWritten, this.currentFile = e.file.name;
                                                    var t = this.streamFiles && !e.file.dir;

                                                    if (t) {
                                                      var r = i(e, t, !1, this.currentSourceOffset, this.zipPlatform, this.encodeFileName);
                                                      this.push({
                                                        data: r.fileRecord,
                                                        meta: {
                                                          percent: 0
                                                        }
                                                      });
                                                    } else this.accumulate = !0;
                                                  }, n.prototype.closedSource = function (e) {
                                                    this.accumulate = !1;
                                                    var t,
                                                        r = this.streamFiles && !e.file.dir,
                                                        n = i(e, r, !0, this.currentSourceOffset, this.zipPlatform, this.encodeFileName);
                                                    if (this.dirRecords.push(n.dirRecord), r) this.push({
                                                      data: (t = e, R.DATA_DESCRIPTOR + I(t.crc32, 4) + I(t.compressedSize, 4) + I(t.uncompressedSize, 4)),
                                                      meta: {
                                                        percent: 100
                                                      }
                                                    });else for (this.push({
                                                      data: n.fileRecord,
                                                      meta: {
                                                        percent: 0
                                                      }
                                                    }); this.contentBuffer.length;) this.push(this.contentBuffer.shift());
                                                    this.currentFile = null;
                                                  }, n.prototype.flush = function () {
                                                    for (var e = this.bytesWritten, t = 0; t < this.dirRecords.length; t++) this.push({
                                                      data: this.dirRecords[t],
                                                      meta: {
                                                        percent: 100
                                                      }
                                                    });

                                                    var r,
                                                        n,
                                                        i,
                                                        s,
                                                        a,
                                                        o,
                                                        u = this.bytesWritten - e,
                                                        h = (r = this.dirRecords.length, n = u, i = e, s = this.zipComment, a = this.encodeFileName, o = O.transformTo("string", a(s)), R.CENTRAL_DIRECTORY_END + "\0\0\0\0" + I(r, 2) + I(r, 2) + I(n, 4) + I(i, 4) + I(o.length, 2) + o);
                                                    this.push({
                                                      data: h,
                                                      meta: {
                                                        percent: 100
                                                      }
                                                    });
                                                  }, n.prototype.prepareNextSource = function () {
                                                    this.previous = this._sources.shift(), this.openedSource(this.previous.streamInfo), this.isPaused ? this.previous.pause() : this.previous.resume();
                                                  }, n.prototype.registerPrevious = function (e) {
                                                    this._sources.push(e);

                                                    var t = this;
                                                    return e.on("data", function (e) {
                                                      t.processChunk(e);
                                                    }), e.on("end", function () {
                                                      t.closedSource(t.previous.streamInfo), t._sources.length ? t.prepareNextSource() : t.end();
                                                    }), e.on("error", function (e) {
                                                      t.error(e);
                                                    }), this;
                                                  }, n.prototype.resume = function () {
                                                    return !!s.prototype.resume.call(this) && (!this.previous && this._sources.length ? (this.prepareNextSource(), !0) : this.previous || this._sources.length || this.generatedError ? void 0 : (this.end(), !0));
                                                  }, n.prototype.error = function (e) {
                                                    var t = this._sources;
                                                    if (!s.prototype.error.call(this, e)) return !1;

                                                    for (var r = 0; r < t.length; r++) try {
                                                      t[r].error(e);
                                                    } catch (e) {}

                                                    return !0;
                                                  }, n.prototype.lock = function () {
                                                    s.prototype.lock.call(this);

                                                    for (var e = this._sources, t = 0; t < e.length; t++) e[t].lock();
                                                  }, t.exports = n;
                                                }, {
                                                  "../crc32": 4,
                                                  "../signature": 23,
                                                  "../stream/GenericWorker": 28,
                                                  "../utf8": 31,
                                                  "../utils": 32
                                                }],
                                                9: [function (e, t, r) {

                                                  var h = e("../compressions"),
                                                      n = e("./ZipFileWorker");

                                                  r.generateWorker = function (e, a, t) {
                                                    var o = new n(a.streamFiles, t, a.platform, a.encodeFileName),
                                                        u = 0;

                                                    try {
                                                      e.forEach(function (e, t) {
                                                        u++;

                                                        var r = function (e, t) {
                                                          var r = e || t,
                                                              n = h[r];
                                                          if (!n) throw new Error(r + " is not a valid compression method !");
                                                          return n;
                                                        }(t.options.compression, a.compression),
                                                            n = t.options.compressionOptions || a.compressionOptions || {},
                                                            i = t.dir,
                                                            s = t.date;

                                                        t._compressWorker(r, n).withStreamInfo("file", {
                                                          name: e,
                                                          dir: i,
                                                          date: s,
                                                          comment: t.comment || "",
                                                          unixPermissions: t.unixPermissions,
                                                          dosPermissions: t.dosPermissions
                                                        }).pipe(o);
                                                      }), o.entriesCount = u;
                                                    } catch (e) {
                                                      o.error(e);
                                                    }

                                                    return o;
                                                  };
                                                }, {
                                                  "../compressions": 3,
                                                  "./ZipFileWorker": 8
                                                }],
                                                10: [function (e, t, r) {

                                                  function n() {
                                                    if (!(this instanceof n)) return new n();
                                                    if (arguments.length) throw new Error("The constructor with parameters has been removed in JSZip 3.0, please check the upgrade guide.");
                                                    this.files = {}, this.comment = null, this.root = "", this.clone = function () {
                                                      var e = new n();

                                                      for (var t in this) "function" != typeof this[t] && (e[t] = this[t]);

                                                      return e;
                                                    };
                                                  }

                                                  (n.prototype = e("./object")).loadAsync = e("./load"), n.support = e("./support"), n.defaults = e("./defaults"), n.version = "3.5.0", n.loadAsync = function (e, t) {
                                                    return new n().loadAsync(e, t);
                                                  }, n.external = e("./external"), t.exports = n;
                                                }, {
                                                  "./defaults": 5,
                                                  "./external": 6,
                                                  "./load": 11,
                                                  "./object": 15,
                                                  "./support": 30
                                                }],
                                                11: [function (e, t, r) {

                                                  var n = e("./utils"),
                                                      i = e("./external"),
                                                      o = e("./utf8"),
                                                      u = e("./zipEntries"),
                                                      s = e("./stream/Crc32Probe"),
                                                      h = e("./nodejsUtils");

                                                  function f(n) {
                                                    return new i.Promise(function (e, t) {
                                                      var r = n.decompressed.getContentWorker().pipe(new s());
                                                      r.on("error", function (e) {
                                                        t(e);
                                                      }).on("end", function () {
                                                        r.streamInfo.crc32 !== n.decompressed.crc32 ? t(new Error("Corrupted zip : CRC32 mismatch")) : e();
                                                      }).resume();
                                                    });
                                                  }

                                                  t.exports = function (e, s) {
                                                    var a = this;
                                                    return s = n.extend(s || {}, {
                                                      base64: !1,
                                                      checkCRC32: !1,
                                                      optimizedBinaryString: !1,
                                                      createFolders: !1,
                                                      decodeFileName: o.utf8decode
                                                    }), h.isNode && h.isStream(e) ? i.Promise.reject(new Error("JSZip can't accept a stream when loading a zip file.")) : n.prepareContent("the loaded zip file", e, !0, s.optimizedBinaryString, s.base64).then(function (e) {
                                                      var t = new u(s);
                                                      return t.load(e), t;
                                                    }).then(function (e) {
                                                      var t = [i.Promise.resolve(e)],
                                                          r = e.files;
                                                      if (s.checkCRC32) for (var n = 0; n < r.length; n++) t.push(f(r[n]));
                                                      return i.Promise.all(t);
                                                    }).then(function (e) {
                                                      for (var t = e.shift(), r = t.files, n = 0; n < r.length; n++) {
                                                        var i = r[n];
                                                        a.file(i.fileNameStr, i.decompressed, {
                                                          binary: !0,
                                                          optimizedBinaryString: !0,
                                                          date: i.date,
                                                          dir: i.dir,
                                                          comment: i.fileCommentStr.length ? i.fileCommentStr : null,
                                                          unixPermissions: i.unixPermissions,
                                                          dosPermissions: i.dosPermissions,
                                                          createFolders: s.createFolders
                                                        });
                                                      }

                                                      return t.zipComment.length && (a.comment = t.zipComment), a;
                                                    });
                                                  };
                                                }, {
                                                  "./external": 6,
                                                  "./nodejsUtils": 14,
                                                  "./stream/Crc32Probe": 25,
                                                  "./utf8": 31,
                                                  "./utils": 32,
                                                  "./zipEntries": 33
                                                }],
                                                12: [function (e, t, r) {

                                                  var n = e("../utils"),
                                                      i = e("../stream/GenericWorker");

                                                  function s(e, t) {
                                                    i.call(this, "Nodejs stream input adapter for " + e), this._upstreamEnded = !1, this._bindStream(t);
                                                  }

                                                  n.inherits(s, i), s.prototype._bindStream = function (e) {
                                                    var t = this;
                                                    (this._stream = e).pause(), e.on("data", function (e) {
                                                      t.push({
                                                        data: e,
                                                        meta: {
                                                          percent: 0
                                                        }
                                                      });
                                                    }).on("error", function (e) {
                                                      t.isPaused ? this.generatedError = e : t.error(e);
                                                    }).on("end", function () {
                                                      t.isPaused ? t._upstreamEnded = !0 : t.end();
                                                    });
                                                  }, s.prototype.pause = function () {
                                                    return !!i.prototype.pause.call(this) && (this._stream.pause(), !0);
                                                  }, s.prototype.resume = function () {
                                                    return !!i.prototype.resume.call(this) && (this._upstreamEnded ? this.end() : this._stream.resume(), !0);
                                                  }, t.exports = s;
                                                }, {
                                                  "../stream/GenericWorker": 28,
                                                  "../utils": 32
                                                }],
                                                13: [function (e, t, r) {

                                                  var i = e("readable-stream").Readable;

                                                  function n(e, t, r) {
                                                    i.call(this, t), this._helper = e;
                                                    var n = this;
                                                    e.on("data", function (e, t) {
                                                      n.push(e) || n._helper.pause(), r && r(t);
                                                    }).on("error", function (e) {
                                                      n.emit("error", e);
                                                    }).on("end", function () {
                                                      n.push(null);
                                                    });
                                                  }

                                                  e("../utils").inherits(n, i), n.prototype._read = function () {
                                                    this._helper.resume();
                                                  }, t.exports = n;
                                                }, {
                                                  "../utils": 32,
                                                  "readable-stream": 16
                                                }],
                                                14: [function (e, t, r) {

                                                  t.exports = {
                                                    isNode: "undefined" != typeof Buffer,
                                                    newBufferFrom: function (e, t) {
                                                      if (Buffer.from && Buffer.from !== Uint8Array.from) return Buffer.from(e, t);
                                                      if ("number" == typeof e) throw new Error('The "data" argument must not be a number');
                                                      return new Buffer(e, t);
                                                    },
                                                    allocBuffer: function (e) {
                                                      if (Buffer.alloc) return Buffer.alloc(e);
                                                      var t = new Buffer(e);
                                                      return t.fill(0), t;
                                                    },
                                                    isBuffer: function (e) {
                                                      return Buffer.isBuffer(e);
                                                    },
                                                    isStream: function (e) {
                                                      return e && "function" == typeof e.on && "function" == typeof e.pause && "function" == typeof e.resume;
                                                    }
                                                  };
                                                }, {}],
                                                15: [function (e, t, r) {

                                                  function s(e, t, r) {
                                                    var n,
                                                        i = f.getTypeOf(t),
                                                        s = f.extend(r || {}, d);
                                                    s.date = s.date || new Date(), null !== s.compression && (s.compression = s.compression.toUpperCase()), "string" == typeof s.unixPermissions && (s.unixPermissions = parseInt(s.unixPermissions, 8)), s.unixPermissions && 16384 & s.unixPermissions && (s.dir = !0), s.dosPermissions && 16 & s.dosPermissions && (s.dir = !0), s.dir && (e = h(e)), s.createFolders && (n = function (e) {
                                                      "/" === e.slice(-1) && (e = e.substring(0, e.length - 1));
                                                      var t = e.lastIndexOf("/");
                                                      return 0 < t ? e.substring(0, t) : "";
                                                    }(e)) && g.call(this, n, !0);
                                                    var a,
                                                        o = "string" === i && !1 === s.binary && !1 === s.base64;
                                                    r && void 0 !== r.binary || (s.binary = !o), (t instanceof c && 0 === t.uncompressedSize || s.dir || !t || 0 === t.length) && (s.base64 = !1, s.binary = !0, t = "", s.compression = "STORE", i = "string"), a = t instanceof c || t instanceof l ? t : m.isNode && m.isStream(t) ? new _(e, t) : f.prepareContent(e, t, s.binary, s.optimizedBinaryString, s.base64);
                                                    var u = new p(e, a, s);
                                                    this.files[e] = u;
                                                  }

                                                  function h(e) {
                                                    return "/" !== e.slice(-1) && (e += "/"), e;
                                                  }

                                                  var i = e("./utf8"),
                                                      f = e("./utils"),
                                                      l = e("./stream/GenericWorker"),
                                                      a = e("./stream/StreamHelper"),
                                                      d = e("./defaults"),
                                                      c = e("./compressedObject"),
                                                      p = e("./zipObject"),
                                                      o = e("./generate"),
                                                      m = e("./nodejsUtils"),
                                                      _ = e("./nodejs/NodejsStreamInputAdapter"),
                                                      g = function (e, t) {
                                                    return t = void 0 !== t ? t : d.createFolders, e = h(e), this.files[e] || s.call(this, e, null, {
                                                      dir: !0,
                                                      createFolders: t
                                                    }), this.files[e];
                                                  };

                                                  function u(e) {
                                                    return "[object RegExp]" === Object.prototype.toString.call(e);
                                                  }

                                                  var n = {
                                                    load: function () {
                                                      throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.");
                                                    },
                                                    forEach: function (e) {
                                                      var t, r, n;

                                                      for (t in this.files) this.files.hasOwnProperty(t) && (n = this.files[t], (r = t.slice(this.root.length, t.length)) && t.slice(0, this.root.length) === this.root && e(r, n));
                                                    },
                                                    filter: function (r) {
                                                      var n = [];
                                                      return this.forEach(function (e, t) {
                                                        r(e, t) && n.push(t);
                                                      }), n;
                                                    },
                                                    file: function (e, t, r) {
                                                      if (1 !== arguments.length) return e = this.root + e, s.call(this, e, t, r), this;

                                                      if (u(e)) {
                                                        var n = e;
                                                        return this.filter(function (e, t) {
                                                          return !t.dir && n.test(e);
                                                        });
                                                      }

                                                      var i = this.files[this.root + e];
                                                      return i && !i.dir ? i : null;
                                                    },
                                                    folder: function (r) {
                                                      if (!r) return this;
                                                      if (u(r)) return this.filter(function (e, t) {
                                                        return t.dir && r.test(e);
                                                      });
                                                      var e = this.root + r,
                                                          t = g.call(this, e),
                                                          n = this.clone();
                                                      return n.root = t.name, n;
                                                    },
                                                    remove: function (r) {
                                                      r = this.root + r;
                                                      var e = this.files[r];
                                                      if (e || ("/" !== r.slice(-1) && (r += "/"), e = this.files[r]), e && !e.dir) delete this.files[r];else for (var t = this.filter(function (e, t) {
                                                        return t.name.slice(0, r.length) === r;
                                                      }), n = 0; n < t.length; n++) delete this.files[t[n].name];
                                                      return this;
                                                    },
                                                    generate: function (e) {
                                                      throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.");
                                                    },
                                                    generateInternalStream: function (e) {
                                                      var t,
                                                          r = {};

                                                      try {
                                                        if ((r = f.extend(e || {}, {
                                                          streamFiles: !1,
                                                          compression: "STORE",
                                                          compressionOptions: null,
                                                          type: "",
                                                          platform: "DOS",
                                                          comment: null,
                                                          mimeType: "application/zip",
                                                          encodeFileName: i.utf8encode
                                                        })).type = r.type.toLowerCase(), r.compression = r.compression.toUpperCase(), "binarystring" === r.type && (r.type = "string"), !r.type) throw new Error("No output type specified.");
                                                        f.checkSupport(r.type), "darwin" !== r.platform && "freebsd" !== r.platform && "linux" !== r.platform && "sunos" !== r.platform || (r.platform = "UNIX"), "win32" === r.platform && (r.platform = "DOS");
                                                        var n = r.comment || this.comment || "";
                                                        t = o.generateWorker(this, r, n);
                                                      } catch (e) {
                                                        (t = new l("error")).error(e);
                                                      }

                                                      return new a(t, r.type || "string", r.mimeType);
                                                    },
                                                    generateAsync: function (e, t) {
                                                      return this.generateInternalStream(e).accumulate(t);
                                                    },
                                                    generateNodeStream: function (e, t) {
                                                      return (e = e || {}).type || (e.type = "nodebuffer"), this.generateInternalStream(e).toNodejsStream(t);
                                                    }
                                                  };
                                                  t.exports = n;
                                                }, {
                                                  "./compressedObject": 2,
                                                  "./defaults": 5,
                                                  "./generate": 9,
                                                  "./nodejs/NodejsStreamInputAdapter": 12,
                                                  "./nodejsUtils": 14,
                                                  "./stream/GenericWorker": 28,
                                                  "./stream/StreamHelper": 29,
                                                  "./utf8": 31,
                                                  "./utils": 32,
                                                  "./zipObject": 35
                                                }],
                                                16: [function (e, t, r) {
                                                  t.exports = e("stream");
                                                }, {
                                                  stream: void 0
                                                }],
                                                17: [function (e, t, r) {

                                                  var n = e("./DataReader");

                                                  function i(e) {
                                                    n.call(this, e);

                                                    for (var t = 0; t < this.data.length; t++) e[t] = 255 & e[t];
                                                  }

                                                  e("../utils").inherits(i, n), i.prototype.byteAt = function (e) {
                                                    return this.data[this.zero + e];
                                                  }, i.prototype.lastIndexOfSignature = function (e) {
                                                    for (var t = e.charCodeAt(0), r = e.charCodeAt(1), n = e.charCodeAt(2), i = e.charCodeAt(3), s = this.length - 4; 0 <= s; --s) if (this.data[s] === t && this.data[s + 1] === r && this.data[s + 2] === n && this.data[s + 3] === i) return s - this.zero;

                                                    return -1;
                                                  }, i.prototype.readAndCheckSignature = function (e) {
                                                    var t = e.charCodeAt(0),
                                                        r = e.charCodeAt(1),
                                                        n = e.charCodeAt(2),
                                                        i = e.charCodeAt(3),
                                                        s = this.readData(4);
                                                    return t === s[0] && r === s[1] && n === s[2] && i === s[3];
                                                  }, i.prototype.readData = function (e) {
                                                    if (this.checkOffset(e), 0 === e) return [];
                                                    var t = this.data.slice(this.zero + this.index, this.zero + this.index + e);
                                                    return this.index += e, t;
                                                  }, t.exports = i;
                                                }, {
                                                  "../utils": 32,
                                                  "./DataReader": 18
                                                }],
                                                18: [function (e, t, r) {

                                                  var n = e("../utils");

                                                  function i(e) {
                                                    this.data = e, this.length = e.length, this.index = 0, this.zero = 0;
                                                  }

                                                  i.prototype = {
                                                    checkOffset: function (e) {
                                                      this.checkIndex(this.index + e);
                                                    },
                                                    checkIndex: function (e) {
                                                      if (this.length < this.zero + e || e < 0) throw new Error("End of data reached (data length = " + this.length + ", asked index = " + e + "). Corrupted zip ?");
                                                    },
                                                    setIndex: function (e) {
                                                      this.checkIndex(e), this.index = e;
                                                    },
                                                    skip: function (e) {
                                                      this.setIndex(this.index + e);
                                                    },
                                                    byteAt: function (e) {},
                                                    readInt: function (e) {
                                                      var t,
                                                          r = 0;

                                                      for (this.checkOffset(e), t = this.index + e - 1; t >= this.index; t--) r = (r << 8) + this.byteAt(t);

                                                      return this.index += e, r;
                                                    },
                                                    readString: function (e) {
                                                      return n.transformTo("string", this.readData(e));
                                                    },
                                                    readData: function (e) {},
                                                    lastIndexOfSignature: function (e) {},
                                                    readAndCheckSignature: function (e) {},
                                                    readDate: function () {
                                                      var e = this.readInt(4);
                                                      return new Date(Date.UTC(1980 + (e >> 25 & 127), (e >> 21 & 15) - 1, e >> 16 & 31, e >> 11 & 31, e >> 5 & 63, (31 & e) << 1));
                                                    }
                                                  }, t.exports = i;
                                                }, {
                                                  "../utils": 32
                                                }],
                                                19: [function (e, t, r) {

                                                  var n = e("./Uint8ArrayReader");

                                                  function i(e) {
                                                    n.call(this, e);
                                                  }

                                                  e("../utils").inherits(i, n), i.prototype.readData = function (e) {
                                                    this.checkOffset(e);
                                                    var t = this.data.slice(this.zero + this.index, this.zero + this.index + e);
                                                    return this.index += e, t;
                                                  }, t.exports = i;
                                                }, {
                                                  "../utils": 32,
                                                  "./Uint8ArrayReader": 21
                                                }],
                                                20: [function (e, t, r) {

                                                  var n = e("./DataReader");

                                                  function i(e) {
                                                    n.call(this, e);
                                                  }

                                                  e("../utils").inherits(i, n), i.prototype.byteAt = function (e) {
                                                    return this.data.charCodeAt(this.zero + e);
                                                  }, i.prototype.lastIndexOfSignature = function (e) {
                                                    return this.data.lastIndexOf(e) - this.zero;
                                                  }, i.prototype.readAndCheckSignature = function (e) {
                                                    return e === this.readData(4);
                                                  }, i.prototype.readData = function (e) {
                                                    this.checkOffset(e);
                                                    var t = this.data.slice(this.zero + this.index, this.zero + this.index + e);
                                                    return this.index += e, t;
                                                  }, t.exports = i;
                                                }, {
                                                  "../utils": 32,
                                                  "./DataReader": 18
                                                }],
                                                21: [function (e, t, r) {

                                                  var n = e("./ArrayReader");

                                                  function i(e) {
                                                    n.call(this, e);
                                                  }

                                                  e("../utils").inherits(i, n), i.prototype.readData = function (e) {
                                                    if (this.checkOffset(e), 0 === e) return new Uint8Array(0);
                                                    var t = this.data.subarray(this.zero + this.index, this.zero + this.index + e);
                                                    return this.index += e, t;
                                                  }, t.exports = i;
                                                }, {
                                                  "../utils": 32,
                                                  "./ArrayReader": 17
                                                }],
                                                22: [function (e, t, r) {

                                                  var n = e("../utils"),
                                                      i = e("../support"),
                                                      s = e("./ArrayReader"),
                                                      a = e("./StringReader"),
                                                      o = e("./NodeBufferReader"),
                                                      u = e("./Uint8ArrayReader");

                                                  t.exports = function (e) {
                                                    var t = n.getTypeOf(e);
                                                    return n.checkSupport(t), "string" !== t || i.uint8array ? "nodebuffer" === t ? new o(e) : i.uint8array ? new u(n.transformTo("uint8array", e)) : new s(n.transformTo("array", e)) : new a(e);
                                                  };
                                                }, {
                                                  "../support": 30,
                                                  "../utils": 32,
                                                  "./ArrayReader": 17,
                                                  "./NodeBufferReader": 19,
                                                  "./StringReader": 20,
                                                  "./Uint8ArrayReader": 21
                                                }],
                                                23: [function (e, t, r) {

                                                  r.LOCAL_FILE_HEADER = "PK", r.CENTRAL_FILE_HEADER = "PK", r.CENTRAL_DIRECTORY_END = "PK", r.ZIP64_CENTRAL_DIRECTORY_LOCATOR = "PK", r.ZIP64_CENTRAL_DIRECTORY_END = "PK", r.DATA_DESCRIPTOR = "PK\b";
                                                }, {}],
                                                24: [function (e, t, r) {

                                                  var n = e("./GenericWorker"),
                                                      i = e("../utils");

                                                  function s(e) {
                                                    n.call(this, "ConvertWorker to " + e), this.destType = e;
                                                  }

                                                  i.inherits(s, n), s.prototype.processChunk = function (e) {
                                                    this.push({
                                                      data: i.transformTo(this.destType, e.data),
                                                      meta: e.meta
                                                    });
                                                  }, t.exports = s;
                                                }, {
                                                  "../utils": 32,
                                                  "./GenericWorker": 28
                                                }],
                                                25: [function (e, t, r) {

                                                  var n = e("./GenericWorker"),
                                                      i = e("../crc32");

                                                  function s() {
                                                    n.call(this, "Crc32Probe"), this.withStreamInfo("crc32", 0);
                                                  }

                                                  e("../utils").inherits(s, n), s.prototype.processChunk = function (e) {
                                                    this.streamInfo.crc32 = i(e.data, this.streamInfo.crc32 || 0), this.push(e);
                                                  }, t.exports = s;
                                                }, {
                                                  "../crc32": 4,
                                                  "../utils": 32,
                                                  "./GenericWorker": 28
                                                }],
                                                26: [function (e, t, r) {

                                                  var n = e("../utils"),
                                                      i = e("./GenericWorker");

                                                  function s(e) {
                                                    i.call(this, "DataLengthProbe for " + e), this.propName = e, this.withStreamInfo(e, 0);
                                                  }

                                                  n.inherits(s, i), s.prototype.processChunk = function (e) {
                                                    if (e) {
                                                      var t = this.streamInfo[this.propName] || 0;
                                                      this.streamInfo[this.propName] = t + e.data.length;
                                                    }

                                                    i.prototype.processChunk.call(this, e);
                                                  }, t.exports = s;
                                                }, {
                                                  "../utils": 32,
                                                  "./GenericWorker": 28
                                                }],
                                                27: [function (e, t, r) {

                                                  var n = e("../utils"),
                                                      i = e("./GenericWorker");

                                                  function s(e) {
                                                    i.call(this, "DataWorker");
                                                    var t = this;
                                                    this.dataIsReady = !1, this.index = 0, this.max = 0, this.data = null, this.type = "", this._tickScheduled = !1, e.then(function (e) {
                                                      t.dataIsReady = !0, t.data = e, t.max = e && e.length || 0, t.type = n.getTypeOf(e), t.isPaused || t._tickAndRepeat();
                                                    }, function (e) {
                                                      t.error(e);
                                                    });
                                                  }

                                                  n.inherits(s, i), s.prototype.cleanUp = function () {
                                                    i.prototype.cleanUp.call(this), this.data = null;
                                                  }, s.prototype.resume = function () {
                                                    return !!i.prototype.resume.call(this) && (!this._tickScheduled && this.dataIsReady && (this._tickScheduled = !0, n.delay(this._tickAndRepeat, [], this)), !0);
                                                  }, s.prototype._tickAndRepeat = function () {
                                                    this._tickScheduled = !1, this.isPaused || this.isFinished || (this._tick(), this.isFinished || (n.delay(this._tickAndRepeat, [], this), this._tickScheduled = !0));
                                                  }, s.prototype._tick = function () {
                                                    if (this.isPaused || this.isFinished) return !1;
                                                    var e = null,
                                                        t = Math.min(this.max, this.index + 16384);
                                                    if (this.index >= this.max) return this.end();

                                                    switch (this.type) {
                                                      case "string":
                                                        e = this.data.substring(this.index, t);
                                                        break;

                                                      case "uint8array":
                                                        e = this.data.subarray(this.index, t);
                                                        break;

                                                      case "array":
                                                      case "nodebuffer":
                                                        e = this.data.slice(this.index, t);
                                                    }

                                                    return this.index = t, this.push({
                                                      data: e,
                                                      meta: {
                                                        percent: this.max ? this.index / this.max * 100 : 0
                                                      }
                                                    });
                                                  }, t.exports = s;
                                                }, {
                                                  "../utils": 32,
                                                  "./GenericWorker": 28
                                                }],
                                                28: [function (e, t, r) {

                                                  function n(e) {
                                                    this.name = e || "default", this.streamInfo = {}, this.generatedError = null, this.extraStreamInfo = {}, this.isPaused = !0, this.isFinished = !1, this.isLocked = !1, this._listeners = {
                                                      data: [],
                                                      end: [],
                                                      error: []
                                                    }, this.previous = null;
                                                  }

                                                  n.prototype = {
                                                    push: function (e) {
                                                      this.emit("data", e);
                                                    },
                                                    end: function () {
                                                      if (this.isFinished) return !1;
                                                      this.flush();

                                                      try {
                                                        this.emit("end"), this.cleanUp(), this.isFinished = !0;
                                                      } catch (e) {
                                                        this.emit("error", e);
                                                      }

                                                      return !0;
                                                    },
                                                    error: function (e) {
                                                      return !this.isFinished && (this.isPaused ? this.generatedError = e : (this.isFinished = !0, this.emit("error", e), this.previous && this.previous.error(e), this.cleanUp()), !0);
                                                    },
                                                    on: function (e, t) {
                                                      return this._listeners[e].push(t), this;
                                                    },
                                                    cleanUp: function () {
                                                      this.streamInfo = this.generatedError = this.extraStreamInfo = null, this._listeners = [];
                                                    },
                                                    emit: function (e, t) {
                                                      if (this._listeners[e]) for (var r = 0; r < this._listeners[e].length; r++) this._listeners[e][r].call(this, t);
                                                    },
                                                    pipe: function (e) {
                                                      return e.registerPrevious(this);
                                                    },
                                                    registerPrevious: function (e) {
                                                      if (this.isLocked) throw new Error("The stream '" + this + "' has already been used.");
                                                      this.streamInfo = e.streamInfo, this.mergeStreamInfo(), this.previous = e;
                                                      var t = this;
                                                      return e.on("data", function (e) {
                                                        t.processChunk(e);
                                                      }), e.on("end", function () {
                                                        t.end();
                                                      }), e.on("error", function (e) {
                                                        t.error(e);
                                                      }), this;
                                                    },
                                                    pause: function () {
                                                      return !this.isPaused && !this.isFinished && (this.isPaused = !0, this.previous && this.previous.pause(), !0);
                                                    },
                                                    resume: function () {
                                                      if (!this.isPaused || this.isFinished) return !1;
                                                      var e = this.isPaused = !1;
                                                      return this.generatedError && (this.error(this.generatedError), e = !0), this.previous && this.previous.resume(), !e;
                                                    },
                                                    flush: function () {},
                                                    processChunk: function (e) {
                                                      this.push(e);
                                                    },
                                                    withStreamInfo: function (e, t) {
                                                      return this.extraStreamInfo[e] = t, this.mergeStreamInfo(), this;
                                                    },
                                                    mergeStreamInfo: function () {
                                                      for (var e in this.extraStreamInfo) this.extraStreamInfo.hasOwnProperty(e) && (this.streamInfo[e] = this.extraStreamInfo[e]);
                                                    },
                                                    lock: function () {
                                                      if (this.isLocked) throw new Error("The stream '" + this + "' has already been used.");
                                                      this.isLocked = !0, this.previous && this.previous.lock();
                                                    },
                                                    toString: function () {
                                                      var e = "Worker " + this.name;
                                                      return this.previous ? this.previous + " -> " + e : e;
                                                    }
                                                  }, t.exports = n;
                                                }, {}],
                                                29: [function (e, t, r) {

                                                  var h = e("../utils"),
                                                      i = e("./ConvertWorker"),
                                                      s = e("./GenericWorker"),
                                                      f = e("../base64"),
                                                      n = e("../support"),
                                                      a = e("../external"),
                                                      o = null;
                                                  if (n.nodestream) try {
                                                    o = e("../nodejs/NodejsStreamOutputAdapter");
                                                  } catch (e) {}

                                                  function u(e, t, r) {
                                                    var n = t;

                                                    switch (t) {
                                                      case "blob":
                                                      case "arraybuffer":
                                                        n = "uint8array";
                                                        break;

                                                      case "base64":
                                                        n = "string";
                                                    }

                                                    try {
                                                      this._internalType = n, this._outputType = t, this._mimeType = r, h.checkSupport(n), this._worker = e.pipe(new i(n)), e.lock();
                                                    } catch (e) {
                                                      this._worker = new s("error"), this._worker.error(e);
                                                    }
                                                  }

                                                  u.prototype = {
                                                    accumulate: function (e) {
                                                      return o = this, u = e, new a.Promise(function (t, r) {
                                                        var n = [],
                                                            i = o._internalType,
                                                            s = o._outputType,
                                                            a = o._mimeType;
                                                        o.on("data", function (e, t) {
                                                          n.push(e), u && u(t);
                                                        }).on("error", function (e) {
                                                          n = [], r(e);
                                                        }).on("end", function () {
                                                          try {
                                                            var e = function (e, t, r) {
                                                              switch (e) {
                                                                case "blob":
                                                                  return h.newBlob(h.transformTo("arraybuffer", t), r);

                                                                case "base64":
                                                                  return f.encode(t);

                                                                default:
                                                                  return h.transformTo(e, t);
                                                              }
                                                            }(s, function (e, t) {
                                                              var r,
                                                                  n = 0,
                                                                  i = null,
                                                                  s = 0;

                                                              for (r = 0; r < t.length; r++) s += t[r].length;

                                                              switch (e) {
                                                                case "string":
                                                                  return t.join("");

                                                                case "array":
                                                                  return Array.prototype.concat.apply([], t);

                                                                case "uint8array":
                                                                  for (i = new Uint8Array(s), r = 0; r < t.length; r++) i.set(t[r], n), n += t[r].length;

                                                                  return i;

                                                                case "nodebuffer":
                                                                  return Buffer.concat(t);

                                                                default:
                                                                  throw new Error("concat : unsupported type '" + e + "'");
                                                              }
                                                            }(i, n), a);

                                                            t(e);
                                                          } catch (e) {
                                                            r(e);
                                                          }

                                                          n = [];
                                                        }).resume();
                                                      });
                                                      var o, u;
                                                    },
                                                    on: function (e, t) {
                                                      var r = this;
                                                      return "data" === e ? this._worker.on(e, function (e) {
                                                        t.call(r, e.data, e.meta);
                                                      }) : this._worker.on(e, function () {
                                                        h.delay(t, arguments, r);
                                                      }), this;
                                                    },
                                                    resume: function () {
                                                      return h.delay(this._worker.resume, [], this._worker), this;
                                                    },
                                                    pause: function () {
                                                      return this._worker.pause(), this;
                                                    },
                                                    toNodejsStream: function (e) {
                                                      if (h.checkSupport("nodestream"), "nodebuffer" !== this._outputType) throw new Error(this._outputType + " is not supported by this method");
                                                      return new o(this, {
                                                        objectMode: "nodebuffer" !== this._outputType
                                                      }, e);
                                                    }
                                                  }, t.exports = u;
                                                }, {
                                                  "../base64": 1,
                                                  "../external": 6,
                                                  "../nodejs/NodejsStreamOutputAdapter": 13,
                                                  "../support": 30,
                                                  "../utils": 32,
                                                  "./ConvertWorker": 24,
                                                  "./GenericWorker": 28
                                                }],
                                                30: [function (e, t, r) {

                                                  if (r.base64 = !0, r.array = !0, r.string = !0, r.arraybuffer = "undefined" != typeof ArrayBuffer && "undefined" != typeof Uint8Array, r.nodebuffer = "undefined" != typeof Buffer, r.uint8array = "undefined" != typeof Uint8Array, "undefined" == typeof ArrayBuffer) r.blob = !1;else {
                                                    var n = new ArrayBuffer(0);

                                                    try {
                                                      r.blob = 0 === new Blob([n], {
                                                        type: "application/zip"
                                                      }).size;
                                                    } catch (e) {
                                                      try {
                                                        var i = new (self.BlobBuilder || self.WebKitBlobBuilder || self.MozBlobBuilder || self.MSBlobBuilder)();
                                                        i.append(n), r.blob = 0 === i.getBlob("application/zip").size;
                                                      } catch (e) {
                                                        r.blob = !1;
                                                      }
                                                    }
                                                  }

                                                  try {
                                                    r.nodestream = !!e("readable-stream").Readable;
                                                  } catch (e) {
                                                    r.nodestream = !1;
                                                  }
                                                }, {
                                                  "readable-stream": 16
                                                }],
                                                31: [function (e, t, s) {

                                                  for (var o = e("./utils"), u = e("./support"), r = e("./nodejsUtils"), n = e("./stream/GenericWorker"), h = new Array(256), i = 0; i < 256; i++) h[i] = 252 <= i ? 6 : 248 <= i ? 5 : 240 <= i ? 4 : 224 <= i ? 3 : 192 <= i ? 2 : 1;

                                                  function a() {
                                                    n.call(this, "utf-8 decode"), this.leftOver = null;
                                                  }

                                                  function f() {
                                                    n.call(this, "utf-8 encode");
                                                  }

                                                  h[254] = h[254] = 1, s.utf8encode = function (e) {
                                                    return u.nodebuffer ? r.newBufferFrom(e, "utf-8") : function (e) {
                                                      var t,
                                                          r,
                                                          n,
                                                          i,
                                                          s,
                                                          a = e.length,
                                                          o = 0;

                                                      for (i = 0; i < a; i++) 55296 == (64512 & (r = e.charCodeAt(i))) && i + 1 < a && 56320 == (64512 & (n = e.charCodeAt(i + 1))) && (r = 65536 + (r - 55296 << 10) + (n - 56320), i++), o += r < 128 ? 1 : r < 2048 ? 2 : r < 65536 ? 3 : 4;

                                                      for (t = u.uint8array ? new Uint8Array(o) : new Array(o), i = s = 0; s < o; i++) 55296 == (64512 & (r = e.charCodeAt(i))) && i + 1 < a && 56320 == (64512 & (n = e.charCodeAt(i + 1))) && (r = 65536 + (r - 55296 << 10) + (n - 56320), i++), r < 128 ? t[s++] = r : (r < 2048 ? t[s++] = 192 | r >>> 6 : (r < 65536 ? t[s++] = 224 | r >>> 12 : (t[s++] = 240 | r >>> 18, t[s++] = 128 | r >>> 12 & 63), t[s++] = 128 | r >>> 6 & 63), t[s++] = 128 | 63 & r);

                                                      return t;
                                                    }(e);
                                                  }, s.utf8decode = function (e) {
                                                    return u.nodebuffer ? o.transformTo("nodebuffer", e).toString("utf-8") : function (e) {
                                                      var t,
                                                          r,
                                                          n,
                                                          i,
                                                          s = e.length,
                                                          a = new Array(2 * s);

                                                      for (t = r = 0; t < s;) if ((n = e[t++]) < 128) a[r++] = n;else if (4 < (i = h[n])) a[r++] = 65533, t += i - 1;else {
                                                        for (n &= 2 === i ? 31 : 3 === i ? 15 : 7; 1 < i && t < s;) n = n << 6 | 63 & e[t++], i--;

                                                        1 < i ? a[r++] = 65533 : n < 65536 ? a[r++] = n : (n -= 65536, a[r++] = 55296 | n >> 10 & 1023, a[r++] = 56320 | 1023 & n);
                                                      }

                                                      return a.length !== r && (a.subarray ? a = a.subarray(0, r) : a.length = r), o.applyFromCharCode(a);
                                                    }(e = o.transformTo(u.uint8array ? "uint8array" : "array", e));
                                                  }, o.inherits(a, n), a.prototype.processChunk = function (e) {
                                                    var t = o.transformTo(u.uint8array ? "uint8array" : "array", e.data);

                                                    if (this.leftOver && this.leftOver.length) {
                                                      if (u.uint8array) {
                                                        var r = t;
                                                        (t = new Uint8Array(r.length + this.leftOver.length)).set(this.leftOver, 0), t.set(r, this.leftOver.length);
                                                      } else t = this.leftOver.concat(t);

                                                      this.leftOver = null;
                                                    }

                                                    var n = function (e, t) {
                                                      var r;

                                                      for ((t = t || e.length) > e.length && (t = e.length), r = t - 1; 0 <= r && 128 == (192 & e[r]);) r--;

                                                      return r < 0 ? t : 0 === r ? t : r + h[e[r]] > t ? r : t;
                                                    }(t),
                                                        i = t;

                                                    n !== t.length && (u.uint8array ? (i = t.subarray(0, n), this.leftOver = t.subarray(n, t.length)) : (i = t.slice(0, n), this.leftOver = t.slice(n, t.length))), this.push({
                                                      data: s.utf8decode(i),
                                                      meta: e.meta
                                                    });
                                                  }, a.prototype.flush = function () {
                                                    this.leftOver && this.leftOver.length && (this.push({
                                                      data: s.utf8decode(this.leftOver),
                                                      meta: {}
                                                    }), this.leftOver = null);
                                                  }, s.Utf8DecodeWorker = a, o.inherits(f, n), f.prototype.processChunk = function (e) {
                                                    this.push({
                                                      data: s.utf8encode(e.data),
                                                      meta: e.meta
                                                    });
                                                  }, s.Utf8EncodeWorker = f;
                                                }, {
                                                  "./nodejsUtils": 14,
                                                  "./stream/GenericWorker": 28,
                                                  "./support": 30,
                                                  "./utils": 32
                                                }],
                                                32: [function (e, t, o) {

                                                  var u = e("./support"),
                                                      h = e("./base64"),
                                                      r = e("./nodejsUtils"),
                                                      n = e("set-immediate-shim"),
                                                      f = e("./external");

                                                  function i(e) {
                                                    return e;
                                                  }

                                                  function l(e, t) {
                                                    for (var r = 0; r < e.length; ++r) t[r] = 255 & e.charCodeAt(r);

                                                    return t;
                                                  }

                                                  o.newBlob = function (t, r) {
                                                    o.checkSupport("blob");

                                                    try {
                                                      return new Blob([t], {
                                                        type: r
                                                      });
                                                    } catch (e) {
                                                      try {
                                                        var n = new (self.BlobBuilder || self.WebKitBlobBuilder || self.MozBlobBuilder || self.MSBlobBuilder)();
                                                        return n.append(t), n.getBlob(r);
                                                      } catch (e) {
                                                        throw new Error("Bug : can't construct the Blob.");
                                                      }
                                                    }
                                                  };

                                                  var s = {
                                                    stringifyByChunk: function (e, t, r) {
                                                      var n = [],
                                                          i = 0,
                                                          s = e.length;
                                                      if (s <= r) return String.fromCharCode.apply(null, e);

                                                      for (; i < s;) "array" === t || "nodebuffer" === t ? n.push(String.fromCharCode.apply(null, e.slice(i, Math.min(i + r, s)))) : n.push(String.fromCharCode.apply(null, e.subarray(i, Math.min(i + r, s)))), i += r;

                                                      return n.join("");
                                                    },
                                                    stringifyByChar: function (e) {
                                                      for (var t = "", r = 0; r < e.length; r++) t += String.fromCharCode(e[r]);

                                                      return t;
                                                    },
                                                    applyCanBeUsed: {
                                                      uint8array: function () {
                                                        try {
                                                          return u.uint8array && 1 === String.fromCharCode.apply(null, new Uint8Array(1)).length;
                                                        } catch (e) {
                                                          return !1;
                                                        }
                                                      }(),
                                                      nodebuffer: function () {
                                                        try {
                                                          return u.nodebuffer && 1 === String.fromCharCode.apply(null, r.allocBuffer(1)).length;
                                                        } catch (e) {
                                                          return !1;
                                                        }
                                                      }()
                                                    }
                                                  };

                                                  function a(e) {
                                                    var t = 65536,
                                                        r = o.getTypeOf(e),
                                                        n = !0;
                                                    if ("uint8array" === r ? n = s.applyCanBeUsed.uint8array : "nodebuffer" === r && (n = s.applyCanBeUsed.nodebuffer), n) for (; 1 < t;) try {
                                                      return s.stringifyByChunk(e, r, t);
                                                    } catch (e) {
                                                      t = Math.floor(t / 2);
                                                    }
                                                    return s.stringifyByChar(e);
                                                  }

                                                  function d(e, t) {
                                                    for (var r = 0; r < e.length; r++) t[r] = e[r];

                                                    return t;
                                                  }

                                                  o.applyFromCharCode = a;
                                                  var c = {};
                                                  c.string = {
                                                    string: i,
                                                    array: function (e) {
                                                      return l(e, new Array(e.length));
                                                    },
                                                    arraybuffer: function (e) {
                                                      return c.string.uint8array(e).buffer;
                                                    },
                                                    uint8array: function (e) {
                                                      return l(e, new Uint8Array(e.length));
                                                    },
                                                    nodebuffer: function (e) {
                                                      return l(e, r.allocBuffer(e.length));
                                                    }
                                                  }, c.array = {
                                                    string: a,
                                                    array: i,
                                                    arraybuffer: function (e) {
                                                      return new Uint8Array(e).buffer;
                                                    },
                                                    uint8array: function (e) {
                                                      return new Uint8Array(e);
                                                    },
                                                    nodebuffer: function (e) {
                                                      return r.newBufferFrom(e);
                                                    }
                                                  }, c.arraybuffer = {
                                                    string: function (e) {
                                                      return a(new Uint8Array(e));
                                                    },
                                                    array: function (e) {
                                                      return d(new Uint8Array(e), new Array(e.byteLength));
                                                    },
                                                    arraybuffer: i,
                                                    uint8array: function (e) {
                                                      return new Uint8Array(e);
                                                    },
                                                    nodebuffer: function (e) {
                                                      return r.newBufferFrom(new Uint8Array(e));
                                                    }
                                                  }, c.uint8array = {
                                                    string: a,
                                                    array: function (e) {
                                                      return d(e, new Array(e.length));
                                                    },
                                                    arraybuffer: function (e) {
                                                      return e.buffer;
                                                    },
                                                    uint8array: i,
                                                    nodebuffer: function (e) {
                                                      return r.newBufferFrom(e);
                                                    }
                                                  }, c.nodebuffer = {
                                                    string: a,
                                                    array: function (e) {
                                                      return d(e, new Array(e.length));
                                                    },
                                                    arraybuffer: function (e) {
                                                      return c.nodebuffer.uint8array(e).buffer;
                                                    },
                                                    uint8array: function (e) {
                                                      return d(e, new Uint8Array(e.length));
                                                    },
                                                    nodebuffer: i
                                                  }, o.transformTo = function (e, t) {
                                                    if (t = t || "", !e) return t;
                                                    o.checkSupport(e);
                                                    var r = o.getTypeOf(t);
                                                    return c[r][e](t);
                                                  }, o.getTypeOf = function (e) {
                                                    return "string" == typeof e ? "string" : "[object Array]" === Object.prototype.toString.call(e) ? "array" : u.nodebuffer && r.isBuffer(e) ? "nodebuffer" : u.uint8array && e instanceof Uint8Array ? "uint8array" : u.arraybuffer && e instanceof ArrayBuffer ? "arraybuffer" : void 0;
                                                  }, o.checkSupport = function (e) {
                                                    if (!u[e.toLowerCase()]) throw new Error(e + " is not supported by this platform");
                                                  }, o.MAX_VALUE_16BITS = 65535, o.MAX_VALUE_32BITS = -1, o.pretty = function (e) {
                                                    var t,
                                                        r,
                                                        n = "";

                                                    for (r = 0; r < (e || "").length; r++) n += "\\x" + ((t = e.charCodeAt(r)) < 16 ? "0" : "") + t.toString(16).toUpperCase();

                                                    return n;
                                                  }, o.delay = function (e, t, r) {
                                                    n(function () {
                                                      e.apply(r || null, t || []);
                                                    });
                                                  }, o.inherits = function (e, t) {
                                                    function r() {}

                                                    r.prototype = t.prototype, e.prototype = new r();
                                                  }, o.extend = function () {
                                                    var e,
                                                        t,
                                                        r = {};

                                                    for (e = 0; e < arguments.length; e++) for (t in arguments[e]) arguments[e].hasOwnProperty(t) && void 0 === r[t] && (r[t] = arguments[e][t]);

                                                    return r;
                                                  }, o.prepareContent = function (n, e, i, s, a) {
                                                    return f.Promise.resolve(e).then(function (n) {
                                                      return u.blob && (n instanceof Blob || -1 !== ["[object File]", "[object Blob]"].indexOf(Object.prototype.toString.call(n))) && "undefined" != typeof FileReader ? new f.Promise(function (t, r) {
                                                        var e = new FileReader();
                                                        e.onload = function (e) {
                                                          t(e.target.result);
                                                        }, e.onerror = function (e) {
                                                          r(e.target.error);
                                                        }, e.readAsArrayBuffer(n);
                                                      }) : n;
                                                    }).then(function (e) {
                                                      var t,
                                                          r = o.getTypeOf(e);
                                                      return r ? ("arraybuffer" === r ? e = o.transformTo("uint8array", e) : "string" === r && (a ? e = h.decode(e) : i && !0 !== s && (e = l(t = e, u.uint8array ? new Uint8Array(t.length) : new Array(t.length)))), e) : f.Promise.reject(new Error("Can't read the data of '" + n + "'. Is it in a supported JavaScript type (String, Blob, ArrayBuffer, etc) ?"));
                                                    });
                                                  };
                                                }, {
                                                  "./base64": 1,
                                                  "./external": 6,
                                                  "./nodejsUtils": 14,
                                                  "./support": 30,
                                                  "set-immediate-shim": 54
                                                }],
                                                33: [function (e, t, r) {

                                                  var n = e("./reader/readerFor"),
                                                      i = e("./utils"),
                                                      s = e("./signature"),
                                                      a = e("./zipEntry"),
                                                      o = (e("./utf8"), e("./support"));

                                                  function u(e) {
                                                    this.files = [], this.loadOptions = e;
                                                  }

                                                  u.prototype = {
                                                    checkSignature: function (e) {
                                                      if (!this.reader.readAndCheckSignature(e)) {
                                                        this.reader.index -= 4;
                                                        var t = this.reader.readString(4);
                                                        throw new Error("Corrupted zip or bug: unexpected signature (" + i.pretty(t) + ", expected " + i.pretty(e) + ")");
                                                      }
                                                    },
                                                    isSignature: function (e, t) {
                                                      var r = this.reader.index;
                                                      this.reader.setIndex(e);
                                                      var n = this.reader.readString(4) === t;
                                                      return this.reader.setIndex(r), n;
                                                    },
                                                    readBlockEndOfCentral: function () {
                                                      this.diskNumber = this.reader.readInt(2), this.diskWithCentralDirStart = this.reader.readInt(2), this.centralDirRecordsOnThisDisk = this.reader.readInt(2), this.centralDirRecords = this.reader.readInt(2), this.centralDirSize = this.reader.readInt(4), this.centralDirOffset = this.reader.readInt(4), this.zipCommentLength = this.reader.readInt(2);
                                                      var e = this.reader.readData(this.zipCommentLength),
                                                          t = o.uint8array ? "uint8array" : "array",
                                                          r = i.transformTo(t, e);
                                                      this.zipComment = this.loadOptions.decodeFileName(r);
                                                    },
                                                    readBlockZip64EndOfCentral: function () {
                                                      this.zip64EndOfCentralSize = this.reader.readInt(8), this.reader.skip(4), this.diskNumber = this.reader.readInt(4), this.diskWithCentralDirStart = this.reader.readInt(4), this.centralDirRecordsOnThisDisk = this.reader.readInt(8), this.centralDirRecords = this.reader.readInt(8), this.centralDirSize = this.reader.readInt(8), this.centralDirOffset = this.reader.readInt(8), this.zip64ExtensibleData = {};

                                                      for (var e, t, r, n = this.zip64EndOfCentralSize - 44; 0 < n;) e = this.reader.readInt(2), t = this.reader.readInt(4), r = this.reader.readData(t), this.zip64ExtensibleData[e] = {
                                                        id: e,
                                                        length: t,
                                                        value: r
                                                      };
                                                    },
                                                    readBlockZip64EndOfCentralLocator: function () {
                                                      if (this.diskWithZip64CentralDirStart = this.reader.readInt(4), this.relativeOffsetEndOfZip64CentralDir = this.reader.readInt(8), this.disksCount = this.reader.readInt(4), 1 < this.disksCount) throw new Error("Multi-volumes zip are not supported");
                                                    },
                                                    readLocalFiles: function () {
                                                      var e, t;

                                                      for (e = 0; e < this.files.length; e++) t = this.files[e], this.reader.setIndex(t.localHeaderOffset), this.checkSignature(s.LOCAL_FILE_HEADER), t.readLocalPart(this.reader), t.handleUTF8(), t.processAttributes();
                                                    },
                                                    readCentralDir: function () {
                                                      var e;

                                                      for (this.reader.setIndex(this.centralDirOffset); this.reader.readAndCheckSignature(s.CENTRAL_FILE_HEADER);) (e = new a({
                                                        zip64: this.zip64
                                                      }, this.loadOptions)).readCentralPart(this.reader), this.files.push(e);

                                                      if (this.centralDirRecords !== this.files.length && 0 !== this.centralDirRecords && 0 === this.files.length) throw new Error("Corrupted zip or bug: expected " + this.centralDirRecords + " records in central dir, got " + this.files.length);
                                                    },
                                                    readEndOfCentral: function () {
                                                      var e = this.reader.lastIndexOfSignature(s.CENTRAL_DIRECTORY_END);
                                                      if (e < 0) throw this.isSignature(0, s.LOCAL_FILE_HEADER) ? new Error("Corrupted zip: can't find end of central directory") : new Error("Can't find end of central directory : is this a zip file ? If it is, see https://stuk.github.io/jszip/documentation/howto/read_zip.html");
                                                      this.reader.setIndex(e);
                                                      var t = e;

                                                      if (this.checkSignature(s.CENTRAL_DIRECTORY_END), this.readBlockEndOfCentral(), this.diskNumber === i.MAX_VALUE_16BITS || this.diskWithCentralDirStart === i.MAX_VALUE_16BITS || this.centralDirRecordsOnThisDisk === i.MAX_VALUE_16BITS || this.centralDirRecords === i.MAX_VALUE_16BITS || this.centralDirSize === i.MAX_VALUE_32BITS || this.centralDirOffset === i.MAX_VALUE_32BITS) {
                                                        if (this.zip64 = !0, (e = this.reader.lastIndexOfSignature(s.ZIP64_CENTRAL_DIRECTORY_LOCATOR)) < 0) throw new Error("Corrupted zip: can't find the ZIP64 end of central directory locator");
                                                        if (this.reader.setIndex(e), this.checkSignature(s.ZIP64_CENTRAL_DIRECTORY_LOCATOR), this.readBlockZip64EndOfCentralLocator(), !this.isSignature(this.relativeOffsetEndOfZip64CentralDir, s.ZIP64_CENTRAL_DIRECTORY_END) && (this.relativeOffsetEndOfZip64CentralDir = this.reader.lastIndexOfSignature(s.ZIP64_CENTRAL_DIRECTORY_END), this.relativeOffsetEndOfZip64CentralDir < 0)) throw new Error("Corrupted zip: can't find the ZIP64 end of central directory");
                                                        this.reader.setIndex(this.relativeOffsetEndOfZip64CentralDir), this.checkSignature(s.ZIP64_CENTRAL_DIRECTORY_END), this.readBlockZip64EndOfCentral();
                                                      }

                                                      var r = this.centralDirOffset + this.centralDirSize;
                                                      this.zip64 && (r += 20, r += 12 + this.zip64EndOfCentralSize);
                                                      var n = t - r;
                                                      if (0 < n) this.isSignature(t, s.CENTRAL_FILE_HEADER) || (this.reader.zero = n);else if (n < 0) throw new Error("Corrupted zip: missing " + Math.abs(n) + " bytes.");
                                                    },
                                                    prepareReader: function (e) {
                                                      this.reader = n(e);
                                                    },
                                                    load: function (e) {
                                                      this.prepareReader(e), this.readEndOfCentral(), this.readCentralDir(), this.readLocalFiles();
                                                    }
                                                  }, t.exports = u;
                                                }, {
                                                  "./reader/readerFor": 22,
                                                  "./signature": 23,
                                                  "./support": 30,
                                                  "./utf8": 31,
                                                  "./utils": 32,
                                                  "./zipEntry": 34
                                                }],
                                                34: [function (e, t, r) {

                                                  var n = e("./reader/readerFor"),
                                                      s = e("./utils"),
                                                      i = e("./compressedObject"),
                                                      a = e("./crc32"),
                                                      o = e("./utf8"),
                                                      u = e("./compressions"),
                                                      h = e("./support");

                                                  function f(e, t) {
                                                    this.options = e, this.loadOptions = t;
                                                  }

                                                  f.prototype = {
                                                    isEncrypted: function () {
                                                      return 1 == (1 & this.bitFlag);
                                                    },
                                                    useUTF8: function () {
                                                      return 2048 == (2048 & this.bitFlag);
                                                    },
                                                    readLocalPart: function (e) {
                                                      var t, r;
                                                      if (e.skip(22), this.fileNameLength = e.readInt(2), r = e.readInt(2), this.fileName = e.readData(this.fileNameLength), e.skip(r), -1 === this.compressedSize || -1 === this.uncompressedSize) throw new Error("Bug or corrupted zip : didn't get enough information from the central directory (compressedSize === -1 || uncompressedSize === -1)");
                                                      if (null === (t = function (e) {
                                                        for (var t in u) if (u.hasOwnProperty(t) && u[t].magic === e) return u[t];

                                                        return null;
                                                      }(this.compressionMethod))) throw new Error("Corrupted zip : compression " + s.pretty(this.compressionMethod) + " unknown (inner file : " + s.transformTo("string", this.fileName) + ")");
                                                      this.decompressed = new i(this.compressedSize, this.uncompressedSize, this.crc32, t, e.readData(this.compressedSize));
                                                    },
                                                    readCentralPart: function (e) {
                                                      this.versionMadeBy = e.readInt(2), e.skip(2), this.bitFlag = e.readInt(2), this.compressionMethod = e.readString(2), this.date = e.readDate(), this.crc32 = e.readInt(4), this.compressedSize = e.readInt(4), this.uncompressedSize = e.readInt(4);
                                                      var t = e.readInt(2);
                                                      if (this.extraFieldsLength = e.readInt(2), this.fileCommentLength = e.readInt(2), this.diskNumberStart = e.readInt(2), this.internalFileAttributes = e.readInt(2), this.externalFileAttributes = e.readInt(4), this.localHeaderOffset = e.readInt(4), this.isEncrypted()) throw new Error("Encrypted zip are not supported");
                                                      e.skip(t), this.readExtraFields(e), this.parseZIP64ExtraField(e), this.fileComment = e.readData(this.fileCommentLength);
                                                    },
                                                    processAttributes: function () {
                                                      this.unixPermissions = null, this.dosPermissions = null;
                                                      var e = this.versionMadeBy >> 8;
                                                      this.dir = !!(16 & this.externalFileAttributes), 0 == e && (this.dosPermissions = 63 & this.externalFileAttributes), 3 == e && (this.unixPermissions = this.externalFileAttributes >> 16 & 65535), this.dir || "/" !== this.fileNameStr.slice(-1) || (this.dir = !0);
                                                    },
                                                    parseZIP64ExtraField: function (e) {
                                                      if (this.extraFields[1]) {
                                                        var t = n(this.extraFields[1].value);
                                                        this.uncompressedSize === s.MAX_VALUE_32BITS && (this.uncompressedSize = t.readInt(8)), this.compressedSize === s.MAX_VALUE_32BITS && (this.compressedSize = t.readInt(8)), this.localHeaderOffset === s.MAX_VALUE_32BITS && (this.localHeaderOffset = t.readInt(8)), this.diskNumberStart === s.MAX_VALUE_32BITS && (this.diskNumberStart = t.readInt(4));
                                                      }
                                                    },
                                                    readExtraFields: function (e) {
                                                      var t,
                                                          r,
                                                          n,
                                                          i = e.index + this.extraFieldsLength;

                                                      for (this.extraFields || (this.extraFields = {}); e.index + 4 < i;) t = e.readInt(2), r = e.readInt(2), n = e.readData(r), this.extraFields[t] = {
                                                        id: t,
                                                        length: r,
                                                        value: n
                                                      };

                                                      e.setIndex(i);
                                                    },
                                                    handleUTF8: function () {
                                                      var e = h.uint8array ? "uint8array" : "array";
                                                      if (this.useUTF8()) this.fileNameStr = o.utf8decode(this.fileName), this.fileCommentStr = o.utf8decode(this.fileComment);else {
                                                        var t = this.findExtraFieldUnicodePath();
                                                        if (null !== t) this.fileNameStr = t;else {
                                                          var r = s.transformTo(e, this.fileName);
                                                          this.fileNameStr = this.loadOptions.decodeFileName(r);
                                                        }
                                                        var n = this.findExtraFieldUnicodeComment();
                                                        if (null !== n) this.fileCommentStr = n;else {
                                                          var i = s.transformTo(e, this.fileComment);
                                                          this.fileCommentStr = this.loadOptions.decodeFileName(i);
                                                        }
                                                      }
                                                    },
                                                    findExtraFieldUnicodePath: function () {
                                                      var e = this.extraFields[28789];

                                                      if (e) {
                                                        var t = n(e.value);
                                                        return 1 !== t.readInt(1) ? null : a(this.fileName) !== t.readInt(4) ? null : o.utf8decode(t.readData(e.length - 5));
                                                      }

                                                      return null;
                                                    },
                                                    findExtraFieldUnicodeComment: function () {
                                                      var e = this.extraFields[25461];

                                                      if (e) {
                                                        var t = n(e.value);
                                                        return 1 !== t.readInt(1) ? null : a(this.fileComment) !== t.readInt(4) ? null : o.utf8decode(t.readData(e.length - 5));
                                                      }

                                                      return null;
                                                    }
                                                  }, t.exports = f;
                                                }, {
                                                  "./compressedObject": 2,
                                                  "./compressions": 3,
                                                  "./crc32": 4,
                                                  "./reader/readerFor": 22,
                                                  "./support": 30,
                                                  "./utf8": 31,
                                                  "./utils": 32
                                                }],
                                                35: [function (e, t, r) {

                                                  function n(e, t, r) {
                                                    this.name = e, this.dir = r.dir, this.date = r.date, this.comment = r.comment, this.unixPermissions = r.unixPermissions, this.dosPermissions = r.dosPermissions, this._data = t, this._dataBinary = r.binary, this.options = {
                                                      compression: r.compression,
                                                      compressionOptions: r.compressionOptions
                                                    };
                                                  }

                                                  var s = e("./stream/StreamHelper"),
                                                      i = e("./stream/DataWorker"),
                                                      a = e("./utf8"),
                                                      o = e("./compressedObject"),
                                                      u = e("./stream/GenericWorker");
                                                  n.prototype = {
                                                    internalStream: function (e) {
                                                      var t = null,
                                                          r = "string";

                                                      try {
                                                        if (!e) throw new Error("No output type specified.");
                                                        var n = "string" === (r = e.toLowerCase()) || "text" === r;
                                                        "binarystring" !== r && "text" !== r || (r = "string"), t = this._decompressWorker();
                                                        var i = !this._dataBinary;
                                                        i && !n && (t = t.pipe(new a.Utf8EncodeWorker())), !i && n && (t = t.pipe(new a.Utf8DecodeWorker()));
                                                      } catch (e) {
                                                        (t = new u("error")).error(e);
                                                      }

                                                      return new s(t, r, "");
                                                    },
                                                    async: function (e, t) {
                                                      return this.internalStream(e).accumulate(t);
                                                    },
                                                    nodeStream: function (e, t) {
                                                      return this.internalStream(e || "nodebuffer").toNodejsStream(t);
                                                    },
                                                    _compressWorker: function (e, t) {
                                                      if (this._data instanceof o && this._data.compression.magic === e.magic) return this._data.getCompressedWorker();

                                                      var r = this._decompressWorker();

                                                      return this._dataBinary || (r = r.pipe(new a.Utf8EncodeWorker())), o.createWorkerFrom(r, e, t);
                                                    },
                                                    _decompressWorker: function () {
                                                      return this._data instanceof o ? this._data.getContentWorker() : this._data instanceof u ? this._data : new i(this._data);
                                                    }
                                                  };

                                                  for (var h = ["asText", "asBinary", "asNodeBuffer", "asUint8Array", "asArrayBuffer"], f = function () {
                                                    throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.");
                                                  }, l = 0; l < h.length; l++) n.prototype[h[l]] = f;

                                                  t.exports = n;
                                                }, {
                                                  "./compressedObject": 2,
                                                  "./stream/DataWorker": 27,
                                                  "./stream/GenericWorker": 28,
                                                  "./stream/StreamHelper": 29,
                                                  "./utf8": 31
                                                }],
                                                36: [function (e, f, t) {
                                                  (function (t) {

                                                    var r,
                                                        n,
                                                        e = t.MutationObserver || t.WebKitMutationObserver;

                                                    if (e) {
                                                      var i = 0,
                                                          s = new e(h),
                                                          a = t.document.createTextNode("");
                                                      s.observe(a, {
                                                        characterData: !0
                                                      }), r = function () {
                                                        a.data = i = ++i % 2;
                                                      };
                                                    } else if (t.setImmediate || void 0 === t.MessageChannel) r = "document" in t && "onreadystatechange" in t.document.createElement("script") ? function () {
                                                      var e = t.document.createElement("script");
                                                      e.onreadystatechange = function () {
                                                        h(), e.onreadystatechange = null, e.parentNode.removeChild(e), e = null;
                                                      }, t.document.documentElement.appendChild(e);
                                                    } : function () {
                                                      setTimeout(h, 0);
                                                    };else {
                                                      var o = new t.MessageChannel();
                                                      o.port1.onmessage = h, r = function () {
                                                        o.port2.postMessage(0);
                                                      };
                                                    }

                                                    var u = [];

                                                    function h() {
                                                      var e, t;
                                                      n = !0;

                                                      for (var r = u.length; r;) {
                                                        for (t = u, u = [], e = -1; ++e < r;) t[e]();

                                                        r = u.length;
                                                      }

                                                      n = !1;
                                                    }

                                                    f.exports = function (e) {
                                                      1 !== u.push(e) || n || r();
                                                    };
                                                  }).call(this, void 0 !== r ? r : "undefined" != typeof self ? self : "undefined" != typeof window ? window : {});
                                                }, {}],
                                                37: [function (e, t, r) {

                                                  var i = e("immediate");

                                                  function h() {}

                                                  var f = {},
                                                      s = ["REJECTED"],
                                                      a = ["FULFILLED"],
                                                      n = ["PENDING"];

                                                  function o(e) {
                                                    if ("function" != typeof e) throw new TypeError("resolver must be a function");
                                                    this.state = n, this.queue = [], this.outcome = void 0, e !== h && c(this, e);
                                                  }

                                                  function u(e, t, r) {
                                                    this.promise = e, "function" == typeof t && (this.onFulfilled = t, this.callFulfilled = this.otherCallFulfilled), "function" == typeof r && (this.onRejected = r, this.callRejected = this.otherCallRejected);
                                                  }

                                                  function l(t, r, n) {
                                                    i(function () {
                                                      var e;

                                                      try {
                                                        e = r(n);
                                                      } catch (e) {
                                                        return f.reject(t, e);
                                                      }

                                                      e === t ? f.reject(t, new TypeError("Cannot resolve promise with itself")) : f.resolve(t, e);
                                                    });
                                                  }

                                                  function d(e) {
                                                    var t = e && e.then;
                                                    if (e && ("object" == typeof e || "function" == typeof e) && "function" == typeof t) return function () {
                                                      t.apply(e, arguments);
                                                    };
                                                  }

                                                  function c(t, e) {
                                                    var r = !1;

                                                    function n(e) {
                                                      r || (r = !0, f.reject(t, e));
                                                    }

                                                    function i(e) {
                                                      r || (r = !0, f.resolve(t, e));
                                                    }

                                                    var s = p(function () {
                                                      e(i, n);
                                                    });
                                                    "error" === s.status && n(s.value);
                                                  }

                                                  function p(e, t) {
                                                    var r = {};

                                                    try {
                                                      r.value = e(t), r.status = "success";
                                                    } catch (e) {
                                                      r.status = "error", r.value = e;
                                                    }

                                                    return r;
                                                  }

                                                  (t.exports = o).prototype.finally = function (t) {
                                                    if ("function" != typeof t) return this;
                                                    var r = this.constructor;
                                                    return this.then(function (e) {
                                                      return r.resolve(t()).then(function () {
                                                        return e;
                                                      });
                                                    }, function (e) {
                                                      return r.resolve(t()).then(function () {
                                                        throw e;
                                                      });
                                                    });
                                                  }, o.prototype.catch = function (e) {
                                                    return this.then(null, e);
                                                  }, o.prototype.then = function (e, t) {
                                                    if ("function" != typeof e && this.state === a || "function" != typeof t && this.state === s) return this;
                                                    var r = new this.constructor(h);
                                                    return this.state !== n ? l(r, this.state === a ? e : t, this.outcome) : this.queue.push(new u(r, e, t)), r;
                                                  }, u.prototype.callFulfilled = function (e) {
                                                    f.resolve(this.promise, e);
                                                  }, u.prototype.otherCallFulfilled = function (e) {
                                                    l(this.promise, this.onFulfilled, e);
                                                  }, u.prototype.callRejected = function (e) {
                                                    f.reject(this.promise, e);
                                                  }, u.prototype.otherCallRejected = function (e) {
                                                    l(this.promise, this.onRejected, e);
                                                  }, f.resolve = function (e, t) {
                                                    var r = p(d, t);
                                                    if ("error" === r.status) return f.reject(e, r.value);
                                                    var n = r.value;
                                                    if (n) c(e, n);else {
                                                      e.state = a, e.outcome = t;

                                                      for (var i = -1, s = e.queue.length; ++i < s;) e.queue[i].callFulfilled(t);
                                                    }
                                                    return e;
                                                  }, f.reject = function (e, t) {
                                                    e.state = s, e.outcome = t;

                                                    for (var r = -1, n = e.queue.length; ++r < n;) e.queue[r].callRejected(t);

                                                    return e;
                                                  }, o.resolve = function (e) {
                                                    return e instanceof this ? e : f.resolve(new this(h), e);
                                                  }, o.reject = function (e) {
                                                    var t = new this(h);
                                                    return f.reject(t, e);
                                                  }, o.all = function (e) {
                                                    var r = this;
                                                    if ("[object Array]" !== Object.prototype.toString.call(e)) return this.reject(new TypeError("must be an array"));
                                                    var n = e.length,
                                                        i = !1;
                                                    if (!n) return this.resolve([]);

                                                    for (var s = new Array(n), a = 0, t = -1, o = new this(h); ++t < n;) u(e[t], t);

                                                    return o;

                                                    function u(e, t) {
                                                      r.resolve(e).then(function (e) {
                                                        s[t] = e, ++a !== n || i || (i = !0, f.resolve(o, s));
                                                      }, function (e) {
                                                        i || (i = !0, f.reject(o, e));
                                                      });
                                                    }
                                                  }, o.race = function (e) {
                                                    if ("[object Array]" !== Object.prototype.toString.call(e)) return this.reject(new TypeError("must be an array"));
                                                    var t = e.length,
                                                        r = !1;
                                                    if (!t) return this.resolve([]);

                                                    for (var n, i = -1, s = new this(h); ++i < t;) n = e[i], this.resolve(n).then(function (e) {
                                                      r || (r = !0, f.resolve(s, e));
                                                    }, function (e) {
                                                      r || (r = !0, f.reject(s, e));
                                                    });

                                                    return s;
                                                  };
                                                }, {
                                                  immediate: 36
                                                }],
                                                38: [function (e, t, r) {

                                                  var n = {};
                                                  (0, e("./lib/utils/common").assign)(n, e("./lib/deflate"), e("./lib/inflate"), e("./lib/zlib/constants")), t.exports = n;
                                                }, {
                                                  "./lib/deflate": 39,
                                                  "./lib/inflate": 40,
                                                  "./lib/utils/common": 41,
                                                  "./lib/zlib/constants": 44
                                                }],
                                                39: [function (e, t, r) {

                                                  var a = e("./zlib/deflate"),
                                                      o = e("./utils/common"),
                                                      u = e("./utils/strings"),
                                                      i = e("./zlib/messages"),
                                                      s = e("./zlib/zstream"),
                                                      h = Object.prototype.toString,
                                                      f = 0,
                                                      l = -1,
                                                      d = 0,
                                                      c = 8;

                                                  function p(e) {
                                                    if (!(this instanceof p)) return new p(e);
                                                    this.options = o.assign({
                                                      level: l,
                                                      method: c,
                                                      chunkSize: 16384,
                                                      windowBits: 15,
                                                      memLevel: 8,
                                                      strategy: d,
                                                      to: ""
                                                    }, e || {});
                                                    var t = this.options;
                                                    t.raw && 0 < t.windowBits ? t.windowBits = -t.windowBits : t.gzip && 0 < t.windowBits && t.windowBits < 16 && (t.windowBits += 16), this.err = 0, this.msg = "", this.ended = !1, this.chunks = [], this.strm = new s(), this.strm.avail_out = 0;
                                                    var r = a.deflateInit2(this.strm, t.level, t.method, t.windowBits, t.memLevel, t.strategy);
                                                    if (r !== f) throw new Error(i[r]);

                                                    if (t.header && a.deflateSetHeader(this.strm, t.header), t.dictionary) {
                                                      var n;
                                                      if (n = "string" == typeof t.dictionary ? u.string2buf(t.dictionary) : "[object ArrayBuffer]" === h.call(t.dictionary) ? new Uint8Array(t.dictionary) : t.dictionary, (r = a.deflateSetDictionary(this.strm, n)) !== f) throw new Error(i[r]);
                                                      this._dict_set = !0;
                                                    }
                                                  }

                                                  function n(e, t) {
                                                    var r = new p(t);
                                                    if (r.push(e, !0), r.err) throw r.msg || i[r.err];
                                                    return r.result;
                                                  }

                                                  p.prototype.push = function (e, t) {
                                                    var r,
                                                        n,
                                                        i = this.strm,
                                                        s = this.options.chunkSize;
                                                    if (this.ended) return !1;
                                                    n = t === ~~t ? t : !0 === t ? 4 : 0, "string" == typeof e ? i.input = u.string2buf(e) : "[object ArrayBuffer]" === h.call(e) ? i.input = new Uint8Array(e) : i.input = e, i.next_in = 0, i.avail_in = i.input.length;

                                                    do {
                                                      if (0 === i.avail_out && (i.output = new o.Buf8(s), i.next_out = 0, i.avail_out = s), 1 !== (r = a.deflate(i, n)) && r !== f) return this.onEnd(r), !(this.ended = !0);
                                                      0 !== i.avail_out && (0 !== i.avail_in || 4 !== n && 2 !== n) || ("string" === this.options.to ? this.onData(u.buf2binstring(o.shrinkBuf(i.output, i.next_out))) : this.onData(o.shrinkBuf(i.output, i.next_out)));
                                                    } while ((0 < i.avail_in || 0 === i.avail_out) && 1 !== r);

                                                    return 4 === n ? (r = a.deflateEnd(this.strm), this.onEnd(r), this.ended = !0, r === f) : 2 !== n || (this.onEnd(f), !(i.avail_out = 0));
                                                  }, p.prototype.onData = function (e) {
                                                    this.chunks.push(e);
                                                  }, p.prototype.onEnd = function (e) {
                                                    e === f && ("string" === this.options.to ? this.result = this.chunks.join("") : this.result = o.flattenChunks(this.chunks)), this.chunks = [], this.err = e, this.msg = this.strm.msg;
                                                  }, r.Deflate = p, r.deflate = n, r.deflateRaw = function (e, t) {
                                                    return (t = t || {}).raw = !0, n(e, t);
                                                  }, r.gzip = function (e, t) {
                                                    return (t = t || {}).gzip = !0, n(e, t);
                                                  };
                                                }, {
                                                  "./utils/common": 41,
                                                  "./utils/strings": 42,
                                                  "./zlib/deflate": 46,
                                                  "./zlib/messages": 51,
                                                  "./zlib/zstream": 53
                                                }],
                                                40: [function (e, t, r) {

                                                  var d = e("./zlib/inflate"),
                                                      c = e("./utils/common"),
                                                      p = e("./utils/strings"),
                                                      m = e("./zlib/constants"),
                                                      n = e("./zlib/messages"),
                                                      i = e("./zlib/zstream"),
                                                      s = e("./zlib/gzheader"),
                                                      _ = Object.prototype.toString;

                                                  function a(e) {
                                                    if (!(this instanceof a)) return new a(e);
                                                    this.options = c.assign({
                                                      chunkSize: 16384,
                                                      windowBits: 0,
                                                      to: ""
                                                    }, e || {});
                                                    var t = this.options;
                                                    t.raw && 0 <= t.windowBits && t.windowBits < 16 && (t.windowBits = -t.windowBits, 0 === t.windowBits && (t.windowBits = -15)), !(0 <= t.windowBits && t.windowBits < 16) || e && e.windowBits || (t.windowBits += 32), 15 < t.windowBits && t.windowBits < 48 && 0 == (15 & t.windowBits) && (t.windowBits |= 15), this.err = 0, this.msg = "", this.ended = !1, this.chunks = [], this.strm = new i(), this.strm.avail_out = 0;
                                                    var r = d.inflateInit2(this.strm, t.windowBits);
                                                    if (r !== m.Z_OK) throw new Error(n[r]);
                                                    this.header = new s(), d.inflateGetHeader(this.strm, this.header);
                                                  }

                                                  function o(e, t) {
                                                    var r = new a(t);
                                                    if (r.push(e, !0), r.err) throw r.msg || n[r.err];
                                                    return r.result;
                                                  }

                                                  a.prototype.push = function (e, t) {
                                                    var r,
                                                        n,
                                                        i,
                                                        s,
                                                        a,
                                                        o,
                                                        u = this.strm,
                                                        h = this.options.chunkSize,
                                                        f = this.options.dictionary,
                                                        l = !1;
                                                    if (this.ended) return !1;
                                                    n = t === ~~t ? t : !0 === t ? m.Z_FINISH : m.Z_NO_FLUSH, "string" == typeof e ? u.input = p.binstring2buf(e) : "[object ArrayBuffer]" === _.call(e) ? u.input = new Uint8Array(e) : u.input = e, u.next_in = 0, u.avail_in = u.input.length;

                                                    do {
                                                      if (0 === u.avail_out && (u.output = new c.Buf8(h), u.next_out = 0, u.avail_out = h), (r = d.inflate(u, m.Z_NO_FLUSH)) === m.Z_NEED_DICT && f && (o = "string" == typeof f ? p.string2buf(f) : "[object ArrayBuffer]" === _.call(f) ? new Uint8Array(f) : f, r = d.inflateSetDictionary(this.strm, o)), r === m.Z_BUF_ERROR && !0 === l && (r = m.Z_OK, l = !1), r !== m.Z_STREAM_END && r !== m.Z_OK) return this.onEnd(r), !(this.ended = !0);
                                                      u.next_out && (0 !== u.avail_out && r !== m.Z_STREAM_END && (0 !== u.avail_in || n !== m.Z_FINISH && n !== m.Z_SYNC_FLUSH) || ("string" === this.options.to ? (i = p.utf8border(u.output, u.next_out), s = u.next_out - i, a = p.buf2string(u.output, i), u.next_out = s, u.avail_out = h - s, s && c.arraySet(u.output, u.output, i, s, 0), this.onData(a)) : this.onData(c.shrinkBuf(u.output, u.next_out)))), 0 === u.avail_in && 0 === u.avail_out && (l = !0);
                                                    } while ((0 < u.avail_in || 0 === u.avail_out) && r !== m.Z_STREAM_END);

                                                    return r === m.Z_STREAM_END && (n = m.Z_FINISH), n === m.Z_FINISH ? (r = d.inflateEnd(this.strm), this.onEnd(r), this.ended = !0, r === m.Z_OK) : n !== m.Z_SYNC_FLUSH || (this.onEnd(m.Z_OK), !(u.avail_out = 0));
                                                  }, a.prototype.onData = function (e) {
                                                    this.chunks.push(e);
                                                  }, a.prototype.onEnd = function (e) {
                                                    e === m.Z_OK && ("string" === this.options.to ? this.result = this.chunks.join("") : this.result = c.flattenChunks(this.chunks)), this.chunks = [], this.err = e, this.msg = this.strm.msg;
                                                  }, r.Inflate = a, r.inflate = o, r.inflateRaw = function (e, t) {
                                                    return (t = t || {}).raw = !0, o(e, t);
                                                  }, r.ungzip = o;
                                                }, {
                                                  "./utils/common": 41,
                                                  "./utils/strings": 42,
                                                  "./zlib/constants": 44,
                                                  "./zlib/gzheader": 47,
                                                  "./zlib/inflate": 49,
                                                  "./zlib/messages": 51,
                                                  "./zlib/zstream": 53
                                                }],
                                                41: [function (e, t, r) {

                                                  var n = "undefined" != typeof Uint8Array && "undefined" != typeof Uint16Array && "undefined" != typeof Int32Array;
                                                  r.assign = function (e) {
                                                    for (var t = Array.prototype.slice.call(arguments, 1); t.length;) {
                                                      var r = t.shift();

                                                      if (r) {
                                                        if ("object" != typeof r) throw new TypeError(r + "must be non-object");

                                                        for (var n in r) r.hasOwnProperty(n) && (e[n] = r[n]);
                                                      }
                                                    }

                                                    return e;
                                                  }, r.shrinkBuf = function (e, t) {
                                                    return e.length === t ? e : e.subarray ? e.subarray(0, t) : (e.length = t, e);
                                                  };
                                                  var i = {
                                                    arraySet: function (e, t, r, n, i) {
                                                      if (t.subarray && e.subarray) e.set(t.subarray(r, r + n), i);else for (var s = 0; s < n; s++) e[i + s] = t[r + s];
                                                    },
                                                    flattenChunks: function (e) {
                                                      var t, r, n, i, s, a;

                                                      for (t = n = 0, r = e.length; t < r; t++) n += e[t].length;

                                                      for (a = new Uint8Array(n), t = i = 0, r = e.length; t < r; t++) s = e[t], a.set(s, i), i += s.length;

                                                      return a;
                                                    }
                                                  },
                                                      s = {
                                                    arraySet: function (e, t, r, n, i) {
                                                      for (var s = 0; s < n; s++) e[i + s] = t[r + s];
                                                    },
                                                    flattenChunks: function (e) {
                                                      return [].concat.apply([], e);
                                                    }
                                                  };
                                                  r.setTyped = function (e) {
                                                    e ? (r.Buf8 = Uint8Array, r.Buf16 = Uint16Array, r.Buf32 = Int32Array, r.assign(r, i)) : (r.Buf8 = Array, r.Buf16 = Array, r.Buf32 = Array, r.assign(r, s));
                                                  }, r.setTyped(n);
                                                }, {}],
                                                42: [function (e, t, r) {

                                                  var u = e("./common"),
                                                      i = !0,
                                                      s = !0;

                                                  try {
                                                    String.fromCharCode.apply(null, [0]);
                                                  } catch (e) {
                                                    i = !1;
                                                  }

                                                  try {
                                                    String.fromCharCode.apply(null, new Uint8Array(1));
                                                  } catch (e) {
                                                    s = !1;
                                                  }

                                                  for (var h = new u.Buf8(256), n = 0; n < 256; n++) h[n] = 252 <= n ? 6 : 248 <= n ? 5 : 240 <= n ? 4 : 224 <= n ? 3 : 192 <= n ? 2 : 1;

                                                  function f(e, t) {
                                                    if (t < 65537 && (e.subarray && s || !e.subarray && i)) return String.fromCharCode.apply(null, u.shrinkBuf(e, t));

                                                    for (var r = "", n = 0; n < t; n++) r += String.fromCharCode(e[n]);

                                                    return r;
                                                  }

                                                  h[254] = h[254] = 1, r.string2buf = function (e) {
                                                    var t,
                                                        r,
                                                        n,
                                                        i,
                                                        s,
                                                        a = e.length,
                                                        o = 0;

                                                    for (i = 0; i < a; i++) 55296 == (64512 & (r = e.charCodeAt(i))) && i + 1 < a && 56320 == (64512 & (n = e.charCodeAt(i + 1))) && (r = 65536 + (r - 55296 << 10) + (n - 56320), i++), o += r < 128 ? 1 : r < 2048 ? 2 : r < 65536 ? 3 : 4;

                                                    for (t = new u.Buf8(o), i = s = 0; s < o; i++) 55296 == (64512 & (r = e.charCodeAt(i))) && i + 1 < a && 56320 == (64512 & (n = e.charCodeAt(i + 1))) && (r = 65536 + (r - 55296 << 10) + (n - 56320), i++), r < 128 ? t[s++] = r : (r < 2048 ? t[s++] = 192 | r >>> 6 : (r < 65536 ? t[s++] = 224 | r >>> 12 : (t[s++] = 240 | r >>> 18, t[s++] = 128 | r >>> 12 & 63), t[s++] = 128 | r >>> 6 & 63), t[s++] = 128 | 63 & r);

                                                    return t;
                                                  }, r.buf2binstring = function (e) {
                                                    return f(e, e.length);
                                                  }, r.binstring2buf = function (e) {
                                                    for (var t = new u.Buf8(e.length), r = 0, n = t.length; r < n; r++) t[r] = e.charCodeAt(r);

                                                    return t;
                                                  }, r.buf2string = function (e, t) {
                                                    var r,
                                                        n,
                                                        i,
                                                        s,
                                                        a = t || e.length,
                                                        o = new Array(2 * a);

                                                    for (r = n = 0; r < a;) if ((i = e[r++]) < 128) o[n++] = i;else if (4 < (s = h[i])) o[n++] = 65533, r += s - 1;else {
                                                      for (i &= 2 === s ? 31 : 3 === s ? 15 : 7; 1 < s && r < a;) i = i << 6 | 63 & e[r++], s--;

                                                      1 < s ? o[n++] = 65533 : i < 65536 ? o[n++] = i : (i -= 65536, o[n++] = 55296 | i >> 10 & 1023, o[n++] = 56320 | 1023 & i);
                                                    }

                                                    return f(o, n);
                                                  }, r.utf8border = function (e, t) {
                                                    var r;

                                                    for ((t = t || e.length) > e.length && (t = e.length), r = t - 1; 0 <= r && 128 == (192 & e[r]);) r--;

                                                    return r < 0 ? t : 0 === r ? t : r + h[e[r]] > t ? r : t;
                                                  };
                                                }, {
                                                  "./common": 41
                                                }],
                                                43: [function (e, t, r) {

                                                  t.exports = function (e, t, r, n) {
                                                    for (var i = 65535 & e | 0, s = e >>> 16 & 65535 | 0, a = 0; 0 !== r;) {
                                                      for (r -= a = 2e3 < r ? 2e3 : r; s = s + (i = i + t[n++] | 0) | 0, --a;);

                                                      i %= 65521, s %= 65521;
                                                    }

                                                    return i | s << 16 | 0;
                                                  };
                                                }, {}],
                                                44: [function (e, t, r) {

                                                  t.exports = {
                                                    Z_NO_FLUSH: 0,
                                                    Z_PARTIAL_FLUSH: 1,
                                                    Z_SYNC_FLUSH: 2,
                                                    Z_FULL_FLUSH: 3,
                                                    Z_FINISH: 4,
                                                    Z_BLOCK: 5,
                                                    Z_TREES: 6,
                                                    Z_OK: 0,
                                                    Z_STREAM_END: 1,
                                                    Z_NEED_DICT: 2,
                                                    Z_ERRNO: -1,
                                                    Z_STREAM_ERROR: -2,
                                                    Z_DATA_ERROR: -3,
                                                    Z_BUF_ERROR: -5,
                                                    Z_NO_COMPRESSION: 0,
                                                    Z_BEST_SPEED: 1,
                                                    Z_BEST_COMPRESSION: 9,
                                                    Z_DEFAULT_COMPRESSION: -1,
                                                    Z_FILTERED: 1,
                                                    Z_HUFFMAN_ONLY: 2,
                                                    Z_RLE: 3,
                                                    Z_FIXED: 4,
                                                    Z_DEFAULT_STRATEGY: 0,
                                                    Z_BINARY: 0,
                                                    Z_TEXT: 1,
                                                    Z_UNKNOWN: 2,
                                                    Z_DEFLATED: 8
                                                  };
                                                }, {}],
                                                45: [function (e, t, r) {

                                                  var o = function () {
                                                    for (var e, t = [], r = 0; r < 256; r++) {
                                                      e = r;

                                                      for (var n = 0; n < 8; n++) e = 1 & e ? 3988292384 ^ e >>> 1 : e >>> 1;

                                                      t[r] = e;
                                                    }

                                                    return t;
                                                  }();

                                                  t.exports = function (e, t, r, n) {
                                                    var i = o,
                                                        s = n + r;
                                                    e ^= -1;

                                                    for (var a = n; a < s; a++) e = e >>> 8 ^ i[255 & (e ^ t[a])];

                                                    return -1 ^ e;
                                                  };
                                                }, {}],
                                                46: [function (e, t, r) {

                                                  var u,
                                                      d = e("../utils/common"),
                                                      h = e("./trees"),
                                                      c = e("./adler32"),
                                                      p = e("./crc32"),
                                                      n = e("./messages"),
                                                      f = 0,
                                                      l = 0,
                                                      m = -2,
                                                      i = 2,
                                                      _ = 8,
                                                      s = 286,
                                                      a = 30,
                                                      o = 19,
                                                      g = 2 * s + 1,
                                                      v = 15,
                                                      b = 3,
                                                      w = 258,
                                                      y = w + b + 1,
                                                      k = 42,
                                                      x = 113;

                                                  function S(e, t) {
                                                    return e.msg = n[t], t;
                                                  }

                                                  function z(e) {
                                                    return (e << 1) - (4 < e ? 9 : 0);
                                                  }

                                                  function E(e) {
                                                    for (var t = e.length; 0 <= --t;) e[t] = 0;
                                                  }

                                                  function C(e) {
                                                    var t = e.state,
                                                        r = t.pending;
                                                    r > e.avail_out && (r = e.avail_out), 0 !== r && (d.arraySet(e.output, t.pending_buf, t.pending_out, r, e.next_out), e.next_out += r, t.pending_out += r, e.total_out += r, e.avail_out -= r, t.pending -= r, 0 === t.pending && (t.pending_out = 0));
                                                  }

                                                  function A(e, t) {
                                                    h._tr_flush_block(e, 0 <= e.block_start ? e.block_start : -1, e.strstart - e.block_start, t), e.block_start = e.strstart, C(e.strm);
                                                  }

                                                  function I(e, t) {
                                                    e.pending_buf[e.pending++] = t;
                                                  }

                                                  function O(e, t) {
                                                    e.pending_buf[e.pending++] = t >>> 8 & 255, e.pending_buf[e.pending++] = 255 & t;
                                                  }

                                                  function B(e, t) {
                                                    var r,
                                                        n,
                                                        i = e.max_chain_length,
                                                        s = e.strstart,
                                                        a = e.prev_length,
                                                        o = e.nice_match,
                                                        u = e.strstart > e.w_size - y ? e.strstart - (e.w_size - y) : 0,
                                                        h = e.window,
                                                        f = e.w_mask,
                                                        l = e.prev,
                                                        d = e.strstart + w,
                                                        c = h[s + a - 1],
                                                        p = h[s + a];
                                                    e.prev_length >= e.good_match && (i >>= 2), o > e.lookahead && (o = e.lookahead);

                                                    do {
                                                      if (h[(r = t) + a] === p && h[r + a - 1] === c && h[r] === h[s] && h[++r] === h[s + 1]) {
                                                        s += 2, r++;

                                                        do {} while (h[++s] === h[++r] && h[++s] === h[++r] && h[++s] === h[++r] && h[++s] === h[++r] && h[++s] === h[++r] && h[++s] === h[++r] && h[++s] === h[++r] && h[++s] === h[++r] && s < d);

                                                        if (n = w - (d - s), s = d - w, a < n) {
                                                          if (e.match_start = t, o <= (a = n)) break;
                                                          c = h[s + a - 1], p = h[s + a];
                                                        }
                                                      }
                                                    } while ((t = l[t & f]) > u && 0 != --i);

                                                    return a <= e.lookahead ? a : e.lookahead;
                                                  }

                                                  function T(e) {
                                                    var t,
                                                        r,
                                                        n,
                                                        i,
                                                        s,
                                                        a,
                                                        o,
                                                        u,
                                                        h,
                                                        f,
                                                        l = e.w_size;

                                                    do {
                                                      if (i = e.window_size - e.lookahead - e.strstart, e.strstart >= l + (l - y)) {
                                                        for (d.arraySet(e.window, e.window, l, l, 0), e.match_start -= l, e.strstart -= l, e.block_start -= l, t = r = e.hash_size; n = e.head[--t], e.head[t] = l <= n ? n - l : 0, --r;);

                                                        for (t = r = l; n = e.prev[--t], e.prev[t] = l <= n ? n - l : 0, --r;);

                                                        i += l;
                                                      }

                                                      if (0 === e.strm.avail_in) break;
                                                      if (a = e.strm, o = e.window, u = e.strstart + e.lookahead, f = void 0, (h = i) < (f = a.avail_in) && (f = h), r = 0 === f ? 0 : (a.avail_in -= f, d.arraySet(o, a.input, a.next_in, f, u), 1 === a.state.wrap ? a.adler = c(a.adler, o, f, u) : 2 === a.state.wrap && (a.adler = p(a.adler, o, f, u)), a.next_in += f, a.total_in += f, f), e.lookahead += r, e.lookahead + e.insert >= b) for (s = e.strstart - e.insert, e.ins_h = e.window[s], e.ins_h = (e.ins_h << e.hash_shift ^ e.window[s + 1]) & e.hash_mask; e.insert && (e.ins_h = (e.ins_h << e.hash_shift ^ e.window[s + b - 1]) & e.hash_mask, e.prev[s & e.w_mask] = e.head[e.ins_h], e.head[e.ins_h] = s, s++, e.insert--, !(e.lookahead + e.insert < b)););
                                                    } while (e.lookahead < y && 0 !== e.strm.avail_in);
                                                  }

                                                  function R(e, t) {
                                                    for (var r, n;;) {
                                                      if (e.lookahead < y) {
                                                        if (T(e), e.lookahead < y && t === f) return 1;
                                                        if (0 === e.lookahead) break;
                                                      }

                                                      if (r = 0, e.lookahead >= b && (e.ins_h = (e.ins_h << e.hash_shift ^ e.window[e.strstart + b - 1]) & e.hash_mask, r = e.prev[e.strstart & e.w_mask] = e.head[e.ins_h], e.head[e.ins_h] = e.strstart), 0 !== r && e.strstart - r <= e.w_size - y && (e.match_length = B(e, r)), e.match_length >= b) {
                                                        if (n = h._tr_tally(e, e.strstart - e.match_start, e.match_length - b), e.lookahead -= e.match_length, e.match_length <= e.max_lazy_match && e.lookahead >= b) {
                                                          for (e.match_length--; e.strstart++, e.ins_h = (e.ins_h << e.hash_shift ^ e.window[e.strstart + b - 1]) & e.hash_mask, r = e.prev[e.strstart & e.w_mask] = e.head[e.ins_h], e.head[e.ins_h] = e.strstart, 0 != --e.match_length;);

                                                          e.strstart++;
                                                        } else e.strstart += e.match_length, e.match_length = 0, e.ins_h = e.window[e.strstart], e.ins_h = (e.ins_h << e.hash_shift ^ e.window[e.strstart + 1]) & e.hash_mask;
                                                      } else n = h._tr_tally(e, 0, e.window[e.strstart]), e.lookahead--, e.strstart++;
                                                      if (n && (A(e, !1), 0 === e.strm.avail_out)) return 1;
                                                    }

                                                    return e.insert = e.strstart < b - 1 ? e.strstart : b - 1, 4 === t ? (A(e, !0), 0 === e.strm.avail_out ? 3 : 4) : e.last_lit && (A(e, !1), 0 === e.strm.avail_out) ? 1 : 2;
                                                  }

                                                  function D(e, t) {
                                                    for (var r, n, i;;) {
                                                      if (e.lookahead < y) {
                                                        if (T(e), e.lookahead < y && t === f) return 1;
                                                        if (0 === e.lookahead) break;
                                                      }

                                                      if (r = 0, e.lookahead >= b && (e.ins_h = (e.ins_h << e.hash_shift ^ e.window[e.strstart + b - 1]) & e.hash_mask, r = e.prev[e.strstart & e.w_mask] = e.head[e.ins_h], e.head[e.ins_h] = e.strstart), e.prev_length = e.match_length, e.prev_match = e.match_start, e.match_length = b - 1, 0 !== r && e.prev_length < e.max_lazy_match && e.strstart - r <= e.w_size - y && (e.match_length = B(e, r), e.match_length <= 5 && (1 === e.strategy || e.match_length === b && 4096 < e.strstart - e.match_start) && (e.match_length = b - 1)), e.prev_length >= b && e.match_length <= e.prev_length) {
                                                        for (i = e.strstart + e.lookahead - b, n = h._tr_tally(e, e.strstart - 1 - e.prev_match, e.prev_length - b), e.lookahead -= e.prev_length - 1, e.prev_length -= 2; ++e.strstart <= i && (e.ins_h = (e.ins_h << e.hash_shift ^ e.window[e.strstart + b - 1]) & e.hash_mask, r = e.prev[e.strstart & e.w_mask] = e.head[e.ins_h], e.head[e.ins_h] = e.strstart), 0 != --e.prev_length;);

                                                        if (e.match_available = 0, e.match_length = b - 1, e.strstart++, n && (A(e, !1), 0 === e.strm.avail_out)) return 1;
                                                      } else if (e.match_available) {
                                                        if ((n = h._tr_tally(e, 0, e.window[e.strstart - 1])) && A(e, !1), e.strstart++, e.lookahead--, 0 === e.strm.avail_out) return 1;
                                                      } else e.match_available = 1, e.strstart++, e.lookahead--;
                                                    }

                                                    return e.match_available && (n = h._tr_tally(e, 0, e.window[e.strstart - 1]), e.match_available = 0), e.insert = e.strstart < b - 1 ? e.strstart : b - 1, 4 === t ? (A(e, !0), 0 === e.strm.avail_out ? 3 : 4) : e.last_lit && (A(e, !1), 0 === e.strm.avail_out) ? 1 : 2;
                                                  }

                                                  function F(e, t, r, n, i) {
                                                    this.good_length = e, this.max_lazy = t, this.nice_length = r, this.max_chain = n, this.func = i;
                                                  }

                                                  function N() {
                                                    this.strm = null, this.status = 0, this.pending_buf = null, this.pending_buf_size = 0, this.pending_out = 0, this.pending = 0, this.wrap = 0, this.gzhead = null, this.gzindex = 0, this.method = _, this.last_flush = -1, this.w_size = 0, this.w_bits = 0, this.w_mask = 0, this.window = null, this.window_size = 0, this.prev = null, this.head = null, this.ins_h = 0, this.hash_size = 0, this.hash_bits = 0, this.hash_mask = 0, this.hash_shift = 0, this.block_start = 0, this.match_length = 0, this.prev_match = 0, this.match_available = 0, this.strstart = 0, this.match_start = 0, this.lookahead = 0, this.prev_length = 0, this.max_chain_length = 0, this.max_lazy_match = 0, this.level = 0, this.strategy = 0, this.good_match = 0, this.nice_match = 0, this.dyn_ltree = new d.Buf16(2 * g), this.dyn_dtree = new d.Buf16(2 * (2 * a + 1)), this.bl_tree = new d.Buf16(2 * (2 * o + 1)), E(this.dyn_ltree), E(this.dyn_dtree), E(this.bl_tree), this.l_desc = null, this.d_desc = null, this.bl_desc = null, this.bl_count = new d.Buf16(v + 1), this.heap = new d.Buf16(2 * s + 1), E(this.heap), this.heap_len = 0, this.heap_max = 0, this.depth = new d.Buf16(2 * s + 1), E(this.depth), this.l_buf = 0, this.lit_bufsize = 0, this.last_lit = 0, this.d_buf = 0, this.opt_len = 0, this.static_len = 0, this.matches = 0, this.insert = 0, this.bi_buf = 0, this.bi_valid = 0;
                                                  }

                                                  function U(e) {
                                                    var t;
                                                    return e && e.state ? (e.total_in = e.total_out = 0, e.data_type = i, (t = e.state).pending = 0, t.pending_out = 0, t.wrap < 0 && (t.wrap = -t.wrap), t.status = t.wrap ? k : x, e.adler = 2 === t.wrap ? 0 : 1, t.last_flush = f, h._tr_init(t), l) : S(e, m);
                                                  }

                                                  function P(e) {
                                                    var t,
                                                        r = U(e);
                                                    return r === l && ((t = e.state).window_size = 2 * t.w_size, E(t.head), t.max_lazy_match = u[t.level].max_lazy, t.good_match = u[t.level].good_length, t.nice_match = u[t.level].nice_length, t.max_chain_length = u[t.level].max_chain, t.strstart = 0, t.block_start = 0, t.lookahead = 0, t.insert = 0, t.match_length = t.prev_length = b - 1, t.match_available = 0, t.ins_h = 0), r;
                                                  }

                                                  function L(e, t, r, n, i, s) {
                                                    if (!e) return m;
                                                    var a = 1;
                                                    if (-1 === t && (t = 6), n < 0 ? (a = 0, n = -n) : 15 < n && (a = 2, n -= 16), i < 1 || 9 < i || r !== _ || n < 8 || 15 < n || t < 0 || 9 < t || s < 0 || 4 < s) return S(e, m);
                                                    8 === n && (n = 9);
                                                    var o = new N();
                                                    return (e.state = o).strm = e, o.wrap = a, o.gzhead = null, o.w_bits = n, o.w_size = 1 << o.w_bits, o.w_mask = o.w_size - 1, o.hash_bits = i + 7, o.hash_size = 1 << o.hash_bits, o.hash_mask = o.hash_size - 1, o.hash_shift = ~~((o.hash_bits + b - 1) / b), o.window = new d.Buf8(2 * o.w_size), o.head = new d.Buf16(o.hash_size), o.prev = new d.Buf16(o.w_size), o.lit_bufsize = 1 << i + 6, o.pending_buf_size = 4 * o.lit_bufsize, o.pending_buf = new d.Buf8(o.pending_buf_size), o.d_buf = 1 * o.lit_bufsize, o.l_buf = 3 * o.lit_bufsize, o.level = t, o.strategy = s, o.method = r, P(e);
                                                  }

                                                  u = [new F(0, 0, 0, 0, function (e, t) {
                                                    var r = 65535;

                                                    for (r > e.pending_buf_size - 5 && (r = e.pending_buf_size - 5);;) {
                                                      if (e.lookahead <= 1) {
                                                        if (T(e), 0 === e.lookahead && t === f) return 1;
                                                        if (0 === e.lookahead) break;
                                                      }

                                                      e.strstart += e.lookahead, e.lookahead = 0;
                                                      var n = e.block_start + r;
                                                      if ((0 === e.strstart || e.strstart >= n) && (e.lookahead = e.strstart - n, e.strstart = n, A(e, !1), 0 === e.strm.avail_out)) return 1;
                                                      if (e.strstart - e.block_start >= e.w_size - y && (A(e, !1), 0 === e.strm.avail_out)) return 1;
                                                    }

                                                    return e.insert = 0, 4 === t ? (A(e, !0), 0 === e.strm.avail_out ? 3 : 4) : (e.strstart > e.block_start && (A(e, !1), e.strm.avail_out), 1);
                                                  }), new F(4, 4, 8, 4, R), new F(4, 5, 16, 8, R), new F(4, 6, 32, 32, R), new F(4, 4, 16, 16, D), new F(8, 16, 32, 32, D), new F(8, 16, 128, 128, D), new F(8, 32, 128, 256, D), new F(32, 128, 258, 1024, D), new F(32, 258, 258, 4096, D)], r.deflateInit = function (e, t) {
                                                    return L(e, t, _, 15, 8, 0);
                                                  }, r.deflateInit2 = L, r.deflateReset = P, r.deflateResetKeep = U, r.deflateSetHeader = function (e, t) {
                                                    return e && e.state ? 2 !== e.state.wrap ? m : (e.state.gzhead = t, l) : m;
                                                  }, r.deflate = function (e, t) {
                                                    var r, n, i, s;
                                                    if (!e || !e.state || 5 < t || t < 0) return e ? S(e, m) : m;
                                                    if (n = e.state, !e.output || !e.input && 0 !== e.avail_in || 666 === n.status && 4 !== t) return S(e, 0 === e.avail_out ? -5 : m);
                                                    if (n.strm = e, r = n.last_flush, n.last_flush = t, n.status === k) if (2 === n.wrap) e.adler = 0, I(n, 31), I(n, 139), I(n, 8), n.gzhead ? (I(n, (n.gzhead.text ? 1 : 0) + (n.gzhead.hcrc ? 2 : 0) + (n.gzhead.extra ? 4 : 0) + (n.gzhead.name ? 8 : 0) + (n.gzhead.comment ? 16 : 0)), I(n, 255 & n.gzhead.time), I(n, n.gzhead.time >> 8 & 255), I(n, n.gzhead.time >> 16 & 255), I(n, n.gzhead.time >> 24 & 255), I(n, 9 === n.level ? 2 : 2 <= n.strategy || n.level < 2 ? 4 : 0), I(n, 255 & n.gzhead.os), n.gzhead.extra && n.gzhead.extra.length && (I(n, 255 & n.gzhead.extra.length), I(n, n.gzhead.extra.length >> 8 & 255)), n.gzhead.hcrc && (e.adler = p(e.adler, n.pending_buf, n.pending, 0)), n.gzindex = 0, n.status = 69) : (I(n, 0), I(n, 0), I(n, 0), I(n, 0), I(n, 0), I(n, 9 === n.level ? 2 : 2 <= n.strategy || n.level < 2 ? 4 : 0), I(n, 3), n.status = x);else {
                                                      var a = _ + (n.w_bits - 8 << 4) << 8;
                                                      a |= (2 <= n.strategy || n.level < 2 ? 0 : n.level < 6 ? 1 : 6 === n.level ? 2 : 3) << 6, 0 !== n.strstart && (a |= 32), a += 31 - a % 31, n.status = x, O(n, a), 0 !== n.strstart && (O(n, e.adler >>> 16), O(n, 65535 & e.adler)), e.adler = 1;
                                                    }
                                                    if (69 === n.status) if (n.gzhead.extra) {
                                                      for (i = n.pending; n.gzindex < (65535 & n.gzhead.extra.length) && (n.pending !== n.pending_buf_size || (n.gzhead.hcrc && n.pending > i && (e.adler = p(e.adler, n.pending_buf, n.pending - i, i)), C(e), i = n.pending, n.pending !== n.pending_buf_size));) I(n, 255 & n.gzhead.extra[n.gzindex]), n.gzindex++;

                                                      n.gzhead.hcrc && n.pending > i && (e.adler = p(e.adler, n.pending_buf, n.pending - i, i)), n.gzindex === n.gzhead.extra.length && (n.gzindex = 0, n.status = 73);
                                                    } else n.status = 73;
                                                    if (73 === n.status) if (n.gzhead.name) {
                                                      i = n.pending;

                                                      do {
                                                        if (n.pending === n.pending_buf_size && (n.gzhead.hcrc && n.pending > i && (e.adler = p(e.adler, n.pending_buf, n.pending - i, i)), C(e), i = n.pending, n.pending === n.pending_buf_size)) {
                                                          s = 1;
                                                          break;
                                                        }

                                                        s = n.gzindex < n.gzhead.name.length ? 255 & n.gzhead.name.charCodeAt(n.gzindex++) : 0, I(n, s);
                                                      } while (0 !== s);

                                                      n.gzhead.hcrc && n.pending > i && (e.adler = p(e.adler, n.pending_buf, n.pending - i, i)), 0 === s && (n.gzindex = 0, n.status = 91);
                                                    } else n.status = 91;
                                                    if (91 === n.status) if (n.gzhead.comment) {
                                                      i = n.pending;

                                                      do {
                                                        if (n.pending === n.pending_buf_size && (n.gzhead.hcrc && n.pending > i && (e.adler = p(e.adler, n.pending_buf, n.pending - i, i)), C(e), i = n.pending, n.pending === n.pending_buf_size)) {
                                                          s = 1;
                                                          break;
                                                        }

                                                        s = n.gzindex < n.gzhead.comment.length ? 255 & n.gzhead.comment.charCodeAt(n.gzindex++) : 0, I(n, s);
                                                      } while (0 !== s);

                                                      n.gzhead.hcrc && n.pending > i && (e.adler = p(e.adler, n.pending_buf, n.pending - i, i)), 0 === s && (n.status = 103);
                                                    } else n.status = 103;

                                                    if (103 === n.status && (n.gzhead.hcrc ? (n.pending + 2 > n.pending_buf_size && C(e), n.pending + 2 <= n.pending_buf_size && (I(n, 255 & e.adler), I(n, e.adler >> 8 & 255), e.adler = 0, n.status = x)) : n.status = x), 0 !== n.pending) {
                                                      if (C(e), 0 === e.avail_out) return n.last_flush = -1, l;
                                                    } else if (0 === e.avail_in && z(t) <= z(r) && 4 !== t) return S(e, -5);

                                                    if (666 === n.status && 0 !== e.avail_in) return S(e, -5);

                                                    if (0 !== e.avail_in || 0 !== n.lookahead || t !== f && 666 !== n.status) {
                                                      var o = 2 === n.strategy ? function (e, t) {
                                                        for (var r;;) {
                                                          if (0 === e.lookahead && (T(e), 0 === e.lookahead)) {
                                                            if (t === f) return 1;
                                                            break;
                                                          }

                                                          if (e.match_length = 0, r = h._tr_tally(e, 0, e.window[e.strstart]), e.lookahead--, e.strstart++, r && (A(e, !1), 0 === e.strm.avail_out)) return 1;
                                                        }

                                                        return e.insert = 0, 4 === t ? (A(e, !0), 0 === e.strm.avail_out ? 3 : 4) : e.last_lit && (A(e, !1), 0 === e.strm.avail_out) ? 1 : 2;
                                                      }(n, t) : 3 === n.strategy ? function (e, t) {
                                                        for (var r, n, i, s, a = e.window;;) {
                                                          if (e.lookahead <= w) {
                                                            if (T(e), e.lookahead <= w && t === f) return 1;
                                                            if (0 === e.lookahead) break;
                                                          }

                                                          if (e.match_length = 0, e.lookahead >= b && 0 < e.strstart && (n = a[i = e.strstart - 1]) === a[++i] && n === a[++i] && n === a[++i]) {
                                                            s = e.strstart + w;

                                                            do {} while (n === a[++i] && n === a[++i] && n === a[++i] && n === a[++i] && n === a[++i] && n === a[++i] && n === a[++i] && n === a[++i] && i < s);

                                                            e.match_length = w - (s - i), e.match_length > e.lookahead && (e.match_length = e.lookahead);
                                                          }

                                                          if (e.match_length >= b ? (r = h._tr_tally(e, 1, e.match_length - b), e.lookahead -= e.match_length, e.strstart += e.match_length, e.match_length = 0) : (r = h._tr_tally(e, 0, e.window[e.strstart]), e.lookahead--, e.strstart++), r && (A(e, !1), 0 === e.strm.avail_out)) return 1;
                                                        }

                                                        return e.insert = 0, 4 === t ? (A(e, !0), 0 === e.strm.avail_out ? 3 : 4) : e.last_lit && (A(e, !1), 0 === e.strm.avail_out) ? 1 : 2;
                                                      }(n, t) : u[n.level].func(n, t);
                                                      if (3 !== o && 4 !== o || (n.status = 666), 1 === o || 3 === o) return 0 === e.avail_out && (n.last_flush = -1), l;
                                                      if (2 === o && (1 === t ? h._tr_align(n) : 5 !== t && (h._tr_stored_block(n, 0, 0, !1), 3 === t && (E(n.head), 0 === n.lookahead && (n.strstart = 0, n.block_start = 0, n.insert = 0))), C(e), 0 === e.avail_out)) return n.last_flush = -1, l;
                                                    }

                                                    return 4 !== t ? l : n.wrap <= 0 ? 1 : (2 === n.wrap ? (I(n, 255 & e.adler), I(n, e.adler >> 8 & 255), I(n, e.adler >> 16 & 255), I(n, e.adler >> 24 & 255), I(n, 255 & e.total_in), I(n, e.total_in >> 8 & 255), I(n, e.total_in >> 16 & 255), I(n, e.total_in >> 24 & 255)) : (O(n, e.adler >>> 16), O(n, 65535 & e.adler)), C(e), 0 < n.wrap && (n.wrap = -n.wrap), 0 !== n.pending ? l : 1);
                                                  }, r.deflateEnd = function (e) {
                                                    var t;
                                                    return e && e.state ? (t = e.state.status) !== k && 69 !== t && 73 !== t && 91 !== t && 103 !== t && t !== x && 666 !== t ? S(e, m) : (e.state = null, t === x ? S(e, -3) : l) : m;
                                                  }, r.deflateSetDictionary = function (e, t) {
                                                    var r,
                                                        n,
                                                        i,
                                                        s,
                                                        a,
                                                        o,
                                                        u,
                                                        h,
                                                        f = t.length;
                                                    if (!e || !e.state) return m;
                                                    if (2 === (s = (r = e.state).wrap) || 1 === s && r.status !== k || r.lookahead) return m;

                                                    for (1 === s && (e.adler = c(e.adler, t, f, 0)), r.wrap = 0, f >= r.w_size && (0 === s && (E(r.head), r.strstart = 0, r.block_start = 0, r.insert = 0), h = new d.Buf8(r.w_size), d.arraySet(h, t, f - r.w_size, r.w_size, 0), t = h, f = r.w_size), a = e.avail_in, o = e.next_in, u = e.input, e.avail_in = f, e.next_in = 0, e.input = t, T(r); r.lookahead >= b;) {
                                                      for (n = r.strstart, i = r.lookahead - (b - 1); r.ins_h = (r.ins_h << r.hash_shift ^ r.window[n + b - 1]) & r.hash_mask, r.prev[n & r.w_mask] = r.head[r.ins_h], r.head[r.ins_h] = n, n++, --i;);

                                                      r.strstart = n, r.lookahead = b - 1, T(r);
                                                    }

                                                    return r.strstart += r.lookahead, r.block_start = r.strstart, r.insert = r.lookahead, r.lookahead = 0, r.match_length = r.prev_length = b - 1, r.match_available = 0, e.next_in = o, e.input = u, e.avail_in = a, r.wrap = s, l;
                                                  }, r.deflateInfo = "pako deflate (from Nodeca project)";
                                                }, {
                                                  "../utils/common": 41,
                                                  "./adler32": 43,
                                                  "./crc32": 45,
                                                  "./messages": 51,
                                                  "./trees": 52
                                                }],
                                                47: [function (e, t, r) {

                                                  t.exports = function () {
                                                    this.text = 0, this.time = 0, this.xflags = 0, this.os = 0, this.extra = null, this.extra_len = 0, this.name = "", this.comment = "", this.hcrc = 0, this.done = !1;
                                                  };
                                                }, {}],
                                                48: [function (e, t, r) {

                                                  t.exports = function (e, t) {
                                                    var r, n, i, s, a, o, u, h, f, l, d, c, p, m, _, g, v, b, w, y, k, x, S, z, E;

                                                    r = e.state, n = e.next_in, z = e.input, i = n + (e.avail_in - 5), s = e.next_out, E = e.output, a = s - (t - e.avail_out), o = s + (e.avail_out - 257), u = r.dmax, h = r.wsize, f = r.whave, l = r.wnext, d = r.window, c = r.hold, p = r.bits, m = r.lencode, _ = r.distcode, g = (1 << r.lenbits) - 1, v = (1 << r.distbits) - 1;

                                                    e: do {
                                                      p < 15 && (c += z[n++] << p, p += 8, c += z[n++] << p, p += 8), b = m[c & g];

                                                      t: for (;;) {
                                                        if (c >>>= w = b >>> 24, p -= w, 0 == (w = b >>> 16 & 255)) E[s++] = 65535 & b;else {
                                                          if (!(16 & w)) {
                                                            if (0 == (64 & w)) {
                                                              b = m[(65535 & b) + (c & (1 << w) - 1)];
                                                              continue t;
                                                            }

                                                            if (32 & w) {
                                                              r.mode = 12;
                                                              break e;
                                                            }

                                                            e.msg = "invalid literal/length code", r.mode = 30;
                                                            break e;
                                                          }

                                                          y = 65535 & b, (w &= 15) && (p < w && (c += z[n++] << p, p += 8), y += c & (1 << w) - 1, c >>>= w, p -= w), p < 15 && (c += z[n++] << p, p += 8, c += z[n++] << p, p += 8), b = _[c & v];

                                                          r: for (;;) {
                                                            if (c >>>= w = b >>> 24, p -= w, !(16 & (w = b >>> 16 & 255))) {
                                                              if (0 == (64 & w)) {
                                                                b = _[(65535 & b) + (c & (1 << w) - 1)];
                                                                continue r;
                                                              }

                                                              e.msg = "invalid distance code", r.mode = 30;
                                                              break e;
                                                            }

                                                            if (k = 65535 & b, p < (w &= 15) && (c += z[n++] << p, (p += 8) < w && (c += z[n++] << p, p += 8)), u < (k += c & (1 << w) - 1)) {
                                                              e.msg = "invalid distance too far back", r.mode = 30;
                                                              break e;
                                                            }

                                                            if (c >>>= w, p -= w, (w = s - a) < k) {
                                                              if (f < (w = k - w) && r.sane) {
                                                                e.msg = "invalid distance too far back", r.mode = 30;
                                                                break e;
                                                              }

                                                              if (S = d, (x = 0) === l) {
                                                                if (x += h - w, w < y) {
                                                                  for (y -= w; E[s++] = d[x++], --w;);

                                                                  x = s - k, S = E;
                                                                }
                                                              } else if (l < w) {
                                                                if (x += h + l - w, (w -= l) < y) {
                                                                  for (y -= w; E[s++] = d[x++], --w;);

                                                                  if (x = 0, l < y) {
                                                                    for (y -= w = l; E[s++] = d[x++], --w;);

                                                                    x = s - k, S = E;
                                                                  }
                                                                }
                                                              } else if (x += l - w, w < y) {
                                                                for (y -= w; E[s++] = d[x++], --w;);

                                                                x = s - k, S = E;
                                                              }

                                                              for (; 2 < y;) E[s++] = S[x++], E[s++] = S[x++], E[s++] = S[x++], y -= 3;

                                                              y && (E[s++] = S[x++], 1 < y && (E[s++] = S[x++]));
                                                            } else {
                                                              for (x = s - k; E[s++] = E[x++], E[s++] = E[x++], E[s++] = E[x++], 2 < (y -= 3););

                                                              y && (E[s++] = E[x++], 1 < y && (E[s++] = E[x++]));
                                                            }

                                                            break;
                                                          }
                                                        }
                                                        break;
                                                      }
                                                    } while (n < i && s < o);

                                                    n -= y = p >> 3, c &= (1 << (p -= y << 3)) - 1, e.next_in = n, e.next_out = s, e.avail_in = n < i ? i - n + 5 : 5 - (n - i), e.avail_out = s < o ? o - s + 257 : 257 - (s - o), r.hold = c, r.bits = p;
                                                  };
                                                }, {}],
                                                49: [function (e, t, r) {

                                                  var I = e("../utils/common"),
                                                      O = e("./adler32"),
                                                      B = e("./crc32"),
                                                      T = e("./inffast"),
                                                      R = e("./inftrees"),
                                                      D = 1,
                                                      F = 2,
                                                      N = 0,
                                                      U = -2,
                                                      P = 1,
                                                      n = 852,
                                                      i = 592;

                                                  function L(e) {
                                                    return (e >>> 24 & 255) + (e >>> 8 & 65280) + ((65280 & e) << 8) + ((255 & e) << 24);
                                                  }

                                                  function s() {
                                                    this.mode = 0, this.last = !1, this.wrap = 0, this.havedict = !1, this.flags = 0, this.dmax = 0, this.check = 0, this.total = 0, this.head = null, this.wbits = 0, this.wsize = 0, this.whave = 0, this.wnext = 0, this.window = null, this.hold = 0, this.bits = 0, this.length = 0, this.offset = 0, this.extra = 0, this.lencode = null, this.distcode = null, this.lenbits = 0, this.distbits = 0, this.ncode = 0, this.nlen = 0, this.ndist = 0, this.have = 0, this.next = null, this.lens = new I.Buf16(320), this.work = new I.Buf16(288), this.lendyn = null, this.distdyn = null, this.sane = 0, this.back = 0, this.was = 0;
                                                  }

                                                  function a(e) {
                                                    var t;
                                                    return e && e.state ? (t = e.state, e.total_in = e.total_out = t.total = 0, e.msg = "", t.wrap && (e.adler = 1 & t.wrap), t.mode = P, t.last = 0, t.havedict = 0, t.dmax = 32768, t.head = null, t.hold = 0, t.bits = 0, t.lencode = t.lendyn = new I.Buf32(n), t.distcode = t.distdyn = new I.Buf32(i), t.sane = 1, t.back = -1, N) : U;
                                                  }

                                                  function o(e) {
                                                    var t;
                                                    return e && e.state ? ((t = e.state).wsize = 0, t.whave = 0, t.wnext = 0, a(e)) : U;
                                                  }

                                                  function u(e, t) {
                                                    var r, n;
                                                    return e && e.state ? (n = e.state, t < 0 ? (r = 0, t = -t) : (r = 1 + (t >> 4), t < 48 && (t &= 15)), t && (t < 8 || 15 < t) ? U : (null !== n.window && n.wbits !== t && (n.window = null), n.wrap = r, n.wbits = t, o(e))) : U;
                                                  }

                                                  function h(e, t) {
                                                    var r, n;
                                                    return e ? (n = new s(), (e.state = n).window = null, (r = u(e, t)) !== N && (e.state = null), r) : U;
                                                  }

                                                  var f,
                                                      l,
                                                      d = !0;

                                                  function j(e) {
                                                    if (d) {
                                                      var t;

                                                      for (f = new I.Buf32(512), l = new I.Buf32(32), t = 0; t < 144;) e.lens[t++] = 8;

                                                      for (; t < 256;) e.lens[t++] = 9;

                                                      for (; t < 280;) e.lens[t++] = 7;

                                                      for (; t < 288;) e.lens[t++] = 8;

                                                      for (R(D, e.lens, 0, 288, f, 0, e.work, {
                                                        bits: 9
                                                      }), t = 0; t < 32;) e.lens[t++] = 5;

                                                      R(F, e.lens, 0, 32, l, 0, e.work, {
                                                        bits: 5
                                                      }), d = !1;
                                                    }

                                                    e.lencode = f, e.lenbits = 9, e.distcode = l, e.distbits = 5;
                                                  }

                                                  function Z(e, t, r, n) {
                                                    var i,
                                                        s = e.state;
                                                    return null === s.window && (s.wsize = 1 << s.wbits, s.wnext = 0, s.whave = 0, s.window = new I.Buf8(s.wsize)), n >= s.wsize ? (I.arraySet(s.window, t, r - s.wsize, s.wsize, 0), s.wnext = 0, s.whave = s.wsize) : (n < (i = s.wsize - s.wnext) && (i = n), I.arraySet(s.window, t, r - n, i, s.wnext), (n -= i) ? (I.arraySet(s.window, t, r - n, n, 0), s.wnext = n, s.whave = s.wsize) : (s.wnext += i, s.wnext === s.wsize && (s.wnext = 0), s.whave < s.wsize && (s.whave += i))), 0;
                                                  }

                                                  r.inflateReset = o, r.inflateReset2 = u, r.inflateResetKeep = a, r.inflateInit = function (e) {
                                                    return h(e, 15);
                                                  }, r.inflateInit2 = h, r.inflate = function (e, t) {
                                                    var r,
                                                        n,
                                                        i,
                                                        s,
                                                        a,
                                                        o,
                                                        u,
                                                        h,
                                                        f,
                                                        l,
                                                        d,
                                                        c,
                                                        p,
                                                        m,
                                                        _,
                                                        g,
                                                        v,
                                                        b,
                                                        w,
                                                        y,
                                                        k,
                                                        x,
                                                        S,
                                                        z,
                                                        E = 0,
                                                        C = new I.Buf8(4),
                                                        A = [16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15];

                                                    if (!e || !e.state || !e.output || !e.input && 0 !== e.avail_in) return U;
                                                    12 === (r = e.state).mode && (r.mode = 13), a = e.next_out, i = e.output, u = e.avail_out, s = e.next_in, n = e.input, o = e.avail_in, h = r.hold, f = r.bits, l = o, d = u, x = N;

                                                    e: for (;;) switch (r.mode) {
                                                      case P:
                                                        if (0 === r.wrap) {
                                                          r.mode = 13;
                                                          break;
                                                        }

                                                        for (; f < 16;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        if (2 & r.wrap && 35615 === h) {
                                                          C[r.check = 0] = 255 & h, C[1] = h >>> 8 & 255, r.check = B(r.check, C, 2, 0), f = h = 0, r.mode = 2;
                                                          break;
                                                        }

                                                        if (r.flags = 0, r.head && (r.head.done = !1), !(1 & r.wrap) || (((255 & h) << 8) + (h >> 8)) % 31) {
                                                          e.msg = "incorrect header check", r.mode = 30;
                                                          break;
                                                        }

                                                        if (8 != (15 & h)) {
                                                          e.msg = "unknown compression method", r.mode = 30;
                                                          break;
                                                        }

                                                        if (f -= 4, k = 8 + (15 & (h >>>= 4)), 0 === r.wbits) r.wbits = k;else if (k > r.wbits) {
                                                          e.msg = "invalid window size", r.mode = 30;
                                                          break;
                                                        }
                                                        r.dmax = 1 << k, e.adler = r.check = 1, r.mode = 512 & h ? 10 : 12, f = h = 0;
                                                        break;

                                                      case 2:
                                                        for (; f < 16;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        if (r.flags = h, 8 != (255 & r.flags)) {
                                                          e.msg = "unknown compression method", r.mode = 30;
                                                          break;
                                                        }

                                                        if (57344 & r.flags) {
                                                          e.msg = "unknown header flags set", r.mode = 30;
                                                          break;
                                                        }

                                                        r.head && (r.head.text = h >> 8 & 1), 512 & r.flags && (C[0] = 255 & h, C[1] = h >>> 8 & 255, r.check = B(r.check, C, 2, 0)), f = h = 0, r.mode = 3;

                                                      case 3:
                                                        for (; f < 32;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        r.head && (r.head.time = h), 512 & r.flags && (C[0] = 255 & h, C[1] = h >>> 8 & 255, C[2] = h >>> 16 & 255, C[3] = h >>> 24 & 255, r.check = B(r.check, C, 4, 0)), f = h = 0, r.mode = 4;

                                                      case 4:
                                                        for (; f < 16;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        r.head && (r.head.xflags = 255 & h, r.head.os = h >> 8), 512 & r.flags && (C[0] = 255 & h, C[1] = h >>> 8 & 255, r.check = B(r.check, C, 2, 0)), f = h = 0, r.mode = 5;

                                                      case 5:
                                                        if (1024 & r.flags) {
                                                          for (; f < 16;) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          r.length = h, r.head && (r.head.extra_len = h), 512 & r.flags && (C[0] = 255 & h, C[1] = h >>> 8 & 255, r.check = B(r.check, C, 2, 0)), f = h = 0;
                                                        } else r.head && (r.head.extra = null);

                                                        r.mode = 6;

                                                      case 6:
                                                        if (1024 & r.flags && (o < (c = r.length) && (c = o), c && (r.head && (k = r.head.extra_len - r.length, r.head.extra || (r.head.extra = new Array(r.head.extra_len)), I.arraySet(r.head.extra, n, s, c, k)), 512 & r.flags && (r.check = B(r.check, n, c, s)), o -= c, s += c, r.length -= c), r.length)) break e;
                                                        r.length = 0, r.mode = 7;

                                                      case 7:
                                                        if (2048 & r.flags) {
                                                          if (0 === o) break e;

                                                          for (c = 0; k = n[s + c++], r.head && k && r.length < 65536 && (r.head.name += String.fromCharCode(k)), k && c < o;);

                                                          if (512 & r.flags && (r.check = B(r.check, n, c, s)), o -= c, s += c, k) break e;
                                                        } else r.head && (r.head.name = null);

                                                        r.length = 0, r.mode = 8;

                                                      case 8:
                                                        if (4096 & r.flags) {
                                                          if (0 === o) break e;

                                                          for (c = 0; k = n[s + c++], r.head && k && r.length < 65536 && (r.head.comment += String.fromCharCode(k)), k && c < o;);

                                                          if (512 & r.flags && (r.check = B(r.check, n, c, s)), o -= c, s += c, k) break e;
                                                        } else r.head && (r.head.comment = null);

                                                        r.mode = 9;

                                                      case 9:
                                                        if (512 & r.flags) {
                                                          for (; f < 16;) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          if (h !== (65535 & r.check)) {
                                                            e.msg = "header crc mismatch", r.mode = 30;
                                                            break;
                                                          }

                                                          f = h = 0;
                                                        }

                                                        r.head && (r.head.hcrc = r.flags >> 9 & 1, r.head.done = !0), e.adler = r.check = 0, r.mode = 12;
                                                        break;

                                                      case 10:
                                                        for (; f < 32;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        e.adler = r.check = L(h), f = h = 0, r.mode = 11;

                                                      case 11:
                                                        if (0 === r.havedict) return e.next_out = a, e.avail_out = u, e.next_in = s, e.avail_in = o, r.hold = h, r.bits = f, 2;
                                                        e.adler = r.check = 1, r.mode = 12;

                                                      case 12:
                                                        if (5 === t || 6 === t) break e;

                                                      case 13:
                                                        if (r.last) {
                                                          h >>>= 7 & f, f -= 7 & f, r.mode = 27;
                                                          break;
                                                        }

                                                        for (; f < 3;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        switch (r.last = 1 & h, f -= 1, 3 & (h >>>= 1)) {
                                                          case 0:
                                                            r.mode = 14;
                                                            break;

                                                          case 1:
                                                            if (j(r), r.mode = 20, 6 !== t) break;
                                                            h >>>= 2, f -= 2;
                                                            break e;

                                                          case 2:
                                                            r.mode = 17;
                                                            break;

                                                          case 3:
                                                            e.msg = "invalid block type", r.mode = 30;
                                                        }

                                                        h >>>= 2, f -= 2;
                                                        break;

                                                      case 14:
                                                        for (h >>>= 7 & f, f -= 7 & f; f < 32;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        if ((65535 & h) != (h >>> 16 ^ 65535)) {
                                                          e.msg = "invalid stored block lengths", r.mode = 30;
                                                          break;
                                                        }

                                                        if (r.length = 65535 & h, f = h = 0, r.mode = 15, 6 === t) break e;

                                                      case 15:
                                                        r.mode = 16;

                                                      case 16:
                                                        if (c = r.length) {
                                                          if (o < c && (c = o), u < c && (c = u), 0 === c) break e;
                                                          I.arraySet(i, n, s, c, a), o -= c, s += c, u -= c, a += c, r.length -= c;
                                                          break;
                                                        }

                                                        r.mode = 12;
                                                        break;

                                                      case 17:
                                                        for (; f < 14;) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        if (r.nlen = 257 + (31 & h), h >>>= 5, f -= 5, r.ndist = 1 + (31 & h), h >>>= 5, f -= 5, r.ncode = 4 + (15 & h), h >>>= 4, f -= 4, 286 < r.nlen || 30 < r.ndist) {
                                                          e.msg = "too many length or distance symbols", r.mode = 30;
                                                          break;
                                                        }

                                                        r.have = 0, r.mode = 18;

                                                      case 18:
                                                        for (; r.have < r.ncode;) {
                                                          for (; f < 3;) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          r.lens[A[r.have++]] = 7 & h, h >>>= 3, f -= 3;
                                                        }

                                                        for (; r.have < 19;) r.lens[A[r.have++]] = 0;

                                                        if (r.lencode = r.lendyn, r.lenbits = 7, S = {
                                                          bits: r.lenbits
                                                        }, x = R(0, r.lens, 0, 19, r.lencode, 0, r.work, S), r.lenbits = S.bits, x) {
                                                          e.msg = "invalid code lengths set", r.mode = 30;
                                                          break;
                                                        }

                                                        r.have = 0, r.mode = 19;

                                                      case 19:
                                                        for (; r.have < r.nlen + r.ndist;) {
                                                          for (; g = (E = r.lencode[h & (1 << r.lenbits) - 1]) >>> 16 & 255, v = 65535 & E, !((_ = E >>> 24) <= f);) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          if (v < 16) h >>>= _, f -= _, r.lens[r.have++] = v;else {
                                                            if (16 === v) {
                                                              for (z = _ + 2; f < z;) {
                                                                if (0 === o) break e;
                                                                o--, h += n[s++] << f, f += 8;
                                                              }

                                                              if (h >>>= _, f -= _, 0 === r.have) {
                                                                e.msg = "invalid bit length repeat", r.mode = 30;
                                                                break;
                                                              }

                                                              k = r.lens[r.have - 1], c = 3 + (3 & h), h >>>= 2, f -= 2;
                                                            } else if (17 === v) {
                                                              for (z = _ + 3; f < z;) {
                                                                if (0 === o) break e;
                                                                o--, h += n[s++] << f, f += 8;
                                                              }

                                                              f -= _, k = 0, c = 3 + (7 & (h >>>= _)), h >>>= 3, f -= 3;
                                                            } else {
                                                              for (z = _ + 7; f < z;) {
                                                                if (0 === o) break e;
                                                                o--, h += n[s++] << f, f += 8;
                                                              }

                                                              f -= _, k = 0, c = 11 + (127 & (h >>>= _)), h >>>= 7, f -= 7;
                                                            }

                                                            if (r.have + c > r.nlen + r.ndist) {
                                                              e.msg = "invalid bit length repeat", r.mode = 30;
                                                              break;
                                                            }

                                                            for (; c--;) r.lens[r.have++] = k;
                                                          }
                                                        }

                                                        if (30 === r.mode) break;

                                                        if (0 === r.lens[256]) {
                                                          e.msg = "invalid code -- missing end-of-block", r.mode = 30;
                                                          break;
                                                        }

                                                        if (r.lenbits = 9, S = {
                                                          bits: r.lenbits
                                                        }, x = R(D, r.lens, 0, r.nlen, r.lencode, 0, r.work, S), r.lenbits = S.bits, x) {
                                                          e.msg = "invalid literal/lengths set", r.mode = 30;
                                                          break;
                                                        }

                                                        if (r.distbits = 6, r.distcode = r.distdyn, S = {
                                                          bits: r.distbits
                                                        }, x = R(F, r.lens, r.nlen, r.ndist, r.distcode, 0, r.work, S), r.distbits = S.bits, x) {
                                                          e.msg = "invalid distances set", r.mode = 30;
                                                          break;
                                                        }

                                                        if (r.mode = 20, 6 === t) break e;

                                                      case 20:
                                                        r.mode = 21;

                                                      case 21:
                                                        if (6 <= o && 258 <= u) {
                                                          e.next_out = a, e.avail_out = u, e.next_in = s, e.avail_in = o, r.hold = h, r.bits = f, T(e, d), a = e.next_out, i = e.output, u = e.avail_out, s = e.next_in, n = e.input, o = e.avail_in, h = r.hold, f = r.bits, 12 === r.mode && (r.back = -1);
                                                          break;
                                                        }

                                                        for (r.back = 0; g = (E = r.lencode[h & (1 << r.lenbits) - 1]) >>> 16 & 255, v = 65535 & E, !((_ = E >>> 24) <= f);) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        if (g && 0 == (240 & g)) {
                                                          for (b = _, w = g, y = v; g = (E = r.lencode[y + ((h & (1 << b + w) - 1) >> b)]) >>> 16 & 255, v = 65535 & E, !(b + (_ = E >>> 24) <= f);) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          h >>>= b, f -= b, r.back += b;
                                                        }

                                                        if (h >>>= _, f -= _, r.back += _, r.length = v, 0 === g) {
                                                          r.mode = 26;
                                                          break;
                                                        }

                                                        if (32 & g) {
                                                          r.back = -1, r.mode = 12;
                                                          break;
                                                        }

                                                        if (64 & g) {
                                                          e.msg = "invalid literal/length code", r.mode = 30;
                                                          break;
                                                        }

                                                        r.extra = 15 & g, r.mode = 22;

                                                      case 22:
                                                        if (r.extra) {
                                                          for (z = r.extra; f < z;) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          r.length += h & (1 << r.extra) - 1, h >>>= r.extra, f -= r.extra, r.back += r.extra;
                                                        }

                                                        r.was = r.length, r.mode = 23;

                                                      case 23:
                                                        for (; g = (E = r.distcode[h & (1 << r.distbits) - 1]) >>> 16 & 255, v = 65535 & E, !((_ = E >>> 24) <= f);) {
                                                          if (0 === o) break e;
                                                          o--, h += n[s++] << f, f += 8;
                                                        }

                                                        if (0 == (240 & g)) {
                                                          for (b = _, w = g, y = v; g = (E = r.distcode[y + ((h & (1 << b + w) - 1) >> b)]) >>> 16 & 255, v = 65535 & E, !(b + (_ = E >>> 24) <= f);) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          h >>>= b, f -= b, r.back += b;
                                                        }

                                                        if (h >>>= _, f -= _, r.back += _, 64 & g) {
                                                          e.msg = "invalid distance code", r.mode = 30;
                                                          break;
                                                        }

                                                        r.offset = v, r.extra = 15 & g, r.mode = 24;

                                                      case 24:
                                                        if (r.extra) {
                                                          for (z = r.extra; f < z;) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          r.offset += h & (1 << r.extra) - 1, h >>>= r.extra, f -= r.extra, r.back += r.extra;
                                                        }

                                                        if (r.offset > r.dmax) {
                                                          e.msg = "invalid distance too far back", r.mode = 30;
                                                          break;
                                                        }

                                                        r.mode = 25;

                                                      case 25:
                                                        if (0 === u) break e;

                                                        if (c = d - u, r.offset > c) {
                                                          if ((c = r.offset - c) > r.whave && r.sane) {
                                                            e.msg = "invalid distance too far back", r.mode = 30;
                                                            break;
                                                          }

                                                          p = c > r.wnext ? (c -= r.wnext, r.wsize - c) : r.wnext - c, c > r.length && (c = r.length), m = r.window;
                                                        } else m = i, p = a - r.offset, c = r.length;

                                                        for (u < c && (c = u), u -= c, r.length -= c; i[a++] = m[p++], --c;);

                                                        0 === r.length && (r.mode = 21);
                                                        break;

                                                      case 26:
                                                        if (0 === u) break e;
                                                        i[a++] = r.length, u--, r.mode = 21;
                                                        break;

                                                      case 27:
                                                        if (r.wrap) {
                                                          for (; f < 32;) {
                                                            if (0 === o) break e;
                                                            o--, h |= n[s++] << f, f += 8;
                                                          }

                                                          if (d -= u, e.total_out += d, r.total += d, d && (e.adler = r.check = r.flags ? B(r.check, i, d, a - d) : O(r.check, i, d, a - d)), d = u, (r.flags ? h : L(h)) !== r.check) {
                                                            e.msg = "incorrect data check", r.mode = 30;
                                                            break;
                                                          }

                                                          f = h = 0;
                                                        }

                                                        r.mode = 28;

                                                      case 28:
                                                        if (r.wrap && r.flags) {
                                                          for (; f < 32;) {
                                                            if (0 === o) break e;
                                                            o--, h += n[s++] << f, f += 8;
                                                          }

                                                          if (h !== (4294967295 & r.total)) {
                                                            e.msg = "incorrect length check", r.mode = 30;
                                                            break;
                                                          }

                                                          f = h = 0;
                                                        }

                                                        r.mode = 29;

                                                      case 29:
                                                        x = 1;
                                                        break e;

                                                      case 30:
                                                        x = -3;
                                                        break e;

                                                      case 31:
                                                        return -4;

                                                      case 32:
                                                      default:
                                                        return U;
                                                    }

                                                    return e.next_out = a, e.avail_out = u, e.next_in = s, e.avail_in = o, r.hold = h, r.bits = f, (r.wsize || d !== e.avail_out && r.mode < 30 && (r.mode < 27 || 4 !== t)) && Z(e, e.output, e.next_out, d - e.avail_out) ? (r.mode = 31, -4) : (l -= e.avail_in, d -= e.avail_out, e.total_in += l, e.total_out += d, r.total += d, r.wrap && d && (e.adler = r.check = r.flags ? B(r.check, i, d, e.next_out - d) : O(r.check, i, d, e.next_out - d)), e.data_type = r.bits + (r.last ? 64 : 0) + (12 === r.mode ? 128 : 0) + (20 === r.mode || 15 === r.mode ? 256 : 0), (0 == l && 0 === d || 4 === t) && x === N && (x = -5), x);
                                                  }, r.inflateEnd = function (e) {
                                                    if (!e || !e.state) return U;
                                                    var t = e.state;
                                                    return t.window && (t.window = null), e.state = null, N;
                                                  }, r.inflateGetHeader = function (e, t) {
                                                    var r;
                                                    return e && e.state ? 0 == (2 & (r = e.state).wrap) ? U : ((r.head = t).done = !1, N) : U;
                                                  }, r.inflateSetDictionary = function (e, t) {
                                                    var r,
                                                        n = t.length;
                                                    return e && e.state ? 0 !== (r = e.state).wrap && 11 !== r.mode ? U : 11 === r.mode && O(1, t, n, 0) !== r.check ? -3 : Z(e, t, n, n) ? (r.mode = 31, -4) : (r.havedict = 1, N) : U;
                                                  }, r.inflateInfo = "pako inflate (from Nodeca project)";
                                                }, {
                                                  "../utils/common": 41,
                                                  "./adler32": 43,
                                                  "./crc32": 45,
                                                  "./inffast": 48,
                                                  "./inftrees": 50
                                                }],
                                                50: [function (e, t, r) {

                                                  var D = e("../utils/common"),
                                                      F = [3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, 17, 19, 23, 27, 31, 35, 43, 51, 59, 67, 83, 99, 115, 131, 163, 195, 227, 258, 0, 0],
                                                      N = [16, 16, 16, 16, 16, 16, 16, 16, 17, 17, 17, 17, 18, 18, 18, 18, 19, 19, 19, 19, 20, 20, 20, 20, 21, 21, 21, 21, 16, 72, 78],
                                                      U = [1, 2, 3, 4, 5, 7, 9, 13, 17, 25, 33, 49, 65, 97, 129, 193, 257, 385, 513, 769, 1025, 1537, 2049, 3073, 4097, 6145, 8193, 12289, 16385, 24577, 0, 0],
                                                      P = [16, 16, 16, 16, 17, 17, 18, 18, 19, 19, 20, 20, 21, 21, 22, 22, 23, 23, 24, 24, 25, 25, 26, 26, 27, 27, 28, 28, 29, 29, 64, 64];

                                                  t.exports = function (e, t, r, n, i, s, a, o) {
                                                    var u,
                                                        h,
                                                        f,
                                                        l,
                                                        d,
                                                        c,
                                                        p,
                                                        m,
                                                        _,
                                                        g = o.bits,
                                                        v = 0,
                                                        b = 0,
                                                        w = 0,
                                                        y = 0,
                                                        k = 0,
                                                        x = 0,
                                                        S = 0,
                                                        z = 0,
                                                        E = 0,
                                                        C = 0,
                                                        A = null,
                                                        I = 0,
                                                        O = new D.Buf16(16),
                                                        B = new D.Buf16(16),
                                                        T = null,
                                                        R = 0;

                                                    for (v = 0; v <= 15; v++) O[v] = 0;

                                                    for (b = 0; b < n; b++) O[t[r + b]]++;

                                                    for (k = g, y = 15; 1 <= y && 0 === O[y]; y--);

                                                    if (y < k && (k = y), 0 === y) return i[s++] = 20971520, i[s++] = 20971520, o.bits = 1, 0;

                                                    for (w = 1; w < y && 0 === O[w]; w++);

                                                    for (k < w && (k = w), v = z = 1; v <= 15; v++) if (z <<= 1, (z -= O[v]) < 0) return -1;

                                                    if (0 < z && (0 === e || 1 !== y)) return -1;

                                                    for (B[1] = 0, v = 1; v < 15; v++) B[v + 1] = B[v] + O[v];

                                                    for (b = 0; b < n; b++) 0 !== t[r + b] && (a[B[t[r + b]]++] = b);

                                                    if (c = 0 === e ? (A = T = a, 19) : 1 === e ? (A = F, I -= 257, T = N, R -= 257, 256) : (A = U, T = P, -1), v = w, d = s, S = b = C = 0, f = -1, l = (E = 1 << (x = k)) - 1, 1 === e && 852 < E || 2 === e && 592 < E) return 1;

                                                    for (;;) {
                                                      for (p = v - S, _ = a[b] < c ? (m = 0, a[b]) : a[b] > c ? (m = T[R + a[b]], A[I + a[b]]) : (m = 96, 0), u = 1 << v - S, w = h = 1 << x; i[d + (C >> S) + (h -= u)] = p << 24 | m << 16 | _ | 0, 0 !== h;);

                                                      for (u = 1 << v - 1; C & u;) u >>= 1;

                                                      if (0 !== u ? (C &= u - 1, C += u) : C = 0, b++, 0 == --O[v]) {
                                                        if (v === y) break;
                                                        v = t[r + a[b]];
                                                      }

                                                      if (k < v && (C & l) !== f) {
                                                        for (0 === S && (S = k), d += w, z = 1 << (x = v - S); x + S < y && !((z -= O[x + S]) <= 0);) x++, z <<= 1;

                                                        if (E += 1 << x, 1 === e && 852 < E || 2 === e && 592 < E) return 1;
                                                        i[f = C & l] = k << 24 | x << 16 | d - s | 0;
                                                      }
                                                    }

                                                    return 0 !== C && (i[d + C] = v - S << 24 | 64 << 16 | 0), o.bits = k, 0;
                                                  };
                                                }, {
                                                  "../utils/common": 41
                                                }],
                                                51: [function (e, t, r) {

                                                  t.exports = {
                                                    2: "need dictionary",
                                                    1: "stream end",
                                                    0: "",
                                                    "-1": "file error",
                                                    "-2": "stream error",
                                                    "-3": "data error",
                                                    "-4": "insufficient memory",
                                                    "-5": "buffer error",
                                                    "-6": "incompatible version"
                                                  };
                                                }, {}],
                                                52: [function (e, t, r) {

                                                  var o = e("../utils/common");

                                                  function n(e) {
                                                    for (var t = e.length; 0 <= --t;) e[t] = 0;
                                                  }

                                                  var _ = 15,
                                                      i = 16,
                                                      u = [0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 0],
                                                      h = [0, 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 10, 11, 11, 12, 12, 13, 13],
                                                      a = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 3, 7],
                                                      f = [16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15],
                                                      l = new Array(576);
                                                  n(l);
                                                  var d = new Array(60);
                                                  n(d);
                                                  var c = new Array(512);
                                                  n(c);
                                                  var p = new Array(256);
                                                  n(p);
                                                  var m = new Array(29);
                                                  n(m);
                                                  var g,
                                                      v,
                                                      b,
                                                      w = new Array(30);

                                                  function y(e, t, r, n, i) {
                                                    this.static_tree = e, this.extra_bits = t, this.extra_base = r, this.elems = n, this.max_length = i, this.has_stree = e && e.length;
                                                  }

                                                  function s(e, t) {
                                                    this.dyn_tree = e, this.max_code = 0, this.stat_desc = t;
                                                  }

                                                  function k(e) {
                                                    return e < 256 ? c[e] : c[256 + (e >>> 7)];
                                                  }

                                                  function x(e, t) {
                                                    e.pending_buf[e.pending++] = 255 & t, e.pending_buf[e.pending++] = t >>> 8 & 255;
                                                  }

                                                  function S(e, t, r) {
                                                    e.bi_valid > i - r ? (e.bi_buf |= t << e.bi_valid & 65535, x(e, e.bi_buf), e.bi_buf = t >> i - e.bi_valid, e.bi_valid += r - i) : (e.bi_buf |= t << e.bi_valid & 65535, e.bi_valid += r);
                                                  }

                                                  function z(e, t, r) {
                                                    S(e, r[2 * t], r[2 * t + 1]);
                                                  }

                                                  function E(e, t) {
                                                    for (var r = 0; r |= 1 & e, e >>>= 1, r <<= 1, 0 < --t;);

                                                    return r >>> 1;
                                                  }

                                                  function C(e, t, r) {
                                                    var n,
                                                        i,
                                                        s = new Array(_ + 1),
                                                        a = 0;

                                                    for (n = 1; n <= _; n++) s[n] = a = a + r[n - 1] << 1;

                                                    for (i = 0; i <= t; i++) {
                                                      var o = e[2 * i + 1];
                                                      0 !== o && (e[2 * i] = E(s[o]++, o));
                                                    }
                                                  }

                                                  function A(e) {
                                                    var t;

                                                    for (t = 0; t < 286; t++) e.dyn_ltree[2 * t] = 0;

                                                    for (t = 0; t < 30; t++) e.dyn_dtree[2 * t] = 0;

                                                    for (t = 0; t < 19; t++) e.bl_tree[2 * t] = 0;

                                                    e.dyn_ltree[512] = 1, e.opt_len = e.static_len = 0, e.last_lit = e.matches = 0;
                                                  }

                                                  function I(e) {
                                                    8 < e.bi_valid ? x(e, e.bi_buf) : 0 < e.bi_valid && (e.pending_buf[e.pending++] = e.bi_buf), e.bi_buf = 0, e.bi_valid = 0;
                                                  }

                                                  function O(e, t, r, n) {
                                                    var i = 2 * t,
                                                        s = 2 * r;
                                                    return e[i] < e[s] || e[i] === e[s] && n[t] <= n[r];
                                                  }

                                                  function B(e, t, r) {
                                                    for (var n = e.heap[r], i = r << 1; i <= e.heap_len && (i < e.heap_len && O(t, e.heap[i + 1], e.heap[i], e.depth) && i++, !O(t, n, e.heap[i], e.depth));) e.heap[r] = e.heap[i], r = i, i <<= 1;

                                                    e.heap[r] = n;
                                                  }

                                                  function T(e, t, r) {
                                                    var n,
                                                        i,
                                                        s,
                                                        a,
                                                        o = 0;
                                                    if (0 !== e.last_lit) for (; n = e.pending_buf[e.d_buf + 2 * o] << 8 | e.pending_buf[e.d_buf + 2 * o + 1], i = e.pending_buf[e.l_buf + o], o++, 0 === n ? z(e, i, t) : (z(e, (s = p[i]) + 256 + 1, t), 0 !== (a = u[s]) && S(e, i -= m[s], a), z(e, s = k(--n), r), 0 !== (a = h[s]) && S(e, n -= w[s], a)), o < e.last_lit;);
                                                    z(e, 256, t);
                                                  }

                                                  function R(e, t) {
                                                    var r,
                                                        n,
                                                        i,
                                                        s = t.dyn_tree,
                                                        a = t.stat_desc.static_tree,
                                                        o = t.stat_desc.has_stree,
                                                        u = t.stat_desc.elems,
                                                        h = -1;

                                                    for (e.heap_len = 0, e.heap_max = 573, r = 0; r < u; r++) 0 !== s[2 * r] ? (e.heap[++e.heap_len] = h = r, e.depth[r] = 0) : s[2 * r + 1] = 0;

                                                    for (; e.heap_len < 2;) s[2 * (i = e.heap[++e.heap_len] = h < 2 ? ++h : 0)] = 1, e.depth[i] = 0, e.opt_len--, o && (e.static_len -= a[2 * i + 1]);

                                                    for (t.max_code = h, r = e.heap_len >> 1; 1 <= r; r--) B(e, s, r);

                                                    for (i = u; r = e.heap[1], e.heap[1] = e.heap[e.heap_len--], B(e, s, 1), n = e.heap[1], e.heap[--e.heap_max] = r, e.heap[--e.heap_max] = n, s[2 * i] = s[2 * r] + s[2 * n], e.depth[i] = (e.depth[r] >= e.depth[n] ? e.depth[r] : e.depth[n]) + 1, s[2 * r + 1] = s[2 * n + 1] = i, e.heap[1] = i++, B(e, s, 1), 2 <= e.heap_len;);

                                                    e.heap[--e.heap_max] = e.heap[1], function (e, t) {
                                                      var r,
                                                          n,
                                                          i,
                                                          s,
                                                          a,
                                                          o,
                                                          u = t.dyn_tree,
                                                          h = t.max_code,
                                                          f = t.stat_desc.static_tree,
                                                          l = t.stat_desc.has_stree,
                                                          d = t.stat_desc.extra_bits,
                                                          c = t.stat_desc.extra_base,
                                                          p = t.stat_desc.max_length,
                                                          m = 0;

                                                      for (s = 0; s <= _; s++) e.bl_count[s] = 0;

                                                      for (u[2 * e.heap[e.heap_max] + 1] = 0, r = e.heap_max + 1; r < 573; r++) p < (s = u[2 * u[2 * (n = e.heap[r]) + 1] + 1] + 1) && (s = p, m++), u[2 * n + 1] = s, h < n || (e.bl_count[s]++, a = 0, c <= n && (a = d[n - c]), o = u[2 * n], e.opt_len += o * (s + a), l && (e.static_len += o * (f[2 * n + 1] + a)));

                                                      if (0 !== m) {
                                                        do {
                                                          for (s = p - 1; 0 === e.bl_count[s];) s--;

                                                          e.bl_count[s]--, e.bl_count[s + 1] += 2, e.bl_count[p]--, m -= 2;
                                                        } while (0 < m);

                                                        for (s = p; 0 !== s; s--) for (n = e.bl_count[s]; 0 !== n;) h < (i = e.heap[--r]) || (u[2 * i + 1] !== s && (e.opt_len += (s - u[2 * i + 1]) * u[2 * i], u[2 * i + 1] = s), n--);
                                                      }
                                                    }(e, t), C(s, h, e.bl_count);
                                                  }

                                                  function D(e, t, r) {
                                                    var n,
                                                        i,
                                                        s = -1,
                                                        a = t[1],
                                                        o = 0,
                                                        u = 7,
                                                        h = 4;

                                                    for (0 === a && (u = 138, h = 3), t[2 * (r + 1) + 1] = 65535, n = 0; n <= r; n++) i = a, a = t[2 * (n + 1) + 1], ++o < u && i === a || (o < h ? e.bl_tree[2 * i] += o : 0 !== i ? (i !== s && e.bl_tree[2 * i]++, e.bl_tree[32]++) : o <= 10 ? e.bl_tree[34]++ : e.bl_tree[36]++, s = i, h = (o = 0) === a ? (u = 138, 3) : i === a ? (u = 6, 3) : (u = 7, 4));
                                                  }

                                                  function F(e, t, r) {
                                                    var n,
                                                        i,
                                                        s = -1,
                                                        a = t[1],
                                                        o = 0,
                                                        u = 7,
                                                        h = 4;

                                                    for (0 === a && (u = 138, h = 3), n = 0; n <= r; n++) if (i = a, a = t[2 * (n + 1) + 1], !(++o < u && i === a)) {
                                                      if (o < h) for (; z(e, i, e.bl_tree), 0 != --o;);else 0 !== i ? (i !== s && (z(e, i, e.bl_tree), o--), z(e, 16, e.bl_tree), S(e, o - 3, 2)) : o <= 10 ? (z(e, 17, e.bl_tree), S(e, o - 3, 3)) : (z(e, 18, e.bl_tree), S(e, o - 11, 7));
                                                      s = i, h = (o = 0) === a ? (u = 138, 3) : i === a ? (u = 6, 3) : (u = 7, 4);
                                                    }
                                                  }

                                                  n(w);
                                                  var N = !1;

                                                  function U(e, t, r, n) {
                                                    var i, s, a;
                                                    S(e, 0 + (n ? 1 : 0), 3), s = t, a = r, I(i = e), x(i, a), x(i, ~a), o.arraySet(i.pending_buf, i.window, s, a, i.pending), i.pending += a;
                                                  }

                                                  r._tr_init = function (e) {
                                                    N || (function () {
                                                      var e,
                                                          t,
                                                          r,
                                                          n,
                                                          i,
                                                          s = new Array(_ + 1);

                                                      for (n = r = 0; n < 28; n++) for (m[n] = r, e = 0; e < 1 << u[n]; e++) p[r++] = n;

                                                      for (p[r - 1] = n, n = i = 0; n < 16; n++) for (w[n] = i, e = 0; e < 1 << h[n]; e++) c[i++] = n;

                                                      for (i >>= 7; n < 30; n++) for (w[n] = i << 7, e = 0; e < 1 << h[n] - 7; e++) c[256 + i++] = n;

                                                      for (t = 0; t <= _; t++) s[t] = 0;

                                                      for (e = 0; e <= 143;) l[2 * e + 1] = 8, e++, s[8]++;

                                                      for (; e <= 255;) l[2 * e + 1] = 9, e++, s[9]++;

                                                      for (; e <= 279;) l[2 * e + 1] = 7, e++, s[7]++;

                                                      for (; e <= 287;) l[2 * e + 1] = 8, e++, s[8]++;

                                                      for (C(l, 287, s), e = 0; e < 30; e++) d[2 * e + 1] = 5, d[2 * e] = E(e, 5);

                                                      g = new y(l, u, 257, 286, _), v = new y(d, h, 0, 30, _), b = new y(new Array(0), a, 0, 19, 7);
                                                    }(), N = !0), e.l_desc = new s(e.dyn_ltree, g), e.d_desc = new s(e.dyn_dtree, v), e.bl_desc = new s(e.bl_tree, b), e.bi_buf = 0, e.bi_valid = 0, A(e);
                                                  }, r._tr_stored_block = U, r._tr_flush_block = function (e, t, r, n) {
                                                    var i,
                                                        s,
                                                        a = 0;
                                                    0 < e.level ? (2 === e.strm.data_type && (e.strm.data_type = function (e) {
                                                      var t,
                                                          r = 4093624447;

                                                      for (t = 0; t <= 31; t++, r >>>= 1) if (1 & r && 0 !== e.dyn_ltree[2 * t]) return 0;

                                                      if (0 !== e.dyn_ltree[18] || 0 !== e.dyn_ltree[20] || 0 !== e.dyn_ltree[26]) return 1;

                                                      for (t = 32; t < 256; t++) if (0 !== e.dyn_ltree[2 * t]) return 1;

                                                      return 0;
                                                    }(e)), R(e, e.l_desc), R(e, e.d_desc), a = function (e) {
                                                      var t;

                                                      for (D(e, e.dyn_ltree, e.l_desc.max_code), D(e, e.dyn_dtree, e.d_desc.max_code), R(e, e.bl_desc), t = 18; 3 <= t && 0 === e.bl_tree[2 * f[t] + 1]; t--);

                                                      return e.opt_len += 3 * (t + 1) + 5 + 5 + 4, t;
                                                    }(e), i = e.opt_len + 3 + 7 >>> 3, (s = e.static_len + 3 + 7 >>> 3) <= i && (i = s)) : i = s = r + 5, r + 4 <= i && -1 !== t ? U(e, t, r, n) : 4 === e.strategy || s === i ? (S(e, 2 + (n ? 1 : 0), 3), T(e, l, d)) : (S(e, 4 + (n ? 1 : 0), 3), function (e, t, r, n) {
                                                      var i;

                                                      for (S(e, t - 257, 5), S(e, r - 1, 5), S(e, n - 4, 4), i = 0; i < n; i++) S(e, e.bl_tree[2 * f[i] + 1], 3);

                                                      F(e, e.dyn_ltree, t - 1), F(e, e.dyn_dtree, r - 1);
                                                    }(e, e.l_desc.max_code + 1, e.d_desc.max_code + 1, a + 1), T(e, e.dyn_ltree, e.dyn_dtree)), A(e), n && I(e);
                                                  }, r._tr_tally = function (e, t, r) {
                                                    return e.pending_buf[e.d_buf + 2 * e.last_lit] = t >>> 8 & 255, e.pending_buf[e.d_buf + 2 * e.last_lit + 1] = 255 & t, e.pending_buf[e.l_buf + e.last_lit] = 255 & r, e.last_lit++, 0 === t ? e.dyn_ltree[2 * r]++ : (e.matches++, t--, e.dyn_ltree[2 * (p[r] + 256 + 1)]++, e.dyn_dtree[2 * k(t)]++), e.last_lit === e.lit_bufsize - 1;
                                                  }, r._tr_align = function (e) {
                                                    var t;
                                                    S(e, 2, 3), z(e, 256, l), 16 === (t = e).bi_valid ? (x(t, t.bi_buf), t.bi_buf = 0, t.bi_valid = 0) : 8 <= t.bi_valid && (t.pending_buf[t.pending++] = 255 & t.bi_buf, t.bi_buf >>= 8, t.bi_valid -= 8);
                                                  };
                                                }, {
                                                  "../utils/common": 41
                                                }],
                                                53: [function (e, t, r) {

                                                  t.exports = function () {
                                                    this.input = null, this.next_in = 0, this.avail_in = 0, this.total_in = 0, this.output = null, this.next_out = 0, this.avail_out = 0, this.total_out = 0, this.msg = "", this.state = null, this.data_type = 2, this.adler = 0;
                                                  };
                                                }, {}],
                                                54: [function (e, t, r) {

                                                  t.exports = "function" == typeof setImmediate ? setImmediate : function () {
                                                    var e = [].slice.apply(arguments);
                                                    e.splice(1, 0, 0), setTimeout.apply(null, e);
                                                  };
                                                }, {}]
                                              }, {}, [10])(10);
                                            });
                                          }).call(this, void 0 !== r ? r : "undefined" != typeof self ? self : "undefined" != typeof window ? window : {});
                                        }, {}]
                                      }, {}, [1])(1);
                                    });
                                  }).call(this, void 0 !== r ? r : "undefined" != typeof self ? self : "undefined" != typeof window ? window : {});
                                }, {}]
                              }, {}, [1])(1);
                            });
                          }).call(this, void 0 !== r ? r : "undefined" != typeof self ? self : "undefined" != typeof window ? window : {});
                        }, {}]
                      }, {}, [1])(1);
                    });
                  }).call(this, void 0 !== r ? r : "undefined" != typeof self ? self : "undefined" != typeof window ? window : {});
                }, {}]
              }, {}, [1])(1);
            });
          }).call(this, "undefined" != typeof commonjsGlobal ? commonjsGlobal : "undefined" != typeof self ? self : "undefined" != typeof window ? window : {});
        }, {}]
      }, {}, [1])(1);
    });
  });

  function extractFilePaths(pathSpectrum, options = {}) {
    let {
      zipFiles
    } = options;
    const expnoCheck = pathSpectrum.replace(/([.*\w+/]*)([0-9]+\/)[pdata|fid|ser]\/*.*/, '$1$2');
    const procnoCheck = pathSpectrum.match('pdata') ? pathSpectrum.replace(/([.*\w+/]*[0-9]+\/pdata\/[0-9]+\/)*.*/, '$1') : null;
    let filePaths = [];

    for (let file in zipFiles) {
      if (expnoCheck !== file.replace(/([.*/]*)((?<!pdata\/)[0-9]+\/).*/, '$1$2')) {
        continue;
      }

      if (file.match('pdata')) {
        if (!procnoCheck) {
          if (file.match(/[1|2]+[i|r]+[i|r]*/)) continue;
        } else if (procnoCheck !== file.replace(/([.*\w+/]*)(?<!pdata)([0-9]+\/)[pdata|fid|ser]*\/*.*/, '$1$2')) {
          continue;
        }
      }

      if (file.endsWith('/')) continue;
      filePaths.push(file);
    }

    return filePaths;
  }
  async function extractSingleSpectrumZip(pathSpectrum, options = {}) {
    let {
      zipFiles
    } = options;
    const filePaths = extractFilePaths(pathSpectrum, {
      zipFiles
    });
    let zipFolder = new jszip_min();

    for (let file of filePaths) {
      zipFolder.file(file, await zipFiles[file].async('arraybuffer'));
    }

    return {
      extension: 'zip',
      name: pathSpectrum,
      binary: await zipFolder.generateAsync({
        type: 'uint8array',
        compression: 'DEFLATE',
        compressionOptions: {
          level: 9
        }
      })
    };
  }

  const BINARY = 1;
  const TEXT = 2;
  /**
   * Load a zip file using jszip and looking for folders contaning bruker files to parse.
   * @param {*} zipFile - binary or base64 file of the bruker zip file.
   * @param {object} [options={}] - options for jszip and convertFolder functions.
   * @param {boolean} [options.keepOriginal=false] - options to keep a smaller zip file per spectrum.
   * @returns {Promise} - Array of spectrum parsed from fid, ser, 2rr, 1r bruker files.
   */

  function convertZip(zipFile, options = {}) {
    const {
      keepOriginal = false
    } = options;
    const jsZip = new jszip();
    return jsZip.loadAsync(zipFile, options).then(zip => {
      let files = {
        ser: BINARY,
        fid: BINARY,
        acqus: TEXT,
        acqu2s: TEXT,
        procs: TEXT,
        proc2s: TEXT,
        '1r': BINARY,
        '1i': BINARY,
        '2rr': BINARY
      };
      let folders = zip.filter(function (relativePath) {
        if (relativePath.match('__MACOSX')) return false;

        if (relativePath.endsWith('ser') || relativePath.endsWith('fid') || relativePath.endsWith('1r') || relativePath.endsWith('2rr')) {
          return true;
        }

        return false;
      });
      let spectra = new Array(folders.length);

      for (let i = 0; i < folders.length; ++i) {
        let promises = [];
        let name = folders[i].name;
        name = name.substr(0, name.lastIndexOf('/') + 1);
        promises.push(name);
        let currFolder = zip.folder(name);
        let currFiles = currFolder.filter(function (relativePath) {
          return files[relativePath] ? true : false;
        });

        if (name.indexOf('pdata') >= 0) {
          promises.push('acqus');
          promises.push(zip.file(name.replace(/pdata\/[0-9]+\//, 'acqus')).async('string'));
          let acqu2s = zip.file(name.replace(/pdata\/[0-9]+\//, 'acqu2s'));

          if (acqu2s) {
            promises.push('acqu2s');
            promises.push(acqu2s.async('string'));
          }
        }

        for (let j = 0; j < currFiles.length; ++j) {
          let idx = currFiles[j].name.lastIndexOf('/');
          name = currFiles[j].name.substr(idx + 1);
          promises.push(name);

          if (files[name] === BINARY) {
            promises.push(currFiles[j].async('arraybuffer'));
          } else {
            promises.push(currFiles[j].async('string'));
          }
        }

        if (keepOriginal) {
          promises.push('file');
          promises.push(extractSingleSpectrumZip(folders[i].name, {
            zipFiles: zip.files
          }));
        }

        spectra[i] = Promise.all(promises).then(result => {
          let brukerFiles = {};

          for (let k = 1; k < result.length; k += 2) {
            name = result[k];
            brukerFiles[name] = result[k + 1];
          }

          return {
            filename: result[0],
            value: convertFolder(brukerFiles, options)
          };
        });
      }

      return Promise.all(spectra);
    });
  }

  function convertToFloatArray(data) {
    if (isAnyArray(data[0])) {
      return data.map(e => Float64Array.from(e));
    } else if (isAnyArray(data)) {
      return Float64Array.from(data);
    } else if (typeof data === 'object') {
      let keys = Object.keys(data);

      for (let key of keys) {
        data[key] = convertToFloatArray(data[key]);
      }

      return data;
    }

    return data;
  }

  /* eslint-disable no-loss-of-precision */

  /**
   * Returns the group delay for old Bruker NMR spectra
   * @param {number} gspfvs
   * @param {number} decim
   * @return {number}
   */
  function getDigitalFilterParameters(grpdly, dspfvs, decim) {
    let value;

    if (grpdly > 0) {
      value = Number(grpdly);
    } else {
      if (dspfvs > 14) {
        value = 0;
      } else {
        if (!brukerDspTable[dspfvs]) {
          throw new Error('dspfvs not in lookup table');
        } else {
          const dspfvsList = brukerDspTable[dspfvs];
          if (!dspfvsList[decim]) throw new Error('decim not in lookup table');
          value = dspfvsList[decim];
        }
      }
    }

    return value;
  }
  const brukerDspTable = {
    10: {
      2: 44.75,
      3: 33.5,
      4: 66.625,
      6: 59.083333333333333,
      8: 68.5625,
      12: 60.375,
      16: 69.53125,
      24: 61.020833333333333,
      32: 70.015625,
      48: 61.34375,
      64: 70.2578125,
      96: 61.505208333333333,
      128: 70.37890625,
      192: 61.5859375,
      256: 70.439453125,
      384: 61.626302083333333,
      512: 70.4697265625,
      768: 61.646484375,
      1024: 70.48486328125,
      1536: 61.656575520833333,
      2048: 70.492431640625
    },
    11: {
      2: 46,
      3: 36.5,
      4: 48,
      6: 50.166666666666667,
      8: 53.25,
      12: 69.5,
      16: 72.25,
      24: 70.166666666666667,
      32: 72.75,
      48: 70.5,
      64: 73,
      96: 70.666666666666667,
      128: 72.5,
      192: 71.333333333333333,
      256: 72.25,
      384: 71.666666666666667,
      512: 72.125,
      768: 71.833333333333333,
      1024: 72.0625,
      1536: 71.916666666666667,
      2048: 72.03125
    },
    12: {
      2: 46,
      3: 36.5,
      4: 48,
      6: 50.166666666666667,
      8: 53.25,
      12: 69.5,
      16: 71.625,
      24: 70.166666666666667,
      32: 72.125,
      48: 70.5,
      64: 72.375,
      96: 70.666666666666667,
      128: 72.5,
      192: 71.333333333333333,
      256: 72.25,
      384: 71.666666666666667,
      512: 72.125,
      768: 71.833333333333333,
      1024: 72.0625,
      1536: 71.916666666666667,
      2048: 72.03125
    },
    13: {
      2: 2.75,
      3: 2.8333333333333333,
      4: 2.875,
      6: 2.9166666666666667,
      8: 2.9375,
      12: 2.9583333333333333,
      16: 2.96875,
      24: 2.9791666666666667,
      32: 2.984375,
      48: 2.9895833333333333,
      64: 2.9921875,
      96: 2.9947916666666667
    }
  };

  function getNucleusFromMetadata(metaData, info, subfix) {
    let nucleus = [];

    if (metaData[`${subfix}AXNUC`]) {
      nucleus = metaData[`${subfix}AXNUC`];
      if (!Array.isArray(nucleus)) nucleus = [nucleus];
      nucleus = checkForNucleus(nucleus);
    }

    if (nucleus.length < 1 && metaData[`${subfix}NUC1`]) {
      nucleus = metaData[`${subfix}NUC1`];
      if (!Array.isArray(nucleus)) nucleus = [nucleus];
      nucleus = checkForNucleus(nucleus);
    }

    if (nucleus.length === 0) {
      if (metaData['.NUCLEUS']) {
        nucleus = metaData['.NUCLEUS'].split(',').map(nuc => nuc.trim());
      } else if (metaData['.OBSERVENUCLEUS']) {
        nucleus = [metaData['.OBSERVENUCLEUS'].replace(/[^A-Za-z0-9]/g, '')];
      } else {
        nucleus = getNucleusFrom2DExperiment(info.experiment);
      }

      nucleus = checkForNucleus(nucleus);
    }

    if (metaData['2D_X_NUCLEUS'] && metaData['2D_Y_NUCLEUS']) {
      nucleus = [metaData['2D_X_NUCLEUS'].replace(/[^A-Za-z0-9]/g, ''), metaData['2D_Y_NUCLEUS'].replace(/[^A-Za-z0-9]/g, '')];
    }

    return nucleus;
  }
  /**
   * Returns a list of likely nuclei based on an experiment string
   * This is really an hypothesis and should not be used
   * @param {string} experiment
   * @return {string[]}
   */

  function getNucleusFrom2DExperiment(experiment) {
    if (typeof experiment !== 'string') {
      return [];
    }

    experiment = experiment.toLowerCase();

    if (experiment.includes('jres')) {
      return ['1H', 'Hz'];
    }

    if (experiment.includes('hmbc') || experiment.includes('hsqc')) {
      return ['1H', '13C'];
    }

    if (experiment.includes('cosy') || experiment.includes('tocsy')) {
      return ['1H', '1H'];
    }

    return [];
  }

  function checkForNucleus(nucleus) {
    nucleus = nucleus.map(value => value.replace(/[^A-Za-z0-9]/g, '').replace('NA', '').replace('off', ''));
    let beforeLength = nucleus.length;
    nucleus = nucleus.filter(value => value);
    return nucleus.length !== beforeLength ? [] : nucleus;
  }

  /**
   * Returns an experiment string based on a pulse sequence
   * @param {string} pulse
   * @return {string}
   */
  function getSpectrumType(meta = {}, info = {}, options = {}) {
    const {
      subfix = ''
    } = options;
    if (meta === null) meta = {};
    if (typeof meta === 'string') meta = {
      pulseSequence: meta
    };
    let spectyp = info[`${subfix}SPECTYP`];
    spectyp = (Array.isArray(spectyp) ? spectyp[0] : spectyp || '').replace(/^<(.*)>$/, '$1') // eslint-disable-line prefer-named-capture-group
    .toLowerCase();
    if (spectyp.length > 0) return spectyp;
    let pulse = Array.isArray(meta.pulseSequence) ? meta.pulseSequence[0] : meta.pulseSequence || '';

    if (typeof pulse !== 'string') {
      return meta.dimension ? `${meta.dimension}d` : '';
    }

    pulse = pulse.toLowerCase();

    if (pulse.includes('zg') || pulse.includes('single_pulse_dec') || pulse.includes('udeft')) {
      return '1d';
    }

    if (pulse.includes('hsqct') || pulse.includes('invi') && (pulse.includes('ml') || pulse.includes('di'))) {
      return 'hsqctocsy';
    }

    if (pulse.includes('hsqc') || pulse.includes('invi')) {
      return 'hsqc';
    }

    if (pulse.includes('hmbc') || pulse.includes('inv4') && pulse.includes('lp')) {
      return 'hmbc';
    }

    if (pulse.includes('hmqc')) {
      return 'hmqc';
    }

    if (pulse.includes('cosy')) {
      return 'cosy';
    }

    if (pulse.includes('jres')) {
      return 'jres';
    }

    if (pulse.includes('tocsy') || pulse.includes('mlev') || pulse.includes('dipsi')) {
      return 'tocsy';
    }

    if (pulse.includes('noesy')) {
      return 'noesy';
    }

    if (pulse.includes('roesy')) {
      return 'roesy';
    }

    if (pulse.includes('dept')) {
      return 'dept';
    }

    if (pulse.includes('jmod') || pulse.includes('apt')) {
      return 'aptjmod';
    }

    return meta.dimension ? `${meta.dimension}d` : '';
  }

  function getInfoFromJCAMP(metaData, options = {}) {
    const {
      subfix = ''
    } = options;
    const info = {
      dimension: 0,
      nucleus: [],
      isFid: false,
      isFt: false,
      isComplex: false
    };
    let metadataString = JSON.stringify(metaData);
    const separator = metadataString.match('\r\n') ? '\r\n' : '\n';
    let {
      JCAMPDX: jcampdx = '',
      ORIGIN: origin = ''
    } = metaData;
    let creator = String(jcampdx).toLowerCase() + origin.toLowerCase();

    if (creator.includes('mestre') || creator.includes('nova')) {
      creator = 'mnova';
    }

    if (creator.includes('bruker')) {
      creator = 'bruker';
    }

    if (creator === 'mnova') {
      if (metaData.LONGDATE) {
        info.date = metaData.LONGDATE;
      }
    }

    info.nucleus = getNucleusFromMetadata(metaData, info, subfix);
    info.dimension = info.nucleus.length;
    maybeAdd(info, 'title', metaData.TITLE);
    maybeAdd(info, 'solvent', metaData['.SOLVENTNAME']);
    maybeAdd(info, 'temperature', metaData[`${subfix}TE`] || metaData['.TE']);
    maybeAdd(info, 'type', metaData.DATATYPE);

    if (info.type) {
      let typeLowerCase = info.type[0].toUpperCase();

      if (typeLowerCase.indexOf('FID') >= 0) {
        info.isFid = true;
        info.isComplex = true;
      } else if (typeLowerCase.indexOf('SPECTRUM') >= 0) {
        info.isFt = true;
        info.isComplex = true;
      }
    }

    maybeAdd(info, 'pulseSequence', metaData['.PULSESEQUENCE'] || metaData['.PULPROG'] || metaData[`${subfix}PULPROG`]);
    maybeAdd(info, 'experiment', getSpectrumType(info, metaData, {
      subfix
    }));
    maybeAdd(info, 'originFrequency', metaData['.OBSERVEFREQUENCY']);

    if (creator === 'bruker') {
      const gyromagneticRatioConst = gyromagneticRatio[info.nucleus[0]];
      maybeAdd(info, 'probeName', metaData[`${subfix}PROBHD`]);
      maybeAdd(info, 'originFrequency', metaData[`${subfix}SFO1`]);
      maybeAdd(info, 'baseFrequency', metaData[`${subfix}BF1`]);
      const {
        baseFrequency,
        originFrequency
      } = info;
      let fieldStrength = 2 * Math.PI * (baseFrequency[0] / gyromagneticRatioConst) * 1e6;
      let frequencyOffset = baseFrequency.map((bf, i) => (originFrequency[i] - bf) * 1e6);
      maybeAdd(info, 'fieldStrength', fieldStrength);
      maybeAdd(info, 'frequencyOffset', frequencyOffset);
      maybeAdd(info, 'spectralWidth', metaData[`${subfix}SW`]);
      maybeAdd(info, 'numberOfPoints', metaData[`${subfix}TD`]);
      maybeAdd(info, 'sampleName', metaData[`${subfix}NAME`]);

      if (metaData[`${subfix}FNTYPE`] !== undefined) {
        maybeAdd(info, 'acquisitionMode', parseInt(metaData[`${subfix}FNTYPE`], 10));
      }

      let varName = metaData[`${subfix}VARNAME`] ? metaData[`${subfix}VARNAME`].split(',')[0] : '';

      if (varName === 'TIME') {
        let value = typeof metaData.LAST === 'string' || metaData.LAST instanceof String ? metaData.LAST.replace(' ', '').split(',')[0] : metaData.LAST;
        maybeAdd(info, 'acquisitionTime', Number(value));
      }

      const {
        numberOfPoints,
        spectralWidth
      } = info;

      if (!info.acquisitionTime) {
        maybeAdd(info, 'acquisitionTime', Number((numberOfPoints[0] - 1) / (2 * spectralWidth[0] * originFrequency[0])));
      }

      let pulseStrength = 1e6 / (metaData[`${subfix}P`].split(separator)[1].split(' ')[1] * 4);
      maybeAdd(info, 'pulseStrength90', pulseStrength);
      let relaxationTime = metaData[`${subfix}D`].split(separator)[1].split(' ')[1];
      maybeAdd(info, 'relaxationTime', Number(relaxationTime));
      maybeAdd(info, 'numberOfScans', Number(metaData[`${subfix}NS`]));
      let increment;

      if (info.isFid) {
        maybeAdd(info, 'groupDelay', metaData[`${subfix}GRPDLY`] || 0);
        maybeAdd(info, 'DSPFVS', metaData[`${subfix}DSPFVS`]);
        maybeAdd(info, 'DECIM', metaData[`${subfix}DECIM`]);
        let {
          groupDelay,
          DSPFVS,
          DECIM
        } = info;
        let digitalFilterParameters = getDigitalFilterParameters(groupDelay[0], DSPFVS[0], DECIM[0]);
        maybeAdd(info, 'digitalFilter', digitalFilterParameters);
        increment = numberOfPoints.map(nb => {
          return info.acquisitionTime[0] / (nb - 1);
        });
      } else {
        increment = numberOfPoints.map((nb, i) => {
          return spectralWidth[i] / (nb - 1);
        });
      }

      maybeAdd(info, 'increment', increment);

      if (metaData[`${subfix}DATE`]) {
        info.date = new Date(parseInt(metaData[`${subfix}DATE`], 10) * 1000).toISOString();
      }

      if (!info.solvent) {
        maybeAdd(info, 'solvent', Array.isArray(metaData[`${subfix}SOLVENT`]) ? metaData[`${subfix}SOLVENT`][0] : metaData[`${subfix}SOLVENT`]);
      }
    }

    if (metaData.SYMBOL) {
      let symbols = metaData.SYMBOL.split(/[, ]+/);

      if (symbols.includes('R') && symbols.includes('I')) {
        info.isComplex = true;
      }
    }

    for (let key in info) {
      if (info[key].length === 1) info[key] = info[key][0];
    }

    if (!Array.isArray(info.nucleus)) info.nucleus = [info.nucleus];
    return info;
  }

  function maybeAdd(obj, name, value) {
    if (value !== undefined) {
      if (Array.isArray(value)) {
        obj[name] = value.map(v => {
          return removeUnless(v);
        });
      } else {
        obj[name] = [removeUnless(value)];
      }
    }
  }

  function removeUnless(value) {
    if (typeof value === 'string') {
      if (value.startsWith('<') && value.endsWith('>')) {
        value = value.substring(1, value.length - 1);
      }

      value = value.trim();
    }

    return value.match && value.match(/[A-Za-z]/g) ? value : parseFloat(value);
  }

  const defaultOptions = {
    noContour: true,
    xy: true,
    keepRecordsRegExp: /.*/,
    profiling: true
  };
  async function fromBruker(zipFile, options = {}) {
    let parseData = await convertZip(zipFile, Object.assign({}, defaultOptions, options));
    let dataStructure = [];

    for (let element of parseData) {
      let entry = element.value;
      let metadata = Object.assign({}, entry.info, entry.meta);
      let info = getInfoFromJCAMP(metadata);
      if (info.experiment === 'wobble_curve') continue;
      let dimensions = [];
      let dependentVariables = [];
      let dependentVariable = {};

      if (info.dimension === 1) {
        for (let i = 0; i < entry.spectra.length; i++) {
          let data = entry.spectra[i].data;
          data = convertToFloatArray(data);
        }

        dependentVariable.components = entry.spectra;
      } else if (info.dimension === 2) {
        entry.minMax.z = convertToFloatArray(entry.minMax.z);
        dependentVariable.components = entry.minMax;
      }

      let dimension = {
        increment: info.increment,
        numberOfPoints: info.numberOfPoints
      };

      if (info.fid) {
        dimension.coordinatesOffset = {
          magnitude: -info.digitalFilter * info.increment,
          units: 'second'
        };
      } else {
        dimension.coordinatesOffset = {
          magnitude: info.frequencyOffset / info.baseFrequency - 0.5 * info.spectraWidth,
          units: 'ppm'
        };
      }

      dimensions.push(dimension);
      dependentVariables.push(dependentVariable);
      const {
        source
      } = entry;
      dataStructure.push({
        dimensions,
        dependentVariables,
        source,
        info: info,
        meta: metadata,
        timeStamp: new Date().valueOf(),
        version: packageJson.version
      });
    }

    return dataStructure;
  }

  const expectedTypes = ['nd nmr spectrum', 'nd nmr fid', 'nmr spectrum', 'nmr fid'];
  function fromJCAMP(buffer) {
    let parsedData = convert(buffer, {
      noContour: true,
      xy: true,
      keepRecordsRegExp: /.*/,
      profiling: true
    });
    let dataStructure = [];
    let entries = parsedData.flatten;

    for (let entry of entries) {
      if (!isSpectraData(entry)) continue;

      if (entry.spectra && entry.spectra.length > 0 || entry.minMax) {
        let metadata = Object.assign({}, entry.info, entry.meta);
        let info = getInfoFromJCAMP(metadata);
        if (info.experiment === 'wobble_curve') continue;
        let dimensions = [];
        let dependentVariables = [];
        let dependentVariable = {};

        if (info.dimension === 1) {
          for (let i = 0; i < entry.spectra.length; i++) {
            let data = entry.spectra[i].data;
            data = convertToFloatArray(data);
          }

          dependentVariable.components = entry.spectra;
        } else if (info.dimension === 2) {
          entry.minMax.z = convertToFloatArray(entry.minMax.z);
          dependentVariable.components = entry.minMax;
        }

        let dimension = {
          increment: info.increment,
          numberOfPoints: info.numberOfPoints
        };

        if (info.fid) {
          dimension.coordinatesOffset = {
            magnitude: -info.digitalFilter * info.increment,
            units: 'second'
          };
        } else {
          dimension.coordinatesOffset = {
            magnitude: info.frequencyOffset / info.baseFrequency - 0.5 * info.spectraWidth,
            units: 'ppm'
          };
        }

        dimensions.push(dimension);
        dependentVariables.push(dependentVariable);
        dataStructure.push({
          dimensions,
          dependentVariables,
          info,
          meta: metadata,
          timeStamp: new Date().valueOf(),
          version: packageJson.version
        });
      }
    }

    return dataStructure;
  }

  function isSpectraData(entry) {
    const {
      dataType = '',
      dataClass = ''
    } = entry;
    const inputDataType = dataType.toLowerCase();
    const inputDataClass = dataClass.toLowerCase();
    return expectedTypes.some(type => type === inputDataType) && inputDataClass !== 'peak table';
  }

  exports.formatDependentVariable = formatDependentVariable;
  exports.formatLinearDimension = formatLinearDimension;
  exports.fromBruker = fromBruker;
  exports.fromJCAMP = fromJCAMP;
  exports.fromJEOL = fromJEOL;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=nmr-parser.js.map
